/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms;

 

import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.Application;
import org.apache.wicket.Page;
import org.apache.wicket.Session;
import org.apache.wicket.authroles.authentication.AbstractAuthenticatedWebSession;
import org.apache.wicket.authroles.authentication.AuthenticatedWebApplication;
import org.apache.wicket.core.request.mapper.CryptoMapper;
import org.apache.wicket.markup.html.IPackageResourceGuard;
import org.apache.wicket.markup.html.SecurePackageResourceGuard;
import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.request.Request;
import org.apache.wicket.request.Response;
import org.biffo.dms.auth.SignInPage;

import org.biffo.dms.config.WebAppConfig;
import org.biffo.dms.config.WebAppConfigParams;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.entity.Document.DocumentsDB;
import org.biffo.dms.entity.Document.DocumentsDBOS;
import org.biffo.dms.entity.Fragments.FragmentsDB;
import org.biffo.dms.entity.Processing.ProcessesDB;
import org.biffo.dms.entity.Relation.RelationsDB;
import org.biffo.dms.entity.Logentries.LogEntriesDB;
import org.biffo.dms.entity.Tags.TagsDB;
import org.biffo.dms.entity.Tasks.TasksDB;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariablesDB;
import org.biffo.dms.entity.Variables.Impl.DmsVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.Variables.Impl.OpenSearchVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.TesseractVariablesImpl;
import org.biffo.dms.entity.user.Tenant.TenantsDB;
import org.biffo.dms.entity.user.Usergroups.UserGroupsDB;

import org.biffo.dms.entity.user.Userrelation.UserRelationsDB;
import org.biffo.dms.entity.user.Users.UsersDB;
import org.biffo.dms.helpers.CheckDBConnection3;
import org.biffo.dms.helpers.CheckESConnection3;
import org.biffo.dms.lock.Locks;
import org.biffo.dms.opensearch.OpenSearchProcessing;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processQueue.ProcessQueue;
import org.biffo.dms.processQueue.importServer.ImportProcessTask;
import org.biffo.dms.processing.api.Modules;
import org.biffo.dms.resource.DmsImageResource;
import org.biffo.dms.resource.DmsPdfResource;
import org.biffo.dms.web.config.ConfigPage;
import org.biffo.dms.web.home.HomePage;

import de.kleinekuh.os.OsConnection;
import de.kleinekuh.os.indexes.OsIndex;
import de.kleinekuh.os.indexes.OsIndexes;
 

public class DmsWebApplication extends AuthenticatedWebApplication {

	private static final boolean IS_WINDOWS = System.getProperty( "os.name" ).contains( "indow" );
	
	private static final String _className = "DmsWebApplication";
	private static final ProcessQueue _pq = new ProcessQueue();
	private static final Locks _locks = new Locks();
	
	private final DmsProcessingMapper _dmp = new DmsProcessingMapper();
	private static final Modules _modules = new Modules(Application.class.getClassLoader());
	
	private WebAppConfig _wac;
	private boolean _noConfig=false;
	
	
	
	private final static String _driverName = "org.postgresql.Driver";
	private final static String _applicationName = "BIFFO";
	
	private String _applicationVersion;
	
	
	private DocumentsDB _documentsDB;
	private DocumentsDBOS _documentsDBOS;
	private RelationsDB _relationsDB;
	private TagsDB _tagsDB;
	private TasksDB _tasksDB;
	private VariablesDB _variablesDB;
	private FragmentsDB _fragmentsDB;
	
	private UsersDB _usersDB;
	private UserGroupsDB _usergroupsDB;
	private UserRelationsDB _userrelationsDB;
	private LogEntriesDB _logentriesDB;
	private TenantsDB _tenantsDB;
	private ProcessesDB _processesDB;
	
	private CheckDBConnection3 _dbConn;
	private CheckESConnection3 _esConn;
	
	private OsConnection _osConnection;
	private OsIndex _osIndexDms;
	
	private DmsWebMessageHandler _messageHandler;
	
	private OpenSearchProcessing _openSearchProcessing;
	
	
	public DmsWebApplication() {
		try{

			
			Class.forName(_driverName);
			Class.forName("org.h2.Driver");
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	
	
	
	
	private boolean needsDBConfig() {
		if(null==_wac)return true;
		if(!_wac.Exists())return true;
		WebAppConfigParams wacp = _wac.getConfig();
		if(wacp.getDBPassword()==null || wacp.getDBPassword().isEmpty())return true;
		if(wacp.getDBUrl()==null || wacp.getDBUrl().isEmpty())return true;
		if(wacp.getDBUser()==null || wacp.getDBUser().isEmpty())return true;
		return false;
	}
	
	private void addConfigHandler() {
		
		_wac = new WebAppConfig() {
			 
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnConfigChanged(ConfigState configState, WebAppConfigParams wacp) {
				switch (configState){
					case CREATED: 
						break;	
					case CHANGED:
						WebAppConfigParams params = _wac.getConfig();
						String dbURL = params.getDBUrl()+"?user="+params.getDBUser()+"&password="+params.getDBPassword();
						closeDBs();
						if(_dbConn!=null) {
							_dbConn.setDbURL(dbURL);
							_noConfig=false;
						}
						break;
					case DROPPED:
						try {
							_noConfig=true;
							_dbConn.setDbURL("");
							_esConn.setEsURL("");
						} catch (Exception e) {
							e.printStackTrace();
						}

						break;
				}
			}
			@Override
			protected void OnConfigError(DmsException e) {
			}
			
		};
		if(!_wac.Exists() || needsDBConfig()) {
			_noConfig=true;
		}else {
			WebAppConfigParams params = _wac.getConfig();
			String dbURL = params.getDBUrl()+"?user="+params.getDBUser()+"&password="+params.getDBPassword();
			if(_dbConn!=null) {
				_dbConn.setDbURL(dbURL);
				_noConfig=false;
			}else {
				_noConfig=true;
			}
		}
	}
	
	private void addDBHandler() {
		_dbConn = new CheckDBConnection3() {
			@Override
			protected void OnDBConnectionInvalid(Exception e) {
				_noConfig=true;
			}
			
			@Override
			protected void OnDBConnectionValid() {
				try {
					reinitDB();
				} catch (Exception e) {}
			}
		};
		_dbConn.start();
		
		_esConn = new CheckESConnection3() {
			@Override
			protected void OnEsConnectionInvalid(Exception e) {
				super.OnEsConnectionInvalid(e);
			}
			@Override
			protected void OnEsConnectionValid() {
				try {
					reinitOS();
				} catch (Exception e) {}
			}
		};
		_esConn.start();
	}
	

	
	@Override
	protected void init() {
		super.init();
		_applicationVersion= get().getInitParameter("applicationVersion");
		setRootRequestMapper(new CryptoMapper(getRootRequestMapper(), this));
		getStoreSettings().setInmemoryCacheSize(50);
		IPackageResourceGuard packageResourceGuard = getResourceSettings().getPackageResourceGuard();
		if (packageResourceGuard instanceof SecurePackageResourceGuard){
			SecurePackageResourceGuard guard = (SecurePackageResourceGuard)packageResourceGuard;
			guard.addPattern("+*.woff2");
			guard.addPattern("+*.map");
			guard.addPattern("+*.css");
			guard.addPattern("+*.pdf");
		}
		getApplicationSettings().setPageExpiredErrorPage(HomePage.class);
		getApplicationSettings().setAccessDeniedPage(HomePage.class);
		getApplicationSettings().setInternalErrorPage(HomePage.class);
		getDebugSettings().setAjaxDebugModeEnabled(false);
		
		addDBHandler();
		addConfigHandler();
	
		
		
	}
	
	@Override
	protected void onDestroy() {
		stopAllImporters();
		_pq.destroy();
		try{
			if(null!=_dbConn) {
				_dbConn.interrupt();
				closeDBs();
			}
		}catch(Exception e){
		}

		try{
			if(null!=_esConn)_esConn.interrupt();
		}catch(Exception e){
		}
		
		
		try{
			Enumeration<Driver> drivers = DriverManager.getDrivers();
	        while (drivers.hasMoreElements()) {
	            Driver driver = drivers.nextElement();
	            try {
	                DriverManager.deregisterDriver(driver);
	            } catch (SQLException e) {
	            }
	        }
		}catch(Exception e){
		}
		super.onDestroy();
	}

	@Override
	public final Session newSession(Request request, Response response) {
		return new DmsWebSession(request);
	}
	
	@Override 
	protected Class<? extends AbstractAuthenticatedWebSession> getWebSessionClass(){ 
		return DmsWebSession.class; 
	}

	@Override 
	protected Class<? extends WebPage> getSignInPageClass() { 
		return SignInPage.class; 
	}
	  
	
	@Override
	public Class<? extends Page> getHomePage() {
		if(_noConfig) {
			return ConfigPage.class;
		}else{
			return HomePage.class;
		}
	}
	
	public void reinitAll() throws Exception{
		stopAllImporters();		
		try {
			closeDBs();
		}catch(Exception e) {
			e.printStackTrace();
		}
		try {
			reinitDB();
		}catch(Exception e) {
			e.printStackTrace();
		}
		try {
			reinitOS();
		}catch(Exception e) {
			e.printStackTrace();
		}
		startAllImporters();
	}
	
	
	private void reinitDB() throws Exception{
		String dbURL=_dbConn.getDbUrl();
		_documentsDB = new DocumentsDB(dbURL, _driverName);
		_relationsDB = new RelationsDB(dbURL, _driverName);
		_tagsDB = new TagsDB(dbURL, _driverName);
		_tasksDB = new TasksDB(dbURL, _driverName);
		_variablesDB = new VariablesDB(dbURL, _driverName);
		_usersDB = new UsersDB(dbURL, _driverName);
		_usergroupsDB = new UserGroupsDB(dbURL, _driverName);
		_userrelationsDB = new UserRelationsDB(dbURL, _driverName);
		_logentriesDB = new LogEntriesDB(dbURL, _driverName);
		_tenantsDB = new TenantsDB(dbURL, _driverName);
		_processesDB = new ProcessesDB(dbURL, _driverName);
		_fragmentsDB = new FragmentsDB(dbURL, _driverName);

		reinitConfig();
		//startAllImporters();
		
	}
	
	private void closeDBs() {
		if(null!=_documentsDB)_documentsDB.close();
		if(null!=_relationsDB)_relationsDB.close();
		if(null!=_tagsDB)_tagsDB.close();
		if(null!=_tasksDB)_tasksDB.close();
		if(null!=_variablesDB)_variablesDB.close();
		if(null!=_usersDB)_usersDB.close();
		if(null!=_usergroupsDB)_usergroupsDB.close();
		if(null!=_userrelationsDB)_userrelationsDB.close();
		if(null!=_logentriesDB)_logentriesDB.close();
		if(null!=_tenantsDB)_tenantsDB.close();
		if(null!=_processesDB)_processesDB.close();
		if(null!=_fragmentsDB)_fragmentsDB.close();
		
	}
	
	public void reinitConfig() {
		
		_dmp.setDocumentsDB(_documentsDB);
		_dmp.setLogDB(_logentriesDB);
		_dmp.setRelationsDB(_relationsDB);
		_dmp.setTagsDB(_tagsDB);
		_dmp.setVariablesDB(_variablesDB);
		_dmp.setProcessesDB(_processesDB);
		_dmp.setFragmentsDB(_fragmentsDB);
		_dmp.setUsersDB(_usersDB);

		_dmp.setDmsConfig(getDmsConfig());
		_dmp.setTesseractConfig(getTesseractConfig());
		_dmp.setOpensearchConfig(getOpenSearchConfig());
		
		if(_dmp!=null) {
			_esConn.setEsURL(_dmp.getOpensearchConfig().getUrl());
			reinitOS();
			_openSearchProcessing = new OpenSearchProcessing(_dmp);
			_messageHandler = new DmsWebMessageHandler(_dmp);
			_modules.setModuleFolder(_dmp.getDmsConfig().getModuleFolder());
			addResourceReferences();
			_noConfig=false;
		}
		
		
		
	}
	
	
	private void reinitOS() {
		try{
			_osConnection = new OsConnection(_dmp.getOpensearchConfig().getUrl());
			OsIndexes indexes = new OsIndexes(_osConnection);
			_osIndexDms = indexes.get(_dmp.getOpensearchConfig().getIndexname());
			_documentsDBOS = new DocumentsDBOS(_dbConn.getDbUrl(), _driverName, _osConnection, _dmp.getOpensearchConfig().getIndexname());
		}catch(Exception e){
			DmsException dmsException = new DmsException(1000, "E", _className, "reinitOS", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	} 
	
	
	private void stopAllImporters() {
		try {

			int size = (int) _pq.countTasks();
			Iterator<ProcessBridge> itt = _pq.getTasksPage(0, size);
			while(itt.hasNext()) {
				ProcessBridge pb = itt.next();
				pb.setStop(true);
				Thread.sleep(100);
			}
			_pq.pruneFinishedTasks();
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "stopAllImporters", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	}
	
	
	
	private void startAllImporters() {
		try {
			
			DPImporters _qtdp = new DPImporters();
			long size = _qtdp.size();
			Iterator<? extends ImportServerImpl> itt = _qtdp.iterator(0, size);
			while(itt.hasNext()) {
				ImportServerImpl impServer = itt.next();
				if(impServer.isServerAutostart()) {
					getProcessQueue().pruneFinishedTasks();
					if(!getProcessQueue().hasTask(impServer.getProcessName())) {
						getProcessQueue().add( new ImportProcessTask(impServer));
					}
				}
			}
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "startAllImporters", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	}

	
	private void addResourceReferences() {
		mountResource("/png/${params}", new DmsImageResource("png", _dmp.getDmsConfig().getDataFolder()));
		mountResource("/pdf/${params}", new DmsPdfResource("pdf", _dmp.getDmsConfig().getDataFolder()));
	}
	
	public static DmsWebApplication getApplication() {return (DmsWebApplication)get();}
	public String getApplicationName(){return _applicationName;}
	
	public static Locks getLocks(){return _locks;}
	public static ProcessQueue getProcessQueue() {return _pq;}
	public static Modules getModules() {return _modules;}
	public static boolean isWindows() {return IS_WINDOWS;}
	
	private static DmsVariablesImpl getDmsConfig() {
		try {
			DmsVariablesImpl dbVariable = new DmsVariablesImpl();
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", DmsVariablesImpl.getDefaultVariableFilterClass());
			for(Variable entity:entities) {
				dbVariable.setVariable(entity);
			}
			return dbVariable;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getDmsConfig", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
		return null;
	}
	
	private static TesseractVariablesImpl getTesseractConfig() {
		
		try {
			TesseractVariablesImpl dbVariable = new TesseractVariablesImpl();
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", TesseractVariablesImpl.getDefaultVariableFilterClass());
			for(Variable entity:entities) {
				dbVariable.setVariable(entity);
			}
			return dbVariable;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getTesseractConfig", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
		return null;
	}
	
	private static OpenSearchVariablesImpl getOpenSearchConfig() {
		
		try {
			OpenSearchVariablesImpl dbVariable = new OpenSearchVariablesImpl();
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", OpenSearchVariablesImpl.getDefaultVariableFilterClass());
			for(Variable entity:entities) {
				dbVariable.setVariable(entity);
			}
			return dbVariable;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getOpenSearchConfig", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
		return null;
	}
	
	public OsConnection getOsConnection() {return _osConnection;}
	public OsIndex getOsIndex() {return _osIndexDms;}
	
	public DocumentsDB getDocumentsDB() {return _documentsDB;}
	public DocumentsDBOS getDocumentsDBOS() {return _documentsDBOS;}
	public RelationsDB getRelationsDB(){return _relationsDB;}
	public TagsDB getTagsDB() {return _tagsDB;}
	public TasksDB getTasksDB() {return _tasksDB;}
	public VariablesDB getVariablesDB() {return _variablesDB;}
	public LogEntriesDB getLogEntriesDB(){return _logentriesDB;}	
	public ProcessesDB getProcessesDB(){return _processesDB;}
	public FragmentsDB getFragmentsDB() {return _fragmentsDB;}
	
	public TenantsDB getTenantsDB(){return _tenantsDB;}
	public UsersDB getUsersDB(){return _usersDB;}
	public UserGroupsDB getUserGroupsDB() {return _usergroupsDB;}
	public UserRelationsDB getUserRelations(){return _userrelationsDB;}
	

	public DmsProcessingMapper getDmsProcessingMapper() {return _dmp;}
	public DmsWebMessageHandler getMessageHandler() {return _messageHandler;}
	
	public OpenSearchProcessing getOpenSearchProcessing() {return _openSearchProcessing;}
	public String getApplicationVersion() {return _applicationVersion;}
	
	public boolean isNoConfig() { return _noConfig;}
	public WebAppConfig getWebAppConfig() {return _wac;}
	
}
