/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.modulebox;

import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;

 
import de.kleinekuh.components.wicketdnd.DragSource;
import de.kleinekuh.components.wicketdnd.DropTarget;
import de.kleinekuh.components.wicketdnd.IEBackgroundImageCacheFix;
import de.kleinekuh.components.wicketdnd.IECursorFix;
import de.kleinekuh.components.wicketdnd.Location;
import de.kleinekuh.components.wicketdnd.Operation;
import de.kleinekuh.components.wicketdnd.Reject;
import de.kleinekuh.components.wicketdnd.Transfer;
import de.kleinekuh.components.wicketdnd.theme.WindowsTheme;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;

 

public class ModulesBoxPanel extends Panel {

	public enum DnDType{
		NONE,
		DRAG,
		DROP,
		BOTH
	}
	

	private static final long serialVersionUID = 1L;
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private WebMarkupContainer _wmcContent, _wmcModulesBox;
	private ListView<ApiModuleImpl> _lvModules;
	
	private HashSet<ApiModuleImpl> _selection = new HashSet<ApiModuleImpl>();
	private List<ApiModuleImpl> modulelist = new LinkedList<ApiModuleImpl>();
	private String _moduleClassName = "tbpmodule";
	private String _modulesBoxClassName = "modulebox";
	private boolean withEvent=true;
	
	
	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
	}
	
	
	public ModulesBoxPanel(String id) {
		this(id, DnDType.NONE);
	}
	
	
	public ModulesBoxPanel(String id, DnDType dndType) {
		super(id);
		
		_wmcContent = new WebMarkupContainer("content");
		_wmcContent.setOutputMarkupId(true);
		add(_wmcContent);
		addModules();
		if(dndType!=DnDType.NONE)addDND(dndType);
		else withEvent=false;
	}
	
	private void addDND(DnDType dndType) {
		add(new IECursorFix());
		add(new IEBackgroundImageCacheFix());
		
		_wmcContent.add(new WindowsTheme());
		
		if(dndType==DnDType.BOTH || dndType==DnDType.DRAG) addDNDDrag();
		if(dndType==DnDType.BOTH || dndType==DnDType.DROP) addDNDDrop();
		
		
	}
		
	private void addDNDDrag() {
		_wmcContent.add(new DragSource(){

			private static final long serialVersionUID = 1L;

			@Override
			public Set<Operation> getOperations(){return EnumSet.allOf(Operation.class);}

			@Override
			public String[] getTypes(){return new String[] { Transfer.ANY };}

			@Override
			public void onAfterDrop(AjaxRequestTarget target, Transfer transfer) {
			}
			
			public void onDragStart(AjaxRequestTarget target, String dragComponentID) {
				clearSelection();
				target.add(_wmcContent);
			};
			
			public void onBeforeDrop( Component drag, Transfer transfer) throws Reject {
				
				if(drag instanceof ModuleBoxPanel) {
					ModuleBoxPanel tbp = (ModuleBoxPanel)drag;
					transfer.setData(tbp.getModule());
				}else throw new Reject();
			};
			
		}.drag("."+_moduleClassName));
		
	}
	
	private void addDNDDrop() {
		_wmcContent.add(new DropTarget() {
			private static final long serialVersionUID = 1L;
			
			@Override
			public Set<Operation> getOperations(){
				return EnumSet.allOf(Operation.class);
			}

			@Override
			public String[] getTypes(){
				return new String[] { Transfer.ANY };
			}

			@Override
			public void onDrop(AjaxRequestTarget target, Transfer transfer, Location location) throws Reject {
				onModuleDrop(transfer.getData(), target);
			};
		}.dropCenter(".modulebox"));
	}
		
	
	
	private void addModules() {

		try {
			_wmcModulesBox = new WebMarkupContainer("modulebox");
			_wmcModulesBox.add(new AttributeModifier("class", _modulesBoxClassName));
			_wmcModulesBox.setOutputMarkupId(true);
			add(_wmcModulesBox);
			
			_lvModules = new ListView<ApiModuleImpl>("modules", modulelist) {

				private static final long serialVersionUID = 1L;

				@Override
				protected void populateItem(ListItem<ApiModuleImpl> item) {
					final ModuleBoxPanel tbp = new ModuleBoxPanel("module", item.getModelObject()) {
						private static final long serialVersionUID = 1L;
						@Override
						protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {
							if(event.equals("click")) {
								if(isSelected())_selection.add(getModule());
								else _selection.remove(getModule());
								target.add(_wmcContent);
								onModuleSelect(getModule(), isSelected(), target);
							}
						}
					};
					tbp.setOutputMarkupId(true);
					tbp.setWithEvent(withEvent);
					tbp.setModuleClassName(_moduleClassName);
					tbp.setMarkupId(_wmcContent.getMarkupId()+"tbp"+item.getIndex());
					
					if(_selection.contains(item.getModelObject()))tbp.setSelected(true);
					item.add(tbp);
				}
			};
			_lvModules.setOutputMarkupId(true);
			_wmcModulesBox.add(_lvModules);
			_wmcContent.add(_wmcModulesBox);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "addModules", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void clearAll() {
		_selection = new HashSet<ApiModuleImpl>();
		modulelist.clear();
	}
	
	
	public List<ApiModuleImpl> getModules() {return modulelist;}
	public void setModules(List<ApiModuleImpl> modules) {
		clearAll();
		modulelist.addAll(modules);
	}
	
	public void setModule(ApiModuleImpl module) {
		clearAll();
		modulelist.add(module);
	}
	
	public HashSet<ApiModuleImpl> getSelection(){return _selection;}
	public void selectAll(){
		for(ApiModuleImpl module: modulelist)_selection.add(module);
	}
	
	public void clearSelection() {
		_selection = new HashSet<ApiModuleImpl>();
	}
	
	public void setModuleClassName(String moduleClassName) {
		_moduleClassName = moduleClassName;
	}
	
	public void setModulesBoxClassName(String modulesBoxClassName) {
		_modulesBoxClassName = modulesBoxClassName;
	}
	
	
	
	protected void onModuleSelect(ApiModuleImpl module, boolean selection, AjaxRequestTarget target) {}
	protected void onModuleDrop(ApiModuleImpl module, AjaxRequestTarget target) {}
	
}
