/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.pdfviewer;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AbstractDefaultAjaxBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.attributes.CallbackParameter;
import org.apache.wicket.markup.head.CssReferenceHeaderItem;
import org.apache.wicket.markup.head.HeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.head.OnDomReadyHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.request.IRequestHandler;
import org.apache.wicket.request.IRequestParameters;
import org.apache.wicket.request.Response;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.request.flow.RedirectToUrlException;
import org.apache.wicket.request.handler.resource.ResourceReferenceRequestHandler;
import org.apache.wicket.request.resource.CssResourceReference;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.Types.SearchStatusType;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.documents.pdf.PdfViewerResponse;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.models.DocumentModel;
import org.biffo.dms.resource.DmsCrypt;
import org.biffo.dms.session.parameters.DocumentViewParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.user.UserRestrictionType;
import org.biffo.dms.web.components.search.singelfield.SingleFieldSearchInput;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;


public class PdfViewerPanel extends Panel{
	
	private static final String _className = "PdfViewerPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "pdftoolbarbutton";
	
	
	private static final CssResourceReference PDF_VIEWER_CSS = new CssResourceReference(PdfViewerPanel.class, "PdfViewerPanel.css");
	private static JavaScriptResourceReference PDF_JS=  new JavaScriptResourceReference(PdfViewerPanel.class, "PdfDocument.js");
    private static final CssResourceReference PDFJS_VIEWER_CSS = new CssResourceReference(PdfViewerPanel.class, "PdfDocument.css");
    private static JavaScriptResourceReference DND_JS=  new JavaScriptResourceReference(PdfViewerPanel.class, "dragula.min.js");
	private static final CssResourceReference DND_CSS=  new CssResourceReference(PdfViewerPanel.class, "dragula.min.css");

	private JavaScriptResourceReference pdfJSWorker;
	private JavaScriptResourceReference pdfJS;
	private IRequestHandler pdfJSWorkerHandler = null;

	private String _test;
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	private DocumentViewParameters _thisParameters;
	
	private PdfViewerResponse _viewerResponse = new PdfViewerResponse();
    private DocumentOSFilterClass _dsfc = new DocumentOSFilterClass();
    private SingleFieldSearchInput _sfsi;
	
	private String _pdfViewerId, _pdfViewerVar, _pdfRenderBoxVar;
    
    private WebMarkupContainer _wmcFilterBox, _wmcPdfContent;
    private PageTitel _pageTitel = new PageTitel(_viewerResponse);
	
    private Toolbar _toolBar;
	private ToolbarButton _btFirst, _btLast, _btPrev, _btNext, _btnMultiPage, _btnSingelPage, _btnDelete, _btnSplit, _btnPageNumbers;
	private ToolbarButton _btnTurnPageLeft, _btnTurnPageRight, _btEdit, _btCancel,  _btPDF, _btZoomIn, _btZoomOut;
	private ToolbarButton _btnPageTitel;
	private ToolbarButton _btnShowDocumentDetails, _btnHideDocumentDetails;
	private ToolbarButton _btnFilterOn, _btnFilterOff;
    
    
    private Document _document;
    private boolean _reload=false;
    private boolean _isEditable=false;
    private boolean _documentChanged=false;
    
    private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
    private AbstractModalDefaultPanel _amdp;
    
    private String pdfJSVersion="pdfJS_311";    

    public PdfViewerPanel(String id,  DocumentModel documentModel, SearchHelper searchHelper) {
		super(id);
		_document = documentModel.getObject();
		initParameters();
		if(_document.getNumberOfPages()==1) {
			_thisParameters.getActualViewParams().setOnePager(true);
		}
		if(searchHelper!=null && searchHelper.getFoundPages().size()>0) {
			_thisParameters.getActualViewParams().setSearchHelper(searchHelper);
			_thisParameters.getActualViewParams().setSearch(true);
			_dsfc.setSearchString(searchHelper.getSearchString());
			if(!_thisParameters.getActualViewParams().isOnePager()) {
				_thisParameters.getActualViewParams().setMultiView(true);
				_thisParameters.getActualViewParams().setFiltered(true);
			}
		}
		storeParameters();
		
		_pdfViewerId = getMarkupId().replaceAll("id", "");
		_pdfViewerVar  = "pdfV"+getMarkupId().replaceAll("id", "");
		_pdfRenderBoxVar = "pdfB"+getMarkupId().replaceAll("id", "");
		
		addToolBar();
		addSearchBox();
		addPdfControl();
		addModalDefault();
		setToolbar();
		
		applyUserRestrictions();
	}
    
	private void initParameters() {
		
		_thisParameters = (DocumentViewParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENT_PDF_VIEW);
		_dsfc.setId(_document.getId());
		if(_thisParameters.getActualViewParams()==null || _thisParameters.getActualViewParams().getDocumentId()!=_document.getId()) {
			_thisParameters.setActualViewParams(new PdfViewerActualView(_document.getId()));
		}
		
		try {
			pdfJSVersion="pdfJS_311";
			if(_session.getClientInfo().getProperties().isBrowserSafari() ) {
	    		pdfJSVersion = "pdfJS_216";
	    	}
			pdfJSWorker = new JavaScriptResourceReference(PdfViewerPanel.class, pdfJSVersion+"/pdf.worker.js");
			pdfJS = new JavaScriptResourceReference(PdfViewerPanel.class, pdfJSVersion+"/pdf.js");
			pdfJSWorkerHandler = new ResourceReferenceRequestHandler(JavaScriptReferenceHeaderItem.forReference(pdfJSWorker).getReference(), null);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"initParameters",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);		}
	}
	
    @Override
	public void renderHead(IHeaderResponse response) {
    	//super.renderHead(response);

		response.render(CssReferenceHeaderItem.forReference(PDF_VIEWER_CSS));
		response.render(CssReferenceHeaderItem.forReference(PDFJS_VIEWER_CSS));
    	response.render(CssReferenceHeaderItem.forReference(DND_CSS));
    	
    	response.render(getPdfJsAsModule());
    	response.render(JavaScriptReferenceHeaderItem.forReference(PDF_JS));
		response.render(JavaScriptReferenceHeaderItem.forReference(DND_JS));
    	response.render(JavaScriptHeaderItem.forScript("var " + _pdfViewerVar+"=null;", "_pdfViewerVar"+_pdfViewerId));
    	
	}
    
    private HeaderItem getPdfJsAsModule() {
    	
    	HeaderItem hi = new HeaderItem() {

    		private static final long serialVersionUID = 1L;

			@Override
			public void render(Response response) {
				// The terrible JS needs implementing a module file. No sure, why?
				ResourceReferenceRequestHandler pdfJSHandler = new ResourceReferenceRequestHandler(JavaScriptReferenceHeaderItem.forReference(pdfJS).getReference(), null);
				String returnUrlPdfJS = RequestCycle.get().urlFor(pdfJSHandler).toString();
				response.write("<script src=\""+returnUrlPdfJS+"\" type=\"module\"></script>");
			}
			
			@Override
			public Iterable<?> getRenderTokens() {
				return Collections.emptyList();
			}
			
			
		};
		return hi;
    	
    	
    }
    
    
    
    private void addSearchBox() {
    	
		_wmcFilterBox = new WebMarkupContainer("pdfsearchbox");
		_wmcFilterBox.setOutputMarkupId(true);
		add(_wmcFilterBox);
    	
    	_sfsi = new SingleFieldSearchInput("singelfieldsearch", _dsfc, true) {
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnSelectionFilterClick(AjaxRequestTarget target) {
				if(_dsfc.getFilterObject()!=null) {
					SearchHelper sh = buildSearchHelper();
					if(null!=sh && sh.getFilterArray()!=null && sh.getFilterArray().length()>0) {
						_thisParameters.getActualViewParams().setSearchHelper(sh);
						_thisParameters.getActualViewParams().setSearch(true);
						
						target.appendJavaScript(
								_pdfViewerVar + ".clearSearch(false, 'mark');" + 
								_pdfViewerVar + ".search('"+_dsfc.getSearchString()+"');");
						setFilter(target, true);
						storeParameters();
					}
				}else {
					PdfViewerPanel.this.clearSearch();
					target.appendJavaScript(_pdfViewerVar + ".clearSearch();");
					setFilter(target, false);
				}
				setToolbar();
				
			}
			
			@Override
			protected void OnEnterFilterClick(AjaxRequestTarget target) {
				if(_dsfc.getFilterObject()!=null) {
					SearchHelper sh = buildSearchHelper();
					if(null!=sh && sh.getFilterArray()!=null && sh.getFilterArray().length()>0) {
						_thisParameters.getActualViewParams().setSearchHelper(sh);
						_thisParameters.getActualViewParams().setSearch(true);
						target.appendJavaScript(
								_pdfViewerVar + ".clearSearch(false, 'mark');" +
								_pdfViewerVar + ".search('"+_dsfc.getSearchString()+"');");
						setFilter(target, true);
						storeParameters();
					}
				}else {
					PdfViewerPanel.this.clearSearch();
					target.appendJavaScript(_pdfViewerVar + ".clearSearch();");
					setFilter(target, false);
				}
				setToolbar();
			}
			
			@Override
			protected void OnClearFilterClick(AjaxRequestTarget target) {
				PdfViewerPanel.this.clearSearch();
				target.appendJavaScript(
						_pdfViewerVar + ".clearSearch(false, 'mark');" +
						_pdfViewerVar + ".clearSearch();");
				setFilter(target, false);
			}
			
		};
		_sfsi.setWithResultText(true);
		_sfsi.setOutputMarkupId(true);
		_sfsi.setAutoComplete(true);
		_sfsi.setVisible(true);
		_wmcFilterBox.add(_sfsi);
		
	}
    
    private void clearSearch() {
    	_thisParameters.getActualViewParams().setSearchHelper(null);
		_thisParameters.getActualViewParams().setSearch(false);
		_dsfc.setLinkedWhere(null);
		_dsfc.setSingelFieldSearchWhere(null);
		_dsfc.setFilterObject(null);
		_dsfc.setSearchStatusType(SearchStatusType.SINGLEFIELD_SET, false);
		storeParameters();
    }
    
    
    
    private void addPdfControl() {
    	
    	_wmcPdfContent = new WebMarkupContainer("document");
    	_wmcPdfContent.add(new AttributeModifier("id", _pdfRenderBoxVar));
		_wmcPdfContent.setOutputMarkupId(true);
		add(_wmcPdfContent);
		
		add(new AbstractDefaultAjaxBehavior() {

			private static final long serialVersionUID = 1L;
			
            public void renderHead(Component component, IHeaderResponse response) {
                super.renderHead(component, response);
                String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(), _document);
                String url = "../pdf/"+encodedResource;

                CharSequence callbackScript = getCallbackFunction(
                	CallbackParameter.explicit("id"), 
                	CallbackParameter.explicit("event"),
                	CallbackParameter.explicit("scalesingle"),
                	CallbackParameter.explicit("scalemulti"),
                	CallbackParameter.explicit("elementid"),
                	CallbackParameter.explicit("filtered"),
                	CallbackParameter.explicit("actpage"),
                	CallbackParameter.explicit("founditems"),
                    CallbackParameter.explicit("documentpages")
            	);
               
                
                StringBuilder sb = new StringBuilder();
               
                if(_reload) {
                	sb.append(_pdfViewerVar+"=null;");
                	_reload=false;
                }
                sb.append("if("+_pdfViewerVar+"==null){");
                sb.append(getUrlForWorkerSource()+";");
                //sb.append(_pdfViewerVar + " = new PdfDocument('"+getMarkupId()+"', '" + url + "', " + callbackScript +");");
                sb.append(_pdfViewerVar + " = new PdfDocument('"+_pdfViewerId+"', '" + url + "', " + callbackScript +");");
                
                sb.append("}");
                response.render(OnDomReadyHeaderItem.forScript(sb.toString()));

                response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".setShowPageNumbers("+_thisParameters.isPageNumbers()+");"));
            	response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".setScaleMulti("+_thisParameters.getActualViewParams().getZoomFactorMulti()+");"));
            	response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".setScaleSingle("+_thisParameters.getActualViewParams().getZoomFactorSingle()+");"));
            	
                if(_thisParameters.getActualViewParams().isSearch()) {
                	String filterArray = _thisParameters.getActualViewParams().getSearchHelper().getFilterArray();
                	if(null!=filterArray) {
                		
                		response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".clearSearch(false, 'mark');"));
                		response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".search('"+_thisParameters.getActualViewParams().getSearchHelper().getSearchString()+"');"));
    					response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".setFilter("+filterArray+");"));
    					_thisParameters.getActualViewParams().setFiltered(true);
    					if(_thisParameters.getActualViewParams().isMultiView()) {
    						response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+");"));
    					}else {
    						response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+"[0]);"));
    					}
                	}
                }else {
                	response.render(OnDomReadyHeaderItem.forScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+_thisParameters.getActualViewParams().getActPages()+");"));
                }
            }


			protected void respond(AjaxRequestTarget target) {
				
				IRequestParameters requestParams = getComponent().getRequest().getRequestParameters();
				String id = getStringValueFromRequestParam(requestParams,"id");
            	String event = getStringValueFromRequestParam(requestParams,"event");
            	
            	String scaleSingle=getStringValueFromRequestParam(requestParams,"scalesingle");
            	String scaleMulti=getStringValueFromRequestParam(requestParams,"scalemulti");
            	
            	String elementid = getStringValueFromRequestParam(requestParams,"elementid");
            	boolean filtered = getBooleanValueFromRequestParam(requestParams,"filtered");
            	String actpage = getStringValueFromRequestParam(requestParams,"actpage");
            	int founditems = getIntValueFromRequestParam(requestParams,"founditems");
            	String documentpages = getStringValueFromRequestParam(requestParams,"documentpages");
            	try {
					_viewerResponse.setValues(id, event, elementid, filtered, actpage, founditems, documentpages);
				} catch (DmsException e1) {
					e1.printStackTrace();
				}

            	if(!Strings.isEmpty(event)) {
                	if(event.equals("PAGE_DELETED")) _documentChanged=true;
                	else if(event.equals("NEW_ORDER")) _documentChanged=true;
                	else if(event.equals("ONDBLCLICK") && !_isEditable) {
                		_thisParameters.getActualViewParams().setMultiView(false);
                		_thisParameters.getActualViewParams().setAnchorMark(elementid);
    					//String x = elementid.replaceAll("gd"+getMarkupId()+"pg", "");
                		String x = elementid.replaceAll("gd"+_pdfViewerId+"pg", "");
    					
    					target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+x+");");
            			setToolbar();
            			target.add(_toolBar);
                	}else if(event.equals("SEARCH")) {
                	}else if(event.equals("ZOOM_IN")) {
                		PdfViewerPanel.this.processEvent("ZoomIn", target);
                	}else if(event.equals("ZOOM_OUT")) {
                		PdfViewerPanel.this.processEvent("ZoomOut", target);
                	}
                }
                if(actpage!=null && !actpage.isEmpty()) {
                	if(actpage.equals("ALL"))_thisParameters.getActualViewParams().setActPages("'"+actpage+"'");
                	else if(actpage.contains(","))_thisParameters.getActualViewParams().setActPages("["+actpage+"]");
                	else _thisParameters.getActualViewParams().setActPages(actpage);
                	
                	if(scaleSingle.equals("X"))_thisParameters.getActualViewParams().setZoomFactorSingle(-1);
                	else _thisParameters.getActualViewParams().setZoomFactorSingle(Double.valueOf(scaleSingle));
                	if(scaleMulti.equals("X"))_thisParameters.getActualViewParams().setZoomFactorMulti(-1);
                	else _thisParameters.getActualViewParams().setZoomFactorMulti(Double.valueOf(scaleMulti));
                	
                }
                storeParameters(); 
                
        		try {
        			_btnPageTitel.setUtf8Code(_pageTitel.getPageTitel());
        			OnChange(target, _viewerResponse);
        		} catch (Exception e) {
        			DmsException dmsException = new DmsException(
        					1000, 
        					"E", 
        					_className, 
        					"respond",
        					null,
        					e
        				);
        				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
        		}
        		target.add(_btnPageTitel);
			}
			
		}); 
    }
    
    private void addToolBar() {
		_toolBar = new Toolbar("pdftoolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE);
		_btEdit.setDefaultFormProcessing(false);
		_btEdit.setOutputMarkupId(true);
		_btEdit.setCssClass(toolbarCSS);
		
		_btnHideDocumentDetails = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnHideDocumentDetails.setUtf8Code("xf1ce");
		_btnHideDocumentDetails.setEventName("Hide");
		_btnHideDocumentDetails.setDefaultFormProcessing(false);
		_btnHideDocumentDetails.setOutputMarkupId(true);
		_btnHideDocumentDetails.setCssClass(toolbarCSS);
		_btnHideDocumentDetails.setAlternate(ResourceText.getText("altBtnHideDocumentDetails", this));
		_toolBar.add(_btnHideDocumentDetails);
		
 
		_btnShowDocumentDetails = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnShowDocumentDetails.setUtf8Code("xf1cf");
		_btnShowDocumentDetails.setEventName("Show");
		_btnShowDocumentDetails.setDefaultFormProcessing(false);
		_btnShowDocumentDetails.setOutputMarkupId(true);
		_btnShowDocumentDetails.setCssClass(toolbarCSS);
		_btnShowDocumentDetails.setAlternate(ResourceText.getText("altBtnShowDocumentDetails", this));
		_toolBar.add(_btnShowDocumentDetails);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW);
		_btCancel.setDefaultFormProcessing(false);
		_btCancel.setOutputMarkupId(true);
		_btCancel.setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("altBtCancel", this));
		_toolBar.add(_btCancel);
		
		_btnMultiPage = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnMultiPage.setUtf8Code("xe5fa");
		_btnMultiPage.setEventName("Multipage");
		_btnMultiPage.setDefaultFormProcessing(false);
		_btnMultiPage.setOutputMarkupId(true);
		_btnMultiPage.setCssClass(toolbarCSS);
		_btnMultiPage.setAlternate(ResourceText.getText("altBtnMultiPage", this));
		_toolBar.add(_btnMultiPage);
		
		_btnSingelPage = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnSingelPage.setUtf8Code("xe873");
		_btnSingelPage.setEventName("Singelpage");
		_btnSingelPage.setDefaultFormProcessing(false);
		_btnSingelPage.setOutputMarkupId(true);
		_btnSingelPage.setCssClass(toolbarCSS);
		_btnSingelPage.setAlternate(ResourceText.getText("altBtnSingelPage", this));
		_toolBar.add(_btnSingelPage);
		
		_btnPageNumbers = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE);
		_btnPageNumbers.setUtf8Code("xeb8d");
		_btnPageNumbers.setEventName("Pagenumbers");
		_btnPageNumbers.setDefaultFormProcessing(false);
		_btnPageNumbers.setOutputMarkupId(true);
		_btnPageNumbers.setCssClass(toolbarCSS);
		_btnPageNumbers.setAlternate(ResourceText.getText("altBtnPageNumbers", this));
		_toolBar.add(_btnPageNumbers);

		_btnSplit = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.WITH_EVENT);
		_btnSplit.setUtf8Code("xeb13");
		_btnSplit.setEventName("MovePages");
		_btnSplit.setDefaultFormProcessing(false);
		_btnSplit.setOutputMarkupId(true);
		_btnSplit.setCssClass(toolbarCSS);
		_btnSplit.setAlternate(ResourceText.getText("altBtnSplit", this));
		_toolBar.add(_btnSplit);
		
		_btnTurnPageLeft = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.WITH_EVENT);
		_btnTurnPageLeft.setUtf8Code("xe419");
		_btnTurnPageLeft.setEventName("TurnLeft");
		_btnTurnPageLeft.setDefaultFormProcessing(false);
		_btnTurnPageLeft.setOutputMarkupId(true);
		_btnTurnPageLeft.setCssClass(toolbarCSS);
		_btnTurnPageLeft.setAlternate(ResourceText.getText("altBtnTurnPageLeft", this));
		_toolBar.add(_btnTurnPageLeft);
		
		_btnTurnPageRight = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.WITH_EVENT);
		_btnTurnPageRight.setUtf8Code("xe41a");
		_btnTurnPageRight.setEventName("TurnRight");
		_btnTurnPageRight.setDefaultFormProcessing(false);
		_btnTurnPageRight.setOutputMarkupId(true);
		_btnTurnPageRight.setCssClass(toolbarCSS);
		_btnTurnPageRight.setAlternate(ResourceText.getText("altBtnTurnPageRight", this));
		_toolBar.add(_btnTurnPageRight);
		
		_btnDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);;
		_toolBar.add(_btnDelete);
		
		_btZoomIn = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setAlternate("Zoom In").setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_VIEW);
		_btZoomIn.setUtf8Code("xe8ff");
		_btZoomIn.setEventName("ZoomIn");
		_btZoomIn.setDefaultFormProcessing(false);
		_btZoomIn.setOutputMarkupId(true);
		_btZoomIn.setCssClass(toolbarCSS);
		_toolBar.add(_btZoomIn);
		
		_btZoomOut = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setAlternate("Zoom Out").setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_VIEW);
		_btZoomOut.setUtf8Code("xe900");
		_btZoomOut.setEventName("ZoomOut");
		_btZoomOut.setDefaultFormProcessing(false);
		_btZoomOut.setOutputMarkupId(true);
		_btZoomOut.setCssClass(toolbarCSS);
		_toolBar.add(_btZoomOut);
		
		_btPDF = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setAlternate("Download PDF").setToolbarState(ToolbarState.STATE_VIEW);
		_btPDF.setUtf8Code("xf090");
		_btPDF.setEventName("PDF");
		_btPDF.setDefaultFormProcessing(false);
		_btPDF.setOutputMarkupId(true);
		_btPDF.setCssClass(toolbarCSS);
		_toolBar.add(_btPDF);
		
		_btnFilterOn = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setAlternate("Filter Pages").setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_VIEW);
		_btnFilterOn.setUtf8Code("xef4f");
		_btnFilterOn.setEventName("Filter");
		_btnFilterOn.setDefaultFormProcessing(false);
		_btnFilterOn.setOutputMarkupId(true);
		_btnFilterOn.setCssClass("pdftoolbarbutton");
		_btnFilterOn.setAlternate(ResourceText.getText("altBtnFilterOnPages", this));
		_toolBar.add(_btnFilterOn);
		
		_btnFilterOff = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setAlternate("Filter Pages").setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_VIEW);
		_btnFilterOff.setUtf8Code("xeb32");
		_btnFilterOff.setEventName("ClearFilter");
		_btnFilterOff.setDefaultFormProcessing(false);
		_btnFilterOff.setOutputMarkupId(true);
		_btnFilterOff.setCssClass("pdftoolbarbutton");
		_btnFilterOff.setAlternate(ResourceText.getText("altBtnFilterOffPages", this));
		_toolBar.add(_btnFilterOff);
		
		_btFirst = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btFirst.setUtf8Code("xe5dc");
		_btFirst.setEventName("First");
		_btFirst.setDefaultFormProcessing(false);
		_btFirst.setOutputMarkupId(true);
		_btFirst.setCssClass(toolbarCSS);
		_btFirst.setAlternate(ResourceText.getText("altBtnFirst", this));
		_toolBar.add(_btFirst);

		_btPrev = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btPrev.setUtf8Code("xe408");
		_btPrev.setEventName("Prev");
		_btPrev.setDefaultFormProcessing(false);
		_btPrev.setOutputMarkupId(true);
		_btPrev.setCssClass(toolbarCSS);
		_btPrev.setAlternate(ResourceText.getText("altBtnPrev", this));
		_toolBar.add(_btPrev);

		_btNext = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btNext.setUtf8Code("xe409");
		_btNext.setEventName("Next");
		_btNext.setDefaultFormProcessing(false);
		_btNext.setOutputMarkupId(true);
		_btNext.setCssClass(toolbarCSS);
		_btNext.setAlternate(ResourceText.getText("altBtnNext", this));
		_toolBar.add(_btNext);

		_btLast = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btLast.setUtf8Code("xe5dd");
		_btLast.setEventName("Last");
		_btLast.setDefaultFormProcessing(false);
		_btLast.setOutputMarkupId(true);
		_btLast.setCssClass(toolbarCSS);
		_btLast.setAlternate(ResourceText.getText("altBtnLast", this));
		_toolBar.add(_btLast);	
		
		_btnPageTitel = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED_TEXT).setToolbarState(ToolbarState.STATE_VIEW);
		_btnPageTitel.setUtf8Code(_pageTitel.getPageTitel());
		_btnPageTitel.setDefaultFormProcessing(false);
		_btnPageTitel.setOutputMarkupId(true);
		_btnPageTitel.setCssClass("pdftoolbartext");
		_toolBar.add(_btnPageTitel);
		
		_toolBar.setOutputMarkupId(true);
		if(_isEditable)	_toolBar.setViewState(ToolbarState.STATE_CHANGE);
		else _toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		add(_toolBar);
    }
    
    private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(550);
		modalDefault.setResizable(false);
		_amdp = new AbstractModalDefaultPanel(modalDefault) {

			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				if(_amdp.getModalRCValue().equals("DELETE")) {
					modalDefault.close(target);
					target.appendJavaScript(_pdfViewerVar + ".delete("+_pdfRenderBoxVar+");");
				}else if(_amdp.getModalRCValue().equals("SPLIT")) {
					modalDefault.close(target);
					PdfViewerPanel.this.OnSplitt(target, _viewerResponse);
				}
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
			}
		};
		modalDefault.setContent(_amdp);
		add(modalDefault);
	}
    
    private void processEvent(String event, AjaxRequestTarget target) {
    	if(event.equals("Edit")) {
    		target.appendJavaScript("location.href='#"+_test+"';");
 		}else if(event.equals("ZoomIn")) {
 			target.appendJavaScript(_pdfViewerVar +".setScale("+_pdfViewerVar+".getScale()+0.2,"+_pdfRenderBoxVar+");");
 		}else if(event.equals("ZoomOut")) {
 			target.appendJavaScript(_pdfViewerVar +".setScale("+_pdfViewerVar+".getScale()-0.2,"+_pdfRenderBoxVar+");");
 		}else if(event.equals("PDF")) {
			//String url= ProcessingHelpers.getFilePathName("../pdf", _document, false);
			//throw new RedirectToUrlException(url);
			String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(), _document);
            String url = "../pdf/"+encodedResource;
            throw new RedirectToUrlException(url);
			
		}else if(event.equals("Multipage")) {
			setMultiPage(target, true);
		}else if(event.equals("Singelpage")) {
			setMultiPage(target, false);
		}else if(event.equals("Filter")) {
			setFilter(target, true);
		}else if(event.equals("ClearFilter")) {
			setFilter(target, false);
		}else if(event.equals("Pagenumbers")) {
			setPageNumbers(target);
		}else if(event.equals("First")) {
			target.appendJavaScript(_pdfViewerVar + ".first("+_pdfRenderBoxVar+");");
		}else if(event.equals("Prev")) {
			target.appendJavaScript(_pdfViewerVar + ".increment("+_pdfRenderBoxVar+",-1);");
		}else if(event.equals("Next")) {
			target.appendJavaScript(_pdfViewerVar + ".increment("+_pdfRenderBoxVar+",1);");
		}else if(event.equals("Last")) {
			target.appendJavaScript(_pdfViewerVar + ".last("+_pdfRenderBoxVar+");");
		}else if(event.equals("Delete")) {
			if(_viewerResponse.getNumSelections()>0)showModal(target, "DELETE");
		}else if(event.equals("TurnRight")) {
			target.appendJavaScript(_pdfViewerVar + ".turnRight("+_pdfRenderBoxVar+");");
		}else if(event.equals("TurnLeft")) {
			target.appendJavaScript(_pdfViewerVar + ".turnLeft("+_pdfRenderBoxVar+");");
		}else if(event.equals("MovePages")) {
			if(_viewerResponse.getNumSelections()>0) {
				PdfViewerPanel.this.OnSplitt(target, _viewerResponse);
			}
		}else if(event.equals("Hide")) {
			_thisParameters.setDocInfoVisible(false);
			storeParameters();
			OnHide(target);
			setToolbar();
		}else if(event.equals("Show")) {
			_thisParameters.setDocInfoVisible(true);
			storeParameters();
			OnShow(target);
			setToolbar();
		}else if(event.equals("Cancel")) {
			PdfViewerPanel.this.OnClose(target);
		}
    	target.add(_toolBar);
    	
    }
    
    private void setMultiPage(AjaxRequestTarget target, boolean multipage) {
    	_thisParameters.getActualViewParams().setMultiView(multipage);
    	String filterArray = null;
    	if(_thisParameters.getActualViewParams().getSearchHelper()!=null) {
    		filterArray = _thisParameters.getActualViewParams().getSearchHelper().getFilterArray();	
    	}
    	storeParameters();
		setToolbar();
		
		if(_thisParameters.getActualViewParams().isMultiView()) {
			if(_thisParameters.getActualViewParams().isFiltered() && filterArray!=null) {
				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+");");
			}else {
				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", 'ALL');");
				if(_thisParameters.getActualViewParams().getAnchorMark()!=null) {
					_thisParameters.getActualViewParams().setAnchorMark(null);
					storeParameters();
				}
			}
		}else {
			if(_thisParameters.getActualViewParams().isFiltered() && filterArray!=null) {
				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+"[0]);");
			}else {
				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", '1');");
			}
		}
    }
    
    private void setPageNumbers(AjaxRequestTarget target) {
    	if(_thisParameters.isPageNumbers()) {
			_thisParameters.setPageNumbers(false);
		}else {
			_thisParameters.setPageNumbers(true);
		}
    	if(target!=null)target.appendJavaScript(_pdfViewerVar + ".setShowPageNumbers("+_thisParameters.isPageNumbers()+");");
		storeParameters();
    }
    
    private void setFilter(AjaxRequestTarget target, boolean filter) {
    	if(!filter) {
			_thisParameters.getActualViewParams().setFiltered(false);
			storeParameters();
			target.appendJavaScript(_pdfViewerVar + ".removeFilter();");
			if(_thisParameters.getActualViewParams().isMultiView()) {
				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", 'ALL');");
			}
    	}else if(_thisParameters.getActualViewParams().isSearch()){
    		String filterArray = _thisParameters.getActualViewParams().getSearchHelper().getFilterArray();
    		if(filterArray!=null) {
    			_thisParameters.getActualViewParams().setFiltered(true);
    			storeParameters();
    			target.appendJavaScript(_pdfViewerVar + ".setFilter("+filterArray+");");
    			if(_thisParameters.getActualViewParams().isMultiView()) {
    				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+");");
    			}else {
    				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", "+filterArray+"[0]);");
    			}
    		}
		}
		setToolbar();
		target.add(_toolBar);
    }
    
    public void setEditable(AjaxRequestTarget target, boolean editable) {
    	_isEditable=editable;
   		target.appendJavaScript(_pdfViewerVar + ".setEditable("+editable+");");
   		target.appendJavaScript(_pdfViewerVar + ".setSelectable("+editable+");");
   		if(_isEditable) {
   			//if(!_thisParameters.getActualViewParams().isMultiView() && !_thisParameters.getActualViewParams().isOnePager()) {
   			if(!_thisParameters.getActualViewParams().isOnePager()) {
   				_thisParameters.getActualViewParams().setMultiView(true);
   				storeParameters();
   				target.appendJavaScript(_pdfViewerVar + ".renderPages("+_pdfRenderBoxVar+", 'ALL');");
   				//--> Context 
   				target.appendJavaScript(_pdfViewerVar + ".enableDND(true);");
   			}
   			_toolBar.setViewState(ToolbarState.STATE_CHANGE);
   			setToolbar();
   		}else {
   			target.appendJavaScript(_pdfViewerVar + ".unselectAll();");
   			target.appendJavaScript(_pdfViewerVar + ".enableDND(false);");
   			_toolBar.setViewState(ToolbarState.STATE_VIEW);
   			setToolbar();
   		}
   		target.add(_toolBar);
    }
  
	
    private void setToolbar() {
    	setAllButtonsVisible(false);
    	
    	_btPDF.setVisible(true);
    	_btnPageNumbers.setVisible(true);
    	_btZoomIn.setVisible(true);
    	_btZoomOut.setVisible(true);
    	
    	applyUserRestrictions();
    	
    	if(!_isEditable) {
    		//if(_isDocumentInfoVisible) {
    		if(_thisParameters.isDocInfoVisible()) {
    			_btnHideDocumentDetails.setVisible(true);
    			_btCancel.setVisible(false);
    		}else {
    			_btnShowDocumentDetails.setVisible(true);
    			_btCancel.setVisible(true);
    		}
    	}
    	
    	if(_thisParameters.getActualViewParams().isOnePager()) {
    		if(_isEditable) {
    			_btnTurnPageLeft.setVisible(true);
    			_btnTurnPageRight.setVisible(true);
    			_btPDF.setVisible(false);
    		}
    		return;
    	}else {
    		if(!_thisParameters.getActualViewParams().isMultiView() && !_isEditable) {
				_btFirst.setVisible(true);
				_btLast.setVisible(true);
				_btNext.setVisible(true);
				_btPrev.setVisible(true);
				_btnPageTitel.setVisible(true);
				_btnMultiPage.setVisible(true);
			}else if(_thisParameters.getActualViewParams().isMultiView() && !_isEditable) {
				_btnSingelPage.setVisible(true);
			}else if(_thisParameters.getActualViewParams().isMultiView() && _isEditable) {
				_btnDelete.setVisible(true);
				_btnSplit.setVisible(true);
				_btnTurnPageLeft.setVisible(true);
				_btnTurnPageRight.setVisible(true);
				_btPDF.setVisible(false);
			}
    	}
    	
    	
    	if(_thisParameters.getActualViewParams().isSearch()) {
       		if(_thisParameters.getActualViewParams().isFiltered()) {
       			_btnFilterOff.setVisible(true);
       			_btnFilterOn.setVisible(false);
        	}else {
       			_btnFilterOff.setVisible(false);
       			_btnFilterOn.setVisible(true);
        	}
    	}else {
   			_btnFilterOff.setVisible(false);
   			_btnFilterOn.setVisible(false);

    	}
    	
    	
    	
    }
    
    
   
	
	private void setAllButtonsVisible(boolean visible) {
		
		_btnSingelPage.setVisible(visible);
		_btnMultiPage.setVisible(visible);
				
		_btFirst.setVisible(visible);
		_btLast.setVisible(visible);
		_btNext.setVisible(visible);
		_btPrev.setVisible(visible);
		_btnPageTitel.setVisible(visible);
		_btnPageNumbers.setVisible(visible);
		
		_btnDelete.setVisible(visible);
		_btnSplit.setVisible(visible);
		_btnTurnPageLeft.setVisible(visible);
		_btnTurnPageRight.setVisible(visible);
		
		_btnShowDocumentDetails.setVisible(visible);
		_btnHideDocumentDetails.setVisible(visible);
		_btCancel.setVisible(visible);
		
		_btnFilterOff.setVisible(visible);
		_btnFilterOn.setVisible(visible);
		
		_btCancel.setVisible(visible);
		//_btSEARCH.setVisible(visible); 
		_btPDF.setVisible(visible);
		_btZoomIn.setVisible(visible);
		_btZoomOut.setVisible(visible);
 
  
	}
	
	
	
    public void setDocument(Document document) {
    	_document=document;
    	_toolBar.setViewState(ToolbarState.STATE_VIEW);
    	_isEditable=false;
    	if(_document.getNumberOfPages()==1) {
    		_thisParameters.getActualViewParams().setOnePager(true);
    	}
    	//_thisParameters.getActualViewParams().setFiltered(false);
    	//_thisParameters.getActualViewParams().setMultiView(false);
        _documentChanged=false;
        _reload=true;
        storeParameters();
    	setToolbar();
    }
    
    public void highlightFragment(AjaxRequestTarget target, Fragment fragment) {
    	try {
    		String searchFor = "\"" + fragment.getFragmentvalue() +"\"";
    		StringBuilder sb = new StringBuilder();
    		sb.append(_pdfViewerVar + ".clearSearch(false);");
    		sb.append(_pdfViewerVar + ".search('"+searchFor+"', 'mark');");
    		target.appendJavaScript(sb.toString());
    		
    	}catch(Exception e) {
    		DmsException dmsException = new DmsException(
    				1000, 
    				"E", 
    				_className, 
    				"highlightFragment",
    				null,
    				e
    			);
    			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
    	}
    }
    
    
    
    public boolean isDocumentChanged() {
    	return _documentChanged;
    }
    
    public PdfViewerResponse getPdfViewerResponse() {
    	return _viewerResponse;
    }
	
    private String getUrlForWorkerSource() {
    	String rc = "pdfjsLib.GlobalWorkerOptions.workerSrc=";
    	String returnUrl = RequestCycle.get().urlFor(pdfJSWorkerHandler).toString();
    	rc = rc +"'"+returnUrl+"';";
    	return rc;
    }
    private int getIntValueFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			return requestParams.getParameterValue(name).toInt();
    		}
    	}catch(Exception e) {}
    	return -1;
    }
    
    @SuppressWarnings("unused")
	private int[] getIntArrayFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			String temp = getStringValueFromRequestParam(requestParams, name);
    			if(null!=temp) {
    				String strArr[] = temp.split(",");
    				int[] rc = new int[strArr.length];
    				for(int i=0;i<strArr.length;i++) {
    					rc[i] = Integer.valueOf(strArr[i]).intValue();
    				}
    				return rc;
    			}
    		}
    	}catch(Exception e) {}
    	return null;
    }
    
    
    
    private String getStringValueFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			return requestParams.getParameterValue(name).toString(null);
    		}
    	}catch(Exception e) {}
    	return null;
    }
    
    private boolean getBooleanValueFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			String temp = getStringValueFromRequestParam(requestParams, name);
    			if(temp==null)return false;
    			if(temp.toLowerCase().equals("true"))return true;
    			else return false;
    		}
    	}catch(Exception e) {}
    	return false;
    }

    
    @SuppressWarnings("unused")
	private String[] getStringArrayFromRequestParam(IRequestParameters requestParams, String name) {
    	try {
    		if(null!=requestParams.getParameterValue(name)) {
    			String temp = getStringValueFromRequestParam(requestParams, name);
    			if(null!=temp) {
    				String strArr[] = temp.split(",");
    				String[] rc = new String[strArr.length];
    				for(int i=0;i<strArr.length;i++) {
    					rc[i] = strArr[i];
    				}
    				return rc;
    			}
    		}
    	}catch(Exception e) {}
    	return null;
    }

    
     
    
    
    private SearchHelper buildSearchHelper() {
    	try {
	    	SearchHelper sh = new SearchHelper();
			@SuppressWarnings("unchecked")
			HashMap<String, LinkedList<String>> filter = (HashMap<String, LinkedList<String>>)_dsfc.getFilterObject();
			if(null!=filter && filter.size()>0) {
				sh.setIdDocument(_document.getId());
				sh.setFoundPages(filter.get(""+_document.getId()));
				sh.setSearchString(_dsfc.getSearchString());
			}
			return sh;
    	}catch(Exception e){
    		DmsException dmsException = new DmsException(
    				1000, 
    				"E", 
    				_className, 
    				"buildSearchHelper",
    				null,
    				e
    			);
    			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
    	}
    	return null;
    }
    
    
    
    

    
    private void showModal(AjaxRequestTarget target, String modalCustom) {
    	if(modalCustom.equals("DELETE")) {
			_amdp.setModalType(ModalType.DELETE);
			_amdp.setModalRCValue(modalCustom);
			_amdp.setModalText(ResourceText.getText("txtModalDelete", this));
			modalDefault.show(target);
    	}
	}
    
    private void storeParameters() {
    	_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENT_PDF_VIEW, _thisParameters);
    }
    
    private void applyUserRestrictions() {
		if(_btPDF!=null) {
			if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_DOWNLOAD_DOCUMENT))_btPDF.setVisible(true);
			else _btPDF.setVisible(false);
		}
	}
    
    
    protected void OnChange(AjaxRequestTarget target, PdfViewerResponse viewerResponse){}
    protected void OnSplitt(AjaxRequestTarget target, PdfViewerResponse viewerResponse){}
    protected void OnHide(AjaxRequestTarget target){}
    protected void OnShow(AjaxRequestTarget target){}
    protected void OnClose(AjaxRequestTarget target){}
    
    protected void OnContextMenuSelect(AjaxRequestTarget target, String action, String text){}
    
    
    public class PageTitel implements Serializable{
		 
		private static final long serialVersionUID = 1L;
		private final PdfViewerResponse viewerResponse;
		
		public PageTitel(final PdfViewerResponse viewerResponse) {
			this.viewerResponse = viewerResponse;
		}

		public void setPageTitel(String pageTitel) {}
		
		public String getPageTitel() {
			if(!_thisParameters.getActualViewParams().isMultiView() && (_viewerResponse.getActPage()==null || _viewerResponse.getActPage().isEmpty())) return ResourceText.getText("lblPage0", PdfViewerPanel.this) +viewerResponse.getNumDocumentPages();
			else return ResourceText.getText("lblPage", PdfViewerPanel.this) +" "+ (viewerResponse.getActPage()) +" " + ResourceText.getText("lblFrom", PdfViewerPanel.this)+" " +viewerResponse.getNumDocumentPages();
		}
	}

}
