/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.tagbox;

import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;

 
import de.kleinekuh.components.wicketdnd.DragSource;
import de.kleinekuh.components.wicketdnd.DropTarget;
import de.kleinekuh.components.wicketdnd.IEBackgroundImageCacheFix;
import de.kleinekuh.components.wicketdnd.IECursorFix;
import de.kleinekuh.components.wicketdnd.Location;
import de.kleinekuh.components.wicketdnd.Operation;
import de.kleinekuh.components.wicketdnd.Reject;
import de.kleinekuh.components.wicketdnd.Transfer;
import de.kleinekuh.components.wicketdnd.theme.WindowsTheme;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.web.tags.TagsPage;
 

public class TagsBoxPanel extends Panel {

	public enum DnDType{
		NONE,
		DRAG,
		DROP,
		BOTH
	}
	

	private static final long serialVersionUID = 1L;
	
	private WebMarkupContainer _wmcContent, _wmcTagsBox;
	private ListView<Tag> _lvTags;
	
	private HashSet<Tag> _selection = new HashSet<Tag>();
	private List<Tag> taglist = new LinkedList<Tag>();
	private String _tagClassName = "tbptag";
	private String _tagsBoxClassName = "tagbox";
	
	
	@Override
	protected void onBeforeRender() {
		if(null==_wmcTagsBox)addTags();
		super.onBeforeRender();
	}
	
	
	public TagsBoxPanel(String id) {
		this(id, DnDType.NONE);
	}
	
	
	public TagsBoxPanel(String id, DnDType dndType) {
		super(id);
		
		_wmcContent = new WebMarkupContainer("content");
		_wmcContent.setOutputMarkupId(true);
		add(_wmcContent);
		//addTags();
		if(dndType!=DnDType.NONE)addDND(dndType);
	}
	
	private void addDND(DnDType dndType) {
		add(new IECursorFix());
		add(new IEBackgroundImageCacheFix());
		
		_wmcContent.add(new WindowsTheme());
		
		if(dndType==DnDType.BOTH || dndType==DnDType.DRAG) addDNDDrag();
		if(dndType==DnDType.BOTH || dndType==DnDType.DROP) addDNDDrop();
		
		
	}
		
	private void addDNDDrag() {
		_wmcContent.add(new DragSource(){

			private static final long serialVersionUID = 1L;

			@Override
			public Set<Operation> getOperations(){return EnumSet.allOf(Operation.class);}

			@Override
			public String[] getTypes(){return new String[] { Transfer.ANY };}

			@Override
			public void onAfterDrop(AjaxRequestTarget target, Transfer transfer) {
			}
			
			public void onDragStart(AjaxRequestTarget target, String dragComponentID) {
				clearSelection();
				target.add(_wmcContent);
			};
			
			public void onBeforeDrop( Component drag, Transfer transfer) throws Reject {
				
				if(drag instanceof TagBoxPanel) {
					TagBoxPanel tbp = (TagBoxPanel)drag;
					transfer.setData(tbp.getTag());
				}else throw new Reject();
			};
			
		}.drag("."+_tagClassName));
		
	}
	
	private void addDNDDrop() {
		_wmcContent.add(new DropTarget() {
			private static final long serialVersionUID = 1L;
			
			@Override
			public Set<Operation> getOperations(){
				return EnumSet.allOf(Operation.class);
			}

			@Override
			public String[] getTypes(){
				return new String[] { Transfer.ANY };
			}

			@Override
			public void onDrop(AjaxRequestTarget target, Transfer transfer, Location location) throws Reject {
				onTagDrop(transfer.getData(), target);
			};
		}.dropCenter(".tagbox"));
	}
		
	
	
	private void addTags() {

		try {
			
			
			_wmcTagsBox = new WebMarkupContainer("tagsbox");
			_wmcTagsBox.add(new AttributeModifier("class", _tagsBoxClassName));
			_wmcTagsBox.setOutputMarkupId(true);
			add(_wmcTagsBox);
			
			_lvTags = new ListView<Tag>("tags", taglist) {

				private static final long serialVersionUID = 1L;

				@Override
				protected void populateItem(ListItem<Tag> item) {
					final TagBoxPanel tbp = new TagBoxPanel("tag", item.getModelObject()) {
						private static final long serialVersionUID = 1L;
						@Override
						protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {
							if(event.equals("click")) {
								if(isSelected())_selection.add(getTag());
								else _selection.remove(getTag());
								target.add(_wmcContent);
								onTagSelect(getTag(), isSelected(), target);
							}else if(event.equals("dblclick")) {
								_selection.remove(getTag());
								target.add(_wmcContent);
								onTagSelect(getTag(), isSelected(), target);
								
								setResponsePage(new TagsPage(getPage().getPageReference(), getTag().getId()));
								
							}
						}
					};
					tbp.setOutputMarkupId(true);
					tbp.setWithEvent(true);
					tbp.setTagClassName(_tagClassName);
					tbp.setMarkupId(_wmcContent.getMarkupId()+"tbp"+item.getIndex());
					
					if(_selection.contains(item.getModelObject()))tbp.setSelected(true);
					item.add(tbp);
				}
			};
			_lvTags.setOutputMarkupId(true);
			_wmcTagsBox.add(_lvTags);
			_wmcContent.add(_wmcTagsBox);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", "TagsBoxPanel", "addTags", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	private void clearAll() {
		_selection = new HashSet<Tag>();
		taglist.clear();
	}
	
	
	public List<Tag> getTags() {return taglist;}
	public void setTags(List<Tag> tags) {
		clearAll();
		taglist.addAll(tags);
	}
	
	public void setTag(Tag tag) {
		clearAll();
		taglist.add(tag);
	}
	
	public HashSet<Tag> getSelection(){return _selection;}
	public void selectAll(){
		for(Tag tag: taglist)_selection.add(tag);
	}
	
	public void clearSelection() {
		_selection = new HashSet<Tag>();
	}
	
	public void setTagClassName(String tagClassName) {
		_tagClassName = tagClassName;
	}
	
	public void setTagsBoxClassName(String tagsBoxClassName) {
		_tagsBoxClassName = tagsBoxClassName;
	}
	
	protected void onTagSelect(Tag tag, boolean selection, AjaxRequestTarget target) {}
	protected void onTagDrop(Tag tag, AjaxRequestTarget target) {}
	
}
