/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.tagselector;

import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.json.JSONArray;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.select2.Select2;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Tags.TagException;
import org.biffo.dms.entity.Tags.TagFilterClass;

public class TagSelector extends Panel{

	private static final long serialVersionUID = 1L;

	private Select2 _select;
	private List<ISelectData> _lstData;
	
	public TagSelector(String id) {
		super(id);
		
		_lstData = getTags(null);
		_select = new Select2("tagpanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void OnChange(AjaxRequestTarget target, String event, String value) {
				List<Integer> selectedIds = new LinkedList<Integer>();
				try {
				JSONArray savedContent = new JSONArray(value);
				if(null!=savedContent && savedContent.length()>0) {
					for(int i=0;i<savedContent.length();i++) {
						String obj =(String) savedContent.getJSONObject(i).get("id");
						selectedIds.add(Integer.valueOf(obj));
					}
				}
				
				for(int i=0;i<_lstData.size();i++) {
					if(!selectedIds.contains(_lstData.get(i).getId())) {
						_lstData.get(i).setSelected(false);
					}else {
						_lstData.get(i).setSelected(true);
					}
				}
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "OnChange", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
				}
				
				TagSelector.this.OnChange(target);
			}
		};

		_select.setOutputMarkupId(true);
		_select.getSettings().setCloseOnSelect(true);
		_select.getSettings().setSelectOnClose(false);
		_select.getSettings().setWidth("100%");
		_select.getSettings().setMultiple(true);
		_select.getSettings().setData(_lstData);
		_select.getSettings().setPlaceholder("");
		
		StringBuilder sbTemplateResult= new StringBuilder();
		sbTemplateResult.append("function(tag){");
		sbTemplateResult.append("var style = tag[\"style\"];");
		sbTemplateResult.append("var $tag = $(");
		sbTemplateResult.append("'<span class=\"tag\" style=\"'+style+'\"  >' + tag.text + '</span>'");
		sbTemplateResult.append(");");
		sbTemplateResult.append("return $tag;");
		sbTemplateResult.append("}");
		_select.getSettings().setTemplateResult(sbTemplateResult.toString());
		
		add(_select);
		
	}
	
	
	public boolean isDisabled() {
		return _select.isDisabled();
	}
	public void setDisabled(boolean disabled) {
		_select.setDisabled(disabled);
		setEnabled(!disabled);
	}
	
	
	public void setTagSelected(int id, boolean selected) {
		for(int i=0;i<_lstData.size();i++) {
			if(_lstData.get(i).getId()==id) _lstData.get(i).setSelected(selected);
		}
	}
	
	public void unselectAllTags() {
		for(int i=0;i<_lstData.size();i++) {
			_lstData.get(i).setSelected(false);
		}
	}
	
	public void doRefresh() {
		_lstData.clear();
		_lstData.addAll(getTags(null));
		_select.getSettings().replaceData(_lstData);
	}
	
	
	public void setSelectedTags(List<Tag> tagList) {
		unselectAllTags();
		for(Tag tag:tagList) {
			setTagSelected(tag.getId(), true);
		}
	}
	
	
	public void setData(List<ISelectData> lstData, boolean keepSelected) {
	
		if(keepSelected) {
			for(int i=0;i<lstData.size();i++) {
				for(int j=0;j<_lstData.size();j++) {
					if(lstData.get(i).getId()==_lstData.get(j).getId() && _lstData.get(j).getSelected()) {
						lstData.get(i).setSelected(true);
						break;
					}
				}
			}
		}
		_lstData.clear();
		_lstData.addAll(lstData);
		_select.getSettings().replaceData(lstData);
	}
	
	public void setDataByTagFilter(TagFilterClass tfc, boolean keepSelected) {
		List<ISelectData> lstDataTemp = getTags(tfc);
		setData(lstDataTemp, keepSelected);
	}
	
	public List<ISelectData> getDataSelection(boolean onlySelected){
		if(!onlySelected) return _lstData;
		List<ISelectData> rc = new LinkedList<ISelectData>();
		for(ISelectData entity : _lstData) {
			if(entity.getSelected()) rc.add(entity);
		}
		return rc;
	}
	
	private List<ISelectData> getTags(TagFilterClass tagFilterClass){
		List<Tag> tags;
		try {
			List<ISelectData> lstTempData = new LinkedList<ISelectData>();
			tags = StaticLocators.getTags().getEntities(-1, -1, "", tagFilterClass);
		
			for(Tag tag:tags) {
				TagData d1 = new TagData();
				d1.setId(tag.getId());
				d1.setText(tag.getTagValue());
				d1.setBackgroundColor(tag.getTagBgColor());
				d1.setTitel(tag.getBeschreibung());
				d1.setColor(tag.getTagFgColor());
				d1.setSelected(false);
				d1.toString();
				lstTempData.add(d1);
			}
			return lstTempData;
		} catch (TagException e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "getTags", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
		return null;
	}
	
	protected void OnChange(AjaxRequestTarget target){}
	

}
