/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.components.userbox;

import java.util.EnumSet;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;

 
import de.kleinekuh.components.wicketdnd.DragSource;
import de.kleinekuh.components.wicketdnd.DropTarget;
import de.kleinekuh.components.wicketdnd.IEBackgroundImageCacheFix;
import de.kleinekuh.components.wicketdnd.IECursorFix;
import de.kleinekuh.components.wicketdnd.Location;
import de.kleinekuh.components.wicketdnd.Operation;
import de.kleinekuh.components.wicketdnd.Reject;
import de.kleinekuh.components.wicketdnd.Transfer;
import de.kleinekuh.components.wicketdnd.theme.WindowsTheme;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.user.Users.User;

 
public class UsersBoxPanel extends Panel {

	public enum DnDType{
		NONE,
		DRAG,
		DROP,
		BOTH
	}
	

	private static final long serialVersionUID = 1L;
	
	private WebMarkupContainer _wmcContent, _wmcUsersBox;
	private ListView<User> _lvUsers;
	
	private HashSet<User> _selection = new HashSet<User>();
	private List<User> userlist = new LinkedList<User>();
	private String _userClassName = "tbpuser";
	private String _usersBoxClassName = "userbox";
	private boolean withEvent=true;
	
	
	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
	}
	
	
	public UsersBoxPanel(String id) {
		this(id, DnDType.NONE);
	}
	
	
	public UsersBoxPanel(String id, DnDType dndType) {
		super(id);
		
		_wmcContent = new WebMarkupContainer("content");
		_wmcContent.setOutputMarkupId(true);
		add(_wmcContent);
		addUsers();
		if(dndType!=DnDType.NONE)addDND(dndType);
		else withEvent=false;
	}
	
	private void addDND(DnDType dndType) {
		add(new IECursorFix());
		add(new IEBackgroundImageCacheFix());
		
		_wmcContent.add(new WindowsTheme());
		
		if(dndType==DnDType.BOTH || dndType==DnDType.DRAG) addDNDDrag();
		if(dndType==DnDType.BOTH || dndType==DnDType.DROP) addDNDDrop();
		
		
	}
		
	private void addDNDDrag() {
		_wmcContent.add(new DragSource(){

			private static final long serialVersionUID = 1L;

			@Override
			public Set<Operation> getOperations(){return EnumSet.allOf(Operation.class);}

			@Override
			public String[] getTypes(){return new String[] { Transfer.ANY };}

			@Override
			public void onAfterDrop(AjaxRequestTarget target, Transfer transfer) {
			}
			
			public void onDragStart(AjaxRequestTarget target, String dragComponentID) {
				clearSelection();
				target.add(_wmcContent);
			};
			
			public void onBeforeDrop( Component drag, Transfer transfer) throws Reject {
				
				if(drag instanceof UserBoxPanel) {
					UserBoxPanel tbp = (UserBoxPanel)drag;
					transfer.setData(tbp.getUser());
				}else throw new Reject();
			};
			
		}.drag("."+_userClassName));
		
	}
	
	private void addDNDDrop() {
		_wmcContent.add(new DropTarget() {
			private static final long serialVersionUID = 1L;
			
			@Override
			public Set<Operation> getOperations(){
				return EnumSet.allOf(Operation.class);
			}

			@Override
			public String[] getTypes(){
				return new String[] { Transfer.ANY };
			}

			@Override
			public void onDrop(AjaxRequestTarget target, Transfer transfer, Location location) throws Reject {
				onUserDrop(transfer.getData(), target);
			};
		}.dropCenter(".userbox"));
	}
		
	
	
	private void addUsers() {

		try {
			_wmcUsersBox = new WebMarkupContainer("usersbox");
			_wmcUsersBox.add(new AttributeModifier("class", _usersBoxClassName));
			_wmcUsersBox.setOutputMarkupId(true);
			add(_wmcUsersBox);
			
			_lvUsers = new ListView<User>("users", userlist) {

				private static final long serialVersionUID = 1L;

				@Override
				protected void populateItem(ListItem<User> item) {
					final UserBoxPanel tbp = new UserBoxPanel("user", item.getModelObject()) {
						private static final long serialVersionUID = 1L;
						@Override
						protected void onSelect(String event, boolean selection, AjaxRequestTarget target) {
							if(event.equals("click")) {
								if(isSelected())_selection.add(getUser());
								else _selection.remove(getUser());
								target.add(_wmcContent);
								onUserSelect(getUser(), isSelected(), target);
							}
						}
					};
					tbp.setOutputMarkupId(true);
					tbp.setWithEvent(withEvent);
					tbp.setUserClassName(_userClassName);
					tbp.setMarkupId(_wmcContent.getMarkupId()+"tbp"+item.getIndex());
					
					if(_selection.contains(item.getModelObject()))tbp.setSelected(true);
					item.add(tbp);
				}
			};
			_lvUsers.setOutputMarkupId(true);
			_wmcUsersBox.add(_lvUsers);
			_wmcContent.add(_wmcUsersBox);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "addUsers", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	private void clearAll() {
		_selection = new HashSet<User>();
		userlist.clear();
	}
	
	
	public List<User> getUsers() {return userlist;}
	public void setUsers(List<User> users) {
		clearAll();
		userlist.addAll(users);
	}
	
	public void setUser(User user) {
		clearAll();
		userlist.add(user);
	}
	
	public HashSet<User> getSelection(){return _selection;}
	public void selectAll(){
		for(User user: userlist)_selection.add(user);
	}
	
	public void clearSelection() {
		_selection = new HashSet<User>();
	}
	
	public void setUserClassName(String userClassName) {
		_userClassName = userClassName;
	}
	
	public void setUsersBoxClassName(String usersBoxClassName) {
		_usersBoxClassName = usersBoxClassName;
	}
	
	
	
	protected void onUserSelect(User user, boolean selection, AjaxRequestTarget target) {}
	protected void onUserDrop(User use, AjaxRequestTarget target) {}
	
}
