/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.config;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;

public class WebAppConfig implements Serializable {

	
	
	public enum ConfigState{
		CHANGED,
		CREATED,
		DROPPED
	}
	
 
	private final static String _className = "WebAppConfig";
	private static final long serialVersionUID = 1L;

	private final static String tableName = "DMSCONFIG";
	
	private final static String SQL_CREATE_CONFIG = "CREATE TABLE "+tableName+"(ID INT, DBURL VARCHAR, DBUSER VARCHAR, DBPASSWORD VARCHAR);";
	private final static String SQL_DROP_CONFIG = "DROP TABLE "+tableName+";";
	private final static String SQL_READ_CONFIG = "SELECT * FROM "+tableName+";";
	private final static String SQL_INSERT_CONFIG = "INSERT INTO "+tableName+"(ID, DBURL, DBUSER, DBPASSWORD) VALUES(?,?,?,?);";
	private final static String SQL_UPDATE_CONFIG = "UPDATE "+tableName+" SET DBURL=?, DBUSER=?, DBPASSWORD=? where id=?;";

	private Connection _h2conn;

	
	public WebAppConfig() {
		try {
			Class.forName("org.h2.Driver");
			_h2conn = (Connection) DmsWebApplication.get().getServletContext().getAttribute("connection");
			_h2conn.setAutoCommit(true);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", 
				_className,
				"init",
				null,
				e
			);
			OnConfigError(dmsException);
		}
	}
	
	
	
	public void initConfig() {
		try {
			if(_h2conn==null || _h2conn.isClosed()) return;
			if(!Exists())_h2conn.createStatement().execute(SQL_CREATE_CONFIG);
			OnConfigChanged(ConfigState.CREATED, null);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", 
				_className,
				"initConfig",
				null,
				e
			);
			OnConfigError(dmsException);
		}
	}
	
	public void removeConfig() {
		
		try {
			if(_h2conn==null || _h2conn.isClosed()) return;
			if(Exists()) {
				_h2conn.createStatement().execute(SQL_DROP_CONFIG);
				OnConfigChanged(ConfigState.DROPPED, null);	
			}
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", 
				_className,
				"removeConfig",
				null,
				e
			);
			OnConfigError(dmsException);
		}
	}
	
	public boolean Exists() {
		try {
			if(_h2conn==null || _h2conn.isClosed()) return false;
			_h2conn.createStatement().execute(SQL_READ_CONFIG);
		}catch(Exception e) {
			return false;
		}
		return true;
	}
	
	 
	
	public void initParams() {
		WebAppConfigParams params = new WebAppConfigParams();
		setConfig(params);
	}
	
	
	public void setConfig(WebAppConfigParams config) {
		if(null==config) return;
		PreparedStatement ps=null;
		try{
			if(_h2conn==null || _h2conn.isClosed()) return;
			if(config.getConfigId()==-1 || !Exists()){
				config.setConfigId(1);
				ps = _h2conn.prepareStatement(SQL_INSERT_CONFIG);
				ps.setInt(1, config.getConfigId());
				ps.setString(2, config.getDBUrl());
				ps.setString(3, config.getDBUser());
				ps.setString(4, config.getDBPassword());
				ps.execute();
			}else {
				ps = _h2conn.prepareStatement(SQL_UPDATE_CONFIG);
				ps.setString(1, config.getDBUrl());
				ps.setString(2, config.getDBUser());
				ps.setString(3, config.getDBPassword());
				ps.setInt(4, config.getConfigId());
				ps.execute();
			}
			OnConfigChanged(ConfigState.CHANGED, config);
		} catch (Exception e ) {
			DmsException dmsException = new DmsException(1000, "E", 
				_className,
				"setConfig",
				null,
				e
			);
			OnConfigError(dmsException);
        } finally {
            if (ps != null) {
            	try{
            		ps.close();
            	}catch(Exception e){
            	}
            }
        }
	}
	
	public WebAppConfigParams getConfig() {
		 
		try {
			WebAppConfigParams config=new WebAppConfigParams();
			if(_h2conn==null || _h2conn.isClosed() || !Exists()) return null;
			try{
		        PreparedStatement ps = _h2conn.prepareStatement(SQL_READ_CONFIG);
		        ResultSet rs = ps.executeQuery();
		        if(rs.next()){
		        	config.setConfigId(rs.getInt(1));
		        	config.setDBUrl(rs.getString(2));
		        	config.setDBUser(rs.getString(3));
		        	config.setDBPassword(rs.getString(4));
		        }
		        rs.close();
		        ps.close();
		    }catch(Exception e){
		    	DmsException dmsException = new DmsException(1000, "E", 
					_className,
					"getConfig",
					null,
					e
				);
		    	throw dmsException;
		    }
			return config;
		}catch(DmsException e) {
			OnConfigError(e);
	    }catch(Exception e){
	    	DmsException dmsException = new DmsException(1000, "E", 
				_className,
				"getConfig",
				null,
				e
			);
	    	OnConfigError(dmsException);
	    }
		return null;
	}
	
	protected void OnConfigChanged(ConfigState configState, WebAppConfigParams wacp) {};
	protected void OnConfigError(DmsException e) {};
	
	
}
