/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider;


 
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.IFilterStateLocator;
import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
 
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.user.Users.UsersAccessType;

public class DPDocuments extends SortableDataProvider<Document, String> implements IFilterStateLocator<DocumentFilterClass> {
 
	private static final String _className = "DPDocuments";

	private static final long serialVersionUID = 1L;
	private final String _defaultSortColumn = "id";

	private DocumentFilterClass _filterClass = new DocumentFilterClass();
	private long _size=0;
	private String _userCondition="";
	private String _linkedWhereCondition="";
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	public DPDocuments() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);

		if(_session.getUserAccessType()==UsersAccessType.USER) {
			_userCondition = "dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
		}else {
		}
	}
	
	@Override
	public void detach() {
		super.detach();
	}
	
	@Override
	public Iterator<? extends Document> iterator(long first, long count) {
		try {
			prepareUserFilterCondition();
			return StaticLocators.getDocuments().getEntities(first, count, buildDBSortCondition(getSort(),_defaultSortColumn), _filterClass).listIterator();
		} catch (Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"iterator",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	
	@Override
	public IModel<Document> model(Document entity) {
		return Model.of(entity);
	}

	
	public long size() {
		try {
			prepareUserFilterCondition();
			_size = StaticLocators.getDocuments().getNumberOfEntities(_filterClass);
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"size",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);			
			_size=0;
		}	
		return _size;
	}


	@Override
	public DocumentFilterClass getFilterState() {
		return _filterClass;
	}
	
	private String buildDBSortCondition(SortParam<?> sort, String defaultColumn){
		String strSortParam=" order by "+defaultColumn+" ASC";
    	if(null!=sort){
    		String sortProperty = sort.getProperty().toString();
			strSortParam = " ORDER BY "+sortProperty;
			if(sort.isAscending()) strSortParam = strSortParam + " ASC";
			else strSortParam = strSortParam + " DESC";
		}
    	return strSortParam;
	}
	
	@SuppressWarnings("unused")
	private void sortList(List<Document> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"sortList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
 
	private void prepareUserFilterCondition() {
		
		if(!Strings.isEmpty(_userCondition) && !Strings.isEmpty(_linkedWhereCondition)) {
			_filterClass.setLinkedWhere(_linkedWhereCondition + " AND "+ _userCondition);
		}else if(!Strings.isEmpty(_userCondition)) {
			_filterClass.setLinkedWhere(_userCondition);
		}else if(!Strings.isEmpty(_linkedWhereCondition)) {
			_filterClass.setLinkedWhere(_linkedWhereCondition);
		}
	}

	@Override
	public void setFilterState(DocumentFilterClass filterClass) {
		_filterClass = filterClass;
		if(!Strings.isEmpty(filterClass.getLinkedWhere())) {
			_linkedWhereCondition = _filterClass.getLinkedWhere();
		}else {
			_linkedWhereCondition = null;
		}
	}
	
	
}
