/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.config;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.setup.DmsSetup;
import org.biffo.dms.setup.DmsTable;
import org.biffo.dms.setup.DmsSetup.DMS_Tables;

public class DPDmsTables extends SortableDataProvider<DmsTable, String>{

	
	
	private static final String _className = "DPDmsTables";
	private static final long serialVersionUID = 1L;

	private DmsSetup _dmsSetup = new DmsSetup();
	private List<DmsTable> _lstEntities = new LinkedList<DmsTable>();

	private String _defaultSortColumn = "tableName";
	private String _dburl, _username, _password;
	
	
	public DPDmsTables() {
		setSort(_defaultSortColumn, SortOrder.ASCENDING);
	}
	
	@Override
	public Iterator<? extends DmsTable> iterator(long first, long count) {
		sortList(_lstEntities,_defaultSortColumn, getSort().isAscending() );
		return _lstEntities.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<DmsTable> model(DmsTable model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		return _lstEntities.size();
	}
	
	public void refresh() {
		refreshDmsTables();
	}
	
	public void setConnectionParamsUrl(String dburl, String username, String password) {
		_dburl = dburl;
		_username = username;
		_password = password;
	}
 
	
	
	
	private void sortList(List<DmsTable> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
		}
	}
	
	private void refreshDmsTables() {
		try {
			_lstEntities.clear();

			if(!_dmsSetup.checkDBJDBC(_dburl)) {
				DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshDmsTables", 
					_dmsSetup.getLastMessage()
				);
				throw dmsException;
			}

			if(!_dmsSetup.checkDMSDBConnection(_username, _password)) {
				DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshDmsTables", 
					_dmsSetup.getLastMessage()
				);
				throw dmsException;
			}

			try {
				_dmsSetup.checkTables(DMS_Tables.DATA);
			}catch(Exception e) {}
			try {
				_dmsSetup.checkTables(DMS_Tables.USER);
			}catch(Exception e) {}

			_lstEntities.addAll(_dmsSetup.getTables(DMS_Tables.DATA));
			_lstEntities.addAll(_dmsSetup.getTables(DMS_Tables.USER));
			
		}catch(DmsException e) {
			e.printStackTrace();
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	
}
