/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.config;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.helpers.ImportProcessHelpers;



public class DPImportServers extends SortableDataProvider<org.biffo.dms.dataprovider.config.DPImportServers.ImportSource, String>{
 
	private static final String _className = "DPImportServers";
	private static final long serialVersionUID = 1L;
	private DmsWebSession _session = DmsWebSession.getSession();

	public class ImportSource implements Serializable, Comparable<ImportSource>{
		 
		private static final long serialVersionUID = 1L;

		@Override
		public String toString() {
			return "ImportSource [id=" + id + ", name=" + name + ", type=" + type + ", folder=" + folder + "]";
		}


		private int id;
		private String name;
		private String type;
		private String folder;

		public ImportSource() {}
		
		public int getId() {return id;}
		public void setId(int id) {this.id = id;}

		public String getName() {return name;}
		public void setName(String name) {this.name = name;}
		
		public String getType() {return type;}
		public void setType(String type) {this.type = type;}
		
		public String getFolder() {return folder;}
		public void setFolder(String folder) {this.folder = folder;}

		
		@Override
		public int compareTo(ImportSource other) {
			if (getName() == null || other.getName() == null) {
				return 0;
		    }
		    return getName().compareToIgnoreCase(other.getName());
		}

	}
	
	private List<ImportSource> lstImportSources = new LinkedList<ImportSource>();

	private String _defaultSortColumn = "ProcessName";
	
	public DPImportServers() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	
	
	@Override
	public Iterator<? extends ImportSource> iterator(long first, long count) {
		sortList(lstImportSources,_defaultSortColumn, getSort().isAscending() );
		return lstImportSources.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<ImportSource> model(ImportSource model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		if(lstImportSources.size()==0)refreshImportServers();
		return lstImportSources.size();
	}
	
	
	public void refresh() {
		refreshImportServers();
	}
	
	private void sortList(List<ImportSource> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"sortList",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void refreshImportServers() {
		try {
			
			lstImportSources.clear();
			
			ImportSource impWeb = new ImportSource();
			impWeb.setId(1);
			impWeb.setName("Web");
			impWeb.setType("Default");  // ImportFolder
			impWeb.setFolder(DmsWebApplication.getApplication().getDmsProcessingMapper().getDmsConfig().getImportFolder());
			lstImportSources.add(impWeb);
			
			
			List<IVariablesImpl> lstVariables = new LinkedList<IVariablesImpl>();
			lstVariables.addAll(ImportProcessHelpers.getServerFolderProcesses());
			lstVariables.addAll(ImportProcessHelpers.getServerFTPProcesses());
			lstVariables.addAll(ImportProcessHelpers.getServerSFTPProcesses());
			lstVariables.addAll(ImportProcessHelpers.getServerSMTPProcesses());
			
			for(IVariablesImpl serverImpl: lstVariables) {
				ImportSource impSource = new ImportSource();
				impSource.setId(serverImpl.getId());
				impSource.setName(serverImpl.getServerName());
				impSource.setType(serverImpl.getGroupType().getDescription());
				lstImportSources.add(impSource);
			}			
			
			Collections.sort(lstImportSources);
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshImportServers",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	 

	
	public List<ImportSource> getImportSources(){
		if(lstImportSources.isEmpty()) {
			refreshImportServers();
		}
		return lstImportSources;
	}

	 
	
}
