/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.config;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
 

public class DPImporters extends SortableDataProvider<ImportServerImpl, String>{

	private static final String _className = "DPImporters";
	private static final long serialVersionUID = 1L;

	private List<ImportServerImpl> lstImportProcesses = new LinkedList<ImportServerImpl>();

	private String _defaultSortColumn = "ProcessName";
	
	public DPImporters() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	
	
	@Override
	public Iterator<? extends ImportServerImpl> iterator(long first, long count) {
		sortList(lstImportProcesses,_defaultSortColumn, getSort().isAscending() );
		return lstImportProcesses.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<ImportServerImpl> model(ImportServerImpl model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		if(lstImportProcesses.size()==0)refreshImportProcess();
		return lstImportProcesses.size();
	}
	
	public void refresh() {
		refreshImportProcess();
	}
	
	public ImportServerImpl getImportServerById(int id) {
		refreshImportProcess();
		for(ImportServerImpl impServer : lstImportProcesses) {
			if(impServer.getId()==id) return impServer;
		}
		return null;
	}
	
	
	
	private void sortList(List<ImportServerImpl> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"sortList",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	}
	
	private void refreshImportProcess() {
		try {
			lstImportProcesses.clear();
			
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(VariableGroupType.IMPORT_PROCESS);
			vfc.setVariableName("processname");
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity:entities) {
				ImportServerImpl importProcess = new ImportServerImpl();
				importProcess.setVariable(entity);
				vfc.setVariableGroup(entity.getVariableGroup());
				vfc.setVariableName(null);
				List<Variable> entities2 = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
				for(Variable entity2:entities2) {
					importProcess.setVariable(entity2);
				}
				
				
				lstImportProcesses.add(importProcess);
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshImportProcess",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	}
	

}
