/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.config;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
 
 

public class DPSFtpServers extends SortableDataProvider<SFTPServerImpl, String>{

	private static final long serialVersionUID = 1L;
	private static final String _className = "DPSFtpServers";
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private List<SFTPServerImpl> lstServers = new LinkedList<SFTPServerImpl>();

	private String _defaultSortColumn = "serverName";
	
	public DPSFtpServers() {
		setSort(_defaultSortColumn, SortOrder.DESCENDING);
	}
	
	
	@Override
	public Iterator<? extends SFTPServerImpl> iterator(long first, long count) {
		sortList(lstServers,_defaultSortColumn, getSort().isAscending() );
		return lstServers.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<SFTPServerImpl> model(SFTPServerImpl model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		if(lstServers.size()==0)refreshServers();
		return lstServers.size();
	}
	
	
	public void refresh() {
		refreshServers();
	}
	
	private void sortList(List<SFTPServerImpl> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"sortList",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void refreshServers() {
		try {
			lstServers.clear();
			
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(VariableGroupType.SFTP);
			vfc.setVariableName("port");
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", vfc);
			for(Variable entity:entities) {
				SFTPServerImpl server = new SFTPServerImpl();
				server.setVariable(entity);
				server.setVariable(getSingelVariable(server.getServerName(), "hostprivatekey"));
				server.setVariable(getSingelVariable(server.getServerName(), "folder"));
				server.setVariable(getSingelVariable(server.getServerName(), "description"));
				server.setVariable(getSingelVariable(server.getServerName(), "autostart"));
				lstServers.add(server);
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshServers",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private Variable getSingelVariable(String variableGroup, String variableName) {
		try {
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroup(variableGroup);
			vfc.setVariableName(variableName);
			if(StaticLocators.getVariables().getNumberOfEntities(vfc)!=1)return null;
			else return StaticLocators.getVariables().getEntity(vfc);
		}catch(Exception e) {
		}
		return null;
	}

}
