/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.metrics;


import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentAggregationClass;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Fragments.FragmentFilterClass;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.os.documents.OsDocuments;
import de.kleinekuh.os.fields.OsField;
import de.kleinekuh.os.fields.OsFieldType;
import de.kleinekuh.os.fields.OsFields;
import de.kleinekuh.os.rest.query.AggregationQuery;
import de.kleinekuh.os.rest.query.BooleanQuery;
import de.kleinekuh.os.rest.query.term.TermQuery;
import de.kleinekuh.os.rest.query.types.QueryType;
import de.kleinekuh.os.rest.search.OsSearch;


public class DPMetrics  extends SortableDataProvider<Metric, String>{

	private static final String _className = "DPMetrics";
	private DmsWebSession _session = DmsWebSession.getSession();
	private static final long serialVersionUID = 1L;
	private List<Metric> lstValues = new LinkedList<Metric>();
	private String _defaultSortColumn = "id";
	private MetricType _metricType;
	private Component _parent;
	private String _userCondition="";
	private String _linkedWhereCondition="";

	
	public DPMetrics(MetricType metricType, Component parent) {
		_metricType = metricType;
		_parent = parent;
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			_userCondition = "dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
		}else {
		}
		setSort(_defaultSortColumn, SortOrder.ASCENDING);
	}
	
	
	@Override
	public Iterator<? extends Metric> iterator(long first, long count) {
		sortList(lstValues,_defaultSortColumn, getSort().isAscending() );
		return lstValues.subList((int)first, ((int)first+(int)count)).iterator();	
	}

	@Override
	public IModel<Metric> model(Metric model) {
		return Model.of(model);
	}

	@Override
	public long size() {
		if(lstValues.size()==0)refresh();
		return lstValues.size();
	}
	
	public void refresh() {
		
		switch (_metricType) {
			case document: refreshDocumentMetrics();break;
			case tags: refreshTagMetrics();break;
			case opensearch: refreshOpenSearchMetrics();break;
		}
	}
	
	
	
	private void sortList(List<Metric> lstEntities, final String sortKey, final boolean isAscending){
		try{
			if(sortKey!=null){
				Collections.sort(lstEntities, new Comparator<Object>() {
					@SuppressWarnings({ "unchecked", "rawtypes" })
					@Override
					public int compare(Object c1, Object c2) {
						int result=1;
						PropertyModel<Comparable<?>> model1= new PropertyModel<Comparable<?>>(c1, sortKey);
						PropertyModel<Comparable<?>> model2= new PropertyModel<Comparable<?>>(c2, sortKey);

						if(model1.getObject() == null && model2.getObject() == null) result = 0;
						else if(model1.getObject() == null) result = 1;
						else if(model2.getObject() == null) result = -1;
						else result = ((Comparable)model1.getObject()).compareTo(model2.getObject());
						result = isAscending ? result : -result;
						return result;
					}
				});
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"sortList",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	 
		

	 
	
	 
	private void refreshDocumentMetrics() {
		try {
			lstValues.clear();
			DocumentFilterClass dfc = new DocumentFilterClass();
			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, false);
			
			if(!Strings.isEmpty(_userCondition) && !Strings.isEmpty(_linkedWhereCondition)) {
				dfc.setLinkedWhere(_linkedWhereCondition + " AND "+ _userCondition);
			}else if(!Strings.isEmpty(_userCondition)) {
				dfc.setLinkedWhere(_userCondition);
			}else if(!Strings.isEmpty(_linkedWhereCondition)) {
				dfc.setLinkedWhere(_linkedWhereCondition);
			}

			
			
			DocumentAggregationClass fac = new DocumentAggregationClass();
			fac.addSumObjectForSelect("id");
			fac.addSumObjectForSelect("numberofpages");
			
			List<Document> entities = StaticLocators.getDocuments().getEntities(-1, -1, "", dfc, fac);
			if(entities.size()>0) {
				lstValues.add(new Metric(0, ResourceText.getText("mtActiveDocuments", _parent), entities.get(0).getId()));
				lstValues.add(new Metric(3, ResourceText.getText("mtPages", _parent), entities.get(0).getNumberOfPages()));
			}
			
			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, null);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, true);
			lstValues.add(new Metric(1,ResourceText.getText("mtDeletedDocuments", _parent), (int)StaticLocators.getDocuments().getNumberOfEntities(dfc)));

			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, true);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, false);
			lstValues.add(new Metric(2, ResourceText.getText("mtArchivedDocuments", _parent), (int)StaticLocators.getDocuments().getNumberOfEntities(dfc)));
			
			FragmentFilterClass ffc = new FragmentFilterClass();
			if(_session.getUserAccessType()==UsersAccessType.USER) {
				ffc.setLinkedWhere(" dms_fragments.iddocument IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")");
			}
			lstValues.add(new Metric(4, ResourceText.getText("mtFragments", _parent), (int)StaticLocators.getFragments().getNumberOfEntities(ffc)));
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshDocumentMetrics",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);			

		}
	}
	
	private void refreshTagMetrics() {
		try {
			lstValues.clear();
			
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
			rfc.setIdTenant(-1);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);

			List<Tag> entities = StaticLocators.getTags().getEntities(-1, -1, "", null);
			int id=0;
			for(Tag entity:entities) {
				rfc.setIdrelation1(entity.getId());
				lstValues.add(new Metric(id++, entity.getTagName(), (int)StaticLocators.getRelations().getNumberOfEntities(rfc)));
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshTagMetrics",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);			
		}
	}
	
	private void refreshOpenSearchMetrics() {
		try {
			lstValues.clear();

			BooleanQuery bq = new BooleanQuery();
			TermQuery tmA = new TermQuery();
			tmA.setField("9_archive");
			tmA.setValue("false");
			bq.addMust(tmA);
			
			TermQuery tmD = new TermQuery();
			tmD.setField("10_deleted");
			tmD.setValue("false");
			bq.addMust(tmD);
			
			AggregationQuery aquery = new AggregationQuery();
			aquery.setField("4_doccontent");
			aquery.setDistinctName("usage");
			aquery.setQueryType(QueryType.significant_text);
			aquery.setSize(5);
			
			OsSearch search = new OsSearch();
			search.setQuery(bq);
			search.setAggregation(aquery);
			search.setSize(0);

			OsFields bucketFields = new OsFields();
			bucketFields.add(new OsField().setName("key").setType(OsFieldType.TEXT));
			bucketFields.add(new OsField().setName("doc_count").setType(OsFieldType.INTEGER));
			bucketFields.add(new OsField().setName("score").setType(OsFieldType.DOUBLE));
			bucketFields.add(new OsField().setName("bg_count").setType(OsFieldType.INTEGER));

			OsDocuments osDocuments = new OsDocuments(StaticLocators.getOSConnection(), StaticLocators.getOsIndex(), bucketFields);
			osDocuments.refresh(search);
			for(int i=0;i<osDocuments.size();i++) {
				lstValues.add(new Metric(i, osDocuments.get(i).getFieldValue("key").toString(), Integer.valueOf(osDocuments.get(i).getFieldValue("bg_count").toString())));
			}
			
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshOpenSearchMetrics",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	
	 

	
}
