/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.os;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.BitSet;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.os.EntityFieldToOsFieldMapping;
import org.biffo.dms.os.EntityToOsMappingStrategy;
import org.biffo.dms.os.SearchResultHashCodeType;
import org.biffo.dms.os.SearchResultsBuilder;

import de.kleinekuh.os.OsHelpers;
import de.kleinekuh.os.rest.query.BooleanQuery;
import de.kleinekuh.os.rest.query.HighlightQuery;
import de.kleinekuh.os.rest.query.IQuery;
import de.kleinekuh.os.rest.query.RangeQuery;
import de.kleinekuh.os.rest.query.SimpleQuery;
import de.kleinekuh.os.rest.query.SortQuery;
import de.kleinekuh.os.rest.query.match.MultiMatchQuery;
import de.kleinekuh.os.rest.query.term.TermQuery;
import de.kleinekuh.os.rest.query.term.TermsQuery;
import de.kleinekuh.os.rest.query.types.HighlightingType;
import de.kleinekuh.os.rest.query.types.MultiMatchType;
import de.kleinekuh.os.rest.query.types.SortOrderType;
import de.kleinekuh.os.rest.search.OsSearch;

public class DPSearchDocumentsHighlighted implements Serializable {

	private final static SimpleDateFormat _sdf = new SimpleDateFormat("yyyy-MM-dd");
	private static final long serialVersionUID = 1L;
	
	private EntityToOsMappingStrategy _strategy = new EntityToOsMappingStrategy(Document.class);
	private SearchResultsBuilder _searchResultsBuilder = new SearchResultsBuilder();
	private final ArrayList<Document> lstDocuments = new ArrayList<>();
	
	private DmsWebSession _session = DmsWebSession.getSession();
	
	public DPSearchDocumentsHighlighted() {
		buildEntityMapping();
	}

	
	public ArrayList<Document> getHightlightedDocuments(DocumentOSFilterClass dsfc) {
		lstDocuments.clear();
		try {
			OsSearch search = getHighlightSearch(dsfc);
			lstDocuments.addAll(StaticLocators.getDocumentsDBOS().getOsFilteredEntities(-1, -1, "", dsfc, search, _strategy, _searchResultsBuilder));
			return lstDocuments;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "getHightlightedDocuments", null, e, dsfc);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	private OsSearch getHighlightSearch(DocumentOSFilterClass dsfc) {

		OsSearch search = new OsSearch();
		BooleanQuery bq = new BooleanQuery();
		
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			TermQuery tm = new TermQuery();
			tm.setField("98_users");
			tm.setValue(""+_session.getCurrentUserId());
			bq.addMust(tm);
		}
		
		
		if(dsfc.getId()!=-1) {
			TermQuery tm = new TermQuery();
			tm.setField("1_docid");
			tm.setValue(""+dsfc.getId());
			bq.addMust(tm);
		}
		if(dsfc.getIDList()!=null && dsfc.getIDList().size()>0) {
			TermsQuery tsm = new TermsQuery();
			tsm.setField("1_docid");
			tsm.setValues(dsfc.getIDList());
			bq.addMust(tsm);
		}
		
		
		if(dsfc.getSearchString()!=null && !dsfc.getSearchString().isEmpty()) {
			bq.addMust(getDocContentQuery(dsfc));
		}
		
		if(dsfc.hasTags()) {
			bq.addMust(getTagQuery(dsfc));
		}
		
		if(dsfc.getDocumentDatumTimeFrom()!=null || dsfc.getDocumentDatumTimeTo()!=null) {
			bq.addMust(getDateRangeQuery(dsfc));
		}
		setDocumentConditionQuery(dsfc, bq);
		
		HighlightQuery hq = new HighlightQuery();
		hq.addField("4_doccontent");
		hq.addField("2_docname");
		hq.setNoMatchSize(0);
		hq.setNumberOfFragments(0);
		hq.setFragmentSize(18);
		hq.setType(HighlightingType.FVH);
		hq.setRequireFieldMatch(true);
		hq.setPreTags("<mark>");
		hq.setPostTags("</mark>");
		
		SortQuery sq = new SortQuery();
		sq.setField("3_docpage");
		sq.setOrder(SortOrderType.asc);
		
		search.setSource(true);
		search.setQuery(bq);
		search.setHighlight(hq);
		search.setSize(dsfc.getSize());
		search.setSort(sq);
		
		return search;
	}
	
	private IQuery getDocContentQuery(DocumentOSFilterClass dofc) {
		String queryString = OsHelpers.convertToUtf8(dofc.getSearchString());
		SimpleQuery query = new SimpleQuery();
		query.addField("4_doccontent");
		query.addField("2_docname");
		query.setQuery(queryString);
		return query;
		

	}
	
	
	private IQuery getTagQuery(DocumentOSFilterClass dofc) {
		MultiMatchQuery mmq = new MultiMatchQuery();
		mmq.setQuery(dofc.getTags());
		mmq.addField("8_tags");
		mmq.setType(MultiMatchType.cross_fields);
		return mmq;
	}
	
	private IQuery getDateRangeQuery(DocumentOSFilterClass dofc) {
		RangeQuery rq = new RangeQuery();
		rq.setField("12_documentdate");
		if(dofc.getDocumentDatumTimeFrom()!=null) {
			rq.setGte(_sdf.format(dofc.getDocumentDatumTimeFrom()));
		}
		if(dofc.getDocumentDatumTimeTo()!=null) {
			rq.setLte(_sdf.format(dofc.getDocumentDatumTimeTo()));
		}
		return rq;
	}
	
	private void setDocumentConditionQuery(DocumentOSFilterClass dofc, BooleanQuery bq) {
		
		BitSet bytesFalse=BitSet.valueOf(dofc.getStatusTypesFalse());
		BitSet bytesTrue=BitSet.valueOf(dofc.getStatusTypesTrue());
		
		if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())!=false || bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("9_archive");
			if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
		
		if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())!=false || bytesTrue.get(DocumentStatusType.DELETED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("10_deleted");
			if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
	}
	
	
	private void buildEntityMapping() {
		EntityFieldToOsFieldMapping idField = new EntityFieldToOsFieldMapping("1_docid", "Id", true);
		EntityFieldToOsFieldMapping pageField = new EntityFieldToOsFieldMapping("3_docpage", "Page", false);
		EntityFieldToOsFieldMapping contentField = new EntityFieldToOsFieldMapping("4_doccontent", "Resulttext", true);
		EntityFieldToOsFieldMapping docnameField = new EntityFieldToOsFieldMapping("2_docname", "DocName", true);
		_strategy.addFieldMapping(idField);
		_strategy.addFieldMapping(pageField);
		_strategy.addFieldMapping(contentField);
		_strategy.addFieldMapping(docnameField);
		
		_searchResultsBuilder.setSearchResultHashCodeType(SearchResultHashCodeType.foundTerm_documentId);
		_searchResultsBuilder.addContentField(contentField);
		_searchResultsBuilder.setIdField(idField);
		_searchResultsBuilder.setPageField(pageField);
		_searchResultsBuilder.setBuildResultFragment(true);
		_searchResultsBuilder.setOffsetEnd(30);
		_searchResultsBuilder.setOffsetStart(40);
		_searchResultsBuilder.setPattern("(<mark>(.*?)</mark>)");
	}
	
	
	
	
	
}
