/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.os;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.BitSet;
import java.util.HashMap;
import java.util.LinkedList;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
 
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.user.Users.UsersAccessType;

import de.kleinekuh.os.OsHelpers;
import de.kleinekuh.os.documents.OsDocuments;

import de.kleinekuh.os.rest.query.AggregationQuery;
import de.kleinekuh.os.rest.query.BooleanQuery;
import de.kleinekuh.os.rest.query.IQuery;
import de.kleinekuh.os.rest.query.RangeQuery;
import de.kleinekuh.os.rest.query.SimpleQuery;
import de.kleinekuh.os.rest.query.SortQuery;
import de.kleinekuh.os.rest.query.match.MultiMatchQuery;
import de.kleinekuh.os.rest.query.term.TermQuery;
import de.kleinekuh.os.rest.query.types.MultiMatchType;
import de.kleinekuh.os.rest.query.types.SortOrderType;
import de.kleinekuh.os.rest.search.OsSearch;

public class DPSearchDocumentsIds implements Serializable {

	private final static SimpleDateFormat _sdf = new SimpleDateFormat("yyyy-MM-dd");
	private static final long serialVersionUID = 1L;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	
	public DPSearchDocumentsIds() {
	}
	
	public HashMap<String, LinkedList<String>> getDocumentIds(DocumentOSFilterClass dsfc, boolean withPageNumbers){
		return getDocumentIds(dsfc, withPageNumbers, false);
	}
	

	public HashMap<String, LinkedList<String>> getDocumentIds(DocumentOSFilterClass dsfc, boolean withPageNumbers, boolean contentOnly){
		try {
			
			OsDocuments osDocuments = new OsDocuments(StaticLocators.getOSConnection(), StaticLocators.getOsIndex());
			osDocuments.refresh(getIdSearch(dsfc, contentOnly));
			dsfc.setTotalNumberOfDocuments(osDocuments.getTotalNumberOfDocuments());
			HashMap<String, LinkedList<String>> hmDocumentIds = new HashMap<String, LinkedList<String>>();
			for(int i=0;i<osDocuments.size();i++) {
				String strArr[] = osDocuments.get(i).getId().split("_");
				if(strArr.length==2) {
					if(hmDocumentIds.containsKey(strArr[0])) {
						hmDocumentIds.get(strArr[0]).add(strArr[1]);
					}else {
						LinkedList<String> lstX = new LinkedList<>();
						lstX.add(strArr[1]);
						hmDocumentIds.put(strArr[0], lstX);
					}
				}else {
					if(!hmDocumentIds.containsKey(strArr[0])) {
						LinkedList<String> lstX = new LinkedList<>();
						hmDocumentIds.put(strArr[0], lstX);
					}
				}
			}
			return hmDocumentIds;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "getDocumentIds", null, e, dsfc);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	private IQuery getDocContentQuery(DocumentOSFilterClass dofc, boolean contentOnly) {
		String queryString = OsHelpers.convertToUtf8(dofc.getSearchString());
		SimpleQuery smq = new SimpleQuery();
		smq.addField("4_doccontent");
		if(!contentOnly)smq.addField("2_docname");
		smq.setQuery(queryString);
		return smq;
	}
	
	
	private IQuery getTagQuery(DocumentOSFilterClass dofc) {
		MultiMatchQuery mmq = new MultiMatchQuery();
		mmq.setQuery(dofc.getTags());
		mmq.addField("8_tags");
		mmq.setType(MultiMatchType.cross_fields);
		return mmq;
	}
	
	private IQuery getDateRangeQuery(DocumentOSFilterClass dofc) {
		RangeQuery rq = new RangeQuery();
		rq.setField("12_documentdate");
		if(dofc.getDocumentDatumTimeFrom()!=null) {
			rq.setGte(_sdf.format(dofc.getDocumentDatumTimeFrom()));
		}
		if(dofc.getDocumentDatumTimeTo()!=null) {
			rq.setLte(_sdf.format(dofc.getDocumentDatumTimeTo()));
		}
		return rq;
	}
	
	private void setDocumentConditionQuery(DocumentOSFilterClass dofc, BooleanQuery bq) {
		
		BitSet bytesFalse=BitSet.valueOf(dofc.getStatusTypesFalse());
		BitSet bytesTrue=BitSet.valueOf(dofc.getStatusTypesTrue());
		
		if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())!=false || bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("9_archive");
			if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
		
		if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())!=false || bytesTrue.get(DocumentStatusType.DELETED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("10_deleted");
			if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
	}
	
	private AggregationQuery getAggregationQuery_Id() {
		AggregationQuery aggQuery = new AggregationQuery();
		aggQuery.setField("1_docid");
		aggQuery.setDistinctName("1_docid");
		aggQuery.setSize(100000);
		return aggQuery;
	}
	
	private AggregationQuery getAggregationQuery_Id_Page() {
		AggregationQuery aggQuery = new AggregationQuery();
		aggQuery.setField("_id");
		aggQuery.setDistinctName("1_docid");
		aggQuery.setSize(100000);
		return aggQuery;
	}
	
	
	private OsSearch getIdSearch(DocumentOSFilterClass dsfc, boolean contentOnly) {
		
		
		OsSearch search = new OsSearch();
		search.setAggregation(getAggregationQuery_Id());
		 
		
		BooleanQuery bq = new BooleanQuery();
		
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			TermQuery tm = new TermQuery();
			tm.setField("98_users");
			tm.setValue(""+_session.getCurrentUserId());
			bq.addMust(tm);
		}
		
		if(dsfc.getId()!=-1) {
			TermQuery tm = new TermQuery();
			tm.setField("1_docid");
			tm.setValue(""+dsfc.getId());
			bq.addMust(tm);
		}
		
		if(dsfc.getSearchString()!=null && !dsfc.getSearchString().isEmpty()) {
			bq.addMust(getDocContentQuery(dsfc, contentOnly));
			search.setAggregation(getAggregationQuery_Id_Page());	
		}
		
		if(dsfc.hasTags()) {
			bq.addMust(getTagQuery(dsfc));
		}
		
		if(dsfc.getDocumentDatumTimeFrom()!=null || dsfc.getDocumentDatumTimeTo()!=null) {
			bq.addMust(getDateRangeQuery(dsfc));
		}
		setDocumentConditionQuery(dsfc, bq);
		
		search.setSource(false);
		search.setQuery(bq);
		search.setSize(dsfc.getSize());

		SortQuery sq = new SortQuery();
		sq.setField("1_docid");
		sq.setOrder(SortOrderType.desc);
		search.setSort(sq);
		
		return search;
	}
	
	
	
	
}
