/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.dataprovider.os;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.BitSet;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
 
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.os.EntityFieldToOsFieldMapping;
import org.biffo.dms.os.EntityToOsMappingStrategy;
import org.biffo.dms.os.SearchResultHashCodeType;
import org.biffo.dms.os.SearchResults;
import org.biffo.dms.os.SearchResultsBuilder;

import de.kleinekuh.os.OsHelpers;
import de.kleinekuh.os.rest.query.BooleanQuery;
import de.kleinekuh.os.rest.query.HighlightQuery;
import de.kleinekuh.os.rest.query.IQuery;
import de.kleinekuh.os.rest.query.RangeQuery;
import de.kleinekuh.os.rest.query.match.MultiMatchQuery;
import de.kleinekuh.os.rest.query.term.TermQuery;
import de.kleinekuh.os.rest.query.types.HighlightingType;
import de.kleinekuh.os.rest.query.types.MultiMatchType;
import de.kleinekuh.os.rest.search.OsSearch;

public class DPSearchDocumentsSingelField implements Serializable {

	private static final String _className = "DPSearchDocumentsSingelField";
	private final static SimpleDateFormat _sdf = new SimpleDateFormat("yyyy-MM-dd");
	private static final long serialVersionUID = 1L;
	
	private static final EntityToOsMappingStrategy _strategy = new EntityToOsMappingStrategy(Document.class);
	private static final SearchResultsBuilder _searchResultsBuilder = new SearchResultsBuilder();
	
	private final SearchResults dummySearchReslts = new SearchResults();
	private final ArrayList<Document> lstDocuments = new ArrayList<>();
	
	private DmsWebSession _session =  DmsWebSession.getSession();

	public DPSearchDocumentsSingelField() {
		buildEntityMapping();
	}
	
	public SearchResults getSearchResults(DocumentOSFilterClass dsfc) {
		lstDocuments.clear();
		try {
			lstDocuments.addAll(StaticLocators.getDocumentsDBOS().getOsFilteredEntities(-1, -1, "", dsfc, getSingleFieldSearch(dsfc, true, dsfc.getSize()), _strategy, _searchResultsBuilder));
			if(lstDocuments.size()==0) return dummySearchReslts;
			else return lstDocuments.get(0).getSearchResults();
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"getSearchResults",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
		
	}

	private void buildEntityMapping() {
		EntityFieldToOsFieldMapping idField = new EntityFieldToOsFieldMapping("1_docid", "Id", false);
		EntityFieldToOsFieldMapping pageField = new EntityFieldToOsFieldMapping("3_docpage", "Page", false);
		EntityFieldToOsFieldMapping contentField = new EntityFieldToOsFieldMapping("4_doccontent", "Resulttext", false);
		EntityFieldToOsFieldMapping docNameField = new EntityFieldToOsFieldMapping("2_docname", "Resulttext", false);
		_strategy.addFieldMapping(idField);
		_strategy.addFieldMapping(pageField);
		_strategy.addFieldMapping(contentField);
		_strategy.addFieldMapping(docNameField);
		_strategy.setMergeDBFields(false);
		_strategy.setMergeGroupBy(false);
		
		_searchResultsBuilder.setSearchResultHashCodeType(SearchResultHashCodeType.foundTerm);
		_searchResultsBuilder.addContentField(contentField);
		_searchResultsBuilder.addContentField(docNameField);
		_searchResultsBuilder.setIdField(idField);
		_searchResultsBuilder.setPageField(pageField);
		_searchResultsBuilder.setAddChild(true);
		_searchResultsBuilder.setBuildResultFragment(false);
		_searchResultsBuilder.setOffsetEnd(30);
		_searchResultsBuilder.setOffsetStart(40);
		
	}
	
	
	private OsSearch getSingleFieldSearch(DocumentOSFilterClass dsfc, boolean withHighlight, int searchSize) {
		
		BooleanQuery bq = new BooleanQuery();
		
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			TermQuery tm = new TermQuery();
			tm.setField("98_users");
			tm.setValue(""+_session.getCurrentUserId());
			bq.addMust(tm);
		}
		
		
		if(dsfc.getId()!=-1) {
			TermQuery tm = new TermQuery();
			tm.setField("1_docid");
			tm.setValue(""+dsfc.getId());
			bq.addMust(tm);
		}
		
		
		if(dsfc.getSearchString()!=null && !dsfc.getSearchString().isEmpty()) {
			bq.addMust(getDocContentQuery(dsfc));
		}
		
		if(dsfc.hasTags()) {
			bq.addMust(getTagQuery(dsfc));
		}
		
		if(dsfc.getDocumentDatumTimeFrom()!=null || dsfc.getDocumentDatumTimeTo()!=null) {
			bq.addMust(getDateRangeQuery(dsfc));
		}
		
		setDocumentConditionQuery(dsfc, bq);
		
		OsSearch search = new OsSearch();
		search.setSource(false);
		search.setQuery(bq);
		search.addField("1_docid");
		
		if(withHighlight) {
			HighlightQuery hq = new HighlightQuery();
			hq.addField("4_doccontent");
			hq.addField("2_docname");
			hq.setFragmentSize(18);
			hq.setNoMatchSize(0);
			hq.setNumberOfFragments(0);
			hq.setType(HighlightingType.FVH);
			hq.setRequireFieldMatch(true);
			search.setHighlight(hq);
		}
		search.setSize(searchSize);
		return search;
	}
	
	private IQuery getDocContentQuery(DocumentOSFilterClass dofc) {
		String queryString = OsHelpers.convertToUtf8(dofc.getSearchString());
		
		MultiMatchQuery mmq = new MultiMatchQuery();
		mmq.setQuery(queryString);
		mmq.addField("4_doccontent");
		mmq.addField("2_docname");
		mmq.setType(MultiMatchType.phrase_prefix);
		return mmq;

	}
	
	
	private IQuery getTagQuery(DocumentOSFilterClass dofc) {
		MultiMatchQuery mmq = new MultiMatchQuery();
		mmq.setQuery(dofc.getTags());
		mmq.addField("8_tags");
		mmq.setType(MultiMatchType.cross_fields);
		return mmq;
	}
	
	private IQuery getDateRangeQuery(DocumentOSFilterClass dofc) {
		RangeQuery rq = new RangeQuery();
		rq.setField("12_documentdate");
		if(dofc.getDocumentDatumTimeFrom()!=null) {
			rq.setGte(_sdf.format(dofc.getDocumentDatumTimeFrom()));
		}
		if(dofc.getDocumentDatumTimeTo()!=null) {
			rq.setLte(_sdf.format(dofc.getDocumentDatumTimeTo()));
		}
		return rq;
	}
	
	private void setDocumentConditionQuery(DocumentOSFilterClass dofc, BooleanQuery bq) {
		
		BitSet bytesFalse=BitSet.valueOf(dofc.getStatusTypesFalse());
		BitSet bytesTrue=BitSet.valueOf(dofc.getStatusTypesTrue());
		
		if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())!=false || bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("9_archive");
			if(bytesFalse.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.ARCHIVED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
		
		if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())!=false || bytesTrue.get(DocumentStatusType.DELETED.ordinal())!=false){
			TermQuery tm = new TermQuery();
			tm.setField("10_deleted");
			if(bytesFalse.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("false");
			}else if(bytesTrue.get(DocumentStatusType.DELETED.ordinal())){
				tm.setValue("true");
			}
			bq.addMust(tm);
		}
	}
	
}
