/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.helpers;

import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;

import de.kleinekuh.entity.types.RecordStatusType;

 

public class DmsDBHelpersTag {

	private static final String _className = "DmsDBHelpersTag";
	
	public static List<Relation> getRelatedRelations(int idRelation1, RelationType relationType, int idTenant) {
		
		try {
			
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdTenant(idTenant);
			rfc.setRelationType(relationType);
			rfc.setIdrelation1(idRelation1);
			
			List<Relation> rc = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			return rc;
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"getRelatedRelations",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
		return null;
	}
 
	public static void addTagsToDocuments(HashSet<Integer> documentIds, HashSet<Tag> tags, String user, int tennant) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				for(Tag tag : tags) {
					addTagToDocument(documentId, tag, user, tennant);
				}
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"addTagsToDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	public static void addTagToDocument(Integer documentId, Tag tag, String user, int tennant) throws Exception {
		if(tagExistsOnDocument(documentId, tag.getId(), tennant)) return;
		Relation relation = new Relation();
		relation.setIdrelation1(tag.getId());
		relation.setIdrelation2(documentId);
		relation.setErstelltAm(new Date());
		relation.setErstelltVon(user);
		relation.setGeaendertAm(relation.getErstelltAm());
		relation.setGeaendertVon(relation.getErstelltVon());
		relation.setIdTenant(tennant);
		relation.setRelationType(RelationType.TAG_TO_DOCUMENT);
		relation.setRelationStatus(RelationStatusType.ACTIVE);
		StaticLocators.getRelations().addEntity(relation);
		
		StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(documentId));
	}
	
	public static void addTagToDocument(Integer documentId, Integer tagId, String user, int tennant) throws Exception {
		if(tagExistsOnDocument(documentId, tagId, tennant)) return;
		Relation relation = new Relation();
		relation.setIdrelation1(tagId);
		relation.setIdrelation2(documentId);
		relation.setErstelltAm(new Date());
		relation.setErstelltVon(user);
		relation.setGeaendertAm(relation.getErstelltAm());
		relation.setGeaendertVon(relation.getErstelltVon());
		relation.setIdTenant(tennant);
		relation.setRelationType(RelationType.TAG_TO_DOCUMENT);
		relation.setRelationStatus(RelationStatusType.ACTIVE);
		
		
		StaticLocators.getRelations().addEntity(relation);
		
		StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(documentId));
	}
	
	private static boolean tagExistsOnDocument(Integer documentId, Integer tagId, int tennant) throws Exception {
		RelationFilterClass rfc = new RelationFilterClass();
		rfc.setIdTenant(tennant);
		rfc.setIdrelation1(tagId);
		rfc.setIdrelation2(documentId);
		rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
		
		if(StaticLocators.getRelations().getNumberOfEntities(rfc)>0) return true;
		return false;
	}
	
	
	
	private static boolean tagToDocumentRelationExists(HashSet<Tag> tags, int idrelation1) {
		for(Tag tag : tags) {
			if(tag.getId()==idrelation1) return true;
		}
		return false;
	}
	
	
	
	public static void replaceTagsFromDocuments(HashSet<Integer> documentIds, HashSet<Tag> tags, String user, int tennant) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation2(documentId);
				rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
				
				List<Relation> relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						if(!tagToDocumentRelationExists(tags, relationToDelete.getIdrelation1())) {
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());
						}
					}
				}
				for(Tag tag : tags) {
					addTagToDocument(documentId, tag, user, tennant);
				}
				
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"replaceTagsFromDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	 
	
	
	public static void deleteTagsFromDocuments(HashSet<Integer> documentIds, HashSet<Tag> tags, String user, int tennant) throws Exception {
		try {
			for(Integer documentId : documentIds) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdTenant(tennant);
				rfc.setIdrelation2(documentId);
				rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
				
				List<Relation> relationsToDelete = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(relationsToDelete.size()>0) {
					for(Relation relationToDelete : relationsToDelete) {
						if(tagToDocumentRelationExists(tags, relationToDelete.getIdrelation1())) {
							StaticLocators.getRelations().deleteEntityById(relationToDelete.getId());
						}
					}
				}
				StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(documentId));
			}
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"deleteTagsFromDocuments",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());
		}
	}
	
	
	
	
	
	
	
	public static void addReplaceRelations(IVariablesImpl server, List<ISelectData> selectionList, RelationType realtionType, String user) throws Exception{

		if(server.getId()==-1) return;
			
		RelationFilterClass rfc = new RelationFilterClass();
		rfc.setIdrelation2(server.getId());
		rfc.setRecordStatus(RecordStatusType.ACTIVE);
		rfc.setRelationStatus(RelationStatusType.ACTIVE);
		rfc.setRelationType(realtionType);
		
		long numberOfEntities = StaticLocators.getRelations().getNumberOfEntities(rfc);
		if(numberOfEntities>0) {
			List<Relation> entities = StaticLocators.getRelations().getEntities(0, numberOfEntities, "", rfc);
			for(int i=0;i<entities.size();i++) {
				Relation entity = entities.get(i);
				boolean delete = true;
				for(ISelectData selected : selectionList){
					if(selected.getId()==entity.getIdrelation1() && selected.getSelected()) {
						delete = false;
						break;
					}
				}
				if(delete) {
					StaticLocators.getRelations().deleteEntityById(entity.getId());
				}
			}
		}
		

		if(null!=selectionList && selectionList.size()>0){
			for(ISelectData entity : selectionList){
				if(entity.getSelected()) {
					
					Relation relation = new Relation();
					relation.setErstelltVon(user);
					relation.setErstelltAm(new Date());
					relation.setRecordStatus(RecordStatusType.ACTIVE);
					relation.setRelationStatus(RelationStatusType.ACTIVE);
					relation.setGeaendertVon(relation.getErstelltVon());
					relation.setGeaendertAm(relation.getErstelltAm());
					relation.setIdrelation1(entity.getId());
					relation.setIdrelation2(server.getId());
					relation.setRelationType(realtionType);
					int idTemp = StaticLocators.getRelations().getEntityId(relation);
					if(idTemp==-1) {
						StaticLocators.getRelations().addEntity(relation);
					}	
				}
			}
		}
	} 
	
	
	public static int getDefaultPDFImporterID() {
		int idDefaultPanel = -1;
		DPImporters _qtdp = new DPImporters();
		long size = _qtdp.size();
		Iterator<? extends ImportServerImpl> itt = _qtdp.iterator(0, size);
		while(itt.hasNext()) {
			ImportServerImpl isei =itt.next();
			if(isei.getProcessName().equals("Default PDF Importer") && isei.isProcessactive()) {
				idDefaultPanel = isei.getId();
				break;
			}
		}
		return idDefaultPanel;
	}
	
	

	
	
}
