/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.biffo.dms.DmsException;
import org.biffo.dms.StaticLocators;

public class ProcessQueue {

	private List<ProcessBridge> bridges = new ArrayList<ProcessBridge>();
	private ExecutorService executorService = new ThreadPoolExecutor(10, 10, 0L, TimeUnit.MILLISECONDS,	new LinkedBlockingQueue<Runnable>());
	
	public ProcessQueue(){
	}
	
	
 
	
	
	
	public void launch(IProcessQueueTask task) {

		if(isTaskRunning(task.getTaskName())) return;
		
		ProcessBridge bridge = new ProcessBridge();
		bridge.setTaskName("Task-" + countTasks() + 1);
		bridge.setPrimaryTaskName(task.getTaskName());
		add(bridge);
		executorService.execute( new ProcessRunnable(task, bridge));
	}
	
	public void add(IProcessQueueTask task) {

		if(isTaskRunning(task.getTaskName())) return;
		
		ProcessBridge bridge = new ProcessBridge();
		bridge.setTaskName("Task-" + countTasks() + 1);
		bridge.setPrimaryTaskName(task.getTaskName());
		bridge.setStop(true);
		add(bridge);
		executorService.execute( new ProcessRunnable(task, bridge));
	}
	
	
	public void destroy(){
		executorService.shutdown();
		try {
			for(ProcessBridge bridge: this.bridges) {
				bridge.setCancel(true);
			}
			
			if (!executorService.awaitTermination(2, TimeUnit.SECONDS)) {
				executorService.shutdownNow();
			}
		}catch (InterruptedException e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					"ProcessQueue", 
					"iterator",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, null);
		}
	}
	
	public synchronized void refresh(String taskName) {
		
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getTaskName().equals(taskName)) {
				bridge.setRefresh(true);
			}
		}
	}
	
	public synchronized void stop(String taskName) {
		
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getTaskName().equals(taskName)) {
				bridge.setStop(true);
			}
		}
	}

	public synchronized void start(String taskName) {
		
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getTaskName().equals(taskName)) {
				bridge.setStop(false);
			}
		}
	}
	
	
	public synchronized void remove(String taskName) {
		
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getTaskName().equals(taskName)) {
				bridge.setCancel(true);
			}
		}
		pruneFinishedTasks();
	}

	
	public synchronized void add(ProcessBridge bridge) {
		bridges.add(bridge);
	}
	
	public boolean isTaskRunning(String taskName) {
		for(ProcessBridge bridge: this.bridges) {
			if (!bridge.isFinished()) {
				if(bridge.getPrimaryTaskName().equals(taskName)) return true;
			}
		}
		return false;
	}
	
	public boolean isRunning(String taskName) {
		for(ProcessBridge bridge: this.bridges) {
			if (!bridge.isStop()) {
				if(bridge.getTaskName().equals(taskName)) return true;
			}
		}
		return false;
	}
	

	public boolean isStopped(String taskName) {
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getTaskName().equals(taskName)) {
				return bridge.isStop();
			}
		}
		return false;
	}
	
	
	
	public boolean hasTask(String taskName) {
		for(ProcessBridge bridge: this.bridges) {
			if(bridge.getPrimaryTaskName().equals(taskName)) return true;
		}
		return false;
	}
	
	
	public synchronized void pruneFinishedTasks() {
		ArrayList<ProcessBridge> nonFinishedBridges = new ArrayList<ProcessBridge>();
		for(ProcessBridge bridge: this.bridges) {
			if (!bridge.isFinished()) {
				nonFinishedBridges.add(bridge);
			}
		}
		this.bridges = nonFinishedBridges;
	}
	
	public Iterator<ProcessBridge> getTasksPage(int start, int size) {
		int min = Math.min(size, bridges.size());
		return new ArrayList<ProcessBridge>(bridges.subList(start, min)).iterator();
	}
	
	public long countTasks() {
		return bridges.size();
	}
	
	
}
