/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.ftpServer;


import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.ftpserver.FtpServer;
import org.apache.ftpserver.FtpServerFactory;
import org.apache.ftpserver.ftplet.DefaultFtplet;
import org.apache.ftpserver.ftplet.FtpException;
import org.apache.ftpserver.ftplet.FtpRequest;
import org.apache.ftpserver.ftplet.FtpSession;
import org.apache.ftpserver.ftplet.Ftplet;
import org.apache.ftpserver.ftplet.FtpletResult;
import org.apache.ftpserver.listener.ListenerFactory;
import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebMessageHandler;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.FTPServerImpl;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
 
 

public class FTPServerTask implements IProcessQueueTask {

	private static final long serialVersionUID = 1L;
	
	private Application _application;
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.IMPORT_FTP;
	private String _taskName;
	private String _message;
	private String _status;
	
	private FtpServer ftpServer;
	private FTPServerUserManager ftpServerUserManager; 
	
	public FTPServerTask(FTPServerImpl ftpServerImpl) {
		
		_application = Application.get();
		
		_taskName=ftpServerImpl.getServerName();
		_status = "Start "+_processType.getDescription();
		
		FtpServerFactory serverFactory = new FtpServerFactory();
		ListenerFactory lisetnerFactory = new ListenerFactory();
		lisetnerFactory.setPort(ftpServerImpl.getServerPort());
		serverFactory.addListener("default", lisetnerFactory.createListener());
		
		ftpServerUserManager = new FTPServerUserManager(StaticLocators.getDmsProcessingMapper());
		ftpServerUserManager.setDataFolder(ftpServerImpl.getServerFolder());
		serverFactory.setUserManager(ftpServerUserManager);
		
		Map<String, Ftplet> ftplets = new LinkedHashMap<String, Ftplet>();
		ftplets.put(FTPServerLet.class.getName(), new FTPServerLet());
		serverFactory.setFtplets(ftplets);
		ftpServer = serverFactory.createServer();
		
		
	}
	
	public void refresh() {
		ftpServerUserManager.refreshFTPUsers();
	}
	
	
	@Override
	public void doIt(ProcessBridge processBridge) {

		_bridge = processBridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		try {
			
			if(!Application.exists())ThreadContext.setApplication(_application);
		
			ftpServer.start();
			createMessage(new DmsException(2000, "I", 
				"FTPServerTask", "doIt", _taskName+" started", null,	null),
				true
			);
			
			boolean running = true;
			while(running) {
				
				if(ftpServer.isSuspended()) {
					ftpServer.resume();
					createMessage(new DmsException(2000, "I", 
						"FTPServerTask", "doIt", _taskName+" resumed", null, null),
						true
					);
				}
				
				
				if(_bridge.isStop()) {
					ftpServer.stop();
					createMessage(new DmsException(2000, "I", 
						"FTPServerTask", "doIt", _taskName+" stopped", null, null),
						true
					);
					_bridge.setProgress(100);
					_bridge.setStop(true);
					running=false;
				}
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
				}
				
				stopWait(1);
				
			}
			_bridge.setProgress(100);
			_bridge.setStop(true);
			
		}catch(Exception e) {
			createMessage(new DmsException(2000, "E", 
				"FTPServerTask", "doIt", _taskName+" error", null, e),
				false
			);
			_bridge.setProcessException(e);
		}
	}
	
	private synchronized void createMessage(DmsException dmsInfo, boolean withBridgeStatus) {
		StaticLocators.getMessageHandler().createMessage(dmsInfo, null);
		if(null!=_bridge && withBridgeStatus) _bridge.setMessage(dmsInfo.getMessage());
	}
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getMessage() {return _message;}
	@Override
	public synchronized String getTaskName() {return _taskName;}
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}
	@Override
	public synchronized int getSourceDocumentId() {return -1;}
	@Override
	public synchronized int getTargetDocumentId() {return -1;}
	@Override
	public synchronized int getIdProcess() {return -1;}
	@Override
	public synchronized String getStatus() {return _status;}
	
	
	
	private class FTPServerLet extends DefaultFtplet {

		
		private DmsWebMessageHandler _messageHandler;
		
		public FTPServerLet() {
			super();
			_messageHandler = new DmsWebMessageHandler(StaticLocators.getDmsProcessingMapper());
			
		}
		
		@Override
		public FtpletResult onDisconnect(FtpSession session) throws FtpException, IOException {
			_messageHandler.createMessage(new DmsException(2000, "I", 
				"FTPServerLet",	"onDisconnect", _taskName +" user "+session.getUser()+ " disconnected", null, session.getSessionId().toString()), null);
			return super.onDisconnect(session);
		}
		
		@Override
		public FtpletResult onConnect(FtpSession session) throws FtpException, IOException {
			_messageHandler.createMessage(new DmsException(2000, "I", 
				"FTPServerLet",	"onConnect", _taskName +" connect from "+session.getClientAddress(), null, session.getSessionId().toString()), null);
			
			return super.onConnect(session);
		}
		
		@Override
		public FtpletResult onUploadEnd(FtpSession session, FtpRequest request) throws FtpException, IOException {
			_messageHandler.createMessage(new DmsException(
					2000,
					"I", 
					"FTPServerLet",
					"onUploadEnd", 
					_taskName +" received file "+request.getArgument(), 
					null, 
					session.getSessionId().toString()), null);
					

			return super.onUploadEnd(session, request);
		}
		
		
		
		@Override
		public FtpletResult onLogin(FtpSession session, FtpRequest request) throws FtpException, IOException {
			return super.onLogin(session, request);
		}
	}
	

}
