/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.ftpServer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.ftpserver.ftplet.Authentication;
import org.apache.ftpserver.ftplet.AuthenticationFailedException;
import org.apache.ftpserver.ftplet.Authority;
import org.apache.ftpserver.ftplet.FtpException;
import org.apache.ftpserver.ftplet.User;
import org.apache.ftpserver.ftplet.UserManager;
import org.apache.ftpserver.usermanager.ClearTextPasswordEncryptor;
import org.apache.ftpserver.usermanager.UserManagerFactory;
import org.apache.ftpserver.usermanager.UsernamePasswordAuthentication;
import org.apache.ftpserver.usermanager.impl.AbstractUserManager;
import org.apache.ftpserver.usermanager.impl.BaseUser;
import org.apache.ftpserver.usermanager.impl.ConcurrentLoginPermission;
import org.apache.ftpserver.usermanager.impl.WritePermission;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebMessageHandler;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.user.Users.UserFilterClass;
import org.biffo.dms.entity.user.Users.UsersAccessType;

public class FTPServerUserManager extends AbstractUserManager implements UserManagerFactory, Serializable {

 
	private static final String _className = "FTPServerUserManager";
	private static final long serialVersionUID = 1L;

	private DmsProcessingMapper _dmp;
	private DmsWebMessageHandler _messageHandler;

	private List<BaseUser> listUsers = new LinkedList<BaseUser>();
	private String dataFolder;

	
	
	public FTPServerUserManager(DmsProcessingMapper dmp){
		super("admin", new ClearTextPasswordEncryptor());
		_dmp = dmp;
		_messageHandler = new DmsWebMessageHandler(_dmp);
	}
	

	@Override
	public User authenticate(Authentication authentication) throws AuthenticationFailedException {
		User bu = null;
		if(UsernamePasswordAuthentication.class.isAssignableFrom(authentication.getClass())) {
			UsernamePasswordAuthentication upAuth = (UsernamePasswordAuthentication) authentication;
			try {
				if(doesExist(upAuth.getUsername())) {
					bu = getUserByName(upAuth.getUsername());
					if(null!=bu) {
						if(!SaltedMD5.validatePassword(upAuth.getPassword(), bu.getPassword())) {
							bu=null;
						}else {
							try {
								UserFilterClass ufc = new UserFilterClass();
								ufc.setBenutzerName(upAuth.getUsername());
								ufc.setAccessType(UsersAccessType.FTP);
								List<org.biffo.dms.entity.user.Users.User> lstEntities = _dmp.getUsersDB().getEntities(-1, -1, "", ufc);
								if(lstEntities.size()==1) {
									lstEntities.get(0).setLastAccessDate(new Date());
									_dmp.getUsersDB().updateEntity(lstEntities.get(0), false);
								}
							}catch(Exception e) {
							}
						}
					}
				}else {
					_messageHandler.createMessage(new DmsException(1000, "W", 
						_className,
						"authenticate", 
						"User "+upAuth.getUsername() + " not exists. Login from "+upAuth.getUserMetadata().getInetAddress().toString(), null, null), null);	
				}
			} catch (Exception e) {
				_messageHandler.createMessage(new DmsException(1000, "W", 
					_className,
					"authenticate", 
					"User "+upAuth.getUsername() + " not able to authenticate.", null, upAuth.getUserMetadata().getInetAddress().toString()), null);
			}
	     }
	     return bu;
	}

	@Override
	public void delete(String username) throws FtpException {
	}

	@Override
	public boolean doesExist(String username) throws FtpException {
		if(null!=getUserByName(username))return true;
		return false;
	}

	@Override
	public String[] getAllUserNames() throws FtpException {
		String strArr[] = new String[listUsers.size()];
	 	for(int i=0;i<listUsers.size();i++)strArr[i]=listUsers.get(i).getName();
	 	return strArr;
	}

	@Override
	public User getUserByName(String username) throws FtpException {
		for(BaseUser bu: listUsers) {
			if(bu.getName().equals(username)) return bu;
		}
	    return null;
	}

	@Override
	public void save(User user) throws FtpException {
	}

	@Override
	public UserManager createUserManager() {
		return this;
	}
	
	
	public void refreshFTPUsers() {
		
		listUsers = new LinkedList<BaseUser>();
     
		try {
			List<Authority> authorities = new ArrayList<Authority>();
	        authorities.add(new WritePermission());
	        authorities.add(new ConcurrentLoginPermission(100, 100));
	        UserFilterClass ufc = new UserFilterClass();
	        ufc.setAccessType(UsersAccessType.FTP);
	        List<org.biffo.dms.entity.user.Users.User> lstDMSUsers = StaticLocators.getUsers().getEntities(-1, -1, "", ufc);
	        for(int i=0;i<lstDMSUsers.size();i++) {
	        	BaseUser bu = new BaseUser();
	        	bu.setAuthorities(authorities);
	        	bu.setEnabled(true);
	        	bu.setHomeDirectory(dataFolder);
	        	bu.setMaxIdleTime(10000);
	        	bu.setName(lstDMSUsers.get(i).getBenutzerName());
	        	bu.setPassword(lstDMSUsers.get(i).getBenutzerPassword());
	        	listUsers.add(bu);
	        }
		}catch(Exception e) {
			_messageHandler.createMessage(new DmsException(1000, "W", 
				_className,
				"refreshFTPUsers", 
				null, null, e), null);
		}
	}
	
	 
	public void setDataFolder(String dataFolder) {
		this.dataFolder =dataFolder;
		refreshFTPUsers();
	}
	

}
