/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.importServer;



import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.FTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.FolderServerImpl;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.helpers.ImportProcessHelpers;
import org.biffo.dms.processing.api.ModuleProcessing;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processQueue.folderServer.FolderServerTask;
import org.biffo.dms.processQueue.ftpServer.FTPServerTask;
import org.biffo.dms.processQueue.sftpServer.SFTPServerTask;
import org.biffo.dms.processQueue.smtpServer.SMTPServerTask;
import org.biffo.dms.processing.DocumentProcessing;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingType;
import org.biffo.dms.processing.imports.ImportProcessing;

public class ImportProcessTask implements IProcessQueueTask {
	
	
	private static final long serialVersionUID = 1L;

	private ProcessType _processType = ProcessType.IMPORT;
	private final ImportServerImpl importerServer;
	private IVariablesImpl importSource;
	
	private Application _application;
	private ProcessBridge _bridge;
	
	private String taskName;
	private String message;
	private String status;
	
	//private int idProcess;
	private boolean isRebuildPdf=false;
	private boolean isArchiveInput=true;
	private int[] assignedTags = null;
	private int[] assignedUsers = null;
	private int[] assignedModules = null;
	
	private int sourceId, targetId;
	private boolean combinedImportProcess=false;
	private String _folderToWatch;
	private final DmsProcessingMapper _dmp;
	
	private ExecutorService _executor;
	private CompletionService<DocumentProcessing> _completionService;
	
	
	private List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
	
	public ImportProcessTask(ImportServerImpl importerServer) {
		
		_application = Application.get();
		this.importerServer = importerServer;
		
		taskName = importerServer.getProcessName();
		_dmp = StaticLocators.getDmsProcessingMapper();

		if(importerServer.isAssignTags())assignedTags = ImportProcessHelpers.getAssignedTagsToImportProcess(importerServer.getId());
		if(importerServer.isAssignToUsers())assignedUsers = ImportProcessHelpers.getAssignedUsersToImportProcess(importerServer.getId(), -1);
		if(importerServer.isUseKi())assignedModules = ImportProcessHelpers.getAssignedModulesToImportProcess(importerServer.getId());
		
		isRebuildPdf = importerServer.isRebuildPdf();
		isArchiveInput = importerServer.isArchiveafterImport();
		
		retrieveImportSource();
		
		if(null!=importSource) {
			checkAndInitImportSource();
			
			if(importerServer.getQueuesize()==-1)_executor = Executors.newFixedThreadPool(2);
			else _executor = Executors.newFixedThreadPool(importerServer.getQueuesize());
			_completionService=new ExecutorCompletionService<DocumentProcessing>(_executor);
		}
	}
	
	private void checkAndInitImportSource() {
		
		if(importSource.getGroupType()==VariableGroupType.FTP) {
			FTPServerImpl ftpServer = (FTPServerImpl)importSource;
			if(!DmsWebApplication.getProcessQueue().hasTask(ftpServer.getServerName())) {
				DmsWebApplication.getProcessQueue().add( new FTPServerTask((FTPServerImpl)ftpServer));
			}else {
				if(!DmsWebApplication.getProcessQueue().isRunning(ftpServer.getServerName())){
					DmsWebApplication.getProcessQueue().start(ftpServer.getServerName());
				}
			}
			_folderToWatch = ftpServer.getServerFolder();
			_processType = ProcessType.IMPORT_FTP;
			combinedImportProcess=true;
		}else if(importSource.getGroupType()==VariableGroupType.SFTP) {
			SFTPServerImpl sftpServer = (SFTPServerImpl)importSource;
			if(!DmsWebApplication.getProcessQueue().hasTask(sftpServer.getServerName())) {
				DmsWebApplication.getProcessQueue().add( new SFTPServerTask((SFTPServerImpl)sftpServer));
			}else {
				if(!DmsWebApplication.getProcessQueue().isRunning(sftpServer.getServerName())){
					DmsWebApplication.getProcessQueue().start(sftpServer.getServerName());
				}
			}
			_folderToWatch = sftpServer.getServerFolder();
			_processType = ProcessType.IMPORT_SFTP;
			combinedImportProcess=true;
		}else if(importSource.getGroupType()==VariableGroupType.SMTP) {
			SmtpServerImpl smtpServer = (SmtpServerImpl)importSource;
			if(!DmsWebApplication.getProcessQueue().hasTask(smtpServer.getServerName())) {
				DmsWebApplication.getProcessQueue().add( new SMTPServerTask(smtpServer));
			}else {
				if(!DmsWebApplication.getProcessQueue().isRunning(smtpServer.getServerName())){
					DmsWebApplication.getProcessQueue().start(smtpServer.getServerName());
				}
			}
			_folderToWatch = smtpServer.getServerFolder();
			_processType = ProcessType.IMPORT_SMTP;
			combinedImportProcess=true;
		}else if(importSource.getGroupType()==VariableGroupType.FOLDERS) {
			FolderServerImpl server = (FolderServerImpl)importSource;
			if(!DmsWebApplication.getProcessQueue().hasTask(server.getServerName())) {
				DmsWebApplication.getProcessQueue().add( new FolderServerTask(server));
			}else {
				if(!DmsWebApplication.getProcessQueue().isRunning(server.getServerName())){
					DmsWebApplication.getProcessQueue().start(server.getServerName());
				}
			}
			_folderToWatch = server.getServerFolder();
			_processType = ProcessType.IMPORT_FOLDER;
			combinedImportProcess=true;
		}
	}
	
	public void refresh() {
	}
	
	
	@Override
	public void doIt(ProcessBridge processBridge) {

		_bridge = processBridge;
		_bridge.setTaskName(taskName);
		_bridge.setStop(false);
			
		try {
			
			if(!Application.exists())ThreadContext.setApplication(_application);
			
			CheckFolderThread checkFolder = null;
			if(combinedImportProcess) {
				checkFolder = new CheckFolderThread(_application, _folderToWatch) {
	
					private static final long serialVersionUID = 1L;

					@Override
					protected void OnFileReceived(String fileName) {
						org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
						process.setProcessStatusType(ProcessStatusType.START);
						ProcessingHelpers.updateProcess(_dmp, process);
						int idProcess = process.getId();

						try {
							doImport(fileName, idProcess);
						}catch(Exception e) {
							finishProcess(idProcess, ProcessStatusType.ERROR);
							createMessage(new DmsException(1000, "E", 
								"CheckFolderThread", "OnFileReceived", taskName+" error", null, e),
								false
							);
						}
					}
				};
				checkFolder.start();
			}else {
				// TBD
			}
			
			
			boolean running=true;
			while(running) {
				if(processBridge.isRefresh()) {
					refresh();
					processBridge.setRefresh(false);
				}
			
				if(processBridge.isStop()) {
					
					try {
						if(combinedImportProcess)    {
							checkFolder.interrupt();
							_executor.shutdown();
						}
					}catch(Exception e) {
						createMessage(new DmsException(1000, "E", 
							"ImportProcessTask", "doIt", taskName+" error", null, e),
							false
						);
					}
					_bridge.setProgress(100);
					_bridge.setStop(true);
					running=false;
				}
				
				if(combinedImportProcess) {
					try {
						Future<DocumentProcessing> ftDocumentProcessing = _completionService.poll(100, TimeUnit.MILLISECONDS);
						if(ftDocumentProcessing!=null && ftDocumentProcessing.get().getProcessingType()==ProcessingType.DOCUMENT_IMPORT ) {
							DocumentProcessing dp = ftDocumentProcessing.get();
							Document document = (Document)dp.getReturn();
							if(assignedModules!=null) {
								doModule(document, dp.getProcessId());
							}else {
								finishProcess(dp.getProcessId(), ProcessStatusType.END);
							}
						}else if(ftDocumentProcessing!=null && ftDocumentProcessing.get().getProcessingType()==ProcessingType.APPLY_MODULE ) {
							DocumentProcessing dp = ftDocumentProcessing.get();
							finishProcess(dp.getProcessId(), ProcessStatusType.END);
							
						}
					}catch(Exception e) {
						//_bridge.setProcessException(e);
					}
				}
					
				if(processBridge.isCancel()){
					processBridge.setMessage("Import Process canceled");
					processBridge.setStop(true);
				}
				stopWait(10);
			}

			_bridge.setProgress(100);
			_bridge.setStop(true);
			
		}catch(Exception e) {
			createMessage(new DmsException(1000, "E", 
					"ImportProcessTask", "doIt", taskName+" error", null, e),
					false
				);
			_bridge.setProcessException(e);
		}
	}
	
	private void finishProcess(int idProcessing, ProcessStatusType processingStatusType) {
		try {
			org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getProcessById(_dmp, idProcessing);
			if(null!=process) {
				process.setProcessStatusType(processingStatusType);
				process.setNumberOfRecords(1);
				process.setProcessEndTime(new Date());
				ProcessingHelpers.updateProcess(_dmp, process);
			}
		}catch(Exception e) {
			createMessage(new DmsException(1000, "E", 
				"ImportProcessTask", "finishProcess", taskName+" error", null, e),
				false
			);
			_bridge.setProcessException(e);
		}
	}
	
	
	private void doImport(String fileName, int idProcssing) throws DmsException{

		ImportProcessing ip = new ImportProcessing(_dmp, _folderToWatch, fileName );
		ip.setUserId(99999);
		ip.setUserName(taskName);
		ip.setIdTenant(-1);
		ip.setProcessId(idProcssing);
	
		ip.setSelectedTags(assignedTags);
		ip.setSelectedUsers(assignedUsers);
		ip.setRebuildPDF(isRebuildPdf);
	
		ip.setStoreDB(true);
		ip.setCleanUp(true);
		ip.setReimport(false);
		ip.setRemoveInput(true);
		ip.setArchiveDocument(isArchiveInput);
		_completionService.submit(ip);
	}
	
	private void doModule(Document document, int idProcssing) throws DmsException {
		if(null==document) return;
		
		try {
			
			ModuleProcessing mp = new ModuleProcessing(_dmp, document, _application.getClass().getClassLoader());
			mp.setIdTenant(-1);
			mp.setUserId(99999);
			mp.setUserName(taskName);
			mp.setIdTenant(-1);
			mp.setProcessId(idProcssing);
		
			mp.setOverwrite(true);
			mp.setSelectedModules(assignedModules);
			_completionService.submit(mp);
		}catch(DmsException dmsException) {
			throw dmsException;
		}
	}
		
	private synchronized void createMessage(DmsException dmsInfo, boolean withBridgeStatus) {
		StaticLocators.getMessageHandler().createMessage(dmsInfo, null);
		if(null!=_bridge && withBridgeStatus) _bridge.setMessage(dmsInfo.getMessage());
	}
	
	
	private void retrieveImportSource() {
		try {
			
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
			rfc.setIdrelation2(importerServer.getId());
			
			Relation relation = StaticLocators.getRelations().getEntity(rfc);
			if(relation!=null) {
				lstServers.addAll(ImportProcessHelpers.getServerProcesses());
				
				for(IVariablesImpl serverProcess : lstServers) {
					if(serverProcess.getId() == relation.getIdrelation1()) {
						importSource = serverProcess;
					}
				}
			}

		}catch(Exception e) {
			createMessage(new DmsException(1000, "E", 
				"ImportProcessTask", "retrieveImportSource", taskName+" error", null, e),
				false
			);
		}
	}

	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	
	@Override
	public synchronized String getTaskName() {return taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}


	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}


	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getMessage() {return message;}

	@Override
	public synchronized String getStatus() {return status;}

	@Override
	public synchronized int getIdProcess() {return -1;}
	
	
	

}
