/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.sftpServer;

import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.sshd.server.auth.password.PasswordAuthenticator;
import org.apache.sshd.server.auth.password.PasswordChangeRequiredException;
import org.apache.sshd.server.session.ServerSession;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebMessageHandler;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.entity.user.Users.User;

public class SFTPPasswordAuthenticator implements PasswordAuthenticator, Serializable {


	

	private static final long serialVersionUID = 1L;
	
	private DmsProcessingMapper _dmp;
	private DmsWebMessageHandler _messageHandler;
	private List<User> _sftpUsers = new LinkedList<User>();

	public SFTPPasswordAuthenticator(SFTPServerImpl sftpServerImpl, DmsProcessingMapper dmp) {
		_dmp = dmp;
		refreshSFTPUsers(sftpServerImpl.getId());
		_messageHandler = new DmsWebMessageHandler(_dmp);
	}
	
	
	
	@Override
	public boolean authenticate(String username, String password, ServerSession session) throws PasswordChangeRequiredException {
			
		if(null==_sftpUsers || _sftpUsers.size()==0) {
			return false;
		}
		boolean found=false;
		for(User user : _sftpUsers) {
			if(user.getBenutzerName().equals(username)) {
				try {
					if(SaltedMD5.validatePassword(password, user.getBenutzerPassword())){
						found = true;
						user.setLastAccessDate(new Date());
						_dmp.getUsersDB().updateEntity(user, true);
						break;
					}
				} catch (Exception e) {
					found=false;
				}
			}
		}
		if(!found) {
			_messageHandler.createMessage(new DmsException(1000, "W", 
				"SFTPServerTask",
				"authenticate", 
				"User "+username + " not able to authenticate.", null, session.getSessionId().toString()), null);
			return false;
		}
		return true;
	}
	
	private void refreshSFTPUsers(int serverId) {
		
		_sftpUsers = new LinkedList<User>();
     
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.SFTP_USER_TO_SFTP_SERVER);
			rfc.setIdrelation2(serverId);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(entities!=null && entities.size()>0) {
				for(int i=0;i<entities.size();i++) {
					_sftpUsers.add(StaticLocators.getUsers().getEntityById(entities.get(i).getIdrelation1()));
				}
			}
		}catch(Exception e) {
			_messageHandler.createMessage(new DmsException(1000, "E", 
				"SFTPServerTask", "refreshSFTPUsers", "Not able building Userlist.", null,	e),
				null
			);
		}
	}
	
	
}