/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.sftpServer;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystem;
import java.nio.file.Paths;
import java.util.Collections;

import org.apache.sshd.common.file.virtualfs.VirtualFileSystemFactory;
import org.apache.sshd.common.session.Session;
import org.apache.sshd.server.SshServer;
import org.apache.sshd.server.keyprovider.SimpleGeneratorHostKeyProvider;
import org.apache.sshd.server.subsystem.sftp.SftpSubsystemFactory;
import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;


public class SFTPServerTask implements IProcessQueueTask {

	private static final long serialVersionUID = 1L;
	
	private static final String _className = "SFTPServerTask";
	private Application _application;
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.IMPORT_FTP;
	private String _taskName;
	private String _message;
	private String _status;
	
	private SshServer sshdServer;
	private SFTPServerImpl sftpServerImpl;
	private SFTPPasswordAuthenticator authenticator;
	
	public SFTPServerTask(SFTPServerImpl sftpServerImpl) {
		
		_application = Application.get();
		this.sftpServerImpl = sftpServerImpl;
		
		_taskName=sftpServerImpl.getServerName();
		_status = "Start "+_processType.getDescription();
		
		authenticator = new SFTPPasswordAuthenticator(sftpServerImpl, StaticLocators.getDmsProcessingMapper());
	}
	
	 


	@Override
	public void doIt(ProcessBridge processBridge) {
		
		_bridge = processBridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		try {
			
			if(!Application.exists())ThreadContext.setApplication(_application);
			createMessage(new DmsException(2000, "I", 
				_className, "doIt", _taskName+" started", null,	null),
				true
			);
			
			
			if(null==sshdServer || !sshdServer.isClosed()) {
				sshdServer = SshServer.setUpDefaultServer();
				sshdServer.setPort(sftpServerImpl.getServerPort());
				sshdServer.setKeyPairProvider(new SimpleGeneratorHostKeyProvider(new File(sftpServerImpl.getHostPrivateKey())));
				sshdServer.setPasswordAuthenticator(authenticator);
				sshdServer.setSubsystemFactories(Collections.singletonList(new SftpSubsystemFactory()));
				
				sshdServer.setFileSystemFactory(new VirtualFileSystemFactory() {
					@Override
					public FileSystem createFileSystem(Session session) throws IOException {
						setUserHomeDir(session.getUsername(), Paths.get(sftpServerImpl.getServerFolder()));
						return super.createFileSystem(session);
					}
				});
		        sshdServer.start();
			}
			boolean running = true;
			while(running) {

				if(_bridge.isStop()) {
					sshdServer.stop();
					createMessage(new DmsException(2000, "I", 
						"SFTPServerTask", "doIt", _taskName+" stopped", null, null),
						true
					);
					_bridge.setProgress(100);
					_bridge.setStop(true);
					running=false;
				}
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
				}
				
				stopWait(1);
				
			}
			_bridge.setProgress(100);
			_bridge.setStop(true);
			
		}catch(Exception e) {
			createMessage(new DmsException(1000, "E", 
				"SFTPServerTask", "doIt", _taskName+" error", null, e),
				false
			);
			_bridge.setProcessException(e);
		}
	}
	
	private synchronized void createMessage(DmsException dmsInfo, boolean withBridgeStatus) {
		StaticLocators.getMessageHandler().createMessage(dmsInfo, null);
		if(null!=_bridge && withBridgeStatus) _bridge.setMessage(dmsInfo.getMessage());
	}
	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getMessage() {return _message;}
	@Override
	public synchronized String getTaskName() {return _taskName;}
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}
	@Override
	public synchronized int getSourceDocumentId() {return -1;}
	@Override
	public synchronized int getTargetDocumentId() {return -1;}
	@Override
	public synchronized int getIdProcess() {return -1;}
	@Override
	public synchronized String getStatus() {return _status;}

}
