/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.smtpServer;

import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebMessageHandler;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.entity.user.Users.User;
import org.subethamail.smtp.MessageContext;
import org.subethamail.smtp.auth.LoginFailedException;
import org.subethamail.smtp.auth.UsernamePasswordValidator;

 

public class SMTPPasswordAuthenticator implements UsernamePasswordValidator, Serializable {


	private static final long serialVersionUID = 1L;
	
	private DmsProcessingMapper _dmp;
	private DmsWebMessageHandler _messageHandler;
	private List<User> _smtpUsers = new LinkedList<User>();
	
	public SMTPPasswordAuthenticator(SmtpServerImpl smtpServerImpl, DmsProcessingMapper dmp) {
		_dmp = dmp;
		refreshSMTPUsers(smtpServerImpl.getId());
		_messageHandler = new DmsWebMessageHandler(_dmp);
	}
	

	@Override
	public void login(String username, String password, MessageContext messagecontext) throws LoginFailedException {
		if(null==_smtpUsers || _smtpUsers.size()==0) {
			throw new LoginFailedException("No SMTP Users found");
		}
		boolean found=false;
		for(User user : _smtpUsers) {
			if(user.getBenutzerName().equals(username)) {
				try {
					if(SaltedMD5.validatePassword(password, user.getBenutzerPassword())){
						found = true;
						user.setLastAccessDate(new Date());
						_dmp.getUsersDB().updateEntity(user, true);
						break;
					}
				} catch (Exception e) {
					e.printStackTrace();
					found=false;
				}
			}
		}
		if(!found) {
			throw new LoginFailedException("Wrong Password provided.");
		}
		
	}
	
	
	private void refreshSMTPUsers(int serverId) {
	
		_smtpUsers = new LinkedList<User>();
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.SMTP_USER_TO_SMTP_SERVER);
			rfc.setIdrelation2(serverId);
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(entities!=null && entities.size()>0) {
				for(int i=0;i<entities.size();i++) {
					_smtpUsers.add(StaticLocators.getUsers().getEntityById(entities.get(i).getIdrelation1()));
				}
			}
		}catch(Exception e) {
			_messageHandler.createMessage(new DmsException(1000, "E", 
				"SMTPServerTask", "refreshSMTPUsers", "Not able building Userlist.", null,	e),
				null
			);
		}
	}
	

}
