/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.smtpServer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.Set;


import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebMessageHandler;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.subethamail.smtp.MessageContext;
import org.subethamail.smtp.MessageHandler;
import org.subethamail.smtp.MessageHandlerFactory;
import org.subethamail.smtp.RejectException;
import org.subethamail.smtp.TooMuchDataException;
import org.subethamail.smtp.auth.EasyAuthenticationHandlerFactory;
import org.subethamail.smtp.server.SMTPServer;

import jakarta.mail.BodyPart;
import jakarta.mail.Message;
import jakarta.mail.Multipart;
import jakarta.mail.Part;
import jakarta.mail.Session;
import jakarta.mail.internet.MimeMessage;
 

 

public class SMTPServerTask implements IProcessQueueTask {


	private static final long serialVersionUID = 1L;
	private Application _application;
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.IMPORT_SMTP;
	private String _taskName;
	private String _message;
	private String _status;
	
	 
	private SMTPServer _smtpServer;
	private int _smtpServerPort;
	private boolean _withAuthentication=false;
	private String _serverFolder;
	private String _from;
	private List<String> _fileTypes=new LinkedList<String>();
	private SMTPPasswordAuthenticator upv;
 	
 	public SMTPServerTask(SmtpServerImpl smtpServerImpl) {
 		_application = Application.get();
		
		_taskName=smtpServerImpl.getServerName();
		_status = "Start "+_processType.getDescription();
		
		
		_smtpServerPort= smtpServerImpl.getServerPort();
		_withAuthentication = smtpServerImpl.isWithAuthentication();
		_serverFolder = smtpServerImpl.getServerFolder();
		_from = smtpServerImpl.getFrom();
		if(smtpServerImpl.getSupportedFileTypes()!=null && smtpServerImpl.getSupportedFileTypes().length()>0) {
			_fileTypes.addAll(Arrays.asList(smtpServerImpl.getSupportedFileTypes().toLowerCase().split(";")));
		}
		if(_withAuthentication) {
			upv = new SMTPPasswordAuthenticator(smtpServerImpl, StaticLocators.getDmsProcessingMapper());
		}
 	}
 	
  
 	
	
	
	@Override
	public void doIt(ProcessBridge processBridge) {
		_bridge = processBridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		try {
			
			if(null==_smtpServer || !_smtpServer.isRunning()) {
				
				if(_withAuthentication) {
					_smtpServer = SMTPServer
						.port(_smtpServerPort)
						.authenticationHandlerFactory(new EasyAuthenticationHandlerFactory(upv))
						.messageHandlerFactory(new SMTPServerMessageHandler())
						.requireAuth(true)
						.build();
					
				}else {
					_smtpServer = SMTPServer
						.port(_smtpServerPort)
						.messageHandlerFactory(new SMTPServerMessageHandler())
						.requireAuth(false)
						.build();
				
					
				}
				_smtpServer.start();
				createMessage(new DmsException(2000, "I", 
					"SMTPServerTask", "doIt", _taskName+" started", null,	null),
					true
				);
			}
			
			
			
			if(!Application.exists())ThreadContext.setApplication(_application);
			boolean running = true;
			while(running) {
				
				if(_bridge.isStop()) {
					_smtpServer.stop();
					createMessage(new DmsException(2000, "I", 
						"SMTPServerTask", "doIt", _taskName+" stopped", null, null),
						true
					);
					_bridge.setProgress(100);
					_bridge.setStop(true);
					running=false;
				}
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
				}
				
				
				stopWait(1);
				
			}
			_bridge.setProgress(100);
			_bridge.setStop(true);
			
		}catch(Exception e) {
			createMessage(new DmsException(1000, "E", 
				"SMTPServerTask", "doIt", _taskName+" error", null, e),
				false
			);
			_bridge.setProcessException(e);
		}
	}
		
	private synchronized void createMessage(DmsException dmsInfo, boolean withBridgeStatus) {
		StaticLocators.getMessageHandler().createMessage(dmsInfo, null);
		if(null!=_bridge && withBridgeStatus) _bridge.setMessage(dmsInfo.getMessage());
	}
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {
		}
	}
	
	
   
	private class SMTPServerMessageHandler implements MessageHandlerFactory, Serializable{

		private static final long serialVersionUID = 1L;
		
		private DmsWebMessageHandler _messageHandler;
		
		public SMTPServerMessageHandler() {
			_messageHandler = new DmsWebMessageHandler(StaticLocators.getDmsProcessingMapper());
		}
		
		private void writeFile(String fileName, HashMap<String, InputStream> inputFiles) {
			try {
            	_messageHandler.createMessage(
           			new DmsException(2000, "I","SMTPServerTask", "data", "Received file "+fileName, null, null), null
        		);

				File f = new File(_serverFolder+"/" + fileName);
				FileOutputStream fos = new FileOutputStream(f);
				byte[] buf = new byte[4096];
				int bytesRead;
				InputStream is = inputFiles.get(fileName);
				while((bytesRead = is.read(buf))!=-1) {
					fos.write(buf, 0, bytesRead);
				}
				fos.close();
				fos.flush();
				is.close();
			}catch(Exception e) {
            	_messageHandler.createMessage(
               		new DmsException(1000, "E","SMTPServerTask", "data", "Error writing "+fileName, null, e), null
            	);
			}
		}
		
		private boolean isFileTypeSupported(String fileName) {

			if(null==fileName || fileName.isEmpty()) return false;
			if(_fileTypes.size()==0) return true;
			if(_fileTypes.contains("*")) return true;
			
			int iPos=fileName.lastIndexOf(".");
			if(iPos==-1) return false;
			String temp = fileName.substring(iPos+1).toLowerCase();
			
			if(!_fileTypes.contains(temp)) return false;
			return true;
		}
		
		
		
		@Override
		public MessageHandler create(MessageContext messagecontext) {
			
			return new MessageHandler() {
            	
                @Override
                public void from(String from) throws RejectException {
                	
                	if(Strings.isEmpty(from) || !_from.equals(from)){
                		throw new RejectException("From not valid");
                	}
                	_messageHandler.createMessage(
               			new DmsException(2000, "I","SMTPServerTask", "from", "Received message from "+from, null, null), null
            		);
                }

                @Override
                public void recipient(String recipient) throws RejectException {
                }

                @Override
                public String data(InputStream inputStream) throws RejectException, TooMuchDataException, IOException {
                    
            	    Properties props = System.getProperties();
            	    Session session = Session.getInstance(props, null);
                    
            	    try {
            	    	
						MimeMessage msg = new MimeMessage(session, inputStream);
						HashMap<String, InputStream> hmInputFiles = getAttachmentNew(msg);
						
						if(null!=hmInputFiles) {
							Set<String> fileNames = hmInputFiles.keySet();
							for(String fileName : fileNames) {
								if(!isFileTypeSupported(fileName)) {
									_messageHandler.createMessage(
					           			new DmsException(2000, "W","SMTPServerTask", "data", "File "+fileName+ " has an unssported filetype.", null, null), null
					        		);
								}else {
									writeFile(fileName, hmInputFiles);
								}
							}
						}
					} catch (Exception e) {
						_messageHandler.createMessage(
		                   	new DmsException(1000, "E", "SMTPServerTask", "data", null, null, e), null
		                );
					}
                    return null;
                }
                @Override
                public void done() {
                }};
        }
    };
	
	
	private HashMap<String, InputStream> getAttachmentNew(Message message) throws Exception {
	    
		Object content = message.getContent();
	    if (content instanceof String)
	        return null;        

	    if (content instanceof Multipart) {
	        Multipart multipart = (Multipart) content;
	        HashMap<String, InputStream> result = new HashMap<String, InputStream>();

	        for (int i = 0; i < multipart.getCount(); i++) {
	        	result.putAll( getAttachmentNew(multipart.getBodyPart(i)));
	        }
	        return result;
	    }
	    return null;
	}
	
	private HashMap<String, InputStream> getAttachmentNew(BodyPart part) throws Exception {
		HashMap<String, InputStream> result = new HashMap<String, InputStream>();
		Object content = part.getContent();
		if (content instanceof InputStream || content instanceof String) {
	        if (Part.ATTACHMENT.equalsIgnoreCase(part.getDisposition()) || (part.getFileName()!=null && !part.getFileName().isEmpty() )) {
	        	result.put(part.getFileName(), part.getInputStream());
	        	return result;
	        } else {
	        	return new HashMap<String, InputStream>();
	        }
		}

		    
		if (content instanceof Multipart) {
			Multipart multipart = (Multipart) content;
	        for (int i = 0; i < multipart.getCount(); i++) {
	        	BodyPart bodyPart = multipart.getBodyPart(i);
	            result.putAll(getAttachmentNew(bodyPart));
	        }
		}
		return result;
	}
	


	@Override
	public synchronized String getMessage() {return _message;}
	@Override
	public synchronized String getTaskName() {return _taskName;}
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}
	@Override
	public synchronized int getSourceDocumentId() {return -1;}
	@Override
	public synchronized int getTargetDocumentId() {return -1;}
	@Override
	public synchronized int getIdProcess() {return -1;}
	@Override
	public synchronized String getStatus() {return _status;}

}
