/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.DocumentProcessing;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingStatus;
import org.biffo.dms.processing.ProcessingType;
import org.biffo.dms.processing.pdf.MergeProcessing;
 

public class DocumentMergeTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;

	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.MERGE;
	private String _taskName;
	private String _message;
	private String _status;
	
	private Application application;
	private DmsWebSession _session = DmsWebSession.getSession();
	private final List<Document> documents = new LinkedList<Document>();
	
	private int sourceId, targetId;
	private int idProcess;
	
	@SuppressWarnings("unused")
	private int _numberOfErrors;
	private final DmsProcessingMapper _dmp;
	
	public DocumentMergeTask(List<Document> documents) {
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		
		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		this.documents.addAll(documents);
	}

	@Override
	public void doIt(ProcessBridge bridge) {
		_bridge = bridge;
		bridge.setTaskName(_taskName);
		bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		try {
			if(!Application.exists())ThreadContext.setApplication(application);
			
			
			if(doMerge(bridge, idProcess)==null) {
				_numberOfErrors++;
			}
			
			
			stopWait(200);
			process.setProcessStatusType(ProcessStatusType.END);

		}catch(Exception e) {
			bridge.setProgress(100);
			bridge.setStop(true);
			bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		bridge.setProgress(100);
		bridge.setStop(true);
		
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	}
	
	private Document doMerge(ProcessBridge bridge, int idProcssing) throws DmsException{
		try {
			MergeProcessing mp = new MergeProcessing(_dmp, documents) {

				private static final long serialVersionUID = 1L;

				@Override
				protected void OnStatusChanged(ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
							bridge, 
							processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
							null);
				}
			};
			mp.setProcessId(idProcssing);
			mp.setUserId(_session.getCurrentUserId());
			mp.setUserName(_session.getCurrentUser());
			mp.setIdTenant(_session.getCurrentTenantId());
			mp.setCleanUp(true);
			
			DocumentProcessing dp = mp.call();
			if(dp.getProcessingType()==ProcessingType.MERGE_DOCUMENT && null!=dp.getReturn()) {
				return (Document)dp.getReturn();
			}else {
				return null;
			}
			
		}catch(Exception e) {
			return null;
		}
	}
	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}

	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}

	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}

	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getStatus() {return _status;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
