/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;


 
import java.util.Date;


import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.DocumentProcessing;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingStatus;
import org.biffo.dms.processing.ProcessingType;
import org.biffo.dms.processing.imports.ImportProcessing;
 
 
public class DocumentRePdfTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;
	
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.REPDF;
	private String _taskName;
	private String _message;
	private String _status;
	
	private Application application;
	private DmsWebSession _session = DmsWebSession.getSession();
	
	private final Document document;
	@SuppressWarnings("unused")
	private final int[] selectedTags;
	@SuppressWarnings("unused")
	private final int[] selectedUsers;
	@SuppressWarnings("unused")
	private final int[] selectedUserGroups;
	private int sourceId, targetId;
	private int idProcess;
	
	private final DmsProcessingMapper _dmp;
	
	public DocumentRePdfTask(Document document, int[] selectedTags, int[] selectedUsers, int[] selectedUserGroups) {
		
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();

		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		
		this.document = document;
		this.selectedTags = selectedTags;
		this.selectedUsers = selectedUsers;
		this.selectedUserGroups = selectedUserGroups;
		
	}
	

	@Override
	public void doIt(ProcessBridge bridge) {
		
		_bridge = bridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		
		try {
			Document documentTarget = null;
			
			if(!Application.exists())ThreadContext.setApplication(application);
			
			sourceId = document.getId();
			documentTarget = document;
			stopWait(100);
			documentTarget = doRePdf(bridge, documentTarget, idProcess);
			targetId = documentTarget.getId();
			process.setProcessStatusType(ProcessStatusType.END);
			
		}catch(Exception e) {
			_bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		_bridge.setProgress(100);
		_bridge.setStop(true);
	 
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	 
	}
	
	
 
	private Document doRePdf(ProcessBridge bridge, Document document, int idProcssing) throws DmsException{
		try {
			ImportProcessing ip =new ImportProcessing(StaticLocators.getDmsProcessingMapper(), document) {
				private static final long serialVersionUID = 1L;

				@Override
				protected void OnStatusChanged(ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
						bridge, 
						_processType.getDescription(_session.getLocaleParams().getLocaleCode()),
						ProcessHelpers.getDocumentTitle(document)
					);
				}
			};
			ip.setProcessId(idProcssing);
			ip.setRebuildPDF(true);
			ip.setStoreDB(true);
			ip.setCleanUp(true);
			ip.setReimport(true);
			
			DocumentProcessing dp = ip.call();
			if(dp.getProcessingType()==ProcessingType.DOCUMENT_IMPORT && null!=dp.getReturn()) {
				return (Document)dp.getReturn();
			}else {
				return null;
			}
		}catch(Exception e) {
			return null;
		}
	}
	


	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}

	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}

	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized String getStatus() {return _status;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
