/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.pdf.RestoreProcessing;
 

public class DocumentUnarchiveTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;
	
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.UNARCHIVE;
	private String _taskName;
	private String _message;
	private String _status;
	
	private Application application;
	private DmsWebSession _session = DmsWebSession.getSession();
	private final List<Document> documents = new LinkedList<Document>();
	private int sourceId, targetId;
	private int idProcess;
	
	private int _numberOfUpdates;
	@SuppressWarnings("unused")
	private int _numberOfErrors;
 
	private final DmsProcessingMapper _dmp;
	
	public DocumentUnarchiveTask(Document document) {
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		
		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		documents.add(document);
	}
	
	
	public DocumentUnarchiveTask(List<Document> documents) {
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		
		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		this.documents.addAll(documents);
	}
 
	

	@Override
	public void doIt(ProcessBridge bridge) {
		_bridge = bridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		try {
			if(!Application.exists())ThreadContext.setApplication(application);

			for(Document document : documents) {
			
				if(bridge.isCancel()) {
					_bridge.setStop(true);
					break;
				}
				
				if(doUnarchive(bridge, document, idProcess)) {
					_numberOfUpdates++;
				}else{
					_numberOfErrors++;
				}
				stopWait(1000);
			}
			process.setProcessStatusType(ProcessStatusType.END);
			process.setNumberOfUpdates(_numberOfUpdates);
			process.setNumberOfRecords(documents.size());
			
		}catch(Exception e) {
			bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		bridge.setProgress(100);
		bridge.setStop(true);
		
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	}
	
	private boolean doUnarchive(ProcessBridge bridge, Document document, int idProcssing) throws DmsException{
		try {
			RestoreProcessing rp = new RestoreProcessing(_dmp, document) {
				private static final long serialVersionUID = 1L;
				protected void OnStatusChanged(org.biffo.dms.processing.ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
							bridge, 
							processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
							ProcessHelpers.getDocumentTitle(document));
				};
			};
			rp.setProcessId(idProcess);
			rp.setUserId(_session.getCurrentUserId());
			rp.setUserName(_session.getCurrentUser());
			rp.setIdTenant(_session.getCurrentTenantId());
			rp.setCleanUp(true);
			rp.call();
			return true;
		}catch(Exception e) {
			return false;
		}
	}

	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {
		}
	}
	
	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}


	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}


	@Override
	public synchronized int getTargetDocumentId() {return targetId;}


	@Override
	public synchronized String getStatus() {return _status;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
