/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;


import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.util.file.Folder;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.helpers.ImportProcessHelpers;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.DocumentProcessing;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingStatus;
import org.biffo.dms.processing.ProcessingType;
import org.biffo.dms.processing.api.ModuleProcessing;
import org.biffo.dms.processing.imports.ImportProcessing;
 
 
public class DocumentUploadTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;
	private static final SimpleDateFormat _sdf = new SimpleDateFormat("yyyyMMdd_HHmmss");
	private ProcessType _processType = ProcessType.IMPORT_WEB;

	private Application application;
	private ProcessBridge _bridge;
	
	private String _taskName;
	private String _message;
	private String _status;
	
	private ImportServerImpl _importServer;
	private Folder uploadFolder;
	private ArrayList<FileUpload> uploads = new ArrayList<>();

	
	private DmsWebSession _session = DmsWebSession.getSession();
	private int sourceId, targetId;
	private int idProcess;
	
	private int[] assignedTags = null;
	private int[] assignedUsers = null;
	//private int[] assignedUserGroups = null;
	private int[] assignedModules = null;
	
	private boolean isRebuildPdf=false;
	private boolean isArchiveInput=true;

	
	private int _numberOfUploads;
	private int _numberOfDublicates;
	
	private final DmsProcessingMapper _dmp;
 
	
	
	public DocumentUploadTask(ArrayList<FileUpload> uploads, int importProcessId) throws DmsException{
		
		this.application = Application.get();
		this.uploads.addAll(uploads);
		_dmp = StaticLocators.getDmsProcessingMapper();
		
		DPImporters _qtdp = new DPImporters();
		_importServer = _qtdp.getImportServerById(importProcessId);
		if(_importServer==null) {
			DmsException dmsException = new DmsException(1000, "E", 
				"DocumentUploadTask", 
				"init",
				"No ImportProcess "+importProcessId+" found.");
			throw dmsException;
		}
				
		_taskName = "U_"+_sdf.format(new Date())+"_"+_importServer.getProcessName()+"_"+_session.getCurrentUserId();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		
		try {
			uploadFolder = new Folder(StaticLocators.getDmsProcessingMapper().getDmsConfig().getImportFolder());
			uploadFolder.mkdirs();
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", 
				"DocumentUploadTask", 
				"init",
				"Upload-Folder "+uploadFolder+" could not be created.");
			throw dmsException;
		}
		
		if(_importServer.isAssignTags())assignedTags = ImportProcessHelpers.getAssignedTagsToImportProcess(_importServer.getId());
		if(_importServer.isUseKi())assignedModules = ImportProcessHelpers.getAssignedModulesToImportProcess(_importServer.getId());
		assignedUsers = ImportProcessHelpers.getAssignedUsersToImportProcess(_importServer.getId(), _session.getCurrentUserId());
		isRebuildPdf = _importServer.isRebuildPdf();
		isArchiveInput = _importServer.isArchiveafterImport();
	}
	

	@Override
	public void doIt(ProcessBridge bridge) {
		
		_bridge = bridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		try {
			if(!Application.exists())ThreadContext.setApplication(application);
			for (FileUpload upload : uploads){
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
					break;
				}
				
				String fileName = upload.getClientFileName();
				File newFile = new File(uploadFolder, fileName);
				newFile.createNewFile();
				upload.writeTo(newFile);
				
				
				try {
					Document document = doImport(bridge, fileName, idProcess);
					if(assignedModules!=null) {
						doModule(bridge, document, idProcess);
					}
					_numberOfUploads++;
				}catch(DmsException e) {
					//_numberOfErrors++;
				}
				stopWait(250);
			}
			process.setProcessStatusType(ProcessStatusType.END);
			process.setNumberOfInserts(_numberOfUploads);
			process.setNumberOfUpdates(_numberOfDublicates);
			
		}catch(Exception e) {
			_bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		_bridge.setProgress(100);
		_bridge.setStop(true);
	 
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	 
	}
	
	private Document doImport(ProcessBridge bridge, String fileName, int idProcssing) throws DmsException{

		ImportProcessing ip = new ImportProcessing(_dmp, uploadFolder.toString(), fileName ) {

			private static final long serialVersionUID = 1L;
			protected void OnStatusChanged(ProcessingStatus processingStatus) {
				ProcessHelpers.setBridgeStatusMessage(
					bridge, 
					processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
					fileName);
			}
		};
		ip.setUserId(_session.getCurrentUserId());
		ip.setUserName(_session.getCurrentUser());
		ip.setIdTenant(_session.getCurrentTenantId());
		ip.setProcessId(idProcssing);
	
		ip.setSelectedTags(assignedTags);
		ip.setSelectedUsers(assignedUsers);
		ip.setRebuildPDF(isRebuildPdf);
	
		ip.setStoreDB(true);
		ip.setCleanUp(true);
		ip.setReimport(false);
		ip.setRemoveInput(true);
		ip.setArchiveDocument(isArchiveInput);

		
		DocumentProcessing dp = ip.call();
		if(dp.getProcessingType()==ProcessingType.DOCUMENT_IMPORT && null!=dp.getReturn()) {
			return (Document)dp.getReturn();
		}else {
			return null;
		}
	}
	
	private Document doModule(ProcessBridge bridge, Document document, int idProcssing) throws DmsException {
		if(null==document) return null;
		
		try {
			
			ModuleProcessing mp = new ModuleProcessing(_dmp, document, application.getClass().getClassLoader()) {
				private static final long serialVersionUID = 1L;

				protected void OnStatusChanged(ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
							bridge, 
							processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
							ProcessHelpers.getDocumentTitle(document)
					);
				}
			};
			
			mp.setUserId(_session.getCurrentUserId());
			mp.setUserName(_session.getCurrentUser());
			mp.setIdTenant(_session.getCurrentTenantId());
			mp.setProcessId(idProcssing);
		
			mp.setOverwrite(true);
			mp.setSelectedModules(assignedModules);
			
			DocumentProcessing dp = mp.call();
			if(dp.getProcessingType()==ProcessingType.APPLY_MODULE && null!=dp.getReturn()) {
				return (Document)dp.getReturn();
			}else {
				return null;
			}
		}catch(DmsException dmsException) {
			throw dmsException;
		}
	}
	
	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {
		}
	}
	

	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}

	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}

	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized String getStatus() {return _status;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
