/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.DocumentProcessing;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingStatus;
import org.biffo.dms.processing.ProcessingType;
import org.biffo.dms.processing.api.ModuleProcessing;

 
public class ModuleApplyTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;
	
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.APPLY_MODULE;
	
	private String _taskName;
	private String _message;
	private String _status;
	
	private Application application;
	private DmsWebSession _session = DmsWebSession.getSession();
	
	
	private int sourceId, targetId;
	private int idProcess;
	
	private final List<Integer> selectedDocuments=new LinkedList<Integer>();
	private final List<Integer> selectedModules=new LinkedList<Integer>();
	
	private int _numberOfUpdates;
	@SuppressWarnings("unused")
	private int _numberOfDublicates;
	@SuppressWarnings("unused")
	private int _numberOfErrors;
	
	private final DmsProcessingMapper _dmp;
	
	
	public ModuleApplyTask(int moduleId, int documentId) {
		
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		
		selectedDocuments.add(documentId);
		selectedModules.add(moduleId);
	}
	
	public ModuleApplyTask(List<Integer> modules, List<Integer> documents) {
		
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		_taskName = _processType+"_"+_session.getCurrentUserId()+"_"+StaticLocators.getDateTimeStamp();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		
		selectedDocuments.addAll(documents);
		selectedModules.addAll(modules);
	}
	
	 

	@Override
	public void doIt(ProcessBridge bridge) {
		
		_bridge = bridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		if(selectedModules.isEmpty()) {
			_bridge.setCancel(true);
			_bridge.setStop(true);
		}
		
		try {

			if(!Application.exists())ThreadContext.setApplication(application);
			int[] assignedModules = new int[selectedModules.size()];
			for(int i=0;i<selectedModules.size();i++) {
				assignedModules[i] = selectedModules.get(i);
			}
			
			for(int i=0;i<selectedDocuments.size();i++) {
			
				Document document = StaticLocators.getDocuments().getEntityById(selectedDocuments.get(i));
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
					break;
				}
				try {
					if(assignedModules!=null && document!=null) {
						doModule(bridge, assignedModules, document, idProcess);
					}
					_numberOfUpdates++;
				}catch(DmsException e) {
					_numberOfErrors++;
				}
				stopWait(250);
			}
			
			process.setNumberOfUpdates(_numberOfUpdates);
			process.setProcessStatusType(ProcessStatusType.END);
			
		}catch(Exception e) {
			_bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		_bridge.setProgress(100);
		_bridge.setStop(true);
		process.setNumberOfRecords(selectedDocuments.size());
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	 
	}
	
	private Document doModule(ProcessBridge bridge, int[] assignedModules, Document document, int idProcssing) throws DmsException {
		if(null==document) return null;
		
		try {
			
			ModuleProcessing mp = new ModuleProcessing(_dmp, document, application.getClass().getClassLoader()) {
				private static final long serialVersionUID = 1L;

				protected void OnStatusChanged(ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
							bridge, 
							processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
							ProcessHelpers.getDocumentTitle(document)
					);
				}
			};
			
			mp.setUserId(_session.getCurrentUserId());
			mp.setUserName(_session.getCurrentUser());
			mp.setIdTenant(_session.getCurrentTenantId());
			mp.setProcessId(idProcssing);
		
			mp.setOverwrite(true);
			mp.setSelectedModules(assignedModules);
			DocumentProcessing dp = mp.call();
			if(dp.getProcessingType()==ProcessingType.APPLY_MODULE && null!=dp.getReturn()) {
				return (Document)dp.getReturn();
			}else {
				return null;
			}
		}catch(DmsException dmsException) {
			dmsException.printStackTrace();
			throw dmsException;
		}
	}
	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}

	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}

	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized String getStatus() {return _status;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
