/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.processQueue.tasks;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import java.util.List;

import org.apache.wicket.Application;
import org.apache.wicket.ThreadContext;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.util.file.Folder;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsProcessingMapper;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Processing.ProcessStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.processQueue.IProcessQueueTask;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.processing.ProcessingHelpers;
import org.biffo.dms.processing.ProcessingStatus;
import org.biffo.dms.processing.api.ModuleImporter;
 
 
public class ModuleUploadTask implements IProcessQueueTask {
 
	private static final long serialVersionUID = 1L;
	private static final SimpleDateFormat _sdf = new SimpleDateFormat("yyyyMMdd_HHmmss");
	
	private ProcessBridge _bridge;
	private ProcessType _processType = ProcessType.IMPORT_MODULE;
	private String _taskName;
	private String _message;
	private String _status;
	
	private Application application;
	private DmsWebSession _session = DmsWebSession.getSession();
	private int sourceId, targetId;
	private int idProcess;
	
	private Folder uploadFolder;
	private ArrayList<FileUpload> uploads = new ArrayList<>();
 	
	private int _numberOfUploads;
	private int _numberOfDublicates;
	@SuppressWarnings("unused")
	private int _numberOfErrors;
	
	private final DmsProcessingMapper _dmp;
	
	public ModuleUploadTask(ArrayList<FileUpload> uploads, int importProcessId) throws DmsException{
		this.application = Application.get();
		_dmp = StaticLocators.getDmsProcessingMapper();
		this.uploads.addAll(uploads);

		String processName = "Module_Import";
		_taskName = "U_"+_sdf.format(new Date())+"_"+processName+"_"+_session.getCurrentUserId();
		_status = "Start "+_processType.getDescription(_session.getLocaleParams().getLocaleCode());
		
		try {
			uploadFolder = new Folder(_dmp.getDmsConfig().getModuleFolder());
			uploadFolder.mkdirs();		
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", 
				"ModuleUploadTask", 
				"init",
				"Module-Folder "+uploadFolder+" could not be created.");
			throw dmsException;
		}
		

	}
	

	@Override
	public void doIt(ProcessBridge bridge) {
		
		_bridge = bridge;
		_bridge.setTaskName(_taskName);
		_bridge.setStop(false);
		
		org.biffo.dms.entity.Processing.Process process = ProcessingHelpers.getNextProcess(_dmp, _processType);
		process.setProcessStatusType(ProcessStatusType.START);
		ProcessingHelpers.updateProcess(_dmp, process);
		idProcess = process.getId();
		
		
		try {
			
			if(!Application.exists())ThreadContext.setApplication(application);
			for (FileUpload upload : uploads){
				
				if(_bridge.isCancel()) {
					_bridge.setStop(true);
					break;
				}
				doModuleUpload(bridge, upload, idProcess);
				stopWait(1000);
			}
			
			process.setProcessStatusType(ProcessStatusType.END);
			process.setNumberOfInserts(_numberOfUploads);
			process.setNumberOfUpdates(_numberOfDublicates);
			
			
		}catch(Exception e) {
			_bridge.setCancel(true);
			process.setProcessStatusType(ProcessStatusType.ERROR);
		}
		
		_bridge.setProgress(100);
		_bridge.setStop(true);
	 
		process.setProcessEndTime(new Date());
		ProcessingHelpers.updateProcess(_dmp, process);
	 
	}
	
	private void doModuleUpload(ProcessBridge bridge, FileUpload upload, int idProcssing) {
		try {
			
			String fileName = upload.getClientFileName();

			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(VariableGroupType.API);
			vfc.setIdTenant(_session.getCurrentTenantId());
			vfc.setVariableValue(fileName);
			
			List<Variable> entities = _dmp.getVariablesDB().getEntities(-1, -1, "", vfc);
			if(!entities.isEmpty()) {
				//Variable var = entities.get(0);
			}
			
			File newFile = new File(uploadFolder, fileName);
			newFile.createNewFile();
			upload.writeTo(newFile);

			ModuleImporter mi = new ModuleImporter(
					_dmp, 
					newFile.getAbsolutePath(),
					Application.class.getClassLoader()) {

				private static final long serialVersionUID = 1L;
				@Override
				protected void OnStatusChanged(ProcessingStatus processingStatus) {
					ProcessHelpers.setBridgeStatusMessage(
						bridge, 
						processingStatus.getDescription(_session.getLocaleParams().getLocaleCode()),
						fileName);
					
				}
			};
			mi.setUserId(_session.getCurrentUserId());
			mi.setIdTenant(_session.getCurrentTenantId());
			mi.setUserName(_session.getCurrentUser());
			mi.setProcessId(idProcssing);
			mi.setOverwrite(true);
			mi.importModule();
			
		}catch(Exception e) {
		}
	}
	
	
	
	private synchronized void stopWait(int ms) {
		try {
			Thread.sleep(ms);
		}catch(Exception e) {}
	}
	

	@Override
	public synchronized String getTaskName() {return _taskName;}
	
	@Override
	public synchronized ProcessBridge getProcessBridge() {return _bridge;}
	
	@Override
	public synchronized ProcessType getProcessType() {return _processType;}

	@Override
	public synchronized int getSourceDocumentId() {return sourceId;}

	@Override
	public synchronized int getTargetDocumentId() {return targetId;}

	@Override
	public synchronized String getMessage() {return _message;}
	
	@Override
	public synchronized String getStatus() {return _status;}
	
	@Override
	public synchronized int getIdProcess() {return idProcess;}

}
