/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.user;

import java.io.Serializable;
import java.util.Date;

import org.apache.wicket.protocol.http.servlet.ServletWebRequest;
import org.apache.wicket.request.http.WebRequest;
import org.apache.wicket.request.http.WebResponse;
import org.biffo.dms.DmsException;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.entity.Helpers.SaltedMD5;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UserException;
import org.biffo.dms.entity.user.Users.UserFilterClass;
import org.biffo.dms.rest.RestHelpers;
import org.biffo.dms.rest.RestResourceException;

public class UserHandler implements Serializable{

	private static final long serialVersionUID = 1L;
	private static final String _className="UserHandler";

	public UserHandler() {}
	
	public User getUser(WebRequest request, WebResponse response, String sessionId) throws RestResourceException{
		
		String ip = RestHelpers.getClientIpAddr((ServletWebRequest) request);
		String userName = RestHelpers.getUserNameFromRequest(request);
		String userPass = RestHelpers.getPasswordFromRequest(request);
		try {
			User user = checkUser(userName, userPass);
			return user;
		}catch(DmsException ue) {
			
			UserLogObject ulo=null;
			if(ue.getLogObject()!=null && ue.getLogObject() instanceof UserLogObject) {
				ulo = (UserLogObject) ue.getLogObject();
				ulo.ip = ip;
				DmsException dmsException = new DmsException(ue.getCode(), "E", _className, ue.getFunctionName(), null, ue, ulo);
				StaticLocators.getMessageHandler().createMessage(dmsException, null);
			}else {
				ulo = new UserLogObject();
				ulo.ip=ip;
				ulo.userName=userName;
				ulo.userPass = userPass;
				DmsException dmsException = new DmsException(3001, "E", _className, "getUser", null, ue, ulo);
				StaticLocators.getMessageHandler().createMessage(dmsException, null);
			}			
			RestResourceException re = new RestResourceException("Not authorized", 3001);
			throw re;
		}
	}
		
	
	public User getUser(String userName, String userPass, String ip, String sessionId) throws DmsException{
		
		try {
			User user = checkUser(userName, userPass);
			return user;
		}catch(DmsException dmsException) {
			StaticLocators.getMessageHandler().createMessage(dmsException, null);
			throw dmsException;
		}
		
	}
	
	
	private User checkUser(String userName, String userPass) throws DmsException{
		
	
		Date dateActual = new Date();
		User user=null;

		if(null==userName || userName.isEmpty() || null==userPass || userPass.isEmpty()){
			UserLogObject ulo = new UserLogObject();
			ulo.userName = userName;
			ulo.userPass = userPass;
			DmsException dmsException = new DmsException(3009, "E", _className, "checkUser", null, null, ulo);
			throw dmsException;
		}
		userName = stripWildcards(userName);

		try {
			userName = userName.toLowerCase();
			UserFilterClass userFilterClass = new UserFilterClass();
			userFilterClass.setLinkedWhere(" lower(benutzername) = '"+userName+"' ");
			user = StaticLocators.getUsers().getEntity(userFilterClass);
			if(null==user){
				UserLogObject ulo = new UserLogObject();
				ulo.userName = userName;
				ulo.userPass = userPass;
				DmsException dmsException = new DmsException(3010, "E", _className, "checkUser", null, null, ulo);
				throw dmsException;
			}
		}catch(UserException e) {
			DmsException dmsException = new DmsException(3000, "E", _className, "checkUser", null, null, e);
			throw dmsException;
		}
		
		try {
			if(!SaltedMD5.validatePassword(userPass, user.getBenutzerPassword())){
				UserLogObject ulo = new UserLogObject();
				ulo.userName = userName;
				ulo.userPass = userPass;
				DmsException dmsException = new DmsException(3011, "E", _className, "checkUser", null, null, ulo);
				throw dmsException;
			}
		}catch(DmsException e) {
			throw e;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(3000, "E", _className, "checkUser", null, null, e);
			throw dmsException;
		}

		
		
		if(user.isGesperrt()){
			UserLogObject ulo = new UserLogObject();
			ulo.userName = userName;
			DmsException dmsException = new DmsException(3012, "E", _className, "checkUser", null, null, ulo);
			throw dmsException;
		}
		
		switch (user.getAccessType()){
			case ADMIN:
			case USER:
				break;
			default:
				UserLogObject ulo = new UserLogObject();
				ulo.userName = userName;
				ulo.accessType = user.getAccessType().toString();
				DmsException dmsException = new DmsException(3013, "E", _className, "checkUser", null, null, ulo);
				throw dmsException;
		}
		 
		user.setLastAccessDate(dateActual);
		try {
			StaticLocators.getUsers().updateEntity(user, true);
		}catch(Exception e) {
		}
		
		return user;
	}
	
	public static String stripWildcards(String input){
		if(null==input) return input;
		return input.replaceAll("[%*]", "").trim();
	}
	
	
	
	
}
