/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.docbar;

import java.text.SimpleDateFormat;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.cycle.RequestCycle;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.dataprovider.DPDocuments;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.processQueue.tasks.ProcessHelpers;
import org.biffo.dms.resource.DmsCrypt;
import org.biffo.dms.web.helpers.ResourceText;

public class DocBar  extends Panel{

	private static final long serialVersionUID = 1L;
		
	private final String className = "DocBar";
	private final String contextPath = RequestCycle.get().getRequest().getContextPath();
	private DmsWebSession _session =  DmsWebSession.getSession();
	private SimpleDateFormat _sdf = new SimpleDateFormat(_session.getLocaleParams().getDateFormat());
	
	private WebMarkupContainer _wmcNoData;
	private DPDocuments _dpDocuments = new DPDocuments();
	private DataView<Document> _dvDocuments;
	private String documentIDs;
	
	private Label _lblNoData;

	@Override
	protected void onBeforeRender() {
		refreshDocView();
		super.onBeforeRender();
	}
	
	public void setDocumentIDs(String documentIDs) {
		this.documentIDs = documentIDs;
	}
	
	
	
	
	public DocBar(String id, String documentIDs) {
		super(id);
		
		this.documentIDs=documentIDs;
		_dvDocuments = new DataView<Document>("documents", _dpDocuments, 10 ) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void populateItem(Item<Document> item) {
				DocumentPanel fdp = new DocumentPanel("document", item.getModel(), item.getIndex());
				fdp.setOutputMarkupId(true);
				item.setOutputMarkupId(true);
				item.add(fdp);
			}
			
			@Override
			protected void onBeforeRender() {
				checkNodata(null);
				super.onBeforeRender();
			}
			
		};
		_dvDocuments.setOutputMarkupId(true);
		add(_dvDocuments);
		
		addNoData();
	}
	
	private void refreshDocView() {
		try {
			
			if(!Strings.isEmpty(documentIDs)) {
				DocumentFilterClass dfc = new DocumentFilterClass();
				dfc.setLinkedWhere(" id in ("+documentIDs+")");
				_dpDocuments.setFilterState(dfc);
			}else {
				DocumentFilterClass dfc = new DocumentFilterClass();
				dfc.setLinkedWhere(" 1=0 ");
				_dpDocuments.setFilterState(dfc);
			}
		}catch(Exception e) {
			DmsException dmsInfo = new DmsException(1000, "E", 
				className,
				"refreshDocView",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
		}
	}
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		
		String temp = ResourceText.getText("lblNoData", this);
		_lblNoData = new Label("lblnodata",temp);
		_lblNoData.setVisible(false);
		_lblNoData.setEscapeModelStrings(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void checkNodata(AjaxRequestTarget target) {
		long numberOfDocuments = _dpDocuments.size();
		if(numberOfDocuments==0) {
			_lblNoData.setVisible(true);
		}else {
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_wmcNoData);
		}
	}
	
	
	public class DocumentPanel extends Panel {

		private static final long serialVersionUID = 1L;

		private WebMarkupContainer wmcDvImage;
		private final Document document;
		
		public DocumentPanel(final String id, final IModel<Document> model, final int index) {
            super(id);
    		this.document = model.getObject();
    		setOutputMarkupId(true);
    		wmcDvImage = new WebMarkupContainer("docimage");
    		wmcDvImage.setOutputMarkupId(true);
    		
    		String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(),  document);
    		wmcDvImage.add(AttributeModifier.replace("style", "background-image: url('"+contextPath+"/png/" + encodedResource+"')"));
    		try {
    			String docName = ProcessHelpers.getDocumentTitle(document);
    			String altTitle= ResourceText.getText("imgTitle", this, docName, document.getErstelltVon(), _sdf.format(document.getErstelltAm()));
        		wmcDvImage.add(AttributeModifier.append("title", altTitle));
    		}catch(Exception e) {
    		}
    		
    		wmcDvImage.add(AttributeModifier.append("ondblclick", "dslpDblClick(event,'"+document.getId()+"');"));
    		add(wmcDvImage);
        }
	}
	
	
}
