/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.modules;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.modulebox.ModulesBoxPanel;
import org.biffo.dms.components.modulebox.ModulesBoxPanel.DnDType;
import org.biffo.dms.dataprovider.config.DPModules;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class ModulesSelectionPanel extends Panel{

	private static final String _className = "ModulesSelectionPanel";
	private DmsWebSession _session = DmsWebSession.getSession();
 
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "modulePanelToolbar";
	
	private ModulesBoxPanel _mbbp;
	private Toolbar _toolBar;
	
	private ToolbarButton _btnAll, _btnNone, _btnApplyModuleOnDocument;

	public ModulesSelectionPanel(String id) {
		super(id);
		addComponents();
		setButtonsActive(null,false);
	}

	private void addComponents() {
		
		_mbbp = new ModulesBoxPanel("modules", DnDType.DRAG) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onModuleSelect(ApiModuleImpl module, boolean selection, AjaxRequestTarget target) {
				super.onModuleSelect(module, selection, target);
				if(_mbbp.getSelection().size()>0)setButtonsActive(target, true);
				else setButtonsActive(target, false);
			}
		};
		_mbbp.setOutputMarkupId(true);
		add(_mbbp);
		addToolBar();
		
	}

	@Override
	protected void onBeforeRender() {
		refreshModulesList();
		super.onBeforeRender();
	}
	
	public void refreshModulesList() {
		
		
		List<ApiModuleImpl> lstModules = new LinkedList<ApiModuleImpl>();
		try {
			DPModules dpModules = new DPModules();
			long size = dpModules.size();
			Iterator<? extends ApiModuleImpl> itt = dpModules.iterator(0, size);
			while(itt.hasNext()) {
				lstModules.add(itt.next());
			}
			
			Collections.sort(lstModules, new Comparator<ApiModuleImpl>() {
			    @Override
			    public int compare(ApiModuleImpl o1, ApiModuleImpl o2) {
			        return o1.getServerName().compareToIgnoreCase(o2.getServerName());
			    }
			});
			_mbbp.setModules(lstModules);
			
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshModulesList",
					null,
					e
				);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		setButtonsActive(null, false);
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btnApplyModuleOnDocument = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnApplyModuleOnDocument.setUtf8Code("xe39d");
		_btnApplyModuleOnDocument.setEventName("ApplyModules");
		_btnApplyModuleOnDocument.setAlternate(ResourceText.getText("btnApplyModuleOnDocument", this));
		_btnApplyModuleOnDocument.setCssClass(toolbarCSS);
		_toolBar.add(_btnApplyModuleOnDocument);
		
		_btnAll = new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnAll.setAlternate(ResourceText.getText("btnAll", this));
		_toolBar.add(_btnAll);
		
		_btnNone = new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnNone.setAlternate(ResourceText.getText("btnNone", this));
		_toolBar.add(_btnNone);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
	}

	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			_mbbp.selectAll();
			if(_mbbp.getSelection().size()>0)setButtonsActive(target, true);
			else setButtonsActive(target, false);
		}else if(event.equals("None")) {
			_mbbp.clearSelection();
			setButtonsActive(target, false);
		}else if(event.equals("ApplyModules")) {
			onAdd(target, _mbbp.getSelection());
			_mbbp.clearSelection();
		}
		target.add(_mbbp);
	}
	
 
	private void setButtonsActive(AjaxRequestTarget target, boolean active) {
		_btnApplyModuleOnDocument.setEnabled(active); 
		if(target!=null) {
			target.add(_btnApplyModuleOnDocument);
		}
	}
	
	
	
	
	protected void onAdd(AjaxRequestTarget target, HashSet<ApiModuleImpl> moduleslist) {}
	
}
