/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.search.searchbox;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.Types.SearchStatusType;
import org.biffo.dms.components.datepicker.DatePicker;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.tagselector.TagSelector;
import org.biffo.dms.dataprovider.os.DPSearchDocumentsIds;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationException;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.TagFilterClass;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.SearchConditionBuilder;
import org.biffo.dms.helpers.SearchConditionBuilderType;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.session.ILocale;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.UserSessionParameters;
import org.biffo.dms.web.components.search.singelfield.SingleFieldSearchInput;


public class SearchboxPanel extends Panel{

 
	private static final long serialVersionUID = 1L;
	private static final String _className = "SearchboxPanel";

	private SimpleDateFormat _sdf;
	
	private SingleFieldSearchInput _sfsi;
	private DatePicker _documentdateFrom, _documentdateTo;
	private AjaxLink<String> _btnSearch, _btnClear;
	private Form<?> _searchform;
	private TagSelector _tagSelector;
	private CheckBox _combinedsearch;
	
	private DocumentOSFilterClass _dsfc;
	private DPSearchDocumentsIds _dpSPId = new DPSearchDocumentsIds();
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	private UserSessionParameters _thisParameters;

	private final HashMap<String, LinkedList<String>> _hmDocumentPageIds = new HashMap<String, LinkedList<String>>();
	
	public SearchboxPanel(String id) {
		this(id, null);
	}

	public SearchboxPanel(String id, final DocumentOSFilterClass dsfc) {
		super(id);
		
		if(null==dsfc)_dsfc = new DocumentOSFilterClass();
		else _dsfc = dsfc;
		
		initParameters();
		
		_searchform = new Form<Void>("searchform");
		_searchform.setOutputMarkupId(true);
		add(_searchform);
		

		addSingleFieldSearch();
		addDatePickers();
		addTagSelector();
		addCheckBoxen();
		addButtons();

		if(_dsfc.getLinkedWhere()!=null) {
			refreshTagList(null);
			refreshMinMaxDates(null);
			buildSearchResults();
		}else {
			refreshMinMaxDates(null);
			refreshTagList(null);
		}

	}
	
	public SearchHelper getSearchHelper(String documentId) {
		if(documentId == null) return null;
		if(null==_hmDocumentPageIds.get(documentId)) return null;
		SearchHelper sh=new SearchHelper();
		sh.setIdDocument(Integer.valueOf(documentId));
		sh.setSearchString(_dsfc.getSearchString());
		sh.setFoundPages(_hmDocumentPageIds.get(documentId));
		return sh;
	}
	
	public HashMap<String, LinkedList<String>> getDocumentPageIds(){
		return _hmDocumentPageIds;
	}
	
	
	public String getCondition() {
		
		if(_hmDocumentPageIds.size()==0) return null;
		try {
			String strArr[]= _hmDocumentPageIds.keySet().toArray(new String[_hmDocumentPageIds.size()]);
			return String.join(",", strArr);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getCondition", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
		
	}
	
	
	private void buildSearchResults() {
		_dsfc.setSize(0);
		_hmDocumentPageIds.clear();
		HashMap<String, LinkedList<String>> docIds = _dpSPId.getDocumentIds(_dsfc, true, true);
		if(docIds!=null)_hmDocumentPageIds.putAll(docIds);
	}
	
	private void addButtons() {
		
		
		_btnSearch = new AjaxLink<String>("search") {
			private static final long serialVersionUID = 1L;
			@Override
			public void onClick(AjaxRequestTarget target){
				target.appendJavaScript("$('#"+getMarkupId()+"').blur();");
				buildSearchResults();
				
				if(_hmDocumentPageIds.isEmpty()) {
					OnSearchNoData(target);
				}else {
					OnButtonSearch(target);
				}
			}
		};
		_btnSearch.setOutputMarkupId(true);
		_btnSearch.setEnabled(!_thisParameters.isCombiendQuery());
		_searchform.add(_btnSearch);
		
		_btnClear = new AjaxLink<String>("clear") {
			private static final long serialVersionUID = 1L;
			@Override
			public void onClick(AjaxRequestTarget target){
				target.appendJavaScript("$('#"+getMarkupId()+"').blur();");
				clearAll(target);
				OnButtonClear(target);
			}
		};
		_btnClear.setOutputMarkupId(true);
		_searchform.add(_btnClear);
		
	}

	private void addSingleFieldSearch() {
		
		_sfsi = new SingleFieldSearchInput("singelfieldsearch", _dsfc) {
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnSelectionFilterClick(AjaxRequestTarget target) {
				_dsfc.setSingelFieldSearchWhere(_dsfc.getLinkedWhere());
				_dsfc.setSearchStatusType(SearchStatusType.SINGLEFIELD_SET, true);
				
				if(_thisParameters.isCombiendQuery()) {
					refreshTagList(target);
					refreshMinMaxDates(target);
					buildSearchResults();
					if(_hmDocumentPageIds.isEmpty()) {
						OnSearchNoData(target);
					}else {
						OnQueryChanged(target);
					}
				}
			}
			
			@Override
			protected void OnEnterFilterClick(AjaxRequestTarget target) {
				_dsfc.setSingelFieldSearchWhere(_dsfc.getLinkedWhere());
				_dsfc.setSearchStatusType(SearchStatusType.SINGLEFIELD_SET, true);
				
				if(_thisParameters.isCombiendQuery()) {
					refreshTagList(target);
					refreshMinMaxDates(target);
					buildSearchResults();
					if(_hmDocumentPageIds.isEmpty()) {
						OnSearchNoData(target);
					}else {
						OnQueryChanged(target);
					}
				}
			}
			
			@Override
			protected void OnClearFilterClick(AjaxRequestTarget target) {
				_dsfc.setLinkedWhere(null);
				_dsfc.setSingelFieldSearchWhere(null);
				_dsfc.setSearchStatusType(SearchStatusType.SINGLEFIELD_SET, false);
				
				if(_thisParameters.isCombiendQuery()) {
					refreshTagList(target);
					refreshMinMaxDates(target);
					buildSearchResults();
				}
				OnQueryChanged(target);
			}
			
		};
		_sfsi.setOutputMarkupId(true);
		_sfsi.setAutoComplete(_thisParameters.isCombiendQuery());
		_searchform.add(_sfsi);
 
		
	}

	
	private void addDatePickers() {
		
		ILocale sessionParams =_session.getLocaleParams();
		_sdf = new SimpleDateFormat(sessionParams.getDateFormat());

		_documentdateFrom = new DatePicker("documentdatefrom", new PropertyModel<Date>(_dsfc, "DocumentDatumTimeFrom"), true) {
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnChange(AjaxRequestTarget target, Date value) {
				_dsfc.setSearchStatusType(SearchStatusType.DFROM_SET, true);
				buildDateSelectorWhere();
				if(_thisParameters.isCombiendQuery()) {
					refreshTagList(target);
					buildSearchResults();
					if(_hmDocumentPageIds.isEmpty()) {
						OnSearchNoData(target);
					}else {
						OnQueryChanged(target);
					}
				}
			}
			protected void OnClear(AjaxRequestTarget target, Date value) {
				_dsfc.setSearchStatusType(SearchStatusType.DFROM_SET, false);
				buildDateSelectorWhere();
				if(_thisParameters.isCombiendQuery()) {
					refreshMinMaxDates(target);
					refreshTagList(target);
					buildSearchResults();
					OnQueryChanged(target);
				}
			}

       };
       _documentdateFrom.setOutputMarkupId(true);
       _documentdateFrom.getDatePickerOptions().setFormat(sessionParams.getDateFormat());
       _documentdateFrom.getDatePickerOptions().setDaysMin(sessionParams.getDaysMin());
       _documentdateFrom.getDatePickerOptions().setMonths(sessionParams.getMonths());
       _documentdateFrom.getDatePickerOptions().setMonthsShort(sessionParams.getMonthsShort());
       _documentdateFrom.getDatePickerOptions().setAutoHide(true);
       _searchform.add(_documentdateFrom);

 
       _documentdateTo = new DatePicker("documentdateto", new PropertyModel<Date>(_dsfc, "DocumentDatumTimeTo"), true) {
  	 
    	   private static final long serialVersionUID = 1L;
		
			@Override
			protected void OnChange(AjaxRequestTarget target, Date value) {
				_dsfc.setSearchStatusType(SearchStatusType.DTO_SET, true);
				buildDateSelectorWhere();
				if(_thisParameters.isCombiendQuery()) {
					refreshTagList(target);
					buildSearchResults();
					if(_hmDocumentPageIds.isEmpty()) {
						OnSearchNoData(target);
					}else {
						OnQueryChanged(target);
					}
				}
			}
			protected void OnClear(AjaxRequestTarget target, Date value) {
				_dsfc.setSearchStatusType(SearchStatusType.DTO_SET, false);
				buildDateSelectorWhere();
				if(_thisParameters.isCombiendQuery()) {
					refreshMinMaxDates(target);
					refreshTagList(target);
					buildSearchResults();
					OnQueryChanged(target);
				}
			}
   		
       };
       _documentdateTo.setOutputMarkupId(true);
       _documentdateTo.getDatePickerOptions().setFormat(sessionParams.getDateFormat());
       _documentdateTo.getDatePickerOptions().setDaysMin(sessionParams.getDaysMin());
       _documentdateTo.getDatePickerOptions().setMonths(sessionParams.getMonths());
       _documentdateTo.getDatePickerOptions().setMonthsShort(sessionParams.getMonthsShort());
       _documentdateTo.getDatePickerOptions().setAutoHide(true);
       _searchform.add(_documentdateTo);
       

		
	}
	
	private void addTagSelector() {
		_tagSelector = new TagSelector("tagselector") {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnChange(AjaxRequestTarget target) {
				super.OnChange(target);
				buildTagSelectorWhere();
				if(_thisParameters.isCombiendQuery()) {
					refreshMinMaxDates(target);
					buildSearchResults();
					if(_hmDocumentPageIds.isEmpty()) {
						OnSearchNoData(target);
					}else {
						OnQueryChanged(target);
					}
				} 
			}
		};
		_tagSelector.setOutputMarkupId(true);
		if(_dsfc.getSelectedTagIds()!=null) {
			for(Integer tagId:_dsfc.getSelectedTagIds()) {
				_tagSelector.setTagSelected(tagId, true);
			}
		}
		_searchform.add(_tagSelector);
		 
	}
	
	private void addCheckBoxen(){

        
		_combinedsearch = new CheckBox("combinedsearch", new PropertyModel<Boolean>(_thisParameters, "CombiendQuery"));
		_combinedsearch.setOutputMarkupId(true);
		_combinedsearch.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				if(_thisParameters.isCombiendQuery()) {
					_btnSearch.setEnabled(false);
					_thisParameters.setAutocomplete(true);
					_thisParameters.setCombiendQuery(true);
				}else {
					_btnSearch.setEnabled(true);
					_thisParameters.setAutocomplete(false);
					_thisParameters.setCombiendQuery(false);
				}
				SearchboxPanel.this.storeParameters();
				_sfsi.setAutoComplete(_thisParameters.isAutocomplete());
				clearAll(target);
				target.add(_btnSearch);
                target.add(_combinedsearch);
            }
        }); 
		_searchform.add(_combinedsearch);
	}
	

	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
	}
	
	private void buildTagSelectorWhere() {
		_dsfc.setLinkedWhere(null);
		_dsfc.setTags(null);
		List<ISelectData> lstTags = _tagSelector.getDataSelection(true);
		List<Integer> lstSelectedTags = new LinkedList<Integer>();
		if(lstTags!=null && lstTags.size()>0) {
			StringBuilder sb = new StringBuilder();
			for(int i=0;i<lstTags.size();i++) {
				if(i>0)sb.append(",");
				sb.append(lstTags.get(i).getId());
				lstSelectedTags.add(lstTags.get(i).getId());
				_dsfc.addTag(""+lstTags.get(i).getId());
			}
			_dsfc.setSelectedTagIds(lstSelectedTags);

			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
			//rfc.setLinkedWhere(" idrelation1 in ("+sb.toString()+") AND idrelation1 in (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+") ");
			rfc.setLinkedWhere(" idrelation1 in ("+sb.toString()+") ");
			
			//_userCondition = "dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";

			
			List<Relation> tagedRelations = null;
			try {
				tagedRelations = StaticLocators.getRelations().getEntitiesAggIdrelation2(-1, -1, "", rfc);
			} catch (RelationException e) {
				DmsException dmsException = new DmsException(1000, "E", _className, "buildTagSelectorWhere", null, e);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
			if(tagedRelations.size()>0) {
				_dsfc.setTagsSearchWhere(getWhereIDs(tagedRelations.get(0).getRelationIdList()));
				_dsfc.setSearchStatusType(SearchStatusType.TAG_SET, true);
			}
		}else {
			_dsfc.setTagsSearchWhere(null);
			_dsfc.setSelectedTagIds(null);
			_dsfc.setSearchStatusType(SearchStatusType.TAG_SET, false);
		}
	} 
	
	
	private void buildDateSelectorWhere() {
		try {
			_dsfc.setLinkedWhere(null);
			_dsfc.setFilterObject(null);

			String condition = SearchConditionBuilder.buildCondition(_dsfc, new SearchConditionBuilderType[] { SearchConditionBuilderType.SINGLEFIELD, SearchConditionBuilderType.TAGS});
			_dsfc.setLinkedWhere(condition);

			if(!_dsfc.getSearchStatusType(SearchStatusType.DFROM_SET) && !_dsfc.getSearchStatusType(SearchStatusType.DTO_SET)){
				_dsfc.setDateSearchWhere(null);
				return;
			}
			
			
			List<Document> entities =StaticLocators.getDocumentsDBOS().getEntitiesAgg(-1, -1, "", _dsfc);
			if(entities.size()>0) {
				_dsfc.setDateSearchWhere(getWhereIDs(entities.get(0).getDocumentIdList()));
				if(_dsfc.getDocumentDatumTimeFrom()!=null)_dsfc.setSearchStatusType(SearchStatusType.DFROM_SET, true);
				if(_dsfc.getDocumentDatumTimeTo()!=null)_dsfc.setSearchStatusType(SearchStatusType.DTO_SET, true);
			}else {
				_dsfc.setDateSearchWhere(null);
				_dsfc.setSearchStatusType(SearchStatusType.DTO_SET, false);
				_dsfc.setSearchStatusType(SearchStatusType.DFROM_SET, false);
			}
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "buildDateSelectorWhere", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	} 
	
	
	private void refreshTagList(AjaxRequestTarget target) {
		
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);

			String condition = SearchConditionBuilder.buildCondition(_dsfc, new SearchConditionBuilderType[] { SearchConditionBuilderType.SINGLEFIELD, SearchConditionBuilderType.DATE});
			if(null!=condition)	condition = condition.replaceAll("id", "idrelation2");
			else condition =  " idrelation2 in (select id from dms_documents)";
			
			if(_session.getUserAccessType()==UsersAccessType.USER) {
				condition = condition + " AND idrelation2 in (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
			}

			rfc.setLinkedWhere(condition);
						
			List<Relation> tagedRelations = StaticLocators.getRelations().getEntitiesAggIdrelation1(-1, -1, "", rfc);
			String relationIds = tagedRelations.get(0).getRelationIdList();
			if(tagedRelations.size()>0 && relationIds!=null) {
				relationIds = relationIds.substring(1, relationIds.length()-1);
				TagFilterClass tfc = new TagFilterClass(); 
				tfc.setLinkedWhere(" id in ("+relationIds+")");
				_tagSelector.setDataByTagFilter(tfc, true);
				_tagSelector.setDisabled(false);
			}else {
				_tagSelector.setDisabled(true);
			} 
			
		    if(target!=null) {
		    	target.add(_tagSelector);
		    }
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshTagList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		
	} 

	private void refreshMinMaxDates(AjaxRequestTarget target) {

		try {
 

			String condition = SearchConditionBuilder.buildCondition(_dsfc, new SearchConditionBuilderType[] { SearchConditionBuilderType.SINGLEFIELD, SearchConditionBuilderType.TAGS});
			
			
			if(_session.getUserAccessType()==UsersAccessType.USER) {
				if(condition==null) {
					condition = "dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
				}else {
					condition = condition + " AND dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
				}
			}
			_dsfc.setLinkedWhere(condition);
			
			List<Document> entities =StaticLocators.getDocumentsDBOS().getEntities(0, 1, " order by DocumentDatum ASC ", _dsfc);
			if(entities!=null && entities.size()==1 && !_dsfc.getSearchStatusType(SearchStatusType.DFROM_SET)) {
				_dsfc.setdMin(entities.get(0).getDocumentDatum());
				_documentdateFrom.getDatePickerOptions().setStartDate(_sdf.format(_dsfc.getdMin()));
				_documentdateTo.getDatePickerOptions().setStartDate(_sdf.format(_dsfc.getdMin()));	
			}
			entities =StaticLocators.getDocumentsDBOS().getEntities(0, 1, " order by DocumentDatum DESC ", _dsfc);
			if(entities!=null && entities.size()==1 && !_dsfc.getSearchStatusType(SearchStatusType.DTO_SET)) {
				_dsfc.setdMax(entities.get(0).getDocumentDatum());
				_documentdateFrom.getDatePickerOptions().setEndDate(_sdf.format(_dsfc.getdMax()));
				_documentdateTo.getDatePickerOptions().setEndDate(_sdf.format(_dsfc.getdMax()));	
			}
		    if(target!=null) {
		    	target.add(_documentdateFrom);
				target.add(_documentdateTo);
		    } 
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshMinMaxDates", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	} 
	
	
	private void initParameters() {
		_thisParameters = (UserSessionParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.USER_SESSION_PARAMETERS);
	}
	
	private void storeParameters() {
		_session.getSessionParameters().setParametersByType(SessionParameterTypes.USER_SESSION_PARAMETERS, _thisParameters);
	}
	
	private static String getWhereIDs(String retrievedIDs) {
		
		String relationIds = retrievedIDs;//
		if(relationIds!=null) {
			relationIds = relationIds.substring(1, relationIds.length()-1);
			relationIds =" id in ("+relationIds+") ";
		}
		return relationIds;
	}
	
	private void clearAll(AjaxRequestTarget target) {
		_dsfc.setDateSearchWhere(null);
		_dsfc.setLinkedWhere(null);
		_dsfc.setSingelFieldSearchWhere(null);
		_dsfc.setTagsSearchWhere(null);
		_dsfc.setSelectedTagIds(null);
		_dsfc.setTags(null);
		_dsfc.setdMax(null);
		_dsfc.setdMin(null);
 
		_documentdateFrom.clearDate(target);
		_documentdateTo.clearDate(target);
		_sfsi.clear(target);
		_tagSelector.unselectAllTags();
		_hmDocumentPageIds.clear();
		
		refreshMinMaxDates(target);
		refreshTagList(target);
		
		target.add(_documentdateTo);
		target.add(_documentdateFrom);
		target.add(_tagSelector);
		
		_session.clearSearchParameters();
 
	}
	
	
	public void closeAll(AjaxRequestTarget target) {
		_documentdateFrom.hide(target);
		_documentdateTo.hide(target);
	}
	
	public void refresh(AjaxRequestTarget target) {
		try {
			refreshMinMaxDates(target);
			refreshTagList(target);
			buildSearchResults();
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refresh", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	protected void OnSearchNoData(AjaxRequestTarget target){}
	protected void OnButtonSearch(AjaxRequestTarget target){}
	protected void OnButtonClear(AjaxRequestTarget target){}
	protected void OnQueryChanged(AjaxRequestTarget target){}
	
	public boolean hasResults() {
		return !_hmDocumentPageIds.isEmpty();
	}
	
}
