/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.tags;

import java.util.HashSet;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.tagbox.TagsBoxPanel;
import org.biffo.dms.components.tagbox.TagsBoxPanel.DnDType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Tags.TagException;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.tags.TagsPage;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 
public class TaggingPanel extends Panel{

 
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "taggingPanelToolbar";
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private TagsBoxPanel _tsbp;
	private Toolbar _toolBar;
	
	private ToolbarButton _btnAll, _btnNone, _btnRemoveTags, _btnAddTagToDocument, _btnReplaceTags;

	public TaggingPanel(String id) {
		super(id);
		addComponents();
		setButtonsActive(null,false);
	}

	private void addComponents() {
		
		_tsbp = new TagsBoxPanel("tags", DnDType.DRAG) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onTagSelect(Tag tag, boolean selection, AjaxRequestTarget target) {
				super.onTagSelect(tag, selection, target);
				if(_tsbp.getSelection().size()>0)setButtonsActive(target, true);
				else setButtonsActive(target, false);
			}
		};
		_tsbp.setOutputMarkupId(true);
		add(_tsbp);
		addToolBar();
		
	}

	@Override
	protected void onBeforeRender() {
		refreshTagList();
		super.onBeforeRender();
	}
	
	public void refreshTagList() {
		List<Tag> taglist;
		try {
			taglist = StaticLocators.getTags().getEntities(-1, -1, " order by tagname ", null);
			_tsbp.setTags(taglist);
		} catch (TagException e) {
			DmsException dmsException = new DmsException(1000, "E", this.getClass().getSimpleName(), "refreshTagList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		setButtonsActive(null, false);
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btnAddTagToDocument = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnAddTagToDocument.setUtf8Code("xe39d");
		_btnAddTagToDocument.setEventName("AddTags");
		_btnAddTagToDocument.setAlternate(ResourceText.getText("btnAddTagToDocument", this));
		_btnAddTagToDocument.setCssClass(toolbarCSS);
		_toolBar.add(_btnAddTagToDocument);
		
		_btnAll = new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnAll.setAlternate(ResourceText.getText("btnAll", this));
		_toolBar.add(_btnAll);
		
		_btnNone = new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnNone.setAlternate(ResourceText.getText("btnNone", this));
		_toolBar.add(_btnNone);

		ToolbarButton btnAddTag = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		btnAddTag.setUtf8Code("xf05b");
		btnAddTag.setEventName("AddTag");
		btnAddTag.setAlternate(ResourceText.getText("btnAddTag", this));
		btnAddTag.setCssClass(toolbarCSS);
		_toolBar.add(btnAddTag);
		
		_btnReplaceTags = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnReplaceTags.setUtf8Code("xf232");
		_btnReplaceTags.setEventName("ReplaceTags");
		_btnReplaceTags.setAlternate(ResourceText.getText("btnReplaceTags", this));
		_btnReplaceTags.setCssClass(toolbarCSS);
		_toolBar.add(_btnReplaceTags);
		 
		_btnRemoveTags = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnRemoveTags.setUtf8Code("xe14a");
		_btnRemoveTags.setEventName("RemoveTags");
		_btnRemoveTags.setAlternate(ResourceText.getText("btnRemoveTags", this));
		_btnRemoveTags.setCssClass(toolbarCSS);
		_toolBar.add(_btnRemoveTags);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
	}

	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			_tsbp.selectAll();
			if(_tsbp.getSelection().size()>0)setButtonsActive(target, true);
			else setButtonsActive(target, false);
		}else if(event.equals("None")) {
			_tsbp.clearSelection();
			setButtonsActive(target, false);
		}else if(event.equals("AddTags")) {
			onAdd(target, _tsbp.getSelection());
			_tsbp.clearSelection();
		}else if(event.equals("ReplaceTags")) {
			onReplace(target, _tsbp.getSelection());
			_tsbp.clearSelection();
		}else if(event.equals("RemoveTags")) {
			onDelete(target, _tsbp.getSelection());
			_tsbp.clearSelection();
		}else if(event.equals("AddTag")) {
			TagsPage page = new TagsPage(getPage().getPageReference(), -1);
			setResponsePage(page);
		}
		target.add(_tsbp);
	}
	
 
	private void setButtonsActive(AjaxRequestTarget target, boolean active) {
		_btnRemoveTags.setEnabled(active);
		_btnAddTagToDocument.setEnabled(active); 
		_btnReplaceTags.setEnabled(active);
		if(target!=null) {
			target.add(_btnRemoveTags);
			target.add(_btnAddTagToDocument);
			target.add(_btnReplaceTags);
		}
	}
	
	
	
	
	protected void onAdd(AjaxRequestTarget target, HashSet<Tag> taglist) {}
	protected void onReplace(AjaxRequestTarget target, HashSet<Tag> taglist) {}
	protected void onDelete(AjaxRequestTarget target, HashSet<Tag> taglist) {}
	
}
