/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.components.users;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.userbox.UsersBoxPanel;
import org.biffo.dms.components.userbox.UsersBoxPanel.DnDType;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UserException;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.users.UsersPage;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;


public class UsersSelectionPanel extends Panel{

	private static final String _className = "UsersSelectionPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "usersPanelToolbar";
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private UsersBoxPanel _usbp;
	private Toolbar _toolBar;
	
	private ToolbarButton _btnAll, _btnNone, _btnRemoveUsers, _btnAddUsersToDocument, _btnReplaceUsers;

	public UsersSelectionPanel(String id) {
		super(id);
		addComponents();
		setButtonsActive(null,false);
	}

	private void addComponents() {
		
		_usbp = new UsersBoxPanel("users", DnDType.DRAG) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onUserSelect(User user, boolean selection, AjaxRequestTarget target) {
				super.onUserSelect(user, selection, target);
				if(_usbp.getSelection().size()>0)setButtonsActive(target, true);
				else setButtonsActive(target, false);
			}
		};
		_usbp.setOutputMarkupId(true);
		add(_usbp);
		addToolBar();
		
	}

	@Override
	protected void onBeforeRender() {
		refreshUserList();
		super.onBeforeRender();
	}
	
	public void refreshUserList() {
		List<User> userlist=new LinkedList<User>();
		try {
			List<User> tempUserlist = StaticLocators.getUsers().getEntities(-1, -1, " order by benutzername ", null);
			for(int i=0;i<tempUserlist.size();i++) {
				if(_session.getUserAccessType()==UsersAccessType.ADMIN && tempUserlist.get(i).getAccessType()==UsersAccessType.ADMIN)userlist.add(tempUserlist.get(i));
				else if(tempUserlist.get(i).getAccessType()==UsersAccessType.USER)userlist.add(tempUserlist.get(i));
			}
			_usbp.setUsers(userlist);
		} catch (UserException e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshUserList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		setButtonsActive(null, false);
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btnAddUsersToDocument = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnAddUsersToDocument.setUtf8Code("xe39d");
		_btnAddUsersToDocument.setEventName("AddUsers");
		_btnAddUsersToDocument.setAlternate(ResourceText.getText("btnAddUsersToDocument", this));
		_btnAddUsersToDocument.setCssClass(toolbarCSS);
		_toolBar.add(_btnAddUsersToDocument);
		
		_btnAll = new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnAll.setAlternate(ResourceText.getText("btnAll", this));
		_toolBar.add(_btnAll);
		
		_btnNone = new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btnNone.setAlternate(ResourceText.getText("btnNone", this));
		_toolBar.add(_btnNone);

		ToolbarButton btnAddUser = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		btnAddUser.setUtf8Code("xe7fd");
		btnAddUser.setEventName("AddUser");
		btnAddUser.setAlternate(ResourceText.getText("btnAddUser", this));
		btnAddUser.setCssClass(toolbarCSS);
		if(_session.getUserAccessType()==UsersAccessType.ADMIN)	_toolBar.add(btnAddUser);
		
		_btnReplaceUsers = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnReplaceUsers.setUtf8Code("xf232");
		_btnReplaceUsers.setEventName("ReplaceUsers");
		_btnReplaceUsers.setAlternate(ResourceText.getText("btnReplaceUsers", this));
		_btnReplaceUsers.setCssClass(toolbarCSS);
		_toolBar.add(_btnReplaceUsers);
		 
		_btnRemoveUsers = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btnRemoveUsers.setUtf8Code("xe14a");
		_btnRemoveUsers.setEventName("RemoveUsers");
		_btnRemoveUsers.setAlternate(ResourceText.getText("btnRemoveUsers", this));
		_btnRemoveUsers.setCssClass(toolbarCSS);
		_toolBar.add(_btnRemoveUsers);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
	}

	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			_usbp.selectAll();
			if(_usbp.getSelection().size()>0)setButtonsActive(target, true);
			else setButtonsActive(target, false);
		}else if(event.equals("None")) {
			_usbp.clearSelection();
			setButtonsActive(target, false);
		}else if(event.equals("AddUsers")) {
			onAdd(target, _usbp.getSelection());
			_usbp.clearSelection();
		}else if(event.equals("ReplaceUsers")) {
			onReplace(target, _usbp.getSelection());
			_usbp.clearSelection();
		}else if(event.equals("RemoveUsers")) {
			onDelete(target, _usbp.getSelection());
			_usbp.clearSelection();
		} else if(event.equals("AddUser")) {
			UsersPage page = new UsersPage(getPage().getPageReference(), -1);
			setResponsePage(page);
		}
		target.add(_usbp);
	}
	
 
	private void setButtonsActive(AjaxRequestTarget target, boolean active) {
		_btnRemoveUsers.setEnabled(active);
		_btnAddUsersToDocument.setEnabled(active); 
		_btnReplaceUsers.setEnabled(active);
		if(target!=null) {
			target.add(_btnRemoveUsers);
			target.add(_btnAddUsersToDocument);
			target.add(_btnReplaceUsers);
		}
	}
	
	
	
	
	protected void onAdd(AjaxRequestTarget target, HashSet<User> userlist) {}
	protected void onReplace(AjaxRequestTarget target, HashSet<User> userlist) {}
	protected void onDelete(AjaxRequestTarget target, HashSet<User> userlist) {}
	
}
