/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.config;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.NumberTextField;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.Impl.TesseractVariablesImpl;
import org.biffo.dms.helpers.ConfigHelpers;
import org.biffo.dms.helpers.TesseractLanguageType;
import org.biffo.dms.helpers.TesseractOemType;
import org.biffo.dms.helpers.TesseractPsmType;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class TesseractConfigPanel extends Panel{

	private static final String _className = "TesseractConfigPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private Label _lblTitel;
	
	private TextField<String> _binaryFolder, _dataFolder, _executableName;
	private NumberTextField<Integer> _pdftotiffpdi;

	private ErrorLabel _binaryFolder_info, _dataFolder_info, _executableName_info; 
	private ErrorLabel _language_info, _oem_info, _pdftotiffpdi_info, _psm_info;
	
	private DropDownChoice<TesseractOemType> _oem;
	private DropDownChoice<TesseractLanguageType> _language;
	private DropDownChoice<TesseractPsmType> _psm;
	
 	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK, _btCheck;

	private Object _origin;
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();
 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private TesseractVariablesImpl _configVariables = new TesseractVariablesImpl();
	private TesseractVariablesImpl _configVariables_Backup = new TesseractVariablesImpl();
	
	private TesseractOemType _oemType=TesseractOemType._1;
	private TesseractLanguageType _languageType=TesseractLanguageType.deu;
	private TesseractPsmType _psmType=TesseractPsmType._3;
	
	public TesseractConfigPanel(String id, int tesseractConfigId, Object origin) {
		super(id);

		_origin = origin;
		readConfig();
		 
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        
        addDropDownBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		_panelTitel.setTitel(ResourceText.getText("titEditConfig", this));
        
	}
	
	private void addTextFields(){
		
		_binaryFolder = new TextField<String>("binaryfolder");
		_binaryFolder.setOutputMarkupId(true);
		_form.add(_binaryFolder); 
		
		_dataFolder = new TextField<String>("datafolder");
		_dataFolder.setOutputMarkupId(true);
		_form.add(_dataFolder); 
		
		_executableName = new TextField<String>("executablename");
		_executableName.setOutputMarkupId(true);
		_form.add(_executableName); 
		
		_pdftotiffpdi = new NumberTextField<Integer>("pdftotiffpdi");
		_pdftotiffpdi.setOutputMarkupId(true);
		_form.add(_pdftotiffpdi); 
 
		_binaryFolder_info= new ErrorLabel("binaryfolder_info");
		_form.add(_binaryFolder_info);
		
		_dataFolder_info= new ErrorLabel("datafolder_info"); 
		_form.add(_dataFolder_info);
		
		_executableName_info= new ErrorLabel("executablename_info"); 
		_form.add(_executableName_info);
		
		_pdftotiffpdi_info= new ErrorLabel("pdftotiffpdi_info");
		_form.add(_pdftotiffpdi_info);

	}
	

	
	
	
	private void addCheckBoxen(){}
 
	private void addDropDownBoxen(){
		
		_oem = new DropDownChoice<TesseractOemType>("oem",  Arrays.asList(TesseractOemType.values()), new IChoiceRenderer<TesseractOemType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(TesseractOemType type) {
				return type.getDescription();
			}

			public String getIdValue(TesseractOemType type, int index) {
				return ""+type.getCode();
			}
		});
		
		_oem.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 			private static final long serialVersionUID = 1L;
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				_configVariables.setOem(_oemType.getCode());
				target.add(_oem);				
			}
		});
		
		_oem.setOutputMarkupId(true);
	    _form.add(_oem);
	    

        
	    _language = new DropDownChoice<TesseractLanguageType>("language",  Arrays.asList(TesseractLanguageType.values()), new IChoiceRenderer<TesseractLanguageType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(TesseractLanguageType type) {
				return type.getDescription();
			}

			public String getIdValue(TesseractLanguageType type, int index) {
				return ""+type.getCode();
			}
		});
		
	    _language.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 			private static final long serialVersionUID = 1L;
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				_configVariables.setLanguage(_languageType.toString());
				target.add(_language);				
			}
		});
	    _language.setOutputMarkupId(true);
	    _form.add(_language);
	    
		_psm = new DropDownChoice<TesseractPsmType>("psm",  Arrays.asList(TesseractPsmType.values()), new IChoiceRenderer<TesseractPsmType>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(TesseractPsmType type) {
				return type.getDescription();
			}

			public String getIdValue(TesseractPsmType type, int index) {
				return ""+type.getCode();
			}
		});
		
		_psm.add(new AjaxFormComponentUpdatingBehavior("onchange") {
 			private static final long serialVersionUID = 1L;
			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				_configVariables.setPsm(_psmType.getCode());
				target.add(_psm);				
			}
		});
		
		_psm.setOutputMarkupId(true);
	    _form.add(_psm);
	    
		_language_info= new ErrorLabel("language_info");
		_language_info.setOutputMarkupId(true);
		_form.add(_language_info);
		
		_oem_info= new ErrorLabel("oem_info"); 
		_oem_info.setOutputMarkupId(true);
		_form.add(_oem_info);
        
		_psm_info= new ErrorLabel("psm_info"); 
		_psm_info.setOutputMarkupId(true);
		_form.add(_psm_info);
		
        
	}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	@SuppressWarnings("unused")
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(TesseractConfigPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		//_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		
		_btCheck = new ToolbarButton(_form).setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_CHANGE |  ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT);
		_btCheck.setUtf8Code("xf232");
		_btCheck.setAlternate(ResourceText.getText("tbAltCheck", this));
		_btCheck.setEventName("check");
		_btCheck.setCssClass(toolbarCSS);
		_toolBar.add(_btCheck);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		else if(event.equals("check")) {
			if(checkShowError(_configVariables, target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}else{
				infobox.refresh(target, ResourceText.getText("infConfigurationValid", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}
		}
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 _configVariables = _configVariables_Backup; 
			 refreshFormObjects();
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 //closeForm(target);
			 }else {
				 try {
					 //_configVariables = _configVariables_Backup; 
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	
	private void edit(AjaxRequestTarget target){
		 try {
			 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
			 setFormComponentsEditable(true);
	 	}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}	
	
	
 
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private boolean checkShowError(TesseractVariablesImpl config, AjaxRequestTarget target) {
		
		
		boolean error=false;
		
		_binaryFolder_info.setVisible(false);
		_dataFolder_info.setVisible(false);
		_executableName_info.setVisible(false);
		_language_info.setVisible(false);
		_oem_info.setVisible(false);
		_pdftotiffpdi_info.setVisible(false);
		_psm_info.setVisible(false);

		
		if(Strings.isEmpty(config.getBinaryFolder())){
			_binaryFolder_info.setErrorText(ResourceText.getText("errTextBinaryFolderEmpty", this));
			_binaryFolder_info.setVisible(true);
			error = true;
		}else if(!ConfigHelpers.checkFileSystemFolderExists(config.getBinaryFolder())){
			_binaryFolder_info.setErrorText(ResourceText.getText("errTextFolderNotExists", this));
			_binaryFolder_info.setVisible(true);
			error = true;
		}
		
		if(Strings.isEmpty(config.getDataFolder())){
			_dataFolder_info.setErrorText(ResourceText.getText("errTextDataFolderEmpty", this));
			_dataFolder_info.setVisible(true);
			error = true;
		}else if(!ConfigHelpers.checkFileSystemFolderExists(config.getDataFolder())){
			_dataFolder_info.setErrorText(ResourceText.getText("errTextFolderNotExists", this));
			_dataFolder_info.setVisible(true);
			error = true;
		}
		
		if(Strings.isEmpty(config.getExecutableName())){
			_executableName_info.setErrorText(ResourceText.getText("errTextExecutableEmpty", this));
			_executableName_info.setVisible(true);
			error = true;
		}else if(!ConfigHelpers.checkFileSystemFileExists(config.getBinaryFolder()+"/"+ config.getExecutableName())){
			_executableName_info.setErrorText(ResourceText.getText("errTextExecutableNotExists", this));
			_executableName_info.setVisible(true);
			error = true;
		}else if(!ConfigHelpers.checkTesseractExecutable(config.getBinaryFolder()+"/"+ config.getExecutableName())){
			_executableName_info.setErrorText(ResourceText.getText("errTextExecutableNotExecutable", this));
			_executableName_info.setVisible(true);
			error = true;
		}
		
		if(config.getOem()==-1 || _oemType==null) {
			_oem_info.setErrorText(ResourceText.getText("errTextOemEmpty", this));
			_oem_info.setVisible(true);
			error = true;
		}
		
		if(config.getLanguage()==null || _languageType==null) {
			_language_info.setErrorText(ResourceText.getText("errTextLanguageEmpty", this));
			_language_info.setVisible(true);
			error = true;
		}
		
		if(config.getPsm()==-1 || _psmType==null) {
			_psm_info.setErrorText(ResourceText.getText("errTextPsmEmpty", this));
			_psm_info.setVisible(true);
			error = true;
		}
		
		if(config.getPDFToTiffDPI()<200 || config.getPDFToTiffDPI()>600) {
			_pdftotiffpdi_info.setErrorText(ResourceText.getText("errTextPdfToTiffEmpty", this));
			_pdftotiffpdi_info.setVisible(true);
			error = true;
		}
		
		target.add(_binaryFolder_info);
		target.add(_dataFolder_info);
		target.add(_executableName_info);
		target.add(_language_info);
		target.add(_oem_info);
		target.add(_pdftotiffpdi_info);
		target.add(_psm_info);
		
		return error;
		
	}
	
	
 
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_configVariables, target)) {
				infobox.refresh(target, ResourceText.getText("errConfigurationInvalid", this), INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
				return false;
			}else {
				HashMap<String, Variable> hmVariables = _configVariables.getVariables();
				Set<String> keys = hmVariables.keySet();
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					
					int id=StaticLocators.getVariables().getEntityId(var);
					if(id==-1) {
						var.setErstelltAm(aenderungsDatum);
						var.setErstelltVon(_session.getCurrentUser());
						StaticLocators.getVariables().addEntity(var);
					}else {
						StaticLocators.getVariables().updateEntity(var, true);
					}
				}
				
				try {
					DmsWebApplication.getApplication().reinitConfig();
				}catch(Exception e) {
					
					/*infobox.refresh(target, ResourceText.getText("infConfigurationSavedIndexExists", this), INFOBOX_STATUS.warning);
					infobox.setVisible(true);
					target.add(wmcInfo);
					withMessage = false;
					*/
					throw e;
				}
			
				DmsException dmsInfo = new DmsException(2000, "I", 
					_className, 
					"save",
					ResourceText.getText("dmsInfoConfigurationChanged", this),
					null,
					_configVariables
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_toolBar.setViewState(ToolbarState.STATE_CHANGE);
				refreshFormObjects();
			
				if(withMessage) {
					infobox.refresh(target, ResourceText.getText("infConfigurationSaved", this), INFOBOX_STATUS.success);
					infobox.setVisible(true);
					target.add(wmcInfo);
					target.add(_lblTitel);
				}
				return true;
			}
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errConfigurationSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		}
		return false;
		
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_binaryFolder.setEnabled(editable);
			_dataFolder.setEnabled(editable);
			_executableName.setEnabled(editable);
			_language.setEnabled(editable);
			_oem.setEnabled(editable);
			_psm.setEnabled(editable);
			_pdftotiffpdi.setEnabled(editable);
		}catch(Exception e){
		}
	}
	
	
	private void refreshFormObjects(){
		_binaryFolder.setDefaultModel(new PropertyModel<String>(_configVariables, "BinaryFolder"));
		_dataFolder.setDefaultModel(new PropertyModel<String>(_configVariables, "DataFolder"));
		_executableName.setDefaultModel(new PropertyModel<String>(_configVariables, "ExecutableName"));
		_pdftotiffpdi.setDefaultModel(new PropertyModel<Integer>(_configVariables, "PDFToTiffDPI"));
		
		if(null!=_configVariables.getLanguage()) {
			_languageType = TesseractLanguageType.valueOf(_configVariables.getLanguage());
		}
		_language.setDefaultModel(new PropertyModel<TesseractLanguageType>(this, "_languageType"));
		
		if(-1!=_configVariables.getOem()) {
			_oemType = TesseractOemType.getTesseractOemTypeByCode(_configVariables.getOem());
		}
		_oem.setDefaultModel(new PropertyModel<TesseractOemType>(this, "_oemType"));
		
		if(-1!=_configVariables.getPsm()) {
			_psmType = TesseractPsmType.getTesseractPsmTypeByCode(_configVariables.getPsm());
		}
		_psm.setDefaultModel(new PropertyModel<TesseractPsmType>(this, "_psmType"));
		
	}
	
	
	
 
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	
	private void readConfig() {
		try {
			_configVariables = new TesseractVariablesImpl();
			List<Variable> entities = StaticLocators.getVariables().getEntities(-1, -1, "", TesseractVariablesImpl.getDefaultVariableFilterClass());
			for(Variable entity:entities) {
				_configVariables.setVariable(entity);
				_configVariables_Backup.setVariable(entity);
			}
		}catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	 
	
	
	
}
