/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.documents.pdf.PdfViewerResponse;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentException;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.models.DocumentModel;
import org.biffo.dms.processQueue.tasks.DocumentDeleteTask;
import org.biffo.dms.processQueue.tasks.DocumentFullDeleteTask;
import org.biffo.dms.processQueue.tasks.DocumentMergeTask;
import org.biffo.dms.processQueue.tasks.DocumentRePdfTask;
import org.biffo.dms.processQueue.tasks.DocumentSplittTask;
import org.biffo.dms.processQueue.tasks.DocumentUnarchiveTask;
import org.biffo.dms.processQueue.tasks.DocumentUndeleteTask;
import org.biffo.dms.processQueue.tasks.DocumentWriteTask;
 
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.helpers.processing.ProcessingPanel;
import org.biffo.dms.web.modals.AbstractModalDeletePanel;
import org.biffo.dms.web.modals.AbstractModalFullDeletePanel;
import org.biffo.dms.web.modals.AbstractModalMergePanel;
import org.biffo.dms.web.modals.AbstractModalRePDFPanel;
import org.biffo.dms.web.modals.AbstractModalSplittPanel;
import org.biffo.dms.web.modals.AbstractModalSavePanel;
import org.biffo.dms.web.modals.AbstractModalUnarchivePanel;
import org.biffo.dms.web.modals.AbstractModalUndeletePanel;

public class DocumentsProcessingDialogs extends Panel{

	
	private static final String _className= "DocumentsProcessingDialogs";
	private static final long serialVersionUID = 1L;
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private ProcessingPanel processing;
	private CustomModalWindow modalDelete = new CustomModalWindow("modalDelete");
	private CustomModalWindow modalMerge = new CustomModalWindow("modalMerge");
	private CustomModalWindow modalUndelete = new CustomModalWindow("modalUndelete");
	private CustomModalWindow modalFullDelete = new CustomModalWindow("modalFulldelete");
	private CustomModalWindow modalSplitt = new CustomModalWindow("modalSplitt");
	
	private CustomModalWindow modalSave = new CustomModalWindow("modalSave");
	private CustomModalWindow modalUnarchive = new CustomModalWindow("modalUnarchive");
	
	private CustomModalWindow modalRepdf = new CustomModalWindow("modalRepdf");

	private DocumentModel _documentModel;
	private PdfViewerResponse _viewerResponse;
	private int[] _selectedTagIds;
	private int[] _selectedUsers; 
	private int[] _selectedUserGroups;
	
	private HashSet<Integer> _selectionList = new HashSet<Integer>();
	
	public DocumentsProcessingDialogs(String id) {
		super(id);
		addProcessingPanel();
		addModalDelete();
		addModalUndelete();
		addModalMerge();
		addModalFullDelete();
		addModalSplitt();
		
		addModalUnarchive();
		addModalSave();
		addModalRepdf();
 		
	}
	
	private void addProcessingPanel() {
		
		processing = new ProcessingPanel("modalprocess") {
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onFinished(AjaxRequestTarget target, ProcessType processType, int sourceDocumentId, int targetDocumentId, int processId) {
				String infoMessage="";
				Document documentTarget=null;
				if(targetDocumentId>0) {
					try {
						documentTarget = StaticLocators.getDocuments().getEntityById(targetDocumentId);
					} catch (DocumentException e) {
						DmsException dmsException = new DmsException(1000, "E", _className, "onFinished", null, e);
						StaticLocators.getMessageHandler().createMessage(dmsException, _session);
					}
				}
				
				
				switch (processType){
					case DELETE:
						infoMessage = ResourceText.getText("infoDocumentDelete", DocumentsProcessingDialogs.this);
						break;
					
					case MERGE: 
						infoMessage = ResourceText.getText("infoDocumentGrouped", DocumentsProcessingDialogs.this);
						break;
					
					case UNDELETE:
						infoMessage = ResourceText.getText("infoDocumentRestored", DocumentsProcessingDialogs.this);
						break;
					
					case FULLDELETE:
						infoMessage = ResourceText.getText("infoDocumentFulldelete", DocumentsProcessingDialogs.this);
						break;
					
					case OVERWRITE: 
						infoMessage = ResourceText.getText("infoDocumentSaved", DocumentsProcessingDialogs.this);
						break;
					
					case VERSIONING: 
						infoMessage = ResourceText.getText("infoDocumentVersion", DocumentsProcessingDialogs.this);
						break;
					
					case UNARCHIVE:
						if(null!=documentTarget) {
							infoMessage = ResourceText.getText("infoDocumentUnarchived", DocumentsProcessingDialogs.this, documentTarget.getDocName(), documentTarget.getVersion());
						}else {
							infoMessage = ResourceText.getText("infoDocumentUnarchived", DocumentsProcessingDialogs.this);
						}
						break;
					case SPLITT: infoMessage = ResourceText.getText("infoDocumentSplitted", DocumentsProcessingDialogs.this); 
						break;
					case REPDF:
						infoMessage = ResourceText.getText("infoDocumentRepdf", DocumentsProcessingDialogs.this); 
						break;
					case APPLY_MODULE:
						infoMessage = ResourceText.getText("infoDocumentModule", DocumentsProcessingDialogs.this);
						break;
					
				default:
					break;
				}

				_selectionList = new HashSet<Integer>();
				OnProcessFinished(target, processType, infoMessage, sourceDocumentId, targetDocumentId);
			}
			@Override
			protected void onError(AjaxRequestTarget target, ProcessType processType, int processId , Exception e) {
				
				String infoMessage="";
				switch (processType){
					case FULLDELETE:
					case DELETE: 
						infoMessage = ResourceText.getText("errDocumentDelete", DocumentsProcessingDialogs.this)+"\n"+e.getMessage();
						break;
					
					case MERGE: 
						infoMessage = ResourceText.getText("errDocumentGrouped", DocumentsProcessingDialogs.this)+"\n"+e.getMessage();
						break;
					
					case UNDELETE:
						infoMessage = ResourceText.getText("errDocumentRestored", DocumentsProcessingDialogs.this)+"\n"+e.getMessage();
						break;
					
					case OVERWRITE: 
						infoMessage = ResourceText.getText("errDocumentSaved", DocumentsProcessingDialogs.this)+"\n"+e.getMessage();
						break;
					
					case VERSIONING: 
						infoMessage = ResourceText.getText("errDocumentVersion", DocumentsProcessingDialogs.this)+"\n"+e.getMessage();
						break;
					
					case UNARCHIVE: 
						infoMessage = ResourceText.getText("errDocumentUnarchived", DocumentsProcessingDialogs.this)+"\n"+e.getMessage(); 
						break;
					
					case SPLITT: 
						infoMessage = ResourceText.getText("errDocumentSplitted", DocumentsProcessingDialogs.this)+"\n"+e.getMessage(); 
						break;
					case REPDF:
						infoMessage = ResourceText.getText("errDocumentRepdf", DocumentsProcessingDialogs.this)+"\n"+e.getMessage(); 
						break;
					case APPLY_MODULE:
						infoMessage = ResourceText.getText("errDocumentModule", DocumentsProcessingDialogs.this)+"\n"+e.getMessage(); 
						break;
					
				default:
					break;
				}
				_selectionList = new HashSet<Integer>();
				OnProcessError(target, processType, infoMessage);
			}
		};
		processing.setOutputMarkupId(true);
		add(processing);
	}
	
	private void addModalRepdf() {

		modalRepdf.setAutoSize(false);
		modalRepdf.setHeightUnit("px");
		modalRepdf.setInitialHeight(75);
		modalRepdf.setInitialWidth(600);
		modalRepdf.setResizable(false);
		modalRepdf.showUnloadConfirmation(false);
		modalRepdf.setContent(new AbstractModalRePDFPanel(modalRepdf){

    		private static final long serialVersionUID = 1L;
    		
    		private Document document;
    		
			@Override
			public void onRePDF(AjaxRequestTarget target) {
				modalRepdf.close(target);
				try {
					processing.start(target, new DocumentRePdfTask(document, _selectedTagIds, _selectedUsers, _selectedUserGroups));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onRePDF", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalRepdf.close(target);
			}
			
			@Override
			protected void onBeforeRender() {
				document = _documentModel.getObject();
				super.onBeforeRender();
			}
    	});
    	add(modalRepdf);
	}
	
	

	private void addModalDelete() {

		modalDelete.setAutoSize(false);
		modalDelete.setHeightUnit("px");
		modalDelete.setInitialHeight(75);
		modalDelete.setInitialWidth(600);
		modalDelete.setResizable(false);
		modalDelete.showUnloadConfirmation(false);
		modalDelete.setContent(new AbstractModalDeletePanel(modalDelete){

    		private static final long serialVersionUID = 1L;
    		
			@Override
			public void onDelete(AjaxRequestTarget target) {
				List<Document> lstDocuments = new LinkedList<Document>();
				try {
					for(Integer entityToDelete : _selectionList) {
						lstDocuments.add(StaticLocators.getDocuments().getEntityById(entityToDelete));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalDelete.onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				modalDelete.close(target);
				try {
					if(lstDocuments.size()>0) {
						processing.start(target, new DocumentDeleteTask(lstDocuments));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalDelete.onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDelete.close(target);
			}
    	});
    	add(modalDelete);
	}
	
	private void addModalMerge() {

		modalMerge.setAutoSize(false);
		modalMerge.setHeightUnit("px");
		modalMerge.setInitialHeight(75);
		modalMerge.setInitialWidth(600);
		modalMerge.setResizable(false);
		modalMerge.showUnloadConfirmation(false);
		modalMerge.setContent(new AbstractModalMergePanel(modalMerge){

    		private static final long serialVersionUID = 1L;
    		
			@Override
			public void onMerge(AjaxRequestTarget target) {
				List<Document> lstDocuments = new LinkedList<Document>();
				try {
					for(Integer entityToMerge : _selectionList) {
						lstDocuments.add(StaticLocators.getDocuments().getEntityById(entityToMerge));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalMerge.onMerge", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				modalMerge.close(target);
				try {
					if(lstDocuments.size()>1) processing.start(target, new DocumentMergeTask(lstDocuments));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalMerge.onMerge", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalMerge.close(target);
			}
    	});
    	add(modalMerge);
	}
	
	
	private void addModalUndelete() {
		modalUndelete.setAutoSize(false);
		modalUndelete.setHeightUnit("px");
		modalUndelete.setInitialHeight(75);
		modalUndelete.setInitialWidth(600);
		modalUndelete.setResizable(false);
		modalUndelete.showUnloadConfirmation(false);
		modalUndelete.setContent(new AbstractModalUndeletePanel(modalUndelete){

    		private static final long serialVersionUID = 1L;
    		
			@Override
			public void onUndelete(AjaxRequestTarget target) {
				
				List<Document> lstDocuments = new LinkedList<Document>();
				try {
					for(Integer entityToRestore : _selectionList) {
						lstDocuments.add(StaticLocators.getDocuments().getEntityById(entityToRestore));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalUndelete.onUndelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				modalUndelete.close(target);
				try {
					if(lstDocuments.size()>0) processing.start(target, new DocumentUndeleteTask(lstDocuments));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalUndelete.onUndelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalUndelete.close(target);
			}
    	});
    	add(modalUndelete);
	}
	
	
	private void addModalFullDelete() {

		modalFullDelete.setAutoSize(false);
		modalFullDelete.setHeightUnit("px");
		modalFullDelete.setInitialHeight(75);
		modalFullDelete.setInitialWidth(600);
		modalFullDelete.setResizable(false);
		modalFullDelete.showUnloadConfirmation(false);
		modalFullDelete.setContent(new AbstractModalFullDeletePanel(modalFullDelete){

    		private static final long serialVersionUID = 1L;
    		
			@Override
			public void onDelete(AjaxRequestTarget target) {
				List<Document> lstDocuments = new LinkedList<Document>();
				try {
					for(Integer entityToDelete : _selectionList) {
						lstDocuments.add(StaticLocators.getDocuments().getEntityById(entityToDelete));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalFullDelete.onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				modalFullDelete.close(target);
				try {
					if(lstDocuments.size()>0) processing.start(target, new DocumentFullDeleteTask(lstDocuments));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalFullDelete.onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalFullDelete.close(target);
				 
			}
    	});
    	add(modalFullDelete);
	}
	
	private void addModalSave() {

		modalSave.setAutoSize(false);
    	modalSave.setHeightUnit("px");
    	modalSave.setInitialHeight(75);
    	modalSave.setInitialWidth(600);
    	modalSave.setResizable(false);
    	modalSave.showUnloadConfirmation(false);
    	modalSave.setContent(new AbstractModalSavePanel(modalSave){

    		private static final long serialVersionUID = 1L;

    		private Document document;
    		
			@Override
			public void onOverwrite(AjaxRequestTarget target) {
				modalSave.close(target);
				try {
					processing.start(target, new DocumentWriteTask(document, _viewerResponse, _selectedTagIds, _selectedUsers, _selectedUserGroups, true));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalSave.onOverwrite", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
    		
			@Override
			public void onVersioning(AjaxRequestTarget target) {
				modalSave.close(target);
				try {
					processing.start(target, new DocumentWriteTask(document, _viewerResponse, _selectedTagIds, _selectedUsers, _selectedUserGroups, false));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalSave.onVersioning", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
    			modalSave.close(target);
			}
			
			@Override
			protected void onBeforeRender() {
				document = _documentModel.getObject();
				super.onBeforeRender();
			}

    	});
    	add(modalSave);
	}
	
	private void addModalUnarchive() {

		modalUnarchive.setAutoSize(false);
		modalUnarchive.setHeightUnit("px");
		modalUnarchive.setInitialHeight(75);
		modalUnarchive.setInitialWidth(700);
		modalUnarchive.setResizable(false);
		modalUnarchive.showUnloadConfirmation(false);
		modalUnarchive.setContent(new AbstractModalUnarchivePanel(modalUnarchive){

    		private static final long serialVersionUID = 1L;
    		
    		@Override
    		public void onUnarchive(AjaxRequestTarget target) {
    			modalUnarchive.close(target);
				try {
					processing.start(target, new DocumentUnarchiveTask(_documentModel.getObject()));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalUnarchive.onUnarchive", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalUnarchive.close(target);
			}
			
			@Override
			protected void onBeforeRender() {
				super.onBeforeRender();
			}

    	});
    	add(modalUnarchive);
	}
	
	
	private void addModalSplitt() {
		modalSplitt.setAutoSize(false);
		modalSplitt.setHeightUnit("px");
		modalSplitt.setInitialHeight(80);
		modalSplitt.setInitialWidth(650);
		modalSplitt.setResizable(false);
		modalSplitt.setContent(new AbstractModalSplittPanel(modalSplitt){

    		private static final long serialVersionUID = 1L;
    		
    		@Override
    		public void onSplittSingle(AjaxRequestTarget target) {
    			modalSplitt.close(target);
				try {
					//processing.start(target, new DocumentSplittTask(_documentModel.getObject(), new Document(), _viewerResponse, false));
					processing.start(target, new DocumentSplittTask(_documentModel.getObject(), _viewerResponse, false));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalSplitt.onSplittSingle", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
    		}
			@Override
			public void onSplittMulti(AjaxRequestTarget target) {
    			modalSplitt.close(target);
				try {
					//processing.start(target, new DocumentSplittTask(_documentModel.getObject(), new Document(), _viewerResponse, true));
					processing.start(target, new DocumentSplittTask(_documentModel.getObject(), _viewerResponse, true));
				} catch (Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "addModalSplitt.onSplittMulti", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}

			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalSplitt.close(target);
			}
    	});
    	add(modalSplitt);
	}
	
 
	public void processRepdf(AjaxRequestTarget target, DocumentModel documentModel, int[] selectedTagIds, int[] selectedUsers, int[] selectedUserGroups) {
		_selectionList.clear();
		_documentModel = documentModel;
		_selectedTagIds = selectedTagIds;
		_selectedUsers = selectedUsers; 
		_selectedUserGroups = selectedUserGroups;
		modalRepdf.show(target);
	}
	
	
	public void processDelete(AjaxRequestTarget target, HashSet<Integer> selectionList) {
		_selectionList.clear();
		_selectionList.addAll(selectionList);
		modalDelete.show(target);
	}
	
	public void processUndelete(AjaxRequestTarget target, HashSet<Integer> selectionList) {
		_selectionList.clear();
		_selectionList.addAll(selectionList);
		modalUndelete.show(target);
	}

	public void processMerge(AjaxRequestTarget target, HashSet<Integer> selectionList) {
		_selectionList.clear();
		_selectionList.addAll(selectionList);
		modalMerge.show(target);
	}

	public void processFullDelete(AjaxRequestTarget target, HashSet<Integer> selectionList) {
		_selectionList.clear();
		_selectionList.addAll(selectionList);
		modalFullDelete.show(target);
	}
	
	public void processUnarchive(AjaxRequestTarget target, DocumentModel documentModel) {
		_selectionList.clear();
		_documentModel = documentModel;
		modalUnarchive.show(target);
	}
	
	public void processSave(AjaxRequestTarget target, DocumentModel documentModel, PdfViewerResponse viewerResponse, int[] selectedTagIds, int[] selectedUsers, int[] selectedUserGroups) {
		_selectionList.clear();
		_documentModel = documentModel;
		_viewerResponse = viewerResponse;
		_selectedTagIds = selectedTagIds;
		_selectedUsers = selectedUsers; 
		_selectedUserGroups = selectedUserGroups;
		modalSave.show(target);
	}
	
	public void processSplitt(AjaxRequestTarget target, DocumentModel documentModel, PdfViewerResponse viewerResponse) {
		_selectionList.clear();
		_documentModel = documentModel;
		_viewerResponse = viewerResponse;
		modalSplitt.show(target);
	}
	
	
	protected void OnProcessFinished(AjaxRequestTarget target, ProcessType processType, String message, int sourceDocumentId, int targetDocumentId){}
	protected void OnProcessError(AjaxRequestTarget target, ProcessType processType, String message){}
	

}
