/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.small;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.EnumSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.RepeatingView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.flow.RedirectToUrlException;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;

 
import de.kleinekuh.components.wicketdnd.DropTarget;
import de.kleinekuh.components.wicketdnd.IEBackgroundImageCacheFix;
import de.kleinekuh.components.wicketdnd.IECursorFix;
import de.kleinekuh.components.wicketdnd.Location;
import de.kleinekuh.components.wicketdnd.Operation;
import de.kleinekuh.components.wicketdnd.Reject;
import de.kleinekuh.components.wicketdnd.Transfer;
import de.kleinekuh.components.wicketdnd.theme.WindowsTheme;

import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.processQueue.tasks.ProcessHelpers;
import org.biffo.dms.resource.DmsCrypt;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;


public class DocumentSmallPanel extends Panel {
	
	
	private static final String _className = "DocumentSmallPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "documentSmallToolbar";
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	private final Document document;
	private final int index;
	
	private ApplicationPage _appPage = DmsWebApplicationPages.ApplicationPage.DOCUMENTS;
	private WebMarkupContainer wmc, wmcDvImage;
	
	private SimpleDateFormat _sdf = new SimpleDateFormat(_session.getLocaleParams().getDateFormat());
	private Toolbar toolBar;
	private boolean selected=false;
	private ToolbarButton _btView, _btSelect, _btUnselect, _btRestore, _btDelete, _btPDF;
	private RepeatingView rvTags, rvUsers;
	private List<Tag> _taglist = new LinkedList<Tag>();
	private List<User> _userList = new LinkedList<User>();
	private AjaxLink<String> _ajlDvdocnamelink;
	
	private boolean _deleted=false;
	private transient Document _documentSearch;
	
	private boolean showTags=false;
	private boolean showUsers=false;

	private String _docName;
	
	
	@Override
	protected void onBeforeRender() {
		
		
		
		rvTags.removeAll();
		if(showTags) {
			
			refreshTagList();
			for(Tag tag : _taglist) {
				WebMarkupContainer wmc = new WebMarkupContainer(rvTags.newChildId());
				wmc.setOutputMarkupId(true);
				wmc.add(new AttributeModifier("style", "border: 0px solid #"+tag.getTagBgColor()+"; background-color: #"+tag.getTagBgColor()+";color: #"+tag.getTagFgColor()+""));
				Label lbl = new Label("tagvalue", tag.getTagValue());
				lbl.setOutputMarkupId(true);
				wmc.add(lbl);
				rvTags.add(wmc);
			}
		}
		rvUsers.removeAll();
		if(showUsers) {
			
			refreshUsersList();
			for(User user : _userList) {
				WebMarkupContainer wmc = new WebMarkupContainer(rvUsers.newChildId());
				wmc.setOutputMarkupId(true);
				Label lbl = new Label("uservalue", user.getBenutzerName());
				lbl.setOutputMarkupId(true);
				wmc.add(lbl);
				rvUsers.add(wmc);
			}
		}
		super.onBeforeRender();
	}
	
	
	public void setDocname() {
		_docName = ProcessHelpers.getDocumentTitle(document);
		if(_documentSearch!=null && _documentSearch.getDocName()!=null && !_documentSearch.getDocName().isEmpty()) {
			_docName = _documentSearch.getDocName();
		}
	}
	public String getDocname() {
		setDocname();
		return _docName;
	}
	
	
	
	public DocumentSmallPanel(final String id, final IModel<Document> model, final int index) {
		super(id, model);

		this.document = model.getObject();
		this.index = index;
		setOutputMarkupId(true);
		
		wmc = new WebMarkupContainer("wmc");
		wmc.setOutputMarkupId(true);
		add(wmc);
		
		
		_ajlDvdocnamelink = new AjaxLink<String>("dvdocnamelink") {
			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(AjaxRequestTarget target) {
				target.appendJavaScript("dslpDblClick(event,'"+document.getId()+"');");
			}
		};
		_ajlDvdocnamelink.setOutputMarkupId(true);
		wmc.add(_ajlDvdocnamelink);
		
		Label lblDocname = new Label("dvdocname", new PropertyModel<String>(DocumentSmallPanel.this, "Docname"));
		lblDocname.setEscapeModelStrings(false);
		lblDocname.setOutputMarkupId(true);
		_ajlDvdocnamelink.add(lblDocname);
		
		String tempSeitenanzahl = ResourceText.getText("lblOnePage", this);
		if(document.getNumberOfPages()>1) {
			tempSeitenanzahl = document.getNumberOfPages()+" "+ResourceText.getText("lblPages", this);
		}
		Label lblSeitenanzahl = new Label("seitenanzahl", tempSeitenanzahl);
		wmc.add(lblSeitenanzahl);

		String tempDocumentDate="NA";
		try {
			tempDocumentDate = _sdf.format(document.getDocumentDatum());
		}catch(Exception e) {
		}
		Label lblDocumentdate = new Label("documentdate", tempDocumentDate);
		wmc.add(lblDocumentdate);

		Label lblVersion = new Label("version", document.getVersion());
		wmc.add(lblVersion);
		
		
		wmcDvImage = new WebMarkupContainer("dvimage");
		wmcDvImage.setOutputMarkupId(true);
		String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(),document);
		wmcDvImage.add(AttributeModifier.replace("style", "background-image: url('../png/" + encodedResource+"')"));
		
		
		rvTags = new RepeatingView("tag");
		rvTags.setOutputMarkupId(true);
		wmcDvImage.add(rvTags);
		
		rvUsers = new RepeatingView("user");
		rvUsers.setOutputMarkupId(true);
		wmcDvImage.add(rvUsers);

		wmc.add(wmcDvImage);
	    wmcDvImage.add(AttributeModifier.append("ondblclick", "dslpDblClick(event,'"+document.getId()+"');"));
	    wmc.add(AttributeModifier.append("onclick", "dslpClick(event,'"+document.getId()+"');"));
		
		
	    
	    if(document.isChangeable())addDNDDrop();
		if(document.hasObjectStatusTypes(DocumentStatusType.ARCHIVED)){
			_appPage = DmsWebApplicationPages.ApplicationPage.ARCHIVE;
		}else if(document.hasObjectStatusTypes(DocumentStatusType.DELETED)){
			_deleted = true;
			_appPage = DmsWebApplicationPages.ApplicationPage.DELETED;
		}
	    addToolBar();
	    setSelected(false);
		 
	}
	
	
	private void addDNDDrop() {

		add(new IECursorFix());
		add(new IEBackgroundImageCacheFix());
		wmcDvImage.add(new WindowsTheme());
		wmcDvImage.add(new DropTarget() {
			private static final long serialVersionUID = 1L;
			
			@Override
			public Set<Operation> getOperations(){
				return EnumSet.allOf(Operation.class);
			}

			@Override
			public String[] getTypes(){
				return new String[] { Transfer.ANY };
			}

			@Override
			public void onDrop(AjaxRequestTarget target, Transfer transfer, Location location) throws Reject {
				try {
					Object transferObject = transfer.getData();
					if(transferObject instanceof Tag) {
						Tag tag = transfer.getData();
						DmsDBHelpersTag.addTagToDocument(
							DocumentSmallPanel.this.document.getId(), 
							tag, 
							_session.getCurrentUser(),
							_session.getCurrentTenantId()
						);
						refreshTagList();
						OnTagDrop(tag, DocumentSmallPanel.this.document.getId(), target);
					}else if(transferObject instanceof User) {
						User user = transfer.getData();
						DmsDBHelpersUser.addUserToDocument(
							DocumentSmallPanel.this.document.getId(), 
							user.getId(),
							_session.getCurrentUser(),
							_session.getCurrentTenantId()
						);
						refreshUsersList();
						OnUserDrop(user, DocumentSmallPanel.this.document.getId(), target);
					}else if(transferObject instanceof ApiModuleImpl) {
						ApiModuleImpl apiModule =transfer.getData();
						OnModuleDrop(apiModule, DocumentSmallPanel.this.document.getId(), target);
					}
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "refreshTagList", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			};
		}.dropCenter(".dvsimage"));
	}
	
	
	
	private void addToolBar() {

		toolBar = new Toolbar("toolbar") {

			private static final long serialVersionUID = 1L;
			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		if(_deleted) {
			_btRestore = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
			_btRestore.setUtf8Code("xe760");
			_btRestore.setEventName("Restore");
			_btRestore.setCssClass(toolbarCSS);
			_btRestore.setAlternate(ResourceText.getText("tbRestore", this));
			toolBar.add(_btRestore);
		}
		
		_btView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btView.setUtf8Code("xe873");
		_btView.setEventName("View");
		_btView.setDefaultFormProcessing(false);
		_btView.setToolbarState(ToolbarState.STATE_VIEW);
		_btView.setCssClass(toolbarCSS);
		_btView.setOutputMarkupId(true);
		_btView.setAlternate(ResourceText.getText("tbView", this));
		toolBar.add(_btView);
 
		_btDelete  = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		if(_deleted) {
			_btDelete.setAlternate(ResourceText.getText("tbFullDelete", this));
			_btDelete.setUtf8Code("xe92b");
		}else {
			_btDelete.setAlternate(ResourceText.getText("tbDelete", this));
			_btDelete.setUtf8Code("xe872");
		}
		_btDelete.setEventName("Delete");
		_btDelete.setToolbarState(ToolbarState.STATE_VIEW);
		_btDelete.setDefaultFormProcessing(false);
		_btDelete.setCssClass(toolbarCSS);
		_btDelete.setOutputMarkupId(true);
		toolBar.add(_btDelete);
		
		
		_btSelect = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btSelect.setUtf8Code("xf1fe");
		_btSelect.setEventName("Select");
		_btSelect.setDefaultFormProcessing(false);
		_btSelect.setOutputMarkupId(true);
		_btSelect.setCssClass(toolbarCSS);
		_btSelect.setAlternate(ResourceText.getText("tbSelect", this));
		toolBar.add(_btSelect);
		
		_btUnselect = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btUnselect.setUtf8Code("xe909");
		_btUnselect.setEventName("Unselect");
		_btUnselect.setDefaultFormProcessing(false);
		_btUnselect.setOutputMarkupId(true);
		_btUnselect.setVisible(false);
		_btUnselect.setCssClass(toolbarCSS);
		_btUnselect.setAlternate(ResourceText.getText("tbUnselect", this));
		toolBar.add(_btUnselect);
	 

		_btPDF = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
		_btPDF.setUtf8Code("xf090");
		_btPDF.setEventName("PDF");
		_btPDF.setDefaultFormProcessing(false);
		_btPDF.setOutputMarkupId(true);
		_btPDF.setCssClass(toolbarCSS);
		_btPDF.setAlternate(ResourceText.getText("tbDownload", this));
		toolBar.add(_btPDF);
				
		toolBar.setOutputMarkupId(true);
		toolBar.setViewState(ToolbarState.STATE_VIEW);
		wmc.add(toolBar);
		
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Edit")) {
		}else if(event.equals("Delete")) {
			OnDeleteClick(document, target);
		}else if(event.equals("Select")) {
			setSelected(true);
			target.add(wmc);
			OnSelectionClick(document, target, true);
		}else if(event.equals("Unselect")) {
			setSelected(false);
			target.add(wmc);
			OnSelectionClick(document, target, false);
		}else if(event.equals("PDF")) {
			String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(), document);
            String url = "../pdf/"+encodedResource;
            throw new RedirectToUrlException(url);
		}else if(event.equals("View")) {
			DocumentsPage page = new DocumentsPage(_appPage, getPage().getPageReference(), Integer.valueOf(document.getId()));
			setResponsePage(page);
		}else if(event.equals("Restore")) {
			OnRestoreClick(document, target);
		}
	}
	
	

	
	
	
	private void refreshTagList() {
		
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(document.getId());
			rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			List<Tag> lstTags = new LinkedList<Tag>();
			for(Relation relation : alRelations) {
				lstTags.add(StaticLocators.getTags().getEntityById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstTags, new Comparator<Tag>() {
			    @Override
			    public int compare(Tag tag1, Tag tag2) {
			        return tag1.getTagName().compareToIgnoreCase(tag2.getTagName());
			    }
			});
			
			_taglist.clear();
			_taglist.addAll(lstTags);
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshTagList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void refreshUsersList() {
		
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation1(document.getId());
			rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			List<User> lstUsers = new LinkedList<User>();
			for(Relation relation : alRelations) {
				lstUsers.add(StaticLocators.getUsers().getEntityById(relation.getIdrelation2()));
			}
			
			Collections.sort(lstUsers, new Comparator<User>() {
			    @Override
			    public int compare(User user1, User user2) {
			        return user1.getBenutzerName().compareToIgnoreCase(user2.getBenutzerName());
			    }
			});
			
			_userList.clear();
			_userList.addAll(lstUsers);
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshUsersList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
 
	
	
	public boolean isSelected() {return selected;}
	public void setSelected(boolean selected) {
		this.selected = selected;
		if(selected) {
			wmc.add(new AttributeModifier("class", "dvsdocument active"));
			_btSelect.setVisible(false);
			_btUnselect.setVisible(true);
		}else {
			wmc.add(new AttributeModifier("class", "dvsdocument"));
			_btSelect.setVisible(true);
			_btUnselect.setVisible(false);
		}
 
	}
	
	public int getIndex() {return index;}

	protected void OnSelectionClick(Document document, AjaxRequestTarget target, boolean selected){}
	protected void OnDeleteClick(Document document, AjaxRequestTarget target){}
	protected void OnPDFClick(Document document, AjaxRequestTarget target){}
	protected void OnTagDrop(Tag tag, int documentId, AjaxRequestTarget target) {}
	protected void OnUserDrop(User user, int documentId, AjaxRequestTarget target) {}
	protected void OnModuleDrop(ApiModuleImpl apiModulImpl, int documentId, AjaxRequestTarget target) {}
	protected void OnRestoreClick(Document document, AjaxRequestTarget target){}

	public Document getDocumentSearch() {return _documentSearch;}
	public void setDocumentSearch(Document documentSearch) {_documentSearch = documentSearch;}

	public boolean isShowTags() {return showTags;}
	public void setShowTags(boolean showTags) {this.showTags = showTags;}
	
	public boolean isShowUsers() {return showUsers;}
	public void setShowUsers(boolean showUsers) {this.showUsers = showUsers;}
	
	 
	
	
	
	
}
