/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.timeline;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.json.JSONArray;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptReferenceHeaderItem;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.request.resource.JavaScriptResourceReference;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;

import org.biffo.dms.dataprovider.DPDocuments;
import org.biffo.dms.dataprovider.DPRelations;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.DmsDBHelpersUser;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.resource.DmsCrypt;
import org.biffo.dms.session.parameters.DocumentsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.components.search.searchbox.SearchboxPanel;
import org.biffo.dms.web.components.tags.TaggingPanel;
import org.biffo.dms.web.components.users.UsersSelectionPanel;
import org.biffo.dms.web.documents.DocumentStateType;
import org.biffo.dms.web.documents.DocumentsListAjaxBehavior;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.documents.DocumentsProcessingDialogs;
import org.biffo.dms.web.documents.DocumentsPage.Style;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.uploads.UploadPanelViewList;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
import de.kleinekuh.components.vistimeline.TimeLine;
import de.kleinekuh.components.vistimeline.TimeLineItem;
import de.kleinekuh.components.vistimeline.events.TimeLineEvent;
import de.kleinekuh.components.vistimeline.events.TimeLineEventResponse;
import de.kleinekuh.components.vistimeline.events.TimeLineEventType;
import de.kleinekuh.components.vistimeline.events.TimeLineEventWhatType;
import de.kleinekuh.components.vistimeline.options.types.TimeLineAxisOrientation;
 
import de.kleinekuh.components.wicketdnd.DropTarget;
import de.kleinekuh.components.wicketdnd.IEBackgroundImageCacheFix;
import de.kleinekuh.components.wicketdnd.IECursorFix;
import de.kleinekuh.components.wicketdnd.Location;
import de.kleinekuh.components.wicketdnd.Operation;
import de.kleinekuh.components.wicketdnd.Reject;
import de.kleinekuh.components.wicketdnd.Transfer;
import de.kleinekuh.components.wicketdnd.theme.WindowsTheme;



public class DocumentsTimeLineListPanel extends Panel{

	
	private static final JavaScriptResourceReference DMSTIMELINE_JS = new JavaScriptResourceReference(DocumentsTimeLineListPanel.class, "dmstimeline.js");
	
	private static final String _className = "DocumentsTimeLineListPanel";
	private static final long serialVersionUID = 1L;
	private static final ApplicationPage _appPage = DmsWebApplicationPages.ApplicationPage.DOCUMENTS;

	private static final Calendar _calendar = Calendar.getInstance();
	
	private static final String toolbarCSS = "viewListToolbar";
	
	private DPDocuments _dpDocuments = new DPDocuments();
	private DPRelations _dpRelationsTags = new DPRelations();
	private DPRelations _dpRelationsUsers = new DPRelations();
	
	private DocumentOSFilterClass _dsfc = new DocumentOSFilterClass();
	private DocumentFilterClass _dfcDocuments = new DocumentFilterClass();

	private WebMarkupContainer _wmcDocuments, _wmcNoData;
	private HashSet<Integer> _selectionList = new HashSet<Integer>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btUSERs, _btRefresh, _btUPLOAD, _btSEARCH, _btTAGs, _btSmallListView, _btNormalListView, _btMerge, _btDelete;

	private DmsWebSession _session =  DmsWebSession.getSession();
	

	private DocumentsListParameters _thisParameters;
	
	private WebMarkupContainer _wmcLeft;
	private WebMarkupContainer wmcInfo;
	
	private InfoboxPanel infobox;
	private Label _lblNoData;
	
	private DmsPagingNavigator _dpn;
	
	private Date documentStart=null;
	private Date documentEnd=null;	 
	
	private TimeLine timeLine;
	private final ArrayList<TimeLineItem> timelineItems = new ArrayList<TimeLineItem>();

	private UploadPanelViewList _upvl;
	private boolean _uploadExists=true;
	private DocumentsProcessingDialogs _dpd;
	private SearchboxPanel _sbbp;
	
	private TaggingPanel  _tsbp;
	private UsersSelectionPanel _usbp;
	
	
	@Override
	public void renderHead(IHeaderResponse response) {
		super.renderHead(response);
		response.render(JavaScriptReferenceHeaderItem.forReference(DMSTIMELINE_JS));
	}
	
 	
	public DocumentsTimeLineListPanel(String id) {
		super(id);
		
		setVersioned(false);
		initParameters();
		addInfobox();
		addNoData();
		addProcessingDialogs();
 
		
		_wmcDocuments = new WebMarkupContainer("wmcdocuments") {

			private static final long serialVersionUID = 1L;
			@Override
			protected void onBeforeRender() {
				refreshTimeLineItems(null);
				StringBuilder sb = new StringBuilder();
				sb.append("innercontent ");
				boolean visible=false;
				if(_thisParameters.isSearchVisible()) {
					_sbbp.setVisible(true);
					visible=true;
				}else {
					_sbbp.setVisible(false);
				}
				if(_thisParameters.isTagsVisible()) {
					_tsbp.setVisible(true);
					visible=true;
				}else {
					_tsbp.setVisible(false);
				}
				if(_thisParameters.isUploadVisible()) {
					if(_upvl.getNumberOfImporters()>0) {
						_upvl.setVisible(true);
						visible=true;
					}
				}else {
					_upvl.setVisible(false);
				}
				if(_thisParameters.isUsersVisible()) {
					_usbp.setVisible(true);
					visible=true;
				}else {
					_usbp.setVisible(false);
				}
				
				if(visible)sb.append("innercontent_tags_active ");
				super.onBeforeRender();
				_wmcDocuments.add(AttributeModifier.replace("class",sb.toString().trim()));
			}
			
			
		};
		_wmcDocuments.setOutputMarkupId(true);
		add(_wmcDocuments);
		
		addTimeLine();
		addToolBar();
		
		_wmcLeft = new WebMarkupContainer("left");
		_wmcLeft.setOutputMarkupId(true);
		add(_wmcLeft);

		addSearchPanel();
		addUploadPanel();
		addTaggingPanel();
		addUsersPanel();
		addDNDDrop();
		addBehavior();
		
		if(_sbbp.getCondition()!=null) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
		}
		
		_dpn = new DmsPagingNavigator("navigation", null, false, false, true, true) {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnShowTagsChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowTags(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			@Override
			protected void OnShowUsersChanged(AjaxRequestTarget target, boolean enabled) {
				_thisParameters.setShowUsers(enabled);
				storeParameters();
				target.add(_wmcDocuments);
				target.add(_dpn);
			}
			
			
		};
		_dpn.setOutputMarkupId(true);
		_dpn.setShowTags(_thisParameters.isShowTags());
		_dpn.setShowUsers(_thisParameters.isShowUsers());
		
		
		add(_dpn);
		
		if(_dpDocuments.size()==0) {
			checkNodata(null);
		}
	}

	private void addSearchPanel() {
		
		_sbbp = new SearchboxPanel("searchbox", _dsfc) {
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnButtonClear(AjaxRequestTarget target) {
				_dfcDocuments.setLinkedWhere(null);
				_dpDocuments.setFilterState(_dfcDocuments);
				_dpDocuments.size();
				_thisParameters.setFilterObject(_dsfc);
				storeParameters();
				target.add(_wmcDocuments);
			};
			@Override
			protected void OnButtonSearch(AjaxRequestTarget target) {
				super.OnButtonSearch(target);
				String condition = getCondition();
				_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
				_dpDocuments.setFilterState(_dfcDocuments);
				_dpDocuments.size();
				
				_thisParameters.setFilterObject(_dsfc);
				storeParameters();
				target.add(_wmcDocuments);
			}
			
			@Override
			protected void OnQueryChanged(AjaxRequestTarget target) {
				String condition = getCondition();
				_dfcDocuments.setLinkedWhere(" id in ("+condition+")");
				_dpDocuments.setFilterState(_dfcDocuments);
				_dpDocuments.size();
				_thisParameters.setFilterObject(_dsfc);
				storeParameters();
				target.add(_wmcDocuments);
				
			}
			
			@Override
			protected void OnSearchNoData(AjaxRequestTarget target) {
				super.OnSearchNoData(target);
				infobox.refresh(target, ResourceText.getText("lblNoSearchResults", this), INFOBOX_STATUS.warning);
				infobox.setVisible(true);
				target.add(wmcInfo);
			}
			
		};
		_sbbp.setOutputMarkupId(true);
		_wmcLeft.add(_sbbp);
	}
	
	private void addUploadPanel() {
		_upvl = new UploadPanelViewList("uploadpanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void OnUploadPanelViewListFinished(AjaxRequestTarget target, int processId, int  uploads, int dublicates) {

				if(dublicates>0) {
					infobox.refresh(target, ResourceText.getText("warUploadDublicate", this, uploads, dublicates), INFOBOX_STATUS.warning);
				}else{
					infobox.refresh(target, ResourceText.getText("infUploadOK", this, uploads), INFOBOX_STATUS.success);
				};
				
				infobox.setVisible(true);

				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_dpn);
				target.add(_wmcDocuments);
				
			}
		};
		_upvl.setOutputMarkupId(true);
		_wmcLeft.add(_upvl);
		if(_upvl.getNumberOfImporters()==0) {
			_upvl.setVisible(false);
		}
		
	}
	
	
	private void addTimeLine() {
		timeLine = new TimeLine("timeline") {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnTimeLineEvent(TimeLineEventType event, TimeLineEventResponse eventResponse) {
				super.OnTimeLineEvent(event, eventResponse);
				if(event==TimeLineEventType.doubleClick) {
				}else if(event==TimeLineEventType.select) {
					_selectionList.clear();
					for(String item : eventResponse.getItems()) {
						_selectionList.add(Integer.valueOf(item));
					}
				}
			}
		};
		timeLine.setOutputMarkupId(true);
		timeLine.getOptions().setMinHeight(800);
		timeLine.getOptions().getMargin().setAxis(5);
		timeLine.getOptions().setMultiselect(true);
		timeLine.getOptions().setStack(true);
		timeLine.getOptions().setZoomMin(864000000);
		timeLine.getOptions().setVerticalScroll(false);
		timeLine.getOptions().setOrientation(TimeLineAxisOrientation.top);
		timeLine.addTimeLineEvent(new TimeLineEvent(TimeLineEventType.select, TimeLineEventWhatType.ITEM));
		timeLine.getOptions().setTemplate("renderPageTemplate");
		timeLine.getOptions().setCluster(Boolean.FALSE);

		
		TimeLineEvent doubleClickEvent = new TimeLineEvent(TimeLineEventType.doubleClick, TimeLineEventWhatType.ITEM);
		doubleClickEvent.setOwnFunction(true);
		doubleClickEvent.setFunction("function (properties) {dslpDblClick('dblclick',properties.item);}");
		timeLine.addTimeLineEvent(doubleClickEvent);
		
		_wmcDocuments.add(timeLine);
	}
	
	private void addDNDDrop() {

		add(new IECursorFix());
		add(new IEBackgroundImageCacheFix());
		_wmcDocuments.add(new WindowsTheme());
		_wmcDocuments.add(new DropTarget() {
			private static final long serialVersionUID = 1L;
			
			@Override
			public Set<Operation> getOperations(){
				return EnumSet.allOf(Operation.class);
			}

			@Override
			public String[] getTypes(){
				return new String[] { Transfer.ANY };
			}

			@Override
			public void onDrop(AjaxRequestTarget target, Transfer transfer, Location location) throws Reject {
				try {
					Object transferObject = transfer.getData();
					if(location.getComponent().getDefaultModelObject()==null) return;
					String docId = location.getComponent().getDefaultModelObject().toString();
					boolean added=false;
					if(transferObject instanceof Tag) {
						Tag tag = transfer.getData();
						docId = docId.replace("docid_", "");
						DmsDBHelpersTag.addTagToDocument(
								Integer.valueOf(docId), 
								tag, 
								_session.getCurrentUser(),
								_session.getCurrentTenantId()
						);
						_dpRelationsTags.size();
						added=true;
					}else if(transferObject instanceof User) {
						User user = transfer.getData();
						docId = docId.replace("docid_", "");
						DmsDBHelpersUser.addUserToDocument(
								Integer.valueOf(docId), 
								user.getId(),
								_session.getCurrentUser(),
								_session.getCurrentTenantId()
							);
						_dpRelationsUsers.size();
						added=true;
					}
					if(added) {
						TimeLineItem updateItem = buildTimeLineItem(StaticLocators.getDocuments().getEntityById(Integer.valueOf(docId)));
						StringBuilder sb = new StringBuilder();
						sb.append("var dataSet = i"+timeLine.getComponentId()+";");
						sb.append("dataSet.update("+updateItem+");");
						target.appendJavaScript(sb.toString());
						target.add(_wmcDocuments);
						if(_sbbp.isVisible())_sbbp.refresh(target);
					}
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDrop", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			};
		}.dropCenter(".dnd-drop-target"));
	}
	
	private void addTaggingPanel() {
		_tsbp = new TaggingPanel("tags") {
			
			private static final long serialVersionUID = 1L;

			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onAdd(target, taglist);
				try {
					
					if(_selectionList.size()==0) return;
					DmsDBHelpersTag.addTagsToDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					refreshTimeLineItemsFromSelectionList(target);
					_selectionList = new HashSet<Integer>();
					fselectAll(target, false);
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
					_sbbp.refresh(target);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onAdd", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onReplace(target, taglist);
				try {
					DmsDBHelpersTag.replaceTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					refreshTimeLineItemsFromSelectionList(target);
					_selectionList = new HashSet<Integer>();
					fselectAll(target, false);
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
					_sbbp.refresh(target);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onReplace", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}

			}

			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<Tag> taglist) {
				super.onDelete(target, taglist);
				try {
					DmsDBHelpersTag.deleteTagsFromDocuments(filterArchivedDocuments(_selectionList), taglist, _session.getCurrentUser(), _session.getCurrentTenantId());
					refreshTimeLineItemsFromSelectionList(target);
					_selectionList = new HashSet<Integer>();
					fselectAll(target, false);
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
					_sbbp.refresh(target);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
		};
		_tsbp.setOutputMarkupId(true);
		_wmcLeft.add(_tsbp);
	}
	
	private void addUsersPanel() {
		_usbp = new UsersSelectionPanel("users") {
 
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onAdd(AjaxRequestTarget target, HashSet<User> userslist) {
				super.onAdd(target, userslist);
				try {
					DmsDBHelpersUser.addUsersToDocuments(
							filterArchivedDocuments(_selectionList),
							userslist,
							_session.getCurrentUser(),
							_session.getCurrentTenantId(),
							false
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
			@Override
			protected void onReplace(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onReplace(target, userlist);
				try {
					DmsDBHelpersUser.replaceUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId(),
						true
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
				
				
			}
			
			@Override
			protected void onDelete(AjaxRequestTarget target, HashSet<User> userlist) {
				super.onDelete(target, userlist);
				try {
					DmsDBHelpersUser.deleteUsersFromDocuments(
						filterArchivedDocuments(_selectionList),
						userlist,
						_session.getCurrentUser(),
						_session.getCurrentTenantId()
					);
					_selectionList = new HashSet<Integer>();
					_dpDocuments.size();
					target.add(_wmcDocuments);
					target.add(_wmcLeft);
				}catch(Exception e) {
					DmsException dmsException = new DmsException(1000, "E", _className, "onDelete", null, e);
					StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				}
			}
			
		};
		_usbp.setOutputMarkupId(true);
		_wmcLeft.add(_usbp);
 
	}
	
	
	@Override
	protected void onBeforeRender() {
		super.onBeforeRender();
	}
	
	
	private void showLimitReached(AjaxRequestTarget target) {
		infobox.refresh(target, ResourceText.getText("lblLimitReached", this), INFOBOX_STATUS.warning);
		infobox.setPreMessage(true);
		infobox.setVisible(true);
	}
	
	private HashSet<Integer> filterArchivedDocuments(HashSet<Integer> selectionList){
		HashSet<Integer> selectionListFilter = new HashSet<Integer>();
		for(Integer id : selectionList) {
			try {
				Document document = StaticLocators.getDocuments().getEntityById(id);
				if(!document.hasObjectStatusTypes(DocumentStatusType.ARCHIVED)) {
					selectionListFilter.add(id);
				}
			}catch(Exception e) {
				DmsException dmsException = new DmsException(1000, "E", _className, "filterArchiveDocuments", null, e);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
		}
		return selectionListFilter;
	}
	
	
	private void refreshTimeLineItems(AjaxRequestTarget target) {
		timelineItems.clear();
		_dpRelationsTags.size();
		_dpRelationsUsers.size();
		
		long size = _dpDocuments.size();
		if(size>200) {
			size=200;
			showLimitReached(target);
		}
		
		Date startDateTemp=documentStart;
		Date endDateTemp=documentEnd;
		Iterator<? extends Document> itt = _dpDocuments.iterator(0, size);
		while(itt.hasNext()) {
			Document document = itt.next();
			TimeLineItem ti = buildTimeLineItem(document);
			timelineItems.add(ti);
			
			if(documentStart==null) documentStart= document.getDocumentDatum();
			else if(documentStart.after(document.getDocumentDatum()))documentStart=document.getDocumentDatum();
			
			if(documentEnd==null) documentEnd= document.getDocumentDatum();
			else if(documentEnd.before(document.getDocumentDatum()))documentEnd=document.getDocumentDatum();
			
		}
 
		if(documentStart!=null && (startDateTemp==null || !startDateTemp.equals(documentStart))) {
			_calendar.setTime(documentStart);
			_calendar.add(Calendar.YEAR, -1);
			timeLine.getOptions().setMin(_calendar.getTime());
		}

		if(documentEnd !=null && (endDateTemp==null || !endDateTemp.equals(documentEnd))) {
			_calendar.setTime(documentEnd);
			_calendar.add(Calendar.YEAR, 1);
			timeLine.getOptions().setMax(_calendar.getTime());
		}  
		timeLine.setTimeLineItems(timelineItems);
	}
	
	
	private DmsTimeLineItem buildTimeLineItem(Document document) {
		
		DmsTimeLineItem ti = new DmsTimeLineItem(document.getId(), document.getTitle(), document.getDocumentDatum(), null);
		
		String encodedResource = DmsCrypt.getEncodedResource(DmsWebSession.getSession(), document);
		String image = "../png/"+encodedResource;
		ti.setImg(image);

		if(document.getHeight()>document.getWidth()) ti.setStyle("cH");
		else ti.setStyle("cW");

		List<DmsTimeLineTag> tagList = getDocumentTags(document.getId());
		ti.setTags(tagList);
		
		List<DmsTimeLineUser> userList = getUsersList(document.getId());
		ti.setUsers(userList);
		return ti;
	}
	
	

	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		
		_btAll = new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAll", this));
		_toolBar.add(_btAll);
		
		_btNone = new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbNone", this));
		_toolBar.add(_btNone);
		
		_btDelete  = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbDelete", this));
		_btDelete.setUtf8Code("xe872");
		_btDelete.setEventName("Delete");
		_btDelete.setToolbarState(ToolbarState.STATE_VIEW);
		_btDelete.setDefaultFormProcessing(false);
		_btDelete.setCssClass(toolbarCSS);
		_btDelete.setOutputMarkupId(true);
		_toolBar.add(_btDelete);
		
		_btRefresh = new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("btRefresh", this));
		_toolBar.add(_btRefresh);
		
		_btNormalListView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btNormalListView.setUtf8Code("xe8ef");
		_btNormalListView.setEventName("ChangeNormalListView");
		_btNormalListView.setAlternate(ResourceText.getText("btNormalListView", this));
		_toolBar.add(_btNormalListView);

		
		_btSmallListView = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT);
		_btSmallListView.setUtf8Code("xe9b0");
		_btSmallListView.setEventName("ChangeSmallListView");
		_btSmallListView.setAlternate(ResourceText.getText("btListView", this));
		_btSmallListView.setCssClass(toolbarCSS);
		_toolBar.add(_btSmallListView);
		
		
		_btSEARCH = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSEARCH.setUtf8Code("xe8b6");
		_btSEARCH.setEventName("SEARCH");
		_btSEARCH.setAlternate(ResourceText.getText("btSEARCH", this));
		_toolBar.add(_btSEARCH);
		
		_btUSERs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUSERs.setUtf8Code("xe7ef");
		_btUSERs.setEventName("USERS");
		_btUSERs.setAlternate(ResourceText.getText("btUSERs", this));
		_toolBar.add(_btUSERs);

		_btUPLOAD = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btUPLOAD.setUtf8Code("xf09b");
		_btUPLOAD.setEventName("UPLOAD");
		_btUPLOAD.setAlternate(ResourceText.getText("btUPLOAD", this));
		if(_uploadExists)_toolBar.add(_btUPLOAD);
		
		
		_btTAGs = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btTAGs.setUtf8Code("xf05b");
		_btTAGs.setEventName("TAGS");
		_btTAGs.setAlternate(ResourceText.getText("btTAGs", this));
		_toolBar.add(_btTAGs);
	
		
		_btMerge = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btMerge.setUtf8Code("xe14d");
		_btMerge.setEventName("Merge");
		_btMerge.setAlternate("Dokumente zusammenfassen.");
		_toolBar.add(_btMerge);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(_sbbp.isVisible())_sbbp.closeAll(target);
		if(event.equals("All")) {
			fselectAll(target, true);
			target.add(_wmcDocuments);
		}else if(event.equals("None")) {
			fselectAll(target, false);
			target.add(_wmcDocuments);
		}else if(event.equals("Refresh")) {
			if(_sbbp.isVisible()) {
				refreshFromSearchBox(target);
			}else {
				_dpDocuments.size();
				target.add(_dpn);
				target.add(_wmcDocuments);
			}
		}else if(event.equals("SEARCH")) {
			showHideModules(_sbbp, target);
		}else if(event.equals("TAGS")) {
			showHideModules(_tsbp, target);
		}else if(event.equals("UPLOAD")) {
			showHideModules(_upvl, target);
		}else if(event.equals("USERS")) {
			showHideModules(_usbp, target);
		}else if(event.equals("SortNameUp")) {
			_dpDocuments.setSort("DocName", SortOrder.ASCENDING);
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("SortNameDown")) {
			_dpDocuments.setSort("DocName", SortOrder.DESCENDING);
			_dpDocuments.size();
			target.add(_wmcDocuments);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0) {
				_dpd.processDelete(target, _selectionList);
			}
		}else if(event.equals("Merge")) {
			if(_selectionList.size()>1) {
				_dpd.processMerge(target, _selectionList);
			}
		}else if(event.equals("ChangeNormalListView")) {
			DocumentsPage page = new DocumentsPage(_appPage,getPage().getPageReference(), Style.LIST);
			setResponsePage(page);
			storeParameters();
		}else if(event.equals("ChangeSmallListView")) {
			DocumentsPage page = new DocumentsPage(_appPage,getPage().getPageReference(), Style.TAB);
			setResponsePage(page);
			storeParameters();
		}
	}
	
	
	
	public HashSet<Integer> getSelectionIds(){
		return _selectionList;
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		_lblNoData = new Label("lblnodata", ResourceText.getText("lblNoData", this));
		_lblNoData.setVisible(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void initParameters() {
		_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS);
		_thisParameters.setCurrentDisplayStyle(Style.CALENDAR);
		if(_thisParameters.getSortParam()!=null) _dpDocuments.setSort(_thisParameters.getSortParam());
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.DELETED, false);
		_dpDocuments.getFilterState().setObjectStatusType(DocumentStatusType.ARCHIVED, false);
		
		_dpRelationsTags.getFilterState().setRelationType(RelationType.TAG_TO_DOCUMENT);
		_dpRelationsUsers.getFilterState().setRelationType(RelationType.DOCUMENT_TO_USER);
		if(DmsDBHelpersTag.getDefaultPDFImporterID()==-1) {
			_uploadExists=false;
			_thisParameters.setUploadVisible(false);
		};
		
		if(_thisParameters.getFilterObject()!=null && _thisParameters.isSearchVisible()) _dsfc = (DocumentOSFilterClass) _thisParameters.getFilterObject();
		_dsfc.setObjectStatusType(DocumentStatusType.DELETED, false);
		_dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
	}
	
	private void storeParameters() {
		_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENTS, _thisParameters);
	}
	
	private void addBehavior() {
		DocumentsListAjaxBehavior dsab = new DocumentsListAjaxBehavior(_wmcDocuments.getMarkupId()) {
 
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target) {

				DocumentsPage page = null;
				SearchHelper sh=_sbbp.getSearchHelper(""+documentId);
				if(sh!= null && sh.getSearchString()!=null && !sh.getSearchString().isEmpty())page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId, DocumentStateType.VIEW, sh);
				else page = new DocumentsPage(_appPage, getPage().getPageReference(), documentId);
				setResponsePage(page);
			}
			
			@Override
			protected void OnResizeView(AjaxRequestTarget target, int height, int width) {
				super.OnResizeView(target, height, width);
			}
			
		};
		add(dsab);
	}
	
	
	
	
	
	
	private void fselectAll(AjaxRequestTarget target, boolean select){
		_selectionList.clear();
		if(select) {
			for(int i=0;i<timelineItems.size();i++) {
				_selectionList.add(Integer.valueOf(timelineItems.get(i).getId()));
			}
			JSONArray selectedItems = new JSONArray(_selectionList);
			target.appendJavaScript("vt"+timeLine.getComponentId()+".setSelection("+selectedItems+");");
		}else {
			target.appendJavaScript("vt"+timeLine.getComponentId()+".setSelection();");
		}
		
		 
	}
	
	private void addProcessingDialogs() {
		_dpd = new DocumentsProcessingDialogs("processingdialogs") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnProcessError(AjaxRequestTarget target, ProcessType processType, String message) {
				infobox.refresh(target, message, INFOBOX_STATUS.error);
				infobox.setVisible(true);
				fselectAll(target, false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
			@Override
			protected void OnProcessFinished(AjaxRequestTarget target, ProcessType processType, String message, int sourceDocumentId, int targetDocumentId) {
				infobox.refresh(target, message, INFOBOX_STATUS.success);
				infobox.setVisible(true);
				fselectAll(target, false);
				_selectionList = new HashSet<Integer>();
				_dpDocuments.size();
				target.add(wmcInfo);
				target.add(_wmcDocuments);
				checkNodata(target);
			}
			
		};
		_dpd.setOutputMarkupId(true);
		add(_dpd);
		
	}
	
	
	
	private List<DmsTimeLineTag> getDocumentTags(int idDocument) {
		if(!_thisParameters.isShowTags())return null;
		try {
			if(_dpRelationsTags.getRelationById2(idDocument)==null) return null;
			List<DmsTimeLineTag> lstTags = new LinkedList<DmsTimeLineTag>();
			for(Relation relation : _dpRelationsTags.getRelationById2(idDocument)) {
				Tag tag = StaticLocators.getTags().getEntityById(relation.getIdrelation1());
				DmsTimeLineTag dmsTimeLineTag = new DmsTimeLineTag(tag);
				lstTags.add(dmsTimeLineTag);
			}
			
			Collections.sort(lstTags, new Comparator<DmsTimeLineTag>() {
			    @Override
			    public int compare(DmsTimeLineTag tag1, DmsTimeLineTag tag2) {
			        return tag1.getTag().compareToIgnoreCase(tag2.getTag());
			    }
			});
			return lstTags;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getDocumentTags", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	
	private List<DmsTimeLineUser> getUsersList(int idDocument){
		if(!_thisParameters.isShowUsers())return null;
		try {
			if(_dpRelationsUsers.getRelationById1(idDocument)==null) return null;
			List<DmsTimeLineUser> lstUsers = new LinkedList<DmsTimeLineUser>();
			for(Relation relation : _dpRelationsUsers.getRelationById1(idDocument)) {
				User user = StaticLocators.getUsers().getEntityById(relation.getIdrelation2());
				DmsTimeLineUser dmsTimeLineUser = new DmsTimeLineUser(user);
				lstUsers.add(dmsTimeLineUser);
			}
			
			Collections.sort(lstUsers, new Comparator<DmsTimeLineUser>() {
			    @Override
			    public int compare(DmsTimeLineUser user1, DmsTimeLineUser user2) {
			        return user1.getUser().compareToIgnoreCase(user2.getUser());
			    }
			});
			return lstUsers;
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "getUsersList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return null;
	}
	
	
	
	private void refreshTimeLineItemsFromSelectionList(AjaxRequestTarget target) {
		try {
			_dpRelationsTags.size();
			_dpRelationsUsers.size();
			StringBuilder sb = new StringBuilder();
			sb.append("var dataSet = i"+timeLine.getComponentId()+";");
			for(Integer docId : _selectionList) {
				DmsTimeLineItem updateItem = buildTimeLineItem(StaticLocators.getDocuments().getEntityById(Integer.valueOf(docId)));
				sb.append("dataSet.update("+updateItem+");");
			}
			target.appendJavaScript(sb.toString());
		}catch(Exception e) {
			DmsException dmsException = new DmsException(1000, "E", _className, "refreshTimeLineItemsFromSelectionList", null, e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	
	
	private void checkNodata(AjaxRequestTarget target) {
		long numberOfDocuments = 0;
		try {
			DocumentFilterClass dfc = new DocumentFilterClass();
			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, false);
			numberOfDocuments = StaticLocators.getDocuments().getNumberOfEntities(dfc);			
		}catch(Exception e) {
		}
		if(numberOfDocuments==0) {
			_toolBar.setVisible(false);
			_wmcDocuments.setVisible(false);
			_wmcLeft.setVisible(false);
			_dpn.setVisible(false);
			_lblNoData.setVisible(true);
		}else {
			_toolBar.setVisible(true);
			_wmcDocuments.setVisible(true);
			_dpn.setVisible(true);
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_toolBar);
			target.add(_wmcDocuments);
			target.add(_wmcLeft);
			target.add(_wmcNoData);
		}
	}
	
 
	
	private void refreshFromSearchBox(AjaxRequestTarget target) {
		
		_sbbp.refresh(target);
		if(_sbbp.hasResults()) {
			_dfcDocuments.setLinkedWhere(" id in ("+_sbbp.getCondition()+")");
			_dpDocuments.setFilterState(_dfcDocuments);
			_dpDocuments.size();
			_thisParameters.setFilterObject(_dsfc);
			storeParameters();
			refreshTimeLineItems(target);
			target.add(_dpn);
			target.add(_wmcDocuments);
		}
	}
	
	private void showHideModules(Panel panel, AjaxRequestTarget target) {
		
		boolean setVisible = !panel.isVisible();
		boolean hideLeft=true;
		if(panel instanceof TaggingPanel) {
			_thisParameters.setTagsVisible(setVisible);
		}else if(panel instanceof UsersSelectionPanel) {
			_thisParameters.setUsersVisible(setVisible);
		}else if(panel instanceof SearchboxPanel) {
			_thisParameters.setSearchVisible(setVisible);
		}else if(panel instanceof UploadPanelViewList) {
			_thisParameters.setUploadVisible(setVisible);
		}
		panel.setVisible(setVisible);
		if(_tsbp.isVisible())hideLeft=false;
		if(_usbp.isVisible())hideLeft=false;
		if(_sbbp.isVisible())hideLeft=false;
		if(_upvl.isVisible())hideLeft=false;

		
		if(hideLeft) {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.remove('innercontent_tags_active');dslpResize();");
		}else {
			target.appendJavaScript("document.getElementById(\""+_wmcDocuments.getMarkupId()+"\").classList.add('innercontent_tags_active');dslpResize();");
		}
		storeParameters();
		target.add(_wmcLeft);
		
	}
	
}
