/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.view;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.apache.wicket.MarkupContainer;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.datepicker.DatePicker;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.select2.SelectData;
import org.biffo.dms.components.tagselector.TagSelector;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.user.Users.User;

import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.models.DocumentModel;
import org.biffo.dms.session.ILocale;
import org.biffo.dms.user.UserRestrictionType;
import org.biffo.dms.web.documents.DocumentStateType;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.users.UserSelectorPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
import de.kleinekuh.entity.types.RecordStatusType;

public class DefaultPanel extends Panel{
	
	private static final String _className = "DefaultPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btUnarchive, _btRestore, _btREPDF; 
	
	
	private TextField<String> _title;
	private TextArea<String> _keywords;
	private DatePicker _documentdate;
	
	private DmsWebSession _session =  DmsWebSession.getSession();
	
	private DocumentModel _documentModel;
	private Document _document;
	
	private TagSelector _tagSelector;
	private UserSelectorPanel _users;
	private List<ISelectData> _lstUsers = new LinkedList<ISelectData>();
	
	//private boolean _pdfDocumentChanged=false;
	private boolean _editable=true;
	private boolean _archived=false;
	private boolean _deleted=false;
	
	public DefaultPanel(String id, final DocumentModel documentModel) {
		this(id, documentModel, DocumentStateType.VIEW);
	}
		
	public DefaultPanel(String id, final DocumentModel documentModel, DocumentStateType viewState) {
		super(id, documentModel);
		
		_documentModel = documentModel;
		_document = documentModel.getObject();
		if(_document.hasObjectStatusTypes(DocumentStatusType.ARCHIVED)){
			_editable = false;
			_archived = true;
		}else if(_document.hasObjectStatusTypes(DocumentStatusType.DELETED)){
			_editable = false;
			_deleted = true;
		}

		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
		addDefaultTextFields();
		addToolBar();
		addTagSelector();
		addUsersSelector();
		
		refreshFormObjects(null);
		
		if(viewState==DocumentStateType.EDIT) {
			setFormComponentsEditable(true);
			_toolBar.setViewState(ToolbarState.STATE_CHANGE);
		}else {
			setFormComponentsEditable(false);
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
		}
		
		applyUserRestrictions();
		
	}
	
	private void addDefaultTextFields() {
	
		_title = new TextField<String>("title");
		_title.setOutputMarkupId(true);
        _form.add(_title);

        _documentdate = new DatePicker("documentdate", new PropertyModel<Date>(_document, "DocumentDatum"), false) {
 
			private static final long serialVersionUID = 1L;
			@Override
        	protected void OnChange(AjaxRequestTarget target, Date value) {
				DefaultPanel.this._document.setDocumentDatum(value);
        	}
        };
        _documentdate.setOutputMarkupId(true);
        
        
        ILocale sessionParams =_session.getLocaleParams();
        _documentdate.getDatePickerOptions().setLanguage("de-DE");
        _documentdate.getDatePickerOptions().setFormat(sessionParams.getDateFormat());
        _documentdate.getDatePickerOptions().setDaysMin(sessionParams.getDaysMin());
        _documentdate.getDatePickerOptions().setMonths(sessionParams.getMonths());
        _documentdate.getDatePickerOptions().setMonthsShort(sessionParams.getMonthsShort());
        
		_form.add(_documentdate);
        
        
		_keywords= new TextArea<String>("keywords");
		_keywords.setOutputMarkupId(true);
        _form.add(_keywords);
	}
	
	public void setDocumentDatum(AjaxRequestTarget target, Date documentDatum) {
		try {
			_documentdate.setDefaultModelObject(documentDatum);
			target.add(_documentdate);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"setDocumentDatum",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	
	public void refreshFormObjects(AjaxRequestTarget target){
		_title.setDefaultModel(new PropertyModel<String>(_document, "Title"));
		_documentdate.setDefaultModel(new PropertyModel<Date>(_document, "DocumentDatum"));
		_keywords.setDefaultModel(new PropertyModel<String>(_document, "KeyWords"));
		if(null!=target) {
			target.add(_title);
			target.add(_keywords);
			_documentdate.hide(target);
			target.add(_documentdate);
		}
		refreshTagList(false);
		refreshUsersList();
	}

	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		if(_editable) {
			_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
			_btEdit.setAlternate(ResourceText.getText("btEdit", this));
			_toolBar.add(_btEdit);
		}
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("btCancel", this));
		_toolBar.add(_btCancel);

		if(_editable) {
			_btSave=new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
			_btSave.setAlternate(ResourceText.getText("btSave", this));
			_toolBar.add(_btSave);
		}
		
		if(_archived) {
			_btUnarchive = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
			_btUnarchive.setUtf8Code("xe169");
			_btUnarchive.setEventName("Unarchive");
			_btUnarchive.setAlternate(ResourceText.getText("btUnarchive", this));
			_btUnarchive.setDefaultFormProcessing(false);
			_btUnarchive.setOutputMarkupId(true);
			_btUnarchive.setCssClass(toolbarCSS);
			_toolBar.add(_btUnarchive);
		}
		
		if(_deleted) {
			_btRestore = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW);
			_btRestore.setUtf8Code("xe938");
			_btRestore.setEventName("Restore");
			_btRestore.setAlternate(ResourceText.getText("btRestore", this));
			_btRestore.setDefaultFormProcessing(false);
			_btRestore.setOutputMarkupId(true);
			_btRestore.setCssClass(toolbarCSS);
			//_toolBar.add(_btRestore);
		}
		
		if(!_deleted && !_archived) {
			_btREPDF = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
			_btREPDF.setUtf8Code("xf74e");
			_btREPDF.setEventName("Repdf");
			_btREPDF.setAlternate(ResourceText.getText("btRePDF", this));
			_toolBar.add(_btREPDF);
		}
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	
	@Override
	protected void onModelChanged() {
		if(getDefaultModel() instanceof DocumentModel)
			_documentModel = (DocumentModel) getDefaultModel();
			
		_document = _documentModel.getObject();
		super.onModelChanged();
	}
	
	
	
	private void processEvent(String event, AjaxRequestTarget target) {
		
		
		_documentdate.hide(target);
		
		if(event.equals("Cancel")) {
			cancel(target);
		}else if(event.equals("Edit")) {
			edit(target);
		}else if(event.equals("Save")) {
			_documentModel.getObject().setDocumentDatum(_documentdate.getDate());
			_documentModel.getObject().setTitle(_document.getTitle());
			_documentModel.getObject().setKeyWords(_document.getKeyWords());
			OnSave(
				target, 
				getSelectorIds(_tagSelector.getDataSelection(true)),
				getSelectorIds(_users.getDataSelection(true)),
				null
			);
		}else if(event.equals("Ok")) {
		}else if(event.equals("Unarchive")) {
			OnUnarchive(target);
		}else if(event.equals("Restore")) {
			OnUndelete(target);
		}else if(event.equals("Export")) {
		}else if(event.equals("Repdf")) {
			OnRePdf(
				target,
				getSelectorIds(_tagSelector.getDataSelection(true)),
				getSelectorIds(_users.getDataSelection(true)),
				null
			);
		}
		
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void edit(AjaxRequestTarget target){
		 try {
			 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
			 OnViewStateChange(target, DocumentStateType.EDIT);
			 setFormComponentsEditable(true);
	 	}catch(Exception e) {
	 		Exception er = new Exception(ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString());
	 		OnError(target, er);
	 	}
	}
	
	
	public void cancel(AjaxRequestTarget target){
		 try {
			 
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 OnClose(target);
			 }else {
				 try {
					 _document = _documentModel.getObject();
					 refreshFormObjects(target);
					 target.add(_tagSelector);
					 target.add(_users);
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 OnCancel(target);
					 OnViewStateChange(target, DocumentStateType.VIEW);
					 setFormComponentsEditable(false);
				 }catch(Exception e2) {
					 Exception er = new Exception(ResourceText.getText("errChangeCancel", this)+"\n"+ e2.toString());
					 OnError(target, er);
				 }
			 } 
		 }catch(Exception e) {
			 Exception er = new Exception( ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString());
			 OnError(target, er);
		 }
	}
	
	
	private void addTagSelector() {
		_tagSelector = new TagSelector("tagselector");
		_tagSelector.setOutputMarkupId(true);
		refreshTagList(false);
		_form.add(_tagSelector);
	}
	
	private void addUsersSelector() {
		refreshUsersList();
        _users = new UserSelectorPanel("userselector",  _lstUsers);
        _users.setOutputMarkupId(true);
        _users.setDisabled(true);
        _form.add(_users);
	}
	
	
	public void setFormComponentsEditable(boolean editable){
		try{
			_title.setEnabled(editable);
			_keywords.setEnabled(editable);
			_documentdate.setActive(editable);
			_tagSelector.setDisabled(!editable);
			_users.setDisabled(!editable);
				
		}catch(Exception e){}
	}
	
	public void refreshTagList(boolean withTagListRefresh) {
		
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(((DocumentModel)getDefaultModel()).getId());
			rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			List<Tag> lstTags = new LinkedList<Tag>();
			for(Relation relation : alRelations) {
				lstTags.add(StaticLocators.getTags().getEntityById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstTags, new Comparator<Tag>() {
			    @Override
			    public int compare(Tag o1, Tag o2) {
			        return o1.getTagName().compareToIgnoreCase(o2.getTagName());
			    }
			});
			if(withTagListRefresh)_tagSelector.doRefresh();
			_tagSelector.setSelectedTags(lstTags);
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshTagList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	public void refreshUsersList(){
		try{
			
			List<User> availabelUsers = StaticLocators.getUsers().getEntities(-1, -1, " order by benutzername ", null);
			for(User us : availabelUsers) {
				
				if(us.getAccessType()==UsersAccessType.ADMIN || us.getAccessType()==UsersAccessType.USER) {
					boolean found=false;
					for(ISelectData data : _lstUsers) {
						if(data.getId()==us.getId()) {
							data.setSelected(false);
							found=true;
						}
					}
					if(!found) {
						_lstUsers.add(new SelectData(us.getId(), us.getBenutzerName(), false, false));
					}
				}
			}
			
			if(_document.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation1(_document.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.DOCUMENT_TO_USER);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstUsers) {
							if(data.getId()==entity.getIdrelation2()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshUsersList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return;
	} 
	
	
	
	private int[] getSelectorIds(List<ISelectData> lstSelectedTags){
		if(lstSelectedTags==null || lstSelectedTags.isEmpty()) return null;
		int[] iArrRC = new int[lstSelectedTags.size()];
		for(int i=0;i<lstSelectedTags.size();i++) {
			iArrRC[i] = lstSelectedTags.get(i).getId();
		}
		return iArrRC;
		
	}
	
	public void setUnarchived() {
		_btUnarchive.setVisible(false);
	}
	
	@Override
	public MarkupContainer setDefaultModel(IModel<?> model) {
		return super.setDefaultModel(model);
	}
	
	
	protected void OnViewStateChange(AjaxRequestTarget target, DocumentStateType viewState){}
	protected void OnClose(AjaxRequestTarget target){}
	protected void OnSave(AjaxRequestTarget target, int[] selectedTags, int[] selectedUsers, int[] selectedUserGroups){}
	protected void OnCancel(AjaxRequestTarget target){}
	protected void OnUnarchive(AjaxRequestTarget target){}
	protected void OnUndelete(AjaxRequestTarget target){}
	protected void OnError(AjaxRequestTarget target, Exception e){}
	protected void OnRePdf(AjaxRequestTarget target,int[] selectedTagIds, int[] selectedUsers, int[] selectedUserGroups){}
 
	
	
	private void applyUserRestrictions() {
		

		if(_btREPDF!=null) {
			if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_REPDF_DOCUMENT))_btREPDF.setVisible(true);
			else _btREPDF.setVisible(false);
		}

		if(_btEdit!=null) {
			if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_EDIT_DOCUMENT))_btEdit.setVisible(true);
			else _btEdit.setVisible(false);
		}
		
		if(_btUnarchive!=null) {
			if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_UNARCHIVE_DOCUMENT))_btUnarchive.setVisible(true);
			else _btUnarchive.setVisible(false);
		}

		
		if(_btRestore!=null) {
			if(_session.getSessionParameters().getRestrictionType(UserRestrictionType.ALLOW_RESTORE_DOCUMENT))_btRestore.setVisible(true);
			else _btRestore.setVisible(false);
		}
		
	}
	
	
}
