/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.documents.page.view;

 
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.ajax.markup.html.tabs.AjaxTabbedPanel;
import org.apache.wicket.extensions.markup.html.tabs.AbstractTab;
import org.apache.wicket.extensions.markup.html.tabs.ITab;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pdfviewer.PdfViewerPanel;
import org.biffo.dms.documents.pdf.PdfViewerResponse;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Processing.ProcessType;

import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.helpers.DmsDBHelpersFragment;
import org.biffo.dms.helpers.SearchHelper;
import org.biffo.dms.models.DocumentModel;
 
import org.biffo.dms.session.parameters.DocumentViewParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.documents.DocumentStateType;
import org.biffo.dms.web.documents.DocumentsProcessingDialogs;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

public class DocumentPanel extends Panel{

 
	private static final long serialVersionUID = 1L;
	
	private final AjaxTabbedPanel<ITab> _documentTabbedPanel;
	private final List<ITab> _lstPanels = new ArrayList<>();
	private final Form<?> _form;

	private DefaultPanel _defaultPanel;
	private ContentPanel _contentPanel;
	private MetadataPanel _metadataPanel;
	private FragmentsPanel _fragmentsPanel;
	
	private PdfViewerPanel _pvp;
	
	private WebMarkupContainer wmcInfo, _wmcDocumentInfo;
	private InfoboxPanel infobox;

	private Label _lblTitel;
	private PanelTitel _panelTitel = new PanelTitel();
	
	private Object _origin;
	
	private DocumentModel _documentModel;
	private int _documentId;
	
	private PdfViewerResponse viewerResponse;

	
	private DmsWebSession _session =  DmsWebSession.getSession();
	private DocumentViewParameters _thisParameters;
	
	private DocumentsProcessingDialogs _dpd;
	
	@Override
	protected void onBeforeRender() {
		if(_defaultPanel!=null) {
			_defaultPanel.refreshTagList(true);
			_defaultPanel.refreshUsersList();
		}
		super.onBeforeRender();
	}
	
	@Override
	protected void onRender() {
		super.onRender();
		
	}

	
	public DocumentPanel(String id, int documentId,  Object origin) {
		this(id, documentId, origin, DocumentStateType.VIEW);
	}

	public DocumentPanel(String id, int documentId,  Object origin, DocumentStateType viewState) {
		this(id, documentId, origin, viewState, null);
	}
	
	public DocumentPanel(String id, int documentId,  Object origin, DocumentStateType viewState, SearchHelper searchHelper) {
		super(id);
		
		initParameters();
		addProcessingDialogs();
		_origin = origin;
		
		if(origin instanceof PageReference) {
			_origin = ((PageReference)origin).getPage();
		}
		
		_documentId = documentId;
		_documentModel = new DocumentModel(_documentId);
		
		_wmcDocumentInfo = new WebMarkupContainer("documentinfo");
		_wmcDocumentInfo.setOutputMarkupId(true);
		add(_wmcDocumentInfo);
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		_wmcDocumentInfo.add(_form);
		addPanels(viewState);
		
		_documentTabbedPanel = new AjaxTabbedPanel<ITab>("tabs", _lstPanels){
			
			private static final long serialVersionUID = 1L;
			
		    @Override
		    protected WebMarkupContainer newLink(String linkId, final int index) {
		    	
		    	return new AjaxSubmitLink(linkId, _form) {
		    		private static final long serialVersionUID = 1L;
		    		
		    		@Override
		    		protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
		    			setSelectedTab(index);
		    			if (target != null) {
		    				target.add(_documentTabbedPanel);
		    			}
		    			onAjaxUpdate(target);
		    		}
		 
		    		@Override
		    		protected void onError(AjaxRequestTarget target, Form<?> form) {
		    		}
		        };
		    }
		};
		_documentTabbedPanel.setOutputMarkupId(true);
		_documentTabbedPanel.setVisible(_thisParameters.isDocInfoVisible());
		_form.add(_documentTabbedPanel);
		
		addDefaultTextFields();
		addAdditionalControls();
		addInfobox();
		
		if(viewState==DocumentStateType.VIEW) {
			
			if(_documentModel.getObject().hasObjectStatusTypes(DocumentStatusType.ARCHIVED)) {
				_panelTitel.setTitel(ResourceText.getText("titViewDocument_Archived", this));
			}else if(_documentModel.getObject().hasObjectStatusTypes(DocumentStatusType.DELETED)) {
				_panelTitel.setTitel(ResourceText.getText("titViewDocument_Deleted", this));
			}else {
				_panelTitel.setTitel(ResourceText.getText("titViewDocument", this));
			}
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditDocument", this));
			_documentTabbedPanel.setVisible(true);
		}
		
		_pvp = new PdfViewerPanel("pdfx", _documentModel,  searchHelper) {
			
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnChange(AjaxRequestTarget target, PdfViewerResponse viewerResponse) {
				DocumentPanel.this.viewerResponse = viewerResponse; 
			}
			
			@Override
			protected void OnSplitt(AjaxRequestTarget target, PdfViewerResponse viewerResponse) {
				super.OnSplitt(target, viewerResponse);
				DocumentPanel.this.viewerResponse = viewerResponse;
				try {
					_dpd.processSplitt(target, _documentModel, viewerResponse);
				}catch(Exception e) {
				}
			}
			
			@Override
			protected void OnHide(AjaxRequestTarget target) {
				if(_documentTabbedPanel.isVisible()) {
					_documentTabbedPanel.setVisible(false);
				} 
				_pvp.add(new AttributeModifier("class", "pdfViewerFull"));
				target.add(_wmcDocumentInfo);
				target.add(_pvp);
			}
			
			@Override
			protected void OnShow(AjaxRequestTarget target) {
				if(!_documentTabbedPanel.isVisible()) {
					_documentTabbedPanel.setVisible(true);
				}
				_pvp.add(new AttributeModifier("class", "pdfViewerNormal"));
				target.add(_wmcDocumentInfo);
				target.add(_pvp);
			}
			
			@Override
			protected void OnClose(AjaxRequestTarget target) {
				super.OnClose(target);
				closeForm(target);
			}
			
			 
			
		};
		_pvp.setOutputMarkupId(true);
		if(_documentTabbedPanel.isVisible()) {
			_pvp.add(new AttributeModifier("class", "pdfViewerNormal"));
		}else {
			_pvp.add(new AttributeModifier("class", "pdfViewerFull"));
		}
		
		add(_pvp);
		
	}
	
	private void addDefaultTextFields() {}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			if(target!=null)target.add(DocumentPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
			setResponsePage((Page)_origin);
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addPanels(DocumentStateType viewState){

		_lstPanels.add(new AbstractTab(new Model<String>(ResourceText.getText("panOptions", this))){
			private static final long serialVersionUID = 1L;
			@Override
            public Panel getPanel(String panelId){
            	if(_defaultPanel==null){
            		_defaultPanel = new DefaultPanel(panelId, _documentModel, viewState) {
						private static final long serialVersionUID = 1L;
            			@Override
            			protected void OnViewStateChange(AjaxRequestTarget target, DocumentStateType viewState) {
            				if(viewState==DocumentStateType.VIEW) {
            					_panelTitel.setTitel(ResourceText.getText("titViewDocument", this));
            					_pvp.setEditable(target, false);
            				}else {
            					_panelTitel.setTitel(ResourceText.getText("titEditDocument", this));
            					_pvp.setEditable(target, true);
            				}
            				target.add(_lblTitel);
            			}
            			@Override
            			protected void OnClose(AjaxRequestTarget target) {
            				closeForm(target);
            			}
            			
            			@Override
            			protected void OnSave(AjaxRequestTarget target, int[] selectedTagIds, int[] selectedUsers, int[] selectedUserGroups) {
            				_dpd.processSave(target, _documentModel, viewerResponse, selectedTagIds, selectedUsers, selectedUserGroups );
            			}
            			
            			@Override
            			protected void OnRePdf(AjaxRequestTarget target,int[] selectedTagIds, int[] selectedUsers, int[] selectedUserGroups) {
            				_dpd.processRepdf(target, _documentModel, selectedTagIds, selectedUsers, selectedUserGroups);
            			}
            			
            			@Override
            			protected void OnUnarchive(AjaxRequestTarget target) {
            				_dpd.processUnarchive(target, _documentModel);
            			}
            			
            			@Override
            			protected void OnError(AjaxRequestTarget target, Exception e) {
            				infobox.refresh(target, e.getMessage(), INFOBOX_STATUS.error);
            				infobox.setVisible(true);
            				target.add(wmcInfo);
            			}
            			
            			@Override
            			protected void OnCancel(AjaxRequestTarget target) {
            				_pvp.setDocument(_documentModel.getObject());
            				target.add(_pvp);
            			}
            			
            			@Override
            			protected void OnUndelete(AjaxRequestTarget target) {
            				HashSet<Integer> selection = new HashSet<Integer>();
            				selection.add(_documentId);
            				_dpd.processUndelete(target, selection);
            			}
            			
            			
            			
            		};
            		_defaultPanel.setOutputMarkupId(true);
            	}
            	return _defaultPanel;
            }
        });
		
		_lstPanels.add(new AbstractTab(new Model<String>(ResourceText.getText("panContent", this))){
			private static final long serialVersionUID = 1L;
			@Override
            public Panel getPanel(String panelId){
            	if(_contentPanel==null){
            		_contentPanel = new ContentPanel(panelId,_documentModel);
            		_contentPanel.setOutputMarkupId(true);
            	}
            	return _contentPanel;
            }
        });
		
		_lstPanels.add(new AbstractTab(new Model<String>(ResourceText.getText("panMetadata", this))){
			private static final long serialVersionUID = 1L;
			@Override
            public Panel getPanel(String panelId){
            	if(_metadataPanel==null){
            		_metadataPanel = new MetadataPanel(panelId,_documentModel );
            		_metadataPanel.setOutputMarkupId(true);
            	}
            	return _metadataPanel;
            }
        });		
		
		if(DmsDBHelpersFragment.hasFragments(_documentId, _session.getCurrentTenantId())) {
			_lstPanels.add(new AbstractTab(new Model<String>("Fragments")){
				private static final long serialVersionUID = 1L;
				@Override
		        public Panel getPanel(String panelId){
		        	if(_fragmentsPanel==null){
		        		_fragmentsPanel = new FragmentsPanel(panelId, _documentModel) {
							private static final long serialVersionUID = 1L;
							
							@Override
							protected void OnFragmentViewClicked(AjaxRequestTarget target, Fragment fragment) {
								DocumentPanel.this._pvp.highlightFragment(target, fragment);
							};
		        		};
		        		_fragmentsPanel.setOutputMarkupId(true);
		        	}
		        	return _fragmentsPanel;
		        }
		    });		
		}
		
	}
	

	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void refreshPanelContent(AjaxRequestTarget target, ProcessType processType, int targetDocumentId) {
		
		if(processType==ProcessType.OVERWRITE) {
			_documentId = targetDocumentId;
			_documentModel = new DocumentModel(_documentId);
		}else if(processType==ProcessType.VERSIONING) {
			_documentId = targetDocumentId;
			_documentModel = new DocumentModel(_documentId);
		}else if(processType==ProcessType.UNDELETE) {
			 
		}
		
		if(null!=_defaultPanel) {
			_defaultPanel.setDefaultModel(_documentModel);
			_defaultPanel.refreshFormObjects(target);
		}
		if(null!=_contentPanel) {
			_contentPanel.setDefaultModel(_documentModel);
			_contentPanel.refreshFormObjects();
		}
		if(null!=_metadataPanel) {
			_metadataPanel.setDefaultModel(_documentModel);
			_metadataPanel.refreshFormObjects();
		}
		
		
		
		
	}
	
	private void initParameters() {
		_thisParameters = (DocumentViewParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENT_PDF_VIEW);
	}
	
	private void addProcessingDialogs() {
		_dpd = new DocumentsProcessingDialogs("processingdialogs") {

			private static final long serialVersionUID = 1L;
			
			@Override
			protected void OnProcessError(AjaxRequestTarget target, ProcessType processType, String message) {
				infobox.refresh(target, message, INFOBOX_STATUS.error);
				infobox.setVisible(true);
				target.add(wmcInfo);
				refreshPanelContent(target, processType, _documentId);
			}
			
			@Override
			protected void OnProcessFinished(AjaxRequestTarget target, ProcessType processType, String message, int sourceDocumentId, int targetDocumentId) {
				infobox.refresh(target, message, INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				refreshPanelContent(target, processType, targetDocumentId);
			}
			
		};
		_dpd.setOutputMarkupId(true);
		add(_dpd);
		
	}
	
	

}
