/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.fragments;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.FilterForm;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.DPFragmentsAggregated;
import org.biffo.dms.entity.Fragments.Fragment;
import org.biffo.dms.entity.Fragments.FragmentFilterClass;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.documents.DocumentsListAjaxBehavior;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class FragmentsListPanel extends Panel{

	private static final String toolbarCSS = "viewListToolbar";
	private static final long serialVersionUID = 1L;

	
	private DPFragmentsAggregated _qtdp = new DPFragmentsAggregated();
	private Set<Fragment> _selectionList = new HashSet<Fragment>();
	private DefaultDataTable<?, ?> _ddt;
	
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btDelete;
	private FilterForm<?> _form;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private StandardListParameters _thisParameters;

	private WebMarkupContainer wmcInfo, _wmcContent, _wmcNoData;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private Label _lblNoData;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	
	@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
	public FragmentsListPanel(String id) {
		super(id);
		initParameters();
		addToolBar();
		addModalDefault();
		addInfobox();
		addBehavior();
		addNoData();
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		columns.add(new CheckBoxColumn<Fragment>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<Fragment> rowModel) {
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		
		columns.add(new PropertyColumn<Fragment, String>(ResourceText.getModelText("colFragmentName", this), "Fragmentname","Fragmentname"));
		columns.add(new PropertyColumn<Fragment, String>(ResourceText.getModelText("colFragmentValue", this), "Fragmentvalue","Fragmentvalue"));
		columns.add(new PropertyColumn<Fragment, String>(ResourceText.getModelText("colNumberConnection", this), "IdDocument","IdDocument"));
		columns.add(new AbstractColumn<Fragment, String>(ResourceText.getModelText("colDocuments", this), "IdDocument_list"){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<Fragment>> item, String componentId, IModel<Fragment> rowModel){

				FragmentDocuments fds = new FragmentDocuments(componentId, -1, rowModel.getObject().getIdDocument_list());
				fds.setOutputMarkupId(true);
				item.add(fds);
			}
		});
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){
			@Override
			protected void onPageChanged() {
				super.onPageChanged();
			}
		};
		_ddt.setOutputMarkupId(true);
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		
		_form = new FilterForm("dataform", _qtdp);
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
		//add(_form);
		
		_wmcContent = new WebMarkupContainer("wmccontent");
		_wmcContent.setOutputMarkupId(true);
		add(_wmcContent);
		_wmcContent.add(_form);
		
		checkNodata(null);
		
	}
	
	private void addToolBar() {
		
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	public Set<Fragment> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			Fragment ql =(Fragment)iter.next();
			if(select){
				_selectionList.add(ql);
			}else{
				 _selectionList.remove(ql);
			} 
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target); 
		}else if(event.equals("Refresh")) {
			_qtdp.size();
			target.add(_dpn);
			target.add(_ddt);
		}
	}
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.FRAGMENTS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void deleteEntities(AjaxRequestTarget target){
		try{

			List<Integer> fragmentIdsToDelete = new LinkedList<Integer>();
			for(Fragment entityToDelete : _selectionList) {
				
				if(entityToDelete.getFragmentvalue()==null || entityToDelete.getFragmentvalue().isEmpty()) continue;
				
				FragmentFilterClass ffc = new FragmentFilterClass();
				ffc.setFragmentname(entityToDelete.getFragmentname());
				ffc.setFragmentvalue(entityToDelete.getFragmentvalue());
				ffc.setIdTenant(_session.getCurrentTenantId());
				
				List<Fragment> entities = StaticLocators.getFragments().getEntities(-1, -1, "", ffc);
				for(Fragment fragment : entities)fragmentIdsToDelete.add(fragment.getId());
			}
			if(!fragmentIdsToDelete.isEmpty()) {
				for(Integer idToDelete : fragmentIdsToDelete) {
					StaticLocators.getFragments().deleteEntityById(idToDelete);
				}
			}
			
			
			infobox.refresh(target, ResourceText.getText("infFragmentDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errFragmentDeleted", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}
		_selectionList = new HashSet<Fragment>();
		target.add(wmcInfo);
		target.add(_form);
		target.add(_ddt);
		target.add(_dpn);
	} 
	
	
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_form);
				checkNodata(target);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		
		String temp = ResourceText.getText("lblNoData", this);
		_lblNoData = new Label("lblnodata",temp);
		_lblNoData.setVisible(false);
		_lblNoData.setEscapeModelStrings(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void addBehavior() {
		
		DocumentsListAjaxBehavior dsab = new DocumentsListAjaxBehavior(getMarkupId()) {
 
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target) {
				DocumentsPage page = null;
				page = new DocumentsPage(DmsWebApplicationPages.ApplicationPage.DOCUMENTS, getPage().getPageReference(), documentId);
				setResponsePage(page);
			}
		};
		add(dsab);
	}
	
	private void checkNodata(AjaxRequestTarget target) {

		if(_qtdp.size()==0) {
			_toolBar.setVisible(false);
			_wmcContent.setVisible(false);
			_dpn.setVisible(false);
			_lblNoData.setVisible(true);
		}else {
			_toolBar.setVisible(true);
			_wmcContent.setVisible(true);
			_dpn.setVisible(true);
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_toolBar);
			target.add(_wmcContent);
			target.add(_wmcNoData);
		}
	}
	
	
	
}
