/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.home;


import java.util.List;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.DmsWebApplicationPages.ApplicationPage;
import org.biffo.dms.Types.SearchStatusType;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.dataprovider.metrics.MetricType;
import org.biffo.dms.entity.Document.Document;
import org.biffo.dms.entity.Document.DocumentFilterClass;
import org.biffo.dms.entity.Document.DocumentOSFilterClass;
import org.biffo.dms.entity.Document.DocumentStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.session.parameters.DocumentsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.components.docbar.DocBar;
import org.biffo.dms.web.components.search.singelfield.SingleFieldSearchInput;
import org.biffo.dms.web.documents.DocumentsListAjaxBehavior;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.documents.DocumentsPage.Style;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.logs.small.LogsPanelSmallViewList;
import org.biffo.dms.web.metrics.MetricsPanel;
import org.biffo.dms.web.uploads.UploadPanelViewList;


public class HomePanel extends Panel{

 
	private static final String className = "HomePanel";
	private static final long serialVersionUID = 1L;

	private DocumentsListParameters _thisParameters;
	private final static ApplicationPage _appPage = DmsWebApplicationPages.ApplicationPage.HOME;
	private DmsWebSession _session =  DmsWebSession.getSession();
	private LogsPanelSmallViewList _pgsv;
	
	private MetricsPanel _mpdo;
	
	private UploadPanelViewList _upvl;
	private DocBar _docBar;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	
	public HomePanel(String id) {
		super(id);
		
		initParameters();
		addInfobox();
		addBehavior();
		
		_pgsv = new LogsPanelSmallViewList("log");
		_pgsv.setOutputMarkupId(true);
		add(_pgsv);
		
		DocumentOSFilterClass dsfc = new DocumentOSFilterClass();
		dsfc.setObjectStatusType(DocumentStatusType.DELETED, false);
		dsfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
		SingleFieldSearchInput sfsi = new SingleFieldSearchInput("singelfieldsearch", dsfc) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnSelectionFilterClick(AjaxRequestTarget target) {
				doSearchRedirect(target, dsfc);
			}
			
			@Override
			protected void OnEnterFilterClick(AjaxRequestTarget target) {
				doSearchRedirect(target, dsfc);
			};
			
		};
		sfsi.setOutputMarkupId(true);
		add(sfsi);

		addUploadPanel();
		
		_mpdo = new MetricsPanel("documents", MetricType.document);
		_mpdo.setOutputMarkupId(true);
		add(_mpdo);
		_mpdo.setVisible(true);
		
		
		_docBar = new DocBar("docbar", getLastAddedDocumentIDs());
		_docBar.setOutputMarkupId(true);
		add(_docBar);
		
	}
	
	private void doSearchRedirect(AjaxRequestTarget target, DocumentOSFilterClass dsfc) {
		dsfc.setSingelFieldSearchWhere(dsfc.getLinkedWhere());
		dsfc.setSearchStatusType(SearchStatusType.SINGLEFIELD_SET, true);
		_thisParameters.setFilterObject(dsfc);
		
		storeParameters();
		DocumentsPage page = new DocumentsPage(_appPage, getPage().getPageReference(), Style.LIST);
		setResponsePage(page);
	}
	
	
	
	private String getLastAddedDocumentIDs() {
		StringBuilder sb = new StringBuilder();
		try {
			DocumentFilterClass dfc = new DocumentFilterClass();
			dfc.setObjectStatusType(DocumentStatusType.ARCHIVED, false);
			dfc.setObjectStatusType(DocumentStatusType.DELETED, false);
			
			if(_session.getUserAccessType()==UsersAccessType.USER) {
				String userCondition = " dms_documents.ID IN (SELECT dms_relations.Idrelation1 FROM dms_relations WHERE RelationType="+RelationType.DOCUMENT_TO_USER.getCode()+" AND Idrelation2="+_session.getCurrentUserId()+")";
				dfc.setLinkedWhere(userCondition);
			}
			List<Document> entities = StaticLocators.getDocuments().getEntities(0, 10, " order by id desc", dfc);
			for(int i=0;i<entities.size();i++) {
				if(i>0)sb.append(",");
				sb.append(entities.get(i).getId());
			}
		}catch(Exception e) {
			DmsException dmsInfo = new DmsException(1000, "E", 
				className,
				"getLastAddedDocumentIDs",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
		}
		return sb.toString();
	}
	
	
	
	 
		
	private void initParameters() {
		_thisParameters = (DocumentsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.DOCUMENTS);
		_thisParameters.setSearchVisible(true);
	}
	
	private void storeParameters() {
		_session.getSessionParameters().setParametersByType(SessionParameterTypes.DOCUMENTS, _thisParameters);
	}
	
	private void addUploadPanel() {
		_upvl = new UploadPanelViewList("uploadpanel") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void OnUploadPanelViewListFinished(AjaxRequestTarget target, int processId, int  uploads, int dublicates) {

				if(dublicates>0) {
					infobox.refresh(target, ResourceText.getText("warUploadDublicate", this, uploads, dublicates), INFOBOX_STATUS.warning);
				}else{
					infobox.refresh(target, ResourceText.getText("infUploadOK", this, uploads), INFOBOX_STATUS.success);
				};
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_pgsv);
				target.add(_mpdo);
				_docBar.setDocumentIDs(getLastAddedDocumentIDs());
				target.add(_docBar);
			}
		};
		_upvl.setOutputMarkupId(true);
		add(_upvl);
		if(_upvl.getNumberOfImporters()==0) {
			_upvl.setVisible(false);
		}
	}
	
	private void addBehavior() {
		
		DocumentsListAjaxBehavior dsab = new DocumentsListAjaxBehavior(getMarkupId()) {
 
			private static final long serialVersionUID = 1L;
			@Override
			protected void OnDocumentDblClick(int documentId, AjaxRequestTarget target) {
				DocumentsPage page = null;
				page = new DocumentsPage(DmsWebApplicationPages.ApplicationPage.DOCUMENTS, getPage().getPageReference(), documentId);
				setResponsePage(page);
			}
		};
		add(dsab);
	}
	
	 
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	 
}
