/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.logs.normal;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.json.JSONArray;
import org.apache.wicket.ajax.json.JSONObject;
import org.apache.wicket.ajax.markup.html.AjaxLink;
import org.apache.wicket.markup.html.basic.Label;
import org.biffo.dms.DmsWebApplicationPages;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.Types.EntityType;
import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.web.documents.DocumentsPage;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.HtmlFragment;

public class LogDescription extends HtmlFragment {


	private static final long serialVersionUID = 1L;
	
	private final String _fragmentSeperator="!";
	private final String _variableSeperator="\\|";
	
	private final String _regExResourceText = "\\$\\{(.+?)\\}";
	private final Pattern _ptResourceText = Pattern.compile(_regExResourceText);
	private final StringBuilder _sb = new StringBuilder();
	private DmsWebSession _session = DmsWebSession.getSession();

	
	public LogDescription(String id, LogEntry logEntry) {
		this(id, logEntry, null);
	}
	
	
	
	public LogDescription(String id, LogEntry logEntry, Component parent) {
		
		super(id);
		int messageCode = logEntry.getMessageCode();
		if(messageCode==1000) {
 			String value = logEntry.getException();
			if(value==null || value.trim().length()==0) value = logEntry.getLogText1();
			if(value==null || value.trim().length()==0) value = "NA";
			Label label= new Label("lbl0", value);
			label.setOutputMarkupId(true);
	        add(label);
	        _sb.append("<span wicket:id=\"lbl0\"></span>");
			
			
		}else if(messageCode==2000) {
 			String value = logEntry.getLogText1();
			if(value==null || value.trim().length()==0) value = logEntry.getException();
			if(value==null || value.trim().length()==0) value = "NA";
			Label label= new Label("lbl0", value);
			label.setOutputMarkupId(true);
	        add(label);
	        _sb.append("<span wicket:id=\"lbl0\"></span>");
		}else {
			String resourceText = null;
			if(parent!=null)  resourceText = ResourceText.getText(""+messageCode, parent);
			else resourceText = ResourceText.getText(""+messageCode, this);
			buildHtml(resourceText, logEntry);
		}
		
	}

	@Override
	protected String getHtml() {
		return _sb.toString();
	}
	
	private void buildHtml(String resourceText, LogEntry logEntry) {
		
		Matcher m = _ptResourceText.matcher(resourceText);
		int count=0;
		int offset=0;
		while(m.find()) {
	         count++;
	         _sb.append(resourceText.substring(offset, m.start()));
	         
	         String fragment=m.group(1);
	         if(fragment!=null && fragment.trim().length()>0 && logEntry.getLogObject()!=null) {
	        	 buildFragment(fragment, logEntry.getEntityType(), logEntry.getLogObject(), count);
	         }
	         offset=m.end();
		}
		_sb.append(resourceText.substring(offset));
	}
	
	private void buildFragment(String fragment, EntityType entityType, String logObject, int positionInFragment) {
		try {
			String strArr[] = fragment.split(_fragmentSeperator);
			if(strArr==null || strArr.length!=3) return;
	        int posInArray = Integer.valueOf(strArr[1]);
	        JSONObject jsonObject = getJSONObjectFromString(logObject, posInArray);
    		if(null==jsonObject) return;
    		String value = getValueFromJSONObject(jsonObject, strArr[2].split(_variableSeperator));
	        if(strArr[0].equals("L")) {
	        	int entityId = getIdFromJSONObject(jsonObject);
	        	if(entityId==-1) return;
	        	switch(entityType) {
	        		case DOCUMENTS:	
	        		case DOCUMENT:
	        			if(_session.isDocumentAvailable(entityId))addDocumentLink(positionInFragment, entityId, value);
	        			else addLabel(positionInFragment, value);
	        			break;
	        		default: 
	        			break;
	        	}
        		
	        }else if(strArr[0].equals("T")) {
	        	addLabel(positionInFragment, value);
	        }
			
		}catch(Exception e) {}
	}
	
	private void addDocumentLink(int positionInFragment, int documentId,  String value) {
		
		AjaxLink<String> link = new AjaxLink<String>("lnk"+positionInFragment) {

        	private static final long serialVersionUID = 1L;
        	
			@Override
			public void onClick(AjaxRequestTarget target) {
				if(documentId!=-1) {
					DocumentsPage page = new DocumentsPage(
							DmsWebApplicationPages.ApplicationPage.LOGS, 
							getPage().getPageReference(), 
							documentId);
					setResponsePage(page);
				}
			}
        	
        };
        link.setOutputMarkupId(true);
        link.setEnabled(false);
        try {
        	if(null!=StaticLocators.getDocuments().getEntityById(documentId))link.setEnabled(true);
		}catch(Exception e) {}
        add(link);
        _sb.append("<a wicket:id=\"lnk"+positionInFragment+"\">");
		Label label= new Label("lbl"+positionInFragment, value);
		label.setOutputMarkupId(true);
        link.add(label);
        _sb.append("<span wicket:id=\"lbl"+positionInFragment+"\"></span>");
        _sb.append("</a>");
	}
	
	
	
	private void addLabel(int positionInFragment, String value) {
		Label label= new Label("lbl"+positionInFragment, value);
		label.setOutputMarkupId(true);
        add(label);
        _sb.append("<span wicket:id=\"lbl"+positionInFragment+"\"></span>");
	}
	
	
	private JSONObject getJSONObjectFromString(String strIn, int posInArray) {
		
		try {
			if(strIn.startsWith("[{")) {
				JSONArray jsonArray = new JSONArray(strIn);
				return jsonArray.getJSONObject(posInArray);
			}else if(strIn.startsWith("{")) {
				JSONObject jsonObject = new JSONObject(strIn);
				return jsonObject;
			}
		}catch(Exception e) {}
		return null;
	}
	
	private String getValueFromJSONObject(JSONObject jsonObject, String strJsonVars[]) {
		
		String sValue="NA";
		int iValue=-1;
		for(int i=0;i<strJsonVars.length;i++) {
			try {
				if(jsonObject.has("_"+strJsonVars[i])) {
					iValue = jsonObject.optInt("_"+strJsonVars[i], -1);
					if(iValue!=-1) sValue = ""+iValue;
					else sValue=jsonObject.getString("_"+strJsonVars[i]);
					break;
				}else if(jsonObject.has(strJsonVars[i])) {
					iValue = jsonObject.optInt(strJsonVars[i], -1);
					if(iValue!=-1) sValue = ""+iValue;
					else sValue=jsonObject.getString(strJsonVars[i]);
					break;
				}
			}catch(Exception e) {}
		}
		return sValue;
	}
	
	private int getIdFromJSONObject(JSONObject jsonObject) {
		int id = -1;
		try {
			if(jsonObject.has("_id")) {
				id = jsonObject.getInt("_id");
			}else if(jsonObject.has("id")) {
				id = jsonObject.getInt("id");
			}

		}catch(Exception e) {}
		return id;
	}
	
	
	
	

}
