/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.logs.normal;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.filter.FilterForm;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.Types.EntityType;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.DPLogEntries;

import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.entity.Logentries.StatusType;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.session.parameters.LogsListParameters;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

public class LogsPanelViewList extends Panel {
 
 
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	
	
	private LogsListParameters _thisParameters;
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btDelete, _btSEARCH;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private final SimpleDateFormat _sdf = new SimpleDateFormat(_session.getLocaleParams().getDateFormatLong());

	
	private DPLogEntries  _qtdp = new DPLogEntries();
	private DefaultDataTable<?, ?> _ddt;
	private Set<LogEntry> _selectionList = new HashSet<LogEntry>();
	private FilterForm<?> _form;
	
	private WebMarkupContainer wmcInfo, _wmcContent, _wmcNoData, _wmcFilter, _wmcFilterBox;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	private Label _lblNoData;
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");

	
	public LogsPanelViewList(String id) {
		this(id, -1, -1);
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public LogsPanelViewList(String id, int documentId, int processId) {
		super(id);
		
		if(documentId!=-1) {
			_qtdp.getFilterState().setIdEntity(documentId);
		}
		if(processId!=-1) {
			_qtdp.getFilterState().setIdProcess(processId);
		}
		
		if(_session.getUserAccessType()==UsersAccessType.USER) {
			String linkedFilter =" messagecode != 1000 and iduser = "+_session.getCurrentUserId(); 
			_qtdp.getFilterState().setLinkedWhere(linkedFilter);
		}
		
		initParameters();
		setOutputMarkupId(true);
		
		addInfobox();
		addToolBar();
		addNoData();
		addModalDefault();
		
		_form = new FilterForm("dataform", _qtdp);
		_form.setOutputMarkupId(true);

		_wmcFilter = new WebMarkupContainer("filter");
		_wmcFilter.setOutputMarkupId(true);
		
		_wmcFilterBox = new WebMarkupContainer("filterbox");
		_wmcFilterBox.setVisible(_thisParameters.isSearchVisible());
		_wmcFilterBox.setOutputMarkupId(true);
		_wmcFilter.add(_wmcFilterBox);
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		if(_session.getUserAccessType()==UsersAccessType.ADMIN) {
			buildColumns(columns);	
		}else {
			buildUserViewColumns(columns);
		}
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
			
		};
		_ddt.setOutputMarkupId(true);
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		_form.add(_ddt);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
		_wmcContent = new WebMarkupContainer("wmccontent");
		_wmcContent.setOutputMarkupId(true);
		add(_wmcContent);
		_wmcContent.add(_form);
		_wmcContent.add(_wmcFilter);
		
		checkNodata(null);
		
 		
	}
	
	private void buildUserViewColumns(List<IColumn<?,?>> columns) {
		
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colId", this), "Id","Id"));
		
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colAenderungsDatum", this), "AenderungsDatum"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				String label = "NA";
				try{
					label = _sdf.format(rowModel.getObject().getAenderungsDatum());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().getStatusType());
			}
		});

		
	
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colLogDescription", this)){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				
				if(rowModel.getObject()!=null) {
					item.add(new LogDescription(componentId, rowModel.getObject()));
				}else {
					String label = "NA";
					Label lbl = new Label(componentId, label);
					item.add(lbl);
				}
			}
		});
	}
 
	
	
	
	private void buildColumns(List<IColumn<?,?>> columns) {
		
		columns.add(new CheckBoxColumn<LogEntry>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<LogEntry> rowModel) {
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colId", this), "Id","Id"));
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colIdProcess", this), "IdProcess","IdProcess"));
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().getStatusType());
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colUser", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				setUserName(item, componentId, rowModel.getObject());
			}
		});
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colEntityType", this), "EntityType"){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				String label = "NA";
				try{
					if(rowModel.getObject().getEntityType()!=null) {
						label =rowModel.getObject().getEntityType().toString(); 
					}else{
						label = EntityType.UNKNOWN.toString();
					};
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colMessageCode", this), "MessageCode","MessageCode"));
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colClassName", this), "ClassName","ClassName"));
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colFunctionName", this), "FunctionName","FunctionName"));
		columns.add(new PropertyColumn<LogEntry, String>(ResourceText.getModelText("colStatusType", this), "StatusType","StatusType"));
		
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colLogDescription", this)){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				
				if(rowModel.getObject()!=null) {
					item.add(new LogDescription(componentId, rowModel.getObject()));
				}else {
					String label = "NA";
					Label lbl = new Label(componentId, label);
					item.add(lbl);
				}
				
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(ResourceText.getModelText("colAenderungsDatum", this), "AenderungsDatum"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				String label = "NA";
				try{
					label = _sdf.format(rowModel.getObject().getAenderungsDatum());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
	}

	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		
		if(_session.getUserAccessType()==UsersAccessType.ADMIN) {
		
			_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
			_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
			_toolBar.add(_btAll);
					
			_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
			_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
			_toolBar.add(_btNone);
			
			_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
			_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
			_toolBar.add(_btDelete);
			
			_btSEARCH = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW  | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
			_btSEARCH.setAlternate(ResourceText.getText("tbAltSearch", this));
			_btSEARCH.setUtf8Code("xe8b6");
			_btSEARCH.setEventName("SEARCH");
			//if(_session.getUserAccessType()==UsersAccessType.ADMIN) _toolBar.add(btSEARCH);


		}
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
		
	}
	
	public Set<LogEntry> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			LogEntry ql =(LogEntry)iter.next();
 			if(select){
				_selectionList.add(ql);
			}else{
				 _selectionList.remove(ql);
			} 
		}
	}
	
	 
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target); 
		}else if(event.equals("Refresh")) {
			_ddt.size();
			target.add(_ddt);
		} else if(event.equals("SEARCH")) {
			if(_wmcFilterBox.isVisible()) {
				_thisParameters.setSearchVisible(false);
				_wmcFilterBox.setVisible(false);
			}else {
				_thisParameters.setSearchVisible(true);
				_wmcFilterBox.setVisible(true);
			}
			target.add(_wmcFilter);
		}
	}
	
	private void addInfobox() {

		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
		
	} 
	
	 
	
	private void initParameters() {
		_thisParameters = (LogsListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.USERLOGS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void setUserName(Item<ICellPopulator<LogEntry>> item, String componentId, LogEntry logEntry) {

		String temp = "NA";
		try {
			if(logEntry.getIdUser()==99999) {
				temp = "System";
			}else {
				temp = StaticLocators.getUsers().getEntityById(logEntry.getIdUser()).getBenutzerName();
				if(temp==null)temp = "ID: "+logEntry.getIdUser();
			}
		}catch(Exception e) {
			temp = "UserID: "+logEntry.getIdUser();
			
		}
		Label lbl = new Label(componentId, temp);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
	
	
	private void buildStatusCell(Item<ICellPopulator<LogEntry>> item, String componentId, StatusType statusType) {
		
		Label lbl = null;
		switch(statusType){
			case ERROR:
				lbl = new Label(componentId, "<span class=\"statusType error\">&#xe000;</span>");
				break;
			case INFO:
				lbl = new Label(componentId, "<span class=\"statusType info\">&#xe2e6;</span>");
				break;
			case WARN:
				lbl = new Label(componentId, "<span class=\"statusType warning\">&#xe002;</span>");
				break;
			case UNKNOWN:
				lbl = new Label(componentId, "");
				break;
		}
		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
	private void deleteEntities(AjaxRequestTarget target){
		try{

			for(LogEntry entityToDelete : _selectionList) {
				StaticLocators.getLogEntries().deleteEntityById(entityToDelete.getId());
			}
			infobox.refresh(target, ResourceText.getText("infLogEntryDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errLogEntryDeleted", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}
		_selectionList = new HashSet<LogEntry>();
		target.add(wmcInfo);
		target.add(_form);
		target.add(_ddt);
		target.add(_dpn);
	} 
	
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_form);
				checkNodata(target);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	private void addNoData() {
		_wmcNoData = new WebMarkupContainer("nodata");
		_wmcNoData.setOutputMarkupId(true);
		add(_wmcNoData);
		
		String temp = ResourceText.getText("lblNoData", this);
		_lblNoData = new Label("lblnodata",temp);
		_lblNoData.setVisible(false);
		_lblNoData.setEscapeModelStrings(false);
		_lblNoData.setOutputMarkupId(true);
		_wmcNoData.add(_lblNoData);
		
	}
	
	private void checkNodata(AjaxRequestTarget target) {

		if(_qtdp.size()==0) {
			_toolBar.setVisible(false);
			_wmcContent.setVisible(false);
			_dpn.setVisible(false);
			_lblNoData.setVisible(true);
		}else {
			_toolBar.setVisible(true);
			_wmcContent.setVisible(true);
			_dpn.setVisible(true);
			_lblNoData.setVisible(false);			
		}
		
		if(null!=target) {
			target.add(_toolBar);
			target.add(_wmcContent);
			target.add(_wmcNoData);
		}
	}


}
