/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.logs.small;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.wicket.extensions.ajax.markup.html.repeater.data.table.AjaxFallbackDefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.dataprovider.DPLogEntries;
import org.biffo.dms.entity.Logentries.LogEntry;
import org.biffo.dms.entity.Logentries.StatusType;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.web.logs.normal.LogDescription;

public class LogsPanelSmallViewList extends Panel {
	
	public enum LogViewType {
		small,
		document,
		upload
	}

	private static final long serialVersionUID = 1L;
	private final LogViewType viewType;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private final SimpleDateFormat _sdf = new SimpleDateFormat(_session.getLocaleParams().getDateFormatLong());
	
	private DPLogEntries  _qtdp = new DPLogEntries();
	private AjaxFallbackDefaultDataTable<?, ?> _ddt;
	
	
	
	public LogsPanelSmallViewList(String id) {
		this(id, -1, -1, LogViewType.small);
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public LogsPanelSmallViewList(String id, int documentId, int processId, LogViewType logsViewType) {
		super(id);
		
		
		viewType = logsViewType;
		setOutputMarkupId(true);

		if(documentId!=-1)_qtdp.getFilterState().setIdEntity(documentId);
		if(processId!=-1)_qtdp.getFilterState().setIdProcess(processId);

		if(_session.getUserAccessType()==UsersAccessType.USER) {
			String linkedFilter =" messagecode != 1000 and iduser = "+_session.getCurrentUserId(); 
			_qtdp.getFilterState().setLinkedWhere(linkedFilter);
		}
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		if(viewType==LogViewType.small) {
			buildSmallViewColumns(columns);
		}
				
		
		_ddt = new AjaxFallbackDefaultDataTable("datatable", columns, _qtdp, 10) {
			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
			}
			
		};
		_ddt.setOutputMarkupId(true);
		add(_ddt);
		
		
		if(viewType==LogViewType.small) {
			//_ddt.add(new AttributeModifier("class", "logsmallview"));
		}
		
 		
	}
	
	private void buildSmallViewColumns(List<IColumn<?,?>> columns) {
		
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("Zeitpunkt"), "AenderungsDatum"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				String label = "NA";
				try{
					label = _sdf.format(rowModel.getObject().getAenderungsDatum());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("User")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				setUserName(item, componentId, rowModel.getObject());
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
		    public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				buildStatusCell(item, componentId, rowModel.getObject().getStatusType());
			}
		});
		
		columns.add(new AbstractColumn<LogEntry, String>(new Model<String>("LogDescription"), "LogDescription"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<LogEntry>> item, String componentId, IModel<LogEntry> rowModel){
				
				if(rowModel.getObject()!=null) {
					item.add(new LogDescription(componentId, rowModel.getObject()));
				}else {
					String label = "NA";
					Label lbl = new Label(componentId, label);
					item.add(lbl);
				}
				
			}
		});
	}
	 
	
	private void setUserName(Item<ICellPopulator<LogEntry>> item, String componentId, LogEntry logEntry) {

		String temp = "NA";
		try {
			if(logEntry.getIdUser()==99999) {
				temp = "System";
			}else {
				temp = StaticLocators.getUsers().getEntityById(logEntry.getIdUser()).getBenutzerName();
				if(temp==null)temp = "ID: "+logEntry.getIdUser();
			}
		}catch(Exception e) {
			temp = "UserID: "+logEntry.getIdUser();
		}
		Label lbl = new Label(componentId, temp);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}
	
	
	
	private void buildStatusCell(Item<ICellPopulator<LogEntry>> item, String componentId, StatusType statusType) {
		
		Label lbl = null;
		switch(statusType){
			case ERROR:
				lbl = new Label(componentId, "<span class=\"statusType error\">&#xe000;</span>");
				break;
			case INFO:
				lbl = new Label(componentId, "<span class=\"statusType info\">&#xe2e6;</span>");
				break;
			case WARN:
				lbl = new Label(componentId, "<span class=\"statusType warning\">&#xe002;</span>");
				break;
			case UNKNOWN:
				lbl = new Label(componentId, "");
				break;
		}
		lbl.setEscapeModelStrings(false);
		lbl.setOutputMarkupId(true);
		item.add(lbl);
	}


}
