/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.modules;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxSubmitLink;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.NavigationToolbar;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.lang.Bytes;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.components.tagbox.TagsBoxPanel;
import org.biffo.dms.components.upload.UploadField;
import org.biffo.dms.components.upload.UploadFieldFile;
import org.biffo.dms.dataprovider.config.DPModules;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Processing.ProcessType;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.ModuleTagHelper;
import org.biffo.dms.processQueue.tasks.ModuleUploadTask;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.helpers.processing.ProcessingPanel;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;
import org.biffo.dms.web.server.ServerControlPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class ModulesListPanel extends Panel{

	private static final String _className = "ModulesListPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	

	private DPModules _qtdp = new DPModules();
	private Set<ApiModuleImpl> _selectionList = new HashSet<ApiModuleImpl>();
	private DefaultDataTable<?, ?> _ddt;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete, _btStart, _btStop;
 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private DmsWebSession _session = (DmsWebSession)DmsWebSession.getSession();
	private StandardListParameters _thisParameters;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");

	private ProcessingPanel _processingPanel;
	
	private AjaxSubmitLink _alDummySubmit;
	private UploadField _uploadField;
	private ArrayList<FileUpload> uploads = new ArrayList<>();
	
	@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
	public ModulesListPanel(String id) {
		super(id);
		initParameters();
		addInfobox();
		addModalDefault();
		addProcessingPanel();
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<ApiModuleImpl>(Model.of("")) {

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<ApiModuleImpl> rowModel) {
				return new AbstractCheckBoxModel() {
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		columns.add(new PropertyColumn<ApiModuleImpl, String>(ResourceText.getModelText("colId", this), "Id","Id"));

		
		columns.add(new LinkColumn(ResourceText.getModelText("colClassname", this), "Classname", "Classname") {
			@Override
			public void onClick(Item item, String componentId, IModel model) {
				ApiModuleImpl fn = (ApiModuleImpl)model.getObject();
				ModulePanel panel = new ModulePanel(ModulesListPanel.this.getId(), fn,  ModulesListPanel.this);
				ModulesListPanel.this.replaceWith(panel);
			}
		});
		
		columns.add(new AbstractColumn<ApiModuleImpl, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ApiModuleImpl>> item, String componentId, IModel<ApiModuleImpl> rowModel) {
				
				ApiModuleImpl fn = rowModel.getObject();
				if(DmsWebApplication.getModules().isLoaded(fn)) {
					rowModel.getObject().setServerActive(true);
				}else {
					rowModel.getObject().setServerActive(false);
				}
				
				ServerControlPanel controlPanel = new ServerControlPanel(componentId, fn) {
					private static final long serialVersionUID = 1L;
					@Override
					protected void OnServerStart(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ModulesListPanel.this.processEvent("start", target, importProcess);
					}
					@Override
					protected void OnServerStop(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ModulesListPanel.this.processEvent("stop", target, importProcess);
					}
					@Override
					protected void OnServerSync(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ModulesListPanel.this.processEvent("sync", target, importProcess);
					}

					@Override
					protected void OnModuleTest(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ModulesListPanel.this.processEvent("test", target, importProcess);
					}
				};
				controlPanel.setOutputMarkupId(true);
				item.add(controlPanel);
			}
			
		}); 
		
		columns.add(new PropertyColumn<ApiModuleImpl, String>(ResourceText.getModelText("colJar", this), "Jar","Jar"));
		columns.add(new PropertyColumn<ApiModuleImpl, String>(ResourceText.getModelText("colType", this), "Type","Type"));
		columns.add(new PropertyColumn<ApiModuleImpl, String>(ResourceText.getModelText("colVersion", this), "Version","Version"));
		
		columns.add(new PropertyColumn<ApiModuleImpl, String>(ResourceText.getModelText("colAuthor", this), "Author","Author"));
		columns.add(new AbstractColumn<ApiModuleImpl, String>(ResourceText.getModelText("colBeschreibung", this), "Description"){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ApiModuleImpl>> item, String componentId, IModel<ApiModuleImpl> rowModel) {
				String temp = "NA";
				try{
					temp = rowModel.getObject().getDescription();
					if(null!=temp){
						if(temp.length()>40) temp = temp.substring(0, 40) + " ...";
					}else{
						temp="NA";
					}
				}catch(Exception e){}
				item.add(new Label(componentId, temp));
			}
		});
		
		
		columns.add(new AbstractColumn<ApiModuleImpl, String>(ResourceText.getModelText("colServerTags", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ApiModuleImpl>> item, String componentId, IModel<ApiModuleImpl> rowModel) {
				
				
				List<Tag> lstTags = ModuleTagHelper.getTagListFromModule(rowModel.getObject(), _session);
				if(lstTags!=null && lstTags.size()>0) {
					TagsBoxPanel tagsBox = new TagsBoxPanel(componentId);
					tagsBox.setTagsBoxClassName("associatedtags");
					tagsBox.setTagClassName("associatedtag");
					tagsBox.setTags(lstTags);
					tagsBox.setOutputMarkupId(true);
					item.add(tagsBox);
				}else {
					item.add(new Label(componentId, ""));
				}
			}
		});
 
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){
			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		_form = new Form("dataform");
		_ddt.addBottomToolbar(new NavigationToolbar(_ddt));
		_ddt.setOutputMarkupId(true);
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		
		addToolBar();
		add(_form);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false) {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnNbOfItemsChanged(AjaxRequestTarget target, long itemPerPage) {
				target.add(_dpn);
			}
		};
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
		addModuleUpload();
	}
	
	
	private void addModuleUpload() {
		
		Form<?> _uploadform = new Form<Void>("uploadform");
		_uploadform.setOutputMarkupId(true);
		
		
		_uploadField =  new UploadField("fileInput", new PropertyModel<>(this, "uploads")) {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnSelect(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles) {
				super.OnSelect(target, uploadFiles);
				String submit = "document.getElementById('"+_alDummySubmit.getMarkupId()+"').click();";
				target.appendJavaScript(submit);
			}

			@Override
			protected void OnDrop(AjaxRequestTarget target, List<UploadFieldFile> uploadFiles) {
				super.OnDrop(target, uploadFiles);
				String submit = "document.getElementById('"+_alDummySubmit.getMarkupId()+"').click();";
				target.appendJavaScript(submit);
			}

			
			@Override
			protected void OnClear(AjaxRequestTarget target) {
				super.OnClear(target);
			}
		};
		
		_uploadField.setOutputMarkupId(true);
		_uploadField.setLabelVisible(false);
		
		_uploadform.add(_uploadField);
		_uploadform.setMaxSize(Bytes.MAX);
		_uploadform.setFileMaxSize(Bytes.MAX);
		
		_alDummySubmit = new AjaxSubmitLink("submit", _uploadform) {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onSubmit(AjaxRequestTarget target, Form<?> form) {
				
				try {
					if(null!=uploads && uploads.size()>0) {
						_processingPanel.start(target, new ModuleUploadTask(uploads, -1));
					}
				} catch (Exception e) {
					DmsException dmsException = new DmsException(
							1000, 
							"E", 
							_className, 
							"onSubmit", 
							null,
							e);
					StaticLocators.getMessageHandler().createMessage(dmsException, DmsWebSession.getSession());

				}
			}
		};
		_alDummySubmit.setOutputMarkupId(true);
		add(_alDummySubmit);
		add(_uploadform);
		
	} 
	
	
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			ApiModuleImpl ql =(ApiModuleImpl)iter.next();
			if(select){
				if(!_selectionList.contains(ql)) _selectionList.add(ql);
			}else{
				if(_selectionList.contains(ql)) _selectionList.remove(ql);
			}
		}
	}
	
	
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			
			
			for(ApiModuleImpl entityToDelete : _selectionList) {
				
				if(DmsWebApplication.getModules().isLoaded(entityToDelete)) {
					DmsWebApplication.getModules().unloadModule(entityToDelete);
				}
				ArrayList<Relation> relationsForDelete = new ArrayList<Relation>();
				relationsForDelete.addAll(DmsDBHelpersTag.getRelatedRelations(entityToDelete.getId(), RelationType.MODULE_TO_IMPORT_PROCESS, _session.getCurrentTenantId()));
				for(Relation relation : relationsForDelete) {
					StaticLocators.getRelations().deleteEntityById(relation.getId());
				}
	
				try {
					String temp = StaticLocators.getDmsProcessingMapper().getDmsConfig().getModuleFolder()+"/"+entityToDelete.getJar();
					Files.delete(Paths.get(temp));
				}catch(Exception e) {
					DmsException dmsInfo = new DmsException(1000, "E", 
						_className,
						"deleteEntities",
						ResourceText.getText("dmsErrDeleteFileModule", this, entityToDelete.getJar()),
						e,
						entityToDelete
					);
					StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				}
				
				Set<String> keys = entityToDelete.getVariables().keySet();
				for(String key: keys) {
					StaticLocators.getVariables().deleteEntityById(entityToDelete.getVariables().get(key).getId());
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
					_className,
					"deleteEntities",
					ResourceText.getText("dmsInfoModuleDeleted", this, entityToDelete.getJar()),
					null,
					entityToDelete
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}
			infobox.refresh(target, ResourceText.getText("infModuleDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errModuleDeleted", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}
		_selectionList = new HashSet<ApiModuleImpl>();
		target.add(wmcInfo);
		_qtdp.refresh();
		target.add(_form);
		target.add(_dpn);
	} 
	

	
	public Set<ApiModuleImpl> getSelection(){
		return _selectionList;
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target, null);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		
		_btStart = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStart.setUtf8Code("xf0d1");
		_btStart.setEventName("start");
		_btStart.setAlternate(ResourceText.getText("tbAltStart", this));
		_toolBar.add(_btStart);

		_btStop = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStop.setUtf8Code("xef71");
		_btStop.setEventName("stop");
		_btStop.setAlternate(ResourceText.getText("tbAltStop", this));
		_toolBar.add(_btStop);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	
	
	private void processEvent(String event, AjaxRequestTarget target, IVariablesImpl serverImpl) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_form);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_form);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			_qtdp.refresh();
			target.add(_dpn);
			target.add(_form);
		}else if(event.equals("Add")) {
			_uploadField.showUploadDialog(target);
		}else if(event.equals("start")) {
			if(_selectionList.size()>0 || serverImpl!=null) {
				startModules(target, serverImpl);
				target.add(_ddt);
				target.add(_dpn);
			}
		}else if(event.equals("stop")) {
			if(_selectionList.size()>0 || serverImpl!=null) {
				stopModules(target, serverImpl);
				target.add(_ddt);
				target.add(_dpn);
			}
		}else if(event.equals("test") && serverImpl!=null) {
			ApiModuleImpl fn = (ApiModuleImpl)serverImpl;
			ModulesPage mp = new ModulesPage(ModulesListPanel.this.getPage().getPageReference(), fn);
			setResponsePage(mp);	
		}

	}
	
	private void startModules( AjaxRequestTarget target, IVariablesImpl variableImpl) {
		try {
			if(variableImpl!=null) {
				startModule(target, variableImpl);
			}else {
				for(ApiModuleImpl module : _selectionList) {
					startModule(target, module);
				}
			}
			infobox.refresh(target, ResourceText.getText("infModuleStarted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
		}catch(DmsException e) {
			infobox.refresh(target, ResourceText.getText("errModuleStarted", this, e.getExceptionobject().getMessage()), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}
		_selectionList = new HashSet<ApiModuleImpl>();		
		target.add(wmcInfo);
		target.add(_form);
	}
	private void stopModules( AjaxRequestTarget target,  IVariablesImpl variableImpl) {
		
		try {
			if(variableImpl!=null) {
				stopModule(target, variableImpl);
			}else {
				for(ApiModuleImpl module : _selectionList) {
					stopModule(target, module);
				}
			}
			infobox.refresh(target, ResourceText.getText("infModuleStopped", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
		}catch(DmsException e) {
			infobox.refresh(target, ResourceText.getText("errModuleStopped", this, e.getExceptionobject().getMessage()), INFOBOX_STATUS.error);
			infobox.setVisible(true);
		}
		_selectionList = new HashSet<ApiModuleImpl>();
		target.add(wmcInfo);
		target.add(_form);
	}
	

	private void startModule(AjaxRequestTarget target,  IVariablesImpl variableImpl) throws DmsException{
		ApiModuleImpl fn = (ApiModuleImpl)variableImpl;
		DmsWebApplication.getModules().loadModule(fn);
	}

	private void stopModule(AjaxRequestTarget target,  IVariablesImpl variableImpl) throws DmsException{
		ApiModuleImpl fn = (ApiModuleImpl)variableImpl;
		DmsWebApplication.getModules().unloadModule(fn);
	}

	
	
	@SuppressWarnings("unused")
	private void refreshModule(AjaxRequestTarget target,  IVariablesImpl variableImpl) throws DmsException{
		stopModule(target, variableImpl);
		startModule(target, variableImpl);
	}
	
	
	
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.MODULES);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void addProcessingPanel() {
		
 		_processingPanel = new ProcessingPanel("modalprocess") {
			private static final long serialVersionUID = 1L;
			
			@Override
			protected void onFinished(AjaxRequestTarget target, ProcessType processType, int documentSourceId, int documentTargetId, int processId) {
				super.onFinished(target, processType, documentSourceId, documentTargetId, processId);

				_qtdp.refresh();
				_qtdp.size();
				target.add(_ddt);
				target.add(_form);
				target.add(_dpn);
			}
		};
		_processingPanel.setOutputMarkupId(true);
		add(_processingPanel);

 	}
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_form);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	 
	
}

