/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.processes;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.AjaxSelfUpdatingTimerBehavior;
import org.apache.wicket.extensions.ajax.markup.html.repeater.data.table.AjaxFallbackDefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;

import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.util.time.Duration;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.helpers.ImportProcessHelpers;
import org.biffo.dms.processQueue.ProcessBridge;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.server.ServerControlPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
 
public class ProcessListPanel extends Panel {
 
	private static final long serialVersionUID = 1L;
	
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btDelete, _btStart, _btStop;
	private static final String toolbarCSS = "viewListToolbar";
	
	private Set<Object> _selectionList = new HashSet<Object>();
	private DPProcesses _qtdp = new DPProcesses();
	private AjaxFallbackDefaultDataTable<?, ?> _ddt; 
	private Form<Object> _form;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private List<IVariablesImpl> lstProcesses = new LinkedList<IVariablesImpl>();
	private StandardListParameters _thisParameters;
	private DmsWebSession _session = DmsWebSession.getSession();

	
	private void refreshProcessesList() {
		
		try {
			lstProcesses.clear();
			lstProcesses.addAll(ImportProcessHelpers.getImportProcesses());
			lstProcesses.addAll(ImportProcessHelpers.getServerFTPProcesses());
			lstProcesses.addAll(ImportProcessHelpers.getServerSFTPProcesses());
			lstProcesses.addAll(ImportProcessHelpers.getServerSMTPProcesses());
			lstProcesses.addAll(ImportProcessHelpers.getServerFolderProcesses());
			lstProcesses.addAll(ImportProcessHelpers.getModuleProcesses());
		}catch(Exception e) {
		}
		
	}
	
	
 
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public ProcessListPanel(String id) {
		super(id);
		
		initParameters();
		addInfobox();
		
		setOutputMarkupId(true);
		
		refreshProcessesList();
		
		_form = new Form<Object>("dataform");
		_form.setOutputMarkupId(true);
		_form.add( new AjaxSelfUpdatingTimerBehavior(Duration.seconds(5)));
		
		ArrayList<IColumn<?, ?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<ProcessBridge>(Model.of("")) {
 
			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<ProcessBridge> rowModel) {
				return new AbstractCheckBoxModel() {
 
					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		columns.add(new AbstractColumn<ProcessBridge, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ProcessBridge>> item, String componentId, IModel<ProcessBridge> rowModel) {
				
				
				IVariablesImpl varImpl=null;
				String taskName = rowModel.getObject().getTaskName();
				for(int i=0;i<lstProcesses.size();i++) {
					if(lstProcesses.get(i).getServerName().equals(taskName)) {
						varImpl=lstProcesses.get(i);
						break;
					}
				}
				if(null==varImpl) {
					item.add(new Label(componentId, "NA"));
				}else {
					
					if(DmsWebApplication.getProcessQueue().isRunning(varImpl.getServerName())){
						varImpl.setServerActive(true);
					}else {
						varImpl.setServerActive(false);
					}
					
					
					ServerControlPanel serverControlPanel = new ServerControlPanel(componentId, varImpl) {
						
						private static final long serialVersionUID = 1L;
						@Override
						protected void OnServerStart(AjaxRequestTarget target, IVariablesImpl serverImpl) {
							if(!DmsWebApplication.getProcessQueue().isRunning(serverImpl.getServerName())){
								DmsWebApplication.getProcessQueue().start(serverImpl.getServerName());
							}
						}
						@Override
						protected void OnServerStop(AjaxRequestTarget target, IVariablesImpl serverImpl) {
							if(DmsWebApplication.getProcessQueue().isRunning(serverImpl.getServerName())){
								DmsWebApplication.getProcessQueue().stop(serverImpl.getServerName());
							}
						}
						@Override
						protected void OnServerSync(AjaxRequestTarget target, IVariablesImpl serverImpl) {
							if(DmsWebApplication.getProcessQueue().isRunning(serverImpl.getServerName())){
								DmsWebApplication.getProcessQueue().refresh(serverImpl.getServerName());
							}
						}
					};
					serverControlPanel.setOutputMarkupId(true);
					item.add(serverControlPanel);
				}
			}
			
		});

		columns.add( new PropertyColumn<ProcessBridge, String>(ResourceText.getModelText("colTaskName", this), "taskName") );
		columns.add( new PropertyColumn<ProcessBridge, String>(ResourceText.getModelText("colMessage", this), "message") );
		_ddt = new AjaxFallbackDefaultDataTable("processtable", columns, _qtdp, 10);
		_form.add(_ddt);
		
		addToolBar();
		add(_form);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
	}
	
	private void addToolBar(){
		_toolBar = new Toolbar("toolbar") {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		add(_toolBar);

		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
		
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		
		_btStart = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStart.setUtf8Code("xf0d1");
		_btStart.setEventName("start");
		_btStart.setAlternate(ResourceText.getText("tbAltStart", this));
		_toolBar.add(_btStart);

		_btStop = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStop.setUtf8Code("xef71");
		_btStop.setEventName("stop");
		_btStop.setAlternate(ResourceText.getText("tbAltStop", this));
		_toolBar.add(_btStop);
		
		
        add(_toolBar);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
        _toolBar.setVisible(true);
	}
	
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Refresh")) {
			_selectionList = new HashSet<Object>();
			target.add(_ddt);
		}else if(event.equals("start")) {
			for(Object object:_selectionList) {
				if(object instanceof ProcessBridge) {
					((ProcessBridge)object).setStop(false);
				}
			}
			_selectionList = new HashSet<Object>();
			target.add(_ddt);
		}else if(event.equals("stop")) {
			for(Object object : _selectionList) {
				if(object instanceof ProcessBridge) {
					((ProcessBridge)object).setStop(true);
				}
			}
			_selectionList = new HashSet<Object>();
			target.add(_ddt);
		}else if(event.equals("delete")) {
	
		}
	}
	
	
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter = _ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			ProcessBridge ql =(ProcessBridge)iter.next();
			if(select){
				if(!_selectionList.contains(ql)) _selectionList.add(ql);
			}else{
				if(_selectionList.contains(ql)) _selectionList.remove(ql);
			}
		}
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.PROCESSES);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}

	
	
	
	
	
	
}
