/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.importer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.IChoiceRenderer;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.select2.SelectData;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.tagselector.TagSelector;
import org.biffo.dms.dataprovider.config.DPImportServers;
import org.biffo.dms.dataprovider.config.DPImportServers.ImportSource;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.dataprovider.config.DPModules;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableFilterClass;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.modules.ModuleSelectorPanel;
import org.biffo.dms.web.usergroups.UserGroupSelectorPanel;
import org.biffo.dms.web.users.UserSelectorPanel;
 

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

import de.kleinekuh.entity.types.RecordStatusType;

public class ImportServerPanel extends Panel{
	
	
	private static final String _className = "ImportServerPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private Label _lblTitel;
	private TextField<String> _processname, _queuesize;
	private ErrorLabel _processname_info, _importsource_info, _queuesize_info;
	private TextArea<String> _beschreibung;
	@SuppressWarnings("unused")
	private CheckBox _assigntousergroups, _assigntousers, _assigntags, _useki, _rebuildpdf;
	private CheckBox _startautomatic, _archiveinput; 
	
	@SuppressWarnings("unused")
	private UserGroupSelectorPanel _userGroups;
	private UserSelectorPanel _users;
	private TagSelector _tagSelector;
	private ModuleSelectorPanel _moduleSelector;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DropDownChoice<ImportSource> _importsource;
 	
	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;
	
	private Object _origin;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private List<ImportSource> _lstImportSources=new ArrayList<ImportSource>();
	private List<ISelectData> _lstUserGroups = new LinkedList<ISelectData>();
	private List<ISelectData> _lstUsers = new LinkedList<ISelectData>();
	private List<ISelectData> _lstModules = new LinkedList<ISelectData>();
	
	private ImportSource _selectedImportSource;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private ImportServerImpl _importProcess, _importProcess_Backup;
	private boolean _isStarted=false;
	
	public ImportServerPanel(String id, int idImportProcess,  Object origin) {
		this(id, null, origin);
	}
	
	public ImportServerPanel(String id, ImportServerImpl importProcess,  Object origin) {
		super(id);
		
		_origin = origin;
		_importProcess = importProcess;
		_importProcess_Backup = importProcess;

		if(null==_importProcess) {
        	_importProcess = new ImportServerImpl();
        }else {
        	_isStarted = DmsWebApplication.getProcessQueue().isRunning(_importProcess.getProcessName());
        }
		
		refreshUsergroupsList();
		refreshUsersList();
		refreshModulesList();
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
		addToolBar();
		addInfobox();
		addDropDownBoxen();
		addCheckBoxen();
		addAdditionalControls();
		addTagSelector();
		addTextFields();
		

        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		if(-1==_importProcess.getId()) {
			_importProcess = new ImportServerImpl();
			_panelTitel.setTitel(ResourceText.getText("titImporter_New", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titImporter_Edit", this));
		}
		refreshImportSources();
		
		refreshFormObjects();
	}
	
	private void addTextFields(){
		
		_processname = new TextField<String>("processname");
		_processname.setOutputMarkupId(true);
        _form.add(_processname); 

        _processname_info = new ErrorLabel("processname_info");
        _form.add(_processname_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
        
        _queuesize = new TextField<String>("queuesize");
        _queuesize.setOutputMarkupId(true);
        _form.add(_queuesize);
        
        _queuesize_info = new ErrorLabel("queuesize_info");
        _form.add(_queuesize_info);
        
	}
	
	
	private void addDropDownBoxen(){
		
		_importsource = new DropDownChoice<ImportSource>("importsource",   _lstImportSources,  new IChoiceRenderer<ImportSource>() {

			private static final long serialVersionUID = 1L;

			public Object getDisplayValue(ImportSource importsource) {
				return importsource.getName() + " ("+importsource.getType()+")";
			}

			public String getIdValue(ImportSource importsource, int index) {
				return ""+importsource.getId();
			}
		});
		
		
		_importsource.add(new AjaxFormComponentUpdatingBehavior("onchange") {

			private static final long serialVersionUID = 1L;

			@Override
			protected void onUpdate(AjaxRequestTarget target) {
				target.add(_importsource);				
			}
		});
		
		_importsource.setOutputMarkupId(true);
	    _form.add(_importsource);
	    
	    _importsource_info = new ErrorLabel("importsource_info");
        _form.add(_importsource_info);
	     
	}
	
	private void addCheckBoxen(){
        
		_assigntousers = new CheckBox("assigntousers");
		_assigntousers.setOutputMarkupId(true);
		_assigntousers.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				if(_importProcess.isAssignToUsers())_users.setDisabled(false);
				else _users.setDisabled(true);
				target.add(_assigntousers);
                target.add(_users);
            }
        }); 
        _form.add(_assigntousers);
        
		_assigntags = new CheckBox("assigntags");
		_assigntags.setOutputMarkupId(true);
		_assigntags.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				if(_importProcess.isAssignTags())_tagSelector.setDisabled(false);
				else _tagSelector.setDisabled(true);
				target.add(_assigntags);
                target.add(_tagSelector);
            }
        }); 
        _form.add(_assigntags);
        
        _startautomatic = new CheckBox("startautomatic");
		_startautomatic.setOutputMarkupId(true);
		_startautomatic.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(_startautomatic);
            }
        }); 
        _form.add(_startautomatic);

        _useki = new CheckBox("useki");
        _useki.setOutputMarkupId(true);
        _useki.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				if(_importProcess.isUseKi())_moduleSelector.setDisabled(false);
				else _moduleSelector.setDisabled(true);
				target.add(_useki);
                target.add(_moduleSelector);
            }
        }); 
        _form.add(_useki);
        
        _rebuildpdf = new CheckBox("rebuildpdf");
        _rebuildpdf.setOutputMarkupId(true);
        _rebuildpdf.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				target.add(_rebuildpdf);
            }
        }); 
        _form.add(_rebuildpdf);

        
        _archiveinput = new CheckBox("archiveinput");
        _archiveinput.setOutputMarkupId(true);
        _archiveinput.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				target.add(_archiveinput);
            }
        }); 
        _form.add(_archiveinput);
        
        
	}
	
	
	
	private void addAdditionalControls(){
        
        _users = new UserSelectorPanel("users",  _lstUsers);
        _users.setOutputMarkupId(true);
        _users.setDisabled(true);
        _form.add(_users);
        
        
   		_moduleSelector = new ModuleSelectorPanel("moduleselector", _lstModules);
    	_moduleSelector.setOutputMarkupId(true);
    	_moduleSelector.setDisabled(true);
   		_form.add(_moduleSelector);
        
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void addTagSelector() {
		_tagSelector = new TagSelector("tagselector");
		_tagSelector.setOutputMarkupId(true);
		_tagSelector.setDisabled(true);
		refreshTagList();
		_form.add(_tagSelector);
	}
	

	
	
	private void refreshTagList() {
		
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(_importProcess.getId());
			rfc.setRelationType(RelationType.TAG_TO_IMPORT_PROCESS);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			List<Tag> lstTags = new LinkedList<Tag>();
			for(Relation relation : alRelations) {
				lstTags.add(StaticLocators.getTags().getEntityById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstTags, new Comparator<Tag>() {
			    @Override
			    public int compare(Tag o1, Tag o2) {
			        return o1.getTagName().compareToIgnoreCase(o2.getTagName());
			    }
			});
			if(_importProcess.getId()!=-1) _tagSelector.setSelectedTags(lstTags);
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshTagList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void refreshImportSources() {
		try {
			
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(_importProcess.getId());
			rfc.setRecordStatus(RecordStatusType.ACTIVE);
			rfc.setRelationStatus(RelationStatusType.ACTIVE);
			rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
		
			int selectedImportSourceID=1;
			List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(null!=entities && entities.size()==1){
				selectedImportSourceID= entities.get(0).getIdrelation1();
			}
			
			DPImportServers dpImportSources = new DPImportServers();
			_lstImportSources.addAll(dpImportSources.getImportSources());
			
			for(ImportSource importSource : _lstImportSources) {
				if(importSource.getId()==selectedImportSourceID) {
					_selectedImportSource = importSource;
					break;
				}
			} 
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshImportSources",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	private void refreshUsersList(){
		try{
			List<User> availabelUsers = StaticLocators.getUsers().getEntities(-1, -1, " order by benutzername ", null);
			for(User us : availabelUsers) {
				if(us.getAccessType()==UsersAccessType.ADMIN || us.getAccessType()==UsersAccessType.USER) {
					boolean found=false;
					for(ISelectData data : _lstUsers) {
						if(data.getId()==us.getId()) {
							data.setSelected(false);
							found=true;
						}
					}
					if(!found) {
						_lstUsers.add(new SelectData(us.getId(), us.getBenutzerName(), false, false));
					}
				}
			}
			
			if(_importProcess.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation2(_importProcess.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.USER_TO_IMPORT_PROCESS);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstUsers) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshUsersList",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return;
	}
	
	
	
	
	private void refreshModulesList(){
		try{
			
			DPModules dpModules = new DPModules();
			long numberOfModules = dpModules.size();
			Iterator<? extends ApiModuleImpl> itt = dpModules.iterator(0, numberOfModules);
			while(itt.hasNext()) {
				ApiModuleImpl module = itt.next();
				boolean found=false;
				for(ISelectData data : _lstModules) {
					if(data.getId()==module.getId()) {
						data.setSelected(false);
						found=true;
					}
				}
				if(!found) {
					_lstModules.add(new SelectData(module.getId(), module.getClassname(), false, false));
				}
			}
			
			if(_importProcess.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation2(_importProcess.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.MODULE_TO_IMPORT_PROCESS);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstModules) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshModulesList",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);

		}
		return;
	} 
	
	

	private void refreshUsergroupsList(){
		try{
			List<UserGroup> availabelUserGroups = StaticLocators.getUserGroups().getEntities(-1, -1, " order by name", null);
			for(UserGroup ug : availabelUserGroups) {
				boolean found=false;
				for(ISelectData data : _lstUserGroups) {
					if(data.getId()==ug.getId()) {
						data.setSelected(false);
						found=true;
					}
				}
				if(!found) {
					_lstUserGroups.add(new SelectData(ug.getId(), ug.getName(), false, false));
				}
			}
			
 			if(_importProcess.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation2(_importProcess.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.USERGROUP_TO_IMPORT_PROCESS);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstUserGroups) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"refreshUsergroupsList",
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return;
	} 
	
	public ImportSource getSelectedImportSource() {
		return _selectedImportSource;
	}
	public void setSelectedImportSource(ImportSource importSource) {
		_selectedImportSource = importSource;
	}	
	
	private void refreshFormObjects(){
		
		_processname.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "ProcessName"));
		_startautomatic.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "ServerAutostart"));
		_beschreibung.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "Description"));
		_assigntousers.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "AssignToUsers"));
		_assigntags.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "AssignTags"));
		_useki.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "UseKi"));
		_rebuildpdf.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "RebuildPdf"));
		
		_archiveinput.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "ArchiveafterImport"));
		_queuesize.setDefaultModel(new PropertyModel<ImportServerImpl>(_importProcess, "Queuesize"));
		
		_importsource.setDefaultModel(new PropertyModel<ImportServerPanel>(this, "SelectedImportSource"));
		
		
		
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		if(!_isStarted)_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
	
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);
		target.add(_form);
		target.add(_toolBar);
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		if(_isStarted) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("warProcessStarted", this), INFOBOX_STATUS.warning);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	private void setFormComponentsEditable(boolean editable){
		try{
			
			_processname.setEnabled(editable);
			_beschreibung.setEnabled(editable);
			_queuesize.setEnabled(editable);
			
			_assigntousers.setEnabled(editable);
			_assigntags.setEnabled(editable);
			_useki.setEnabled(editable);
			_rebuildpdf.setEnabled(editable);
			_startautomatic.setEnabled(editable);
			_archiveinput.setEnabled(editable);
			_importsource.setEnabled(editable);
			_users.setDisabled(!editable);
			_tagSelector.setDisabled(!editable);
			_moduleSelector.setDisabled(!editable);
			
			if(editable && _importProcess.isAssignTags()) {
				_tagSelector.setDisabled(false);
			}else {
				_tagSelector.setDisabled(true);
			}
			
			if(editable && _importProcess.isAssignToUsers()) {
				_users.setDisabled(false);
			}else {
				_users.setDisabled(true);
			}
			
			if(editable && _importProcess.isUseKi()) {
				_moduleSelector.setDisabled(false);
			}else {
				_moduleSelector.setDisabled(true);
			}
			
		}catch(Exception e){
		}
	}
	
	private boolean checkShowError(ImportServerImpl importProcess, AjaxRequestTarget target) {
		
		boolean error=false;
		if(importProcess==null) return false;
		
		if(Strings.isEmpty(importProcess.getProcessName())){
			_processname_info.setErrorText(ResourceText.getText("errProcessName_NoName", this));
			_processname_info.setVisible(true);
			error = true;
		}else if(checkProcessNameExists()){
			_processname_info.setErrorText(ResourceText.getText("errProcessName_Dublicate", this));
			_processname_info.setVisible(true);
			error = true;
		}else {
			_processname_info.setVisible(false);
		}
		
		if(!_selectedImportSource.getType().equals("Default")) {
			
			ImportServerImpl tempImportServer = checkImportSourceAllreadyUsed();
			if(tempImportServer!=null && tempImportServer.getId()!=_importProcess.getId()){
				_importsource_info.setErrorText(ResourceText.getText("errImportSource_Used", this, tempImportServer.getProcessName()));
				_importsource_info.setVisible(true);
				error = true;
			}else {
				_importsource_info.setVisible(false);
			}
		}
		
		
		target.add(_importsource_info);
		target.add(_processname_info);
		return error;
		
	}
	
	private ImportServerImpl checkImportSourceAllreadyUsed() {
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
			rfc.setIdrelation1(_selectedImportSource.getId());
			List<Relation> lstRelations = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			if(lstRelations==null || lstRelations.size()==0) return null;
			
			DPImporters dpImporters = new DPImporters();
			return dpImporters.getImportServerById(lstRelations.get(0).getIdrelation2());
		}catch(Exception e) {
			return null;
		}
		 
	}
	
	

	private boolean checkProcessNameExists() {
		try {
			
			VariableFilterClass vfc = new VariableFilterClass();
			vfc.setVariableGroupType(VariableGroupType.IMPORT_PROCESS);
			vfc.setVariableGroup(_importProcess.getProcessName());
			vfc.setVariableName("processname");
			Variable variable = StaticLocators.getVariables().getEntity(vfc);
			if(null==variable) return false;
			else if(variable.getId()==_importProcess.getId()) return false;
		}catch(Exception e) {
		}
		return true;
	}	
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_importProcess, target))return false;
			
			HashMap<String, Variable> hmVariables = _importProcess.getVariables();
			Set<String> keys = hmVariables.keySet();
			if(_importProcess.getId()!=-1) {
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					if(var.getId()==-1) {
						var.setVariableGroup(key);
					}
					StaticLocators.getVariables().updateEntity(var, true);
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
					_className, 
					"save",
					ResourceText.getText("dmsInfoImportProcessChanged", this, _importProcess.getServerName()),
					null,
					_importProcess
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}else {
				Date erstellungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setErstelltAm(erstellungsDatum);
					var.setGeaendertAm(var.getErstelltAm());

					var.setErstelltVon(_session.getCurrentUser());
					var.setGeaendertVon(var.getErstelltVon());
					var.setUUID(EntityUUID.createVariableUUID(var));
					int varID = StaticLocators.getVariables().getEntityId(var);
					if(-1==varID) {
						varID=StaticLocators.getVariables().addEntity(var);
						if(var.getVariableName().equals("processname"))_importProcess.setId(varID);
					}else {
						StaticLocators.getVariables().updateEntity(var, true);
					}
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
					_className, 
					"save",
					ResourceText.getText("dmsInfoImportProcessCreated", this, _importProcess.getServerName()),
					null,
					_importProcess
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				
			}

			saveSingleRelation(_importProcess, RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
			DmsDBHelpersTag.addReplaceRelations(_importProcess, _lstUserGroups, RelationType.USERGROUP_TO_IMPORT_PROCESS, _session.getCurrentUser());
			DmsDBHelpersTag.addReplaceRelations(_importProcess, _lstUsers, RelationType.USER_TO_IMPORT_PROCESS, _session.getCurrentUser());
			DmsDBHelpersTag.addReplaceRelations(_importProcess, _tagSelector.getDataSelection(true), RelationType.TAG_TO_IMPORT_PROCESS, _session.getCurrentUser());
			DmsDBHelpersTag.addReplaceRelations(_importProcess, _lstModules, RelationType.MODULE_TO_IMPORT_PROCESS, _session.getCurrentUser());
			
			_importProcess_Backup = _importProcess;
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infoProcessSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errProcessNotSaved", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	

	private void saveSingleRelation(ImportServerImpl importProcess, RelationType relationType) throws Exception{
		
			
		RelationFilterClass rfc = new RelationFilterClass();
		rfc.setIdrelation2(importProcess.getId());
		rfc.setRecordStatus(RecordStatusType.ACTIVE);
		rfc.setRelationStatus(RelationStatusType.ACTIVE);
		rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
		List<Relation> lstRealtions = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
		for(Relation relation : lstRealtions)StaticLocators.getRelations().deleteEntityById(relation.getId());
			
		Relation relation = new Relation();
		relation.setErstelltVon(_session.getCurrentUser());
		relation.setErstelltAm(new Date());
		relation.setRecordStatus(RecordStatusType.ACTIVE);
		relation.setRelationStatus(RelationStatusType.ACTIVE);
		relation.setGeaendertVon(relation.getErstelltVon());
		relation.setGeaendertAm(relation.getErstelltAm());
		relation.setIdrelation1(_selectedImportSource.getId());
		relation.setIdrelation2(_importProcess.getId());
		relation.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
		int idTemp = StaticLocators.getRelations().getEntityId(relation);
		if(idTemp==-1) {
			StaticLocators.getRelations().addEntity(relation);
		}
	}
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _importProcess = _importProcess_Backup;
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			 infobox.setVisible(true);
			 target.add(wmcInfo);			 
		 }

	}
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_importProcess.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+".\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}
	
	 
		
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(ImportServerPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}

	
}

 
