/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.importer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.event.Broadcast;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.modulebox.ModulesBoxPanel;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.components.tagbox.TagsBoxPanel;
import org.biffo.dms.components.userbox.UsersBoxPanel;
import org.biffo.dms.dataprovider.config.DPImporters;
import org.biffo.dms.dataprovider.config.DPModules;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.entity.Variables.Impl.ImportServerImpl;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.helpers.ImportProcessHelpers;
import org.biffo.dms.entity.Variables.Impl.ApiModuleImpl;
import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;

import org.biffo.dms.processQueue.ProcessLaunchedEvent;
import org.biffo.dms.processQueue.importServer.ImportProcessTask;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.administration.AdministrationPage;
import org.biffo.dms.web.administration.AdministrationTabedPanel.TabContent;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;
import org.biffo.dms.web.server.ServerControlPanel;


import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 


public class ImportServerPanelViewList extends Panel{

	private static final String _className = "ImportServerPanelViewList";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	
	
	private DPImporters _qtdp = new DPImporters();
	private DefaultDataTable<?, ?> _ddt;
	private Set<ImportServerImpl> _selectionList = new HashSet<ImportServerImpl>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete, _btStart, _btStop;
	private Form<?> _form;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	private StandardListParameters _thisParameters;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	private List<IVariablesImpl> lstServers = new LinkedList<IVariablesImpl>();
	
	@Override
	protected void onBeforeRender() {
		if(_qtdp!=null) {
			_qtdp.refresh();
			_qtdp.size();
		}
		super.onBeforeRender();
	}
	
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public ImportServerPanelViewList(String id) {
		super(id);
		
		initParameters();
		setOutputMarkupId(true);
		addInfobox();
		addModalDefault();
		_form = new Form("dataform");
		_form.setOutputMarkupId(true);
		add(_form);
		
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<ImportServerImpl>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<ImportServerImpl> rowModel) {
				
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		
		
		columns.add(new LinkColumn(ResourceText.getModelText("colServerName", this), "serverName", "serverName") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(Item item, String componentId, IModel model) {
				ImportServerImpl fn = (ImportServerImpl)model.getObject();
				ImportServerPanel panel = new ImportServerPanel(ImportServerPanelViewList.this.getId(), fn, ImportServerPanelViewList.this);
				ImportServerPanelViewList.this.replaceWith(panel);
			}
			
		});
		columns.add(new AbstractColumn<ImportServerImpl, String>(new Model<String>(ResourceText.getText("colServerAutostart", this))){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ImportServerImpl>> item, String componentId, IModel<ImportServerImpl> rowModel) {
				Label lbl = null;
				if(rowModel.getObject().isServerAutostart()) {
					lbl = new Label(componentId, "&#xe834;");
				}else {
					lbl = new Label(componentId, "&#xe835;");
				}
				lbl.add(new AttributeModifier("class", "tablesymbol"));
				lbl.setEscapeModelStrings(false);
				item.add(lbl);
			}
			
		});
		

		columns.add(new AbstractColumn<IVariablesImpl, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<IVariablesImpl>> item, String componentId, IModel<IVariablesImpl> rowModel) {
				
				ImportServerImpl fn = (ImportServerImpl)rowModel.getObject();
				if(DmsWebApplication.getProcessQueue().isRunning(fn.getProcessName())) {
					rowModel.getObject().setServerActive(true);
				}else {
					rowModel.getObject().setServerActive(false);
				}
				
				ServerControlPanel ftpServerControlPanel = new ServerControlPanel(componentId, fn) {
					private static final long serialVersionUID = 1L;
					@Override
					protected void OnServerStart(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ImportServerPanelViewList.this.processEvent("start", target, importProcess);
					}
					@Override
					protected void OnServerStop(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ImportServerPanelViewList.this.processEvent("stop", target, importProcess);
					}
					@Override
					protected void OnServerSync(AjaxRequestTarget target, IVariablesImpl importProcess) {
						ImportServerPanelViewList.this.processEvent("sync", target, importProcess);
					}
				};
				ftpServerControlPanel.setOutputMarkupId(true);
				item.add(ftpServerControlPanel);
			}
			
		}); 
		
		columns.add(new AbstractColumn<ImportServerImpl, String>(ResourceText.getModelText("colServerBeschreibung", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ImportServerImpl>> item, String componentId, IModel<ImportServerImpl> rowModel) {
				
				String temp = "NA";
				try {
					if(null!=rowModel.getObject().getDescription()) {
						temp = rowModel.getObject().getDescription();
						if(temp.length()>100)temp = temp.substring(0,100);
					};
				}catch(Exception e) {}
				Label lbl = new Label(componentId, temp);
				item.add(lbl);
				
				
			}
		});
		
		columns.add(new AbstractColumn<ImportServerImpl, String>(ResourceText.getModelText("colServerSource", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<ImportServerImpl>> item, String componentId, IModel<ImportServerImpl> rowModel) {
				
				String temp = "NA";
				try{
					
					RelationFilterClass rfc = new RelationFilterClass();
					rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
					rfc.setIdrelation2(rowModel.getObject().getId());

					Relation relation = StaticLocators.getRelations().getEntity(rfc);
					for(IVariablesImpl serverProcess : lstServers) {
						if(serverProcess.getId() == relation.getIdrelation1()) {
							temp = serverProcess.getServerName();
							break;
						}
					}
				}catch(Exception e){}
				item.add(new Label(componentId, temp));	
			}
			
		});
		
		columns.add(new AbstractColumn<IVariablesImpl, String>(ResourceText.getModelText("colServerUser", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<IVariablesImpl>> item, String componentId, IModel<IVariablesImpl> rowModel) {
				ImportServerImpl fn = (ImportServerImpl)rowModel.getObject();
				
				if(fn.isAssignToUsers()) {
					UsersBoxPanel usersBox = new UsersBoxPanel(componentId);
					usersBox.setUsers(refreshUsersList(fn.getId()));
					usersBox.setUsersBoxClassName("associatedusers");
					usersBox.setUserClassName("associateduser");
					usersBox.setOutputMarkupId(true);
					item.add(usersBox);
				}else {
					item.add(new Label(componentId, "NA"));
				}
			}
		});
		

		
		columns.add(new AbstractColumn<IVariablesImpl, String>(ResourceText.getModelText("colServerTags", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<IVariablesImpl>> item, String componentId, IModel<IVariablesImpl> rowModel) {
				ImportServerImpl fn = (ImportServerImpl)rowModel.getObject();
				if(fn.isAssignTags()) {
					TagsBoxPanel tagsBox = new TagsBoxPanel(componentId);
					tagsBox.setTagsBoxClassName("associatedtags");
					tagsBox.setTagClassName("associatedtag");
					tagsBox.setTags(refreshTagList(fn.getId()));
					tagsBox.setOutputMarkupId(true);
					item.add(tagsBox);
				}else {
					item.add(new Label(componentId, "NA"));
				}
			}
		});
		

		
		columns.add(new AbstractColumn<IVariablesImpl, String>(ResourceText.getModelText("colServerModule", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<IVariablesImpl>> item, String componentId, IModel<IVariablesImpl> rowModel) {
				ImportServerImpl fn = (ImportServerImpl)rowModel.getObject();
				
				if(fn.isUseKi()){
					ModulesBoxPanel modulesBox = new ModulesBoxPanel(componentId);
					modulesBox.setModules(refreshModulesList(fn.getId()));
					modulesBox.setModulesBoxClassName("associatedmodules");
					modulesBox.setModuleClassName("associatedmodule");
					modulesBox.setOutputMarkupId(true);
					item.add(modulesBox);
				}else {
					item.add(new Label(componentId, "NA"));
				}
			}
		}); 
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setOutputMarkupId(true);
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		addToolBar(); 
		_form.add(_ddt);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
	}
	
	private void deleteRelation(int processId, int tenantId, RelationType relationType) {
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setRelationType(relationType);
			rfc.setIdrelation2(processId);
			rfc.setIdTenant(tenantId);
			List<Relation> lstRelations = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
			for(Relation relation: lstRelations) {
				StaticLocators.getRelations().deleteEntityById(relation.getId());
			}
		}catch(Exception e) {
			
			DmsException dmsException = new DmsException(
					1000, 
					"E", 
					_className, 
					"deleteRelation",
					null,
					null,
					e
				);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			for(ImportServerImpl importProcess : _selectionList) {
				if(DmsWebApplication.getProcessQueue().hasTask(importProcess.getProcessName())) {
					if(!DmsWebApplication.getProcessQueue().isStopped(importProcess.getProcessName())) {
						throw new Exception(ResourceText.getText("errServerStarted", this, importProcess.getProcessName()));
					}else {
						DmsWebApplication.getProcessQueue().remove(importProcess.getProcessName());
					}
				}
				
				deleteRelation(importProcess.getId(), _session.getCurrentTenantId(), RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
				deleteRelation(importProcess.getId(), _session.getCurrentTenantId(), RelationType.TAG_TO_IMPORT_PROCESS);
				deleteRelation(importProcess.getId(), _session.getCurrentTenantId(), RelationType.USER_TO_IMPORT_PROCESS);
				deleteRelation(importProcess.getId(), _session.getCurrentTenantId(), RelationType.USERGROUP_TO_IMPORT_PROCESS);

				
				Set<String> keys = importProcess.getVariables().keySet();
				for(String key: keys) {
					StaticLocators.getVariables().deleteEntityById(importProcess.getVariables().get(key).getId());
				}
				DmsException dmsException = new DmsException(
						2000, 
						"I", 
						_className, 
						"deleteEntities",
						ResourceText.getText("dmsInfoServerDeleted", this, importProcess.getProcessName()),
						null,
						importProcess
					);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
				
			}
			_selectionList = new HashSet<ImportServerImpl>();
			infobox.refresh(target, ResourceText.getText("infImportProcessDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
			target.add(_form);
						
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errImportProcessDeleted", this)+"\n"+e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		} 
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target, null);
			}
		};
		
		
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		_btStart = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStart.setUtf8Code("xf0d1");
		_btStart.setEventName("start");
		_btStart.setAlternate(ResourceText.getText("tbAltStart", this));
		_toolBar.add(_btStart);

		_btStop = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStop.setUtf8Code("xef71");
		_btStop.setEventName("stop");
		_btStop.setAlternate(ResourceText.getText("tbAltStop", this));
		_toolBar.add(_btStop);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}

	
	public Set<ImportServerImpl> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		
		int idDefaultPanel = DmsDBHelpersTag.getDefaultPDFImporterID();
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			ImportServerImpl ql =(ImportServerImpl)iter.next();
			if(ql.getId()!=idDefaultPanel) {
				if(select){
					_selectionList.add(ql);
				}else{
					_selectionList.remove(ql);
				}
			}
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target, IVariablesImpl serverImpl) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			_qtdp.refresh();
			_qtdp.size();
			target.add(_dpn);
			target.add(_ddt);
		}else if(event.equals("start")) {
			if(_selectionList.size()>0 || serverImpl!=null)startImportProcesses(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
		}else if(event.equals("stop")) {
			if(_selectionList.size()>0 || serverImpl!=null)stopImportProcesses(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
		}else if(event.equals("sync")) {
			if(_selectionList.size()>0 || serverImpl!=null)refreshTask(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
			target.add(_form);
		}else if(event.equals("Add")) {
			AdministrationPage ap = new AdministrationPage(getPage().getPageReference(), TabContent.IMPORTERS, true);
			setResponsePage(ap);
		}
	}
	
	
	private void startImportProcesses( AjaxRequestTarget target, IVariablesImpl importProcessHelper) {
		if(importProcessHelper!=null) {
			startTask(target, importProcessHelper);
		}else {
			for(ImportServerImpl importServer : _selectionList) {
				startTask(target, importServer);
			}
		}
		_selectionList = new HashSet<ImportServerImpl>();
		infobox.refresh(target, ResourceText.getText("infImportProcessStarted", this), INFOBOX_STATUS.success);
		infobox.setVisible(true);
		target.add(wmcInfo);
		target.add(_form);
	}
	private void stopImportProcesses( AjaxRequestTarget target,  IVariablesImpl importProcessHelper) {
		if(importProcessHelper!=null) {
			stopTask(target, importProcessHelper);
		}else {
			for(ImportServerImpl importProcess : _selectionList) {
				stopTask(target, importProcess);
			}
		}
		_selectionList = new HashSet<ImportServerImpl>();
		infobox.refresh(target, ResourceText.getText("infImportProcessStopped", this), INFOBOX_STATUS.success);
		infobox.setVisible(true);
		target.add(wmcInfo);
		target.add(_form);
	}
	

	private void startTask(AjaxRequestTarget target,  IVariablesImpl importProcessHelper) {
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		ImportServerImpl fn = (ImportServerImpl)importProcessHelper;
		if(!DmsWebApplication.getProcessQueue().hasTask(fn.getProcessName())) {
			DmsWebApplication.getProcessQueue().add( new ImportProcessTask(fn));
			getPage().send(getPage(), Broadcast.DEPTH, new ProcessLaunchedEvent(target));
		}else {
			if(!DmsWebApplication.getProcessQueue().isRunning(fn.getProcessName())){
				DmsWebApplication.getProcessQueue().start(fn.getProcessName());
			}
		}
	}

	private void stopTask(AjaxRequestTarget target,  IVariablesImpl importProcessHelper) {
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		ImportServerImpl fn = (ImportServerImpl)importProcessHelper;
		if(DmsWebApplication.getProcessQueue().hasTask(fn.getProcessName())) {
			if(DmsWebApplication.getProcessQueue().isRunning(fn.getProcessName())){
				DmsWebApplication.getProcessQueue().stop(fn.getProcessName());
			}
		}
	}

	private void removeTask(AjaxRequestTarget target,  IVariablesImpl importProcessHelper) {
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		ImportServerImpl fn = (ImportServerImpl)importProcessHelper;
		if(DmsWebApplication.getProcessQueue().hasTask(fn.getProcessName())) {
			if(DmsWebApplication.getProcessQueue().isRunning(fn.getProcessName())){
				DmsWebApplication.getProcessQueue().stop(fn.getProcessName());
			}
			DmsWebApplication.getProcessQueue().remove(fn.getProcessName());
		}
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
	}
	
	private void refreshTask(AjaxRequestTarget target,  IVariablesImpl importProcessHelper) {
		removeTask(target, importProcessHelper);
		startTask(target, importProcessHelper);
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.IMPORTERS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
		
		try {
			lstServers.addAll(ImportProcessHelpers.getServerProcesses());
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"initParameters",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		
	}
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_qtdp.refresh();
				_qtdp.size();
				target.add(_dpn);
				target.add(_ddt);
				target.add(_form);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }

	private List<Tag> refreshTagList(int implId) {
		List<Tag> lstTags = new LinkedList<Tag>();
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(implId);
			rfc.setRelationType(RelationType.TAG_TO_IMPORT_PROCESS);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			
			for(Relation relation : alRelations) {
				lstTags.add(StaticLocators.getTags().getEntityById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstTags, new Comparator<Tag>() {
			    @Override
			    public int compare(Tag o1, Tag o2) {
			        return o1.getTagName().compareToIgnoreCase(o2.getTagName());
			    }
			});
		}catch(Exception e) { 
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshTagList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return lstTags;
	}
	
	private List<User> refreshUsersList(int implId) {
		List<User> lstUsers = new LinkedList<User>();
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(implId);
			rfc.setRelationType(RelationType.USER_TO_IMPORT_PROCESS);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			
			for(Relation relation : alRelations) {
				lstUsers.add(StaticLocators.getUsers().getEntityById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstUsers, new Comparator<User>() {
			    @Override
			    public int compare(User o1, User o2) {
			        return o1.getBenutzerName().compareToIgnoreCase(o2.getBenutzerName());
			    }
			});
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshUsersList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return lstUsers;
	}
	
	 
	private List<ApiModuleImpl> refreshModulesList(int implId) {
		List<ApiModuleImpl> lstModules = new LinkedList<ApiModuleImpl>();
		try {
			RelationFilterClass rfc = new RelationFilterClass();
			rfc.setIdrelation2(implId);
			rfc.setRelationType(RelationType.MODULE_TO_IMPORT_PROCESS);
		
			ArrayList<Relation> alRelations = new ArrayList<Relation>();
			alRelations.addAll(StaticLocators.getRelations().getEntities(-1, -1, "", rfc));
			
			DPModules dpModules = new DPModules();
			for(Relation relation : alRelations) {
				lstModules.add(dpModules.getApiModuleById(relation.getIdrelation1()));
			}
			
			Collections.sort(lstModules, new Comparator<ApiModuleImpl>() {
			    @Override
			    public int compare(ApiModuleImpl o1, ApiModuleImpl o2) {
			        return o1.getServerName().compareToIgnoreCase(o2.getServerName());
			    }
			});
			
			
			
		}catch(Exception e) {
			DmsException dmsException = new DmsException(
				1000, 
				"E", 
				_className, 
				"refreshModulesList",
				null,
				e
			);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
		return lstModules;
	}
	
	
	

}
