/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.sftp;


import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.select2.SelectData;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.SFTPServerImpl;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UserFilterClass;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.ConfigHelpers;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.users.UserSelectorPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
import de.kleinekuh.entity.types.RecordStatusType;

public class SFTPServerPanel extends Panel{

	private static final String _className="SFTPServerPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextField<String> _sftpname, _sftpport, _hostprivatekey, _uploadfolder;
	private ErrorLabel _sftpname_info, _sftpport_info, _hostprivatekey_info, _uploadfolder_info, _sftpusers_info;
	
	
	private TextArea<String> _beschreibung;
	private Label _lblTitel;
	
	private UserSelectorPanel _users;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;
	
	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();
	 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private SFTPServerImpl _sftpServer, _sftpServer_Backup;
	private boolean _isStarted=false;
	
	
	private List<ISelectData> _lstUsers = new LinkedList<ISelectData>();
	
	public SFTPServerPanel(String id, int idSftpServer,  Object origin) {
		this(id, null, origin);
	}
	
	public SFTPServerPanel(String id, SFTPServerImpl sftpServer,  Object origin) {
		super(id);
		_origin = origin;
		_sftpServer = sftpServer;
		_sftpServer_Backup = sftpServer;

        if(null==_sftpServer) {
        	_sftpServer = new SFTPServerImpl();
        }else {
        	_isStarted = DmsWebApplication.getProcessQueue().isRunning(_sftpServer.getServerName());
        }

		refreshUsersList();
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		if(-1==_sftpServer.getId()) {
			_sftpServer = new SFTPServerImpl();
			_panelTitel.setTitel(ResourceText.getText("titNewSFTPServer", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditSFTPServer", this));
		}
	 
		
		refreshFormObjects();
	
	}
	
	private void addTextFields(){
		
				
		_sftpname= new TextField<String>("sftpname");
		_sftpname.setOutputMarkupId(true);
        _form.add(_sftpname); 
        _sftpname_info = new ErrorLabel("sftpname_info");
        _form.add(_sftpname_info);
		
		_sftpport= new TextField<String>("sftpport");
		_sftpport.setOutputMarkupId(true);
	    _form.add(_sftpport);
	    _sftpport_info = new ErrorLabel("sftpport_info");
        _form.add(_sftpport_info);
		
		_hostprivatekey= new TextField<String>("hostprivatekey");
		_hostprivatekey.setOutputMarkupId(true);
		_form.add(_hostprivatekey);
	    
	    _hostprivatekey_info = new ErrorLabel("hostprivatekey_info");
	    _form.add(_hostprivatekey_info);
        
        _uploadfolder= new TextField<String>("uploadfolder");
        _uploadfolder.setOutputMarkupId(true);
        _form.add(_uploadfolder);
        
        _uploadfolder_info = new ErrorLabel("uploadfolder_info");
        _form.add(_uploadfolder_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	
	private void addCheckBoxen(){
	}
	
	private void addAdditionalControls(){
        
        _users = new UserSelectorPanel("users",  _lstUsers);
        _users.setOutputMarkupId(true);
        _users.setDisabled(true);
        _form.add(_users);
        
        _sftpusers_info = new ErrorLabel("sftpusers_info");
        _form.add(_sftpusers_info);
		
		
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(SFTPServerPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		if(!_isStarted)_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Edit")) edit(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);

		target.add(_form);
		target.add(_toolBar);
	}
	
	private boolean checkShowError(SFTPServerImpl serverImpl, AjaxRequestTarget target) {
		
		boolean error=false;
		if(serverImpl==null) return false;
		
		 
		if(Strings.isEmpty(serverImpl.getServerName())){
			
			_sftpname_info.setErrorText(ResourceText.getText("errTextServerNameWrong", this));
			_sftpname_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkServerNameExists(_sftpServer.getId(), _sftpServer.getServerName(), VariableGroupType.SFTP)){
			_sftpname_info.setErrorText(ResourceText.getText("errTextServerNameExists", this));
			_sftpname_info.setVisible(true);
			error = true;
		}else {
			_sftpname_info.setVisible(false);
		}
		
		if(_sftpServer.getServerPort()<=0){
			_sftpport_info.setErrorText(ResourceText.getText("errTextServerPortWrong", this));
			_sftpport_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkServerPortExists(_sftpServer.getId(), _sftpServer.getServerPort(), VariableGroupType.SFTP)){
			_sftpport_info.setErrorText(ResourceText.getText("errTextServerPortExists", this));
			_sftpport_info.setVisible(true);
			error=true;
		}else if(ConfigHelpers.portUsed("localhost", _sftpServer.getServerPort())){
			_sftpport_info.setErrorText(ResourceText.getText("errTextServerPortUsed", this));
			_sftpport_info.setVisible(true);
		}else {
			_sftpport_info.setVisible(false);
		}
		
 
		
		if(Strings.isEmpty(_sftpServer.getServerFolder())){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderWrong", this));
			_uploadfolder_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkUploadFolderExists(_sftpServer.getId(), _sftpServer.getVariableByKey("folder").getId(), _sftpServer.getServerFolder(), VariableGroupType.SFTP)){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderUsed", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.checkFileSystemFolderExists(_sftpServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotExists", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.isWritable(_sftpServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotWritable", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else {
			_uploadfolder_info.setVisible(false);
		}
		
		_hostprivatekey_info.setVisible(false);
		String tempHostPrivateKey = _sftpServer.getHostPrivateKey();
		if(Strings.isEmpty(tempHostPrivateKey)){
			_hostprivatekey_info.setErrorText(ResourceText.getText("errTextHostPrivateKeyWrong", this));
			_hostprivatekey_info.setVisible(true);
			error = true;
		}else {
			File fIn = new File(tempHostPrivateKey);
			String tempFolder = fIn.getParent();
			String tempFile = fIn.getName();
			if(!tempHostPrivateKey.endsWith(tempFile)) {
				_hostprivatekey_info.setErrorText(ResourceText.getText("errTextHostPrivateKeyWrong", this));
				_hostprivatekey_info.setVisible(true);
				error=true;
			}else if(!ConfigHelpers.checkFileSystemFolderExists(tempFolder)) {
				_hostprivatekey_info.setErrorText(ResourceText.getText("errTextServerFolderNotExists", this));
				_hostprivatekey_info.setVisible(true);
				error=true;
			}else if(!ConfigHelpers.isWritable(tempFolder)) {
				_hostprivatekey_info.setErrorText(ResourceText.getText("errTextServerFolderNotWritable", this));
				_hostprivatekey_info.setVisible(true);
				error=true;
			}
			
		}
		
		
		if(_users.getSelectionList().size()==0) {
			_sftpusers_info.setErrorText(ResourceText.getText("errUsersNotSelected", this));
			_sftpusers_info.setVisible(true);
			error=true;
		}else {
			_sftpusers_info.setVisible(false);
		}
		
		target.add(_sftpname_info);
		target.add(_sftpport_info);		
		target.add(_uploadfolder_info);
		target.add(_sftpusers_info);
		target.add(_hostprivatekey_info);
	 
		return error;
		
	}
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_sftpServer, target))return false;
			
			HashMap<String, Variable> hmVariables = _sftpServer.getVariables();
			Set<String> keys = hmVariables.keySet();
			if(_sftpServer.getId()!=-1) {
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					StaticLocators.getVariables().updateEntity(var, true);
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoSFTPServerChanged", this, _sftpServer.getServerName()),
						null,
						_sftpServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}else {
				
				Date erstellungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setErstelltAm(erstellungsDatum);
					var.setGeaendertAm(var.getErstelltAm());
					var.setErstelltVon(_session.getCurrentUser());
					var.setGeaendertVon(var.getErstelltVon());
					var.setUUID(EntityUUID.createVariableUUID(var));
					int varID = StaticLocators.getVariables().getEntityId(var);
					if(-1==varID) {
						varID = StaticLocators.getVariables().addEntity(var);
						if(var.getVariableName().equals("port"))_sftpServer.setId(varID);
					}else {
						StaticLocators.getVariables().updateEntity(var, true);
					}
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoSFTPServerCreated", this, _sftpServer.getServerName()),
						null,
						_sftpServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				
			}
			
			DmsDBHelpersTag.addReplaceRelations(_sftpServer, _lstUsers, RelationType.SFTP_USER_TO_SFTP_SERVER, _session.getCurrentUser());
			
			_sftpServer_Backup = _sftpServer;
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infSFTPServerSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errSFTPServerSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _sftpServer = _sftpServer_Backup;
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_sftpServer.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}
	
	 
		
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private void refreshFormObjects(){
	    _sftpname.setDefaultModel(new PropertyModel<String>(_sftpServer, "ServerName"));
	    _sftpport.setDefaultModel(new PropertyModel<String>(_sftpServer, "ServerPort"));
        _uploadfolder.setDefaultModel(new PropertyModel<String>(_sftpServer, "ServerFolder"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_sftpServer, "Description"));
		_hostprivatekey.setDefaultModel(new PropertyModel<String>(_sftpServer, "HostPrivateKey"));
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_sftpname.setEnabled(editable);
			_sftpport.setEnabled(editable);
			_uploadfolder.setEnabled(editable);
			_beschreibung.setEnabled(editable);
			_hostprivatekey.setEnabled(editable);
			_users.setDisabled(!editable);
		}catch(Exception e){
		}
	}
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		if(_isStarted) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("warSFTPServerStarted", this), INFOBOX_STATUS.warning);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 

	
	
	private void refreshUsersList(){
		try{
			
			UserFilterClass ufc = new UserFilterClass();
			ufc.setAccessType(UsersAccessType.SFTP);
			
			List<User> availabelUsers = StaticLocators.getUsers().getEntities(-1, -1, " order by benutzername ", ufc);
			for(User us : availabelUsers) {
				boolean found=false;
				for(ISelectData data : _lstUsers) {
					if(data.getId()==us.getId()) {
						data.setSelected(false);
						found=true;
					}
				}
				if(!found) {
					_lstUsers.add(new SelectData(us.getId(), us.getBenutzerName(), false, false));
				}
			}
			
			if(_sftpServer!=null && _sftpServer.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation2(_sftpServer.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.SFTP_USER_TO_SFTP_SERVER);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstUsers) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsInfo = new DmsException(1000, "E", 
				_className, 
				"refreshUsersList",
				null,
				e,
				_sftpServer
			);
			StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
		}
		return;
	}
	
}
