/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.smtp;


import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.select2.ISelectData;
import org.biffo.dms.components.select2.SelectData;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Variables.Variable;
import org.biffo.dms.entity.Variables.VariableGroupType;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.entity.user.Users.User;
import org.biffo.dms.entity.user.Users.UserFilterClass;
import org.biffo.dms.entity.user.Users.UsersAccessType;
import org.biffo.dms.helpers.ConfigHelpers;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.users.UserSelectorPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
import de.kleinekuh.entity.types.RecordStatusType;

public class SMTPServerPanel extends Panel{

	private static final String _className = "SMTPServerPanel";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";
	
	private TextField<String> _smtpname, _smtpport, _from, _uploadfolder, _supportedfiletypes;
	private ErrorLabel _smtpname_info, _smtpport_info, _from_info, _uploadfolder_info, _supportedfiletypes_info, _smtpusers_info;

	private CheckBox _withauth; 
		
	private TextArea<String> _beschreibung;
	private Label _lblTitel;
	
	private UserSelectorPanel _users;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;
	
	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	
	private DmsWebSession _session = DmsWebSession.getSession();
	 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	
	private SmtpServerImpl _smtpServer, _smtpServer_Backup;
	private boolean _isStarted=false;
	
	
	private List<ISelectData> _lstUsers = new LinkedList<ISelectData>();
	
	public SMTPServerPanel(String id, int idSmtpServer,  Object origin) {
		this(id, null, origin);
	}
	
	public SMTPServerPanel(String id, SmtpServerImpl smtpServer,  Object origin) {
		super(id);
		_origin = origin;
		_smtpServer = smtpServer;
		_smtpServer_Backup = smtpServer;

        if(null==_smtpServer) {
        	_smtpServer = new SmtpServerImpl();
        }else {
        	_isStarted = DmsWebApplication.getProcessQueue().isRunning(_smtpServer.getServerName());
        }

		refreshUsersList();
		
		_form = new Form<Void>("form");
		_form.setOutputMarkupId(true);
		add(_form);
		
        addTextFields();
        addCheckBoxen();
        addAdditionalControls();
        
        addToolBar();
        addInfobox();
        
        setFormComponentsEditable(false);
		setOutputMarkupId(true);
		if(-1==_smtpServer.getId()) {
			_smtpServer = new SmtpServerImpl();
			_panelTitel.setTitel(ResourceText.getText("titNewSMTPServer", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditSMTPServer", this));
		}
	 
		
		refreshFormObjects();
	
	}
	
	private void addTextFields(){
		
				
		_smtpname= new TextField<String>("smtpname");
		_smtpname.setOutputMarkupId(true);
        _form.add(_smtpname); 
        _smtpname_info = new ErrorLabel("smtpname_info");
        _form.add(_smtpname_info);
		
		_smtpport= new TextField<String>("smtpport");
		_smtpport.setOutputMarkupId(true);
	    _form.add(_smtpport);
	    _smtpport_info = new ErrorLabel("smtpport_info");
        _form.add(_smtpport_info);
		
		_from= new TextField<String>("from");
		_from.setOutputMarkupId(true);
	    _form.add(_from);
	    _from_info = new ErrorLabel("from_info");
        _form.add(_from_info);
        
        
        _supportedfiletypes= new TextField<String>("supportedfiletypes");
        _supportedfiletypes.setOutputMarkupId(true);
	    _form.add(_supportedfiletypes);
	    _supportedfiletypes_info = new ErrorLabel("supportedfiletypes_info");
        _form.add(_supportedfiletypes_info);
        
        

        
        _uploadfolder= new TextField<String>("uploadfolder");
        _uploadfolder.setOutputMarkupId(true);
        _form.add(_uploadfolder);
        
        _uploadfolder_info = new ErrorLabel("uploadfolder_info");
        _form.add(_uploadfolder_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	
	private void addCheckBoxen(){
        
		_withauth = new CheckBox("withauth");
		_withauth.setOutputMarkupId(true);
		_withauth.add(new AjaxFormComponentUpdatingBehavior("onclick") {
			private static final long serialVersionUID = 1L;

			@Override
            protected void onUpdate(AjaxRequestTarget target) {
				if(_smtpServer.isWithAuthentication())_users.setDisabled(false);
				else _users.setDisabled(true);
				target.add(_withauth);
                target.add(_users);
            }
        }); 
        _form.add(_withauth);
        
        
	}
	
	private void addAdditionalControls(){
        
        _users = new UserSelectorPanel("users",  _lstUsers);
        _users.setOutputMarkupId(true);
        _users.setDisabled(true);
        _form.add(_users);
        
        _smtpusers_info = new ErrorLabel("smtpusers_info");
        _form.add(_smtpusers_info);
		
		
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(SMTPServerPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		if(!_isStarted)_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("Edit")) edit(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Save")) save(target, true);
		else if(event.equals("Ok")) ok(target);

		target.add(_form);
		target.add(_toolBar);
	}
	
	private boolean checkShowError(SmtpServerImpl serverImpl, AjaxRequestTarget target) {
		
		boolean error=false;
		if(serverImpl==null) return false;
		
		 
		if(Strings.isEmpty(serverImpl.getServerName())){
			_smtpname_info.setErrorText(ResourceText.getText("errTextServerNameWrong", this));
			_smtpname_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkServerNameExists(_smtpServer.getId(), _smtpServer.getServerName(), VariableGroupType.SMTP)){
			_smtpname_info.setErrorText(ResourceText.getText("errTextServerNameExists", this));
			_smtpname_info.setVisible(true);
			error = true;
		}else {
			_smtpname_info.setVisible(false);
		}
		
		if(_smtpServer.getServerPort()<=0){
			_smtpport_info.setErrorText(ResourceText.getText("errTextServerPortWrong", this));
			_smtpport_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkServerPortExists(_smtpServer.getId(), _smtpServer.getServerPort(), VariableGroupType.SMTP)){
			_smtpport_info.setErrorText(ResourceText.getText("errTextServerPortExists", this));
			_smtpport_info.setVisible(true);
			error=true;
		}else if(ConfigHelpers.portUsed("localhost", _smtpServer.getServerPort())){
			_smtpport_info.setErrorText(ResourceText.getText("errTextServerPortUsed", this));
			_smtpport_info.setVisible(true);
		}else {
			_smtpport_info.setVisible(false);
		}
		
 
		
		if(Strings.isEmpty(_smtpServer.getServerFolder())){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderWrong", this));
			_uploadfolder_info.setVisible(true);
			error = true;
		}else if(ConfigHelpers.checkUploadFolderExists(_smtpServer.getId(), _smtpServer.getVariableByKey("folder").getId(), _smtpServer.getServerFolder(), VariableGroupType.SMTP)){
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderUsed", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.checkFileSystemFolderExists(_smtpServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotExists", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else if(!ConfigHelpers.isWritable(_smtpServer.getServerFolder())) {
			_uploadfolder_info.setErrorText(ResourceText.getText("errTextServerFolderNotWritable", this));
			_uploadfolder_info.setVisible(true);
			error=true;
		}else {
			_uploadfolder_info.setVisible(false);
		}
		
		if(_smtpServer.isWithAuthentication() && _users.getSelectionList().size()==0) {
			_smtpusers_info.setErrorText(ResourceText.getText("errUsersNotSelected", this));
			_smtpusers_info.setVisible(true);
			error=true;
		}else {
			_smtpusers_info.setVisible(false);
		}
		
		if(Strings.isEmpty(serverImpl.getFrom())){
			_from_info.setErrorText(ResourceText.getText("errTextFromNotExists", this));
			_from_info.setVisible(true);
			error=true;
		}else {
			_from_info.setVisible(false);
		}
		
		
		target.add(_smtpname_info);
		target.add(_smtpport_info);		
		target.add(_uploadfolder_info);
		target.add(_smtpusers_info);
		target.add(_from_info);
	 
		return error;
		
	}
	
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			
			if(checkShowError(_smtpServer, target))return false;
			
			boolean b=false;
			if(b)return b; 
			
			HashMap<String, Variable> hmVariables = _smtpServer.getVariables();
			Set<String> keys = hmVariables.keySet();
			if(_smtpServer.getId()!=-1) {
				Date aenderungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setGeaendertAm(aenderungsDatum);
					var.setGeaendertVon(_session.getCurrentUser());
					var.setUUID(EntityUUID.createVariableUUID(var));
					StaticLocators.getVariables().updateEntity(var, true);
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoSMTPServerChanged", this, _smtpServer.getServerName()),
						null,
						_smtpServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}else {
				
				Date erstellungsDatum = new Date();
				for(String key: keys) {
					Variable var = hmVariables.get(key);
					var.setErstelltAm(erstellungsDatum);
					var.setGeaendertAm(var.getErstelltAm());
					var.setErstelltVon(_session.getCurrentUser());
					var.setGeaendertVon(var.getErstelltVon());
					var.setUUID(EntityUUID.createVariableUUID(var));
					int varID = StaticLocators.getVariables().getEntityId(var);
					if(-1==varID) {
						varID = StaticLocators.getVariables().addEntity(var);
						if(var.getVariableName().equals("port"))_smtpServer.setId(varID);
					}else {
						StaticLocators.getVariables().updateEntity(var, true);
					}
				}
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className, 
						"save",
						ResourceText.getText("dmsInfoSMTPServerCreated", this, _smtpServer.getServerName()),
						null,
						_smtpServer
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				
			}
			
			DmsDBHelpersTag.addReplaceRelations(_smtpServer, _lstUsers, RelationType.SMTP_USER_TO_SMTP_SERVER, _session.getCurrentUser());
			
			_smtpServer_Backup = _smtpServer;
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infSMTPServerSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errSMTPServerSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 _smtpServer = _smtpServer_Backup;
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
					 target.add(_form);
					 target.add(_toolBar);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);					 
				 }
			 } 
		 }catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);			 
		 }

	}
	
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_smtpServer.getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);	 		
	 	}
	}
	
	 
		
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private void refreshFormObjects(){
		
	    _smtpname.setDefaultModel(new PropertyModel<String>(_smtpServer, "ServerName"));
	    _smtpport.setDefaultModel(new PropertyModel<String>(_smtpServer, "ServerPort"));
        _uploadfolder.setDefaultModel(new PropertyModel<String>(_smtpServer, "ServerFolder"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_smtpServer, "Description"));
		_supportedfiletypes.setDefaultModel(new PropertyModel<String>(_smtpServer, "SupportedFileTypes"));
		_from.setDefaultModel(new PropertyModel<String>(_smtpServer, "From"));
		_withauth.setDefaultModel(new PropertyModel<Boolean>(_smtpServer, "WithAuthentication"));
		 
		
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_smtpname.setEnabled(editable);
			_smtpport.setEnabled(editable);
			_uploadfolder.setEnabled(editable);
			_beschreibung.setEnabled(editable);
			_withauth.setEnabled(editable);
			_from.setEnabled(editable);
			_supportedfiletypes.setEnabled(editable);
			
			_users.setDisabled(!editable);
			if(editable && _smtpServer.isWithAuthentication()) {
				_users.setDisabled(false);
			}else {
				_users.setDisabled(true);
			}
			
		}catch(Exception e){
		}
	}
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		if(_isStarted) {
			infobox = new InfoboxPanel("infobox", ResourceText.getText("warSMTPServerStarted", this), INFOBOX_STATUS.warning);
			infobox.setVisible(true);
		}else {
			infobox = new InfoboxPanel("infobox");
			infobox.setVisible(false);
		}
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 

	
	
	private void refreshUsersList(){
		try{
			
			UserFilterClass ufc = new UserFilterClass();
			ufc.setAccessType(UsersAccessType.SMTP);
			
			List<User> availabelUsers = StaticLocators.getUsers().getEntities(-1, -1, " order by benutzername ", ufc);
			for(User us : availabelUsers) {
				boolean found=false;
				for(ISelectData data : _lstUsers) {
					if(data.getId()==us.getId()) {
						data.setSelected(false);
						found=true;
					}
				}
				if(!found) {
					_lstUsers.add(new SelectData(us.getId(), us.getBenutzerName(), false, false));
				}
			}
			
			if(_smtpServer!=null && _smtpServer.getId()!=-1) {
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation2(_smtpServer.getId());
				rfc.setRecordStatus(RecordStatusType.ACTIVE);
				rfc.setRelationStatus(RelationStatusType.ACTIVE);
				rfc.setRelationType(RelationType.SMTP_USER_TO_SMTP_SERVER);
			
				List<Relation> entities = StaticLocators.getRelations().getEntities(-1, -1, "", rfc);
				if(null!=entities && entities.size()>0){
					for(Relation entity : entities){
						for(ISelectData data : _lstUsers) {
							if(data.getId()==entity.getIdrelation1()) {
								data.setSelected(true);
							}
						}
					}
				}
			}
		}catch(Exception e){
			DmsException dmsInfo = new DmsException(1000, "E", 
				_className, 
				"refreshUsersList",
				null,
				e,
				_smtpServer
			);
			StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
		}
		return;
	}
	
}
