/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.server.smtp;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.event.Broadcast;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebApplication;
import org.biffo.dms.DmsWebSession;
 
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.config.DPSmtpServers;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationType;

import org.biffo.dms.entity.Variables.Impl.IVariablesImpl;
import org.biffo.dms.entity.Variables.Impl.SmtpServerImpl;
import org.biffo.dms.processQueue.ProcessLaunchedEvent;
import org.biffo.dms.processQueue.smtpServer.SMTPServerTask;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.administration.AdministrationPage;
import org.biffo.dms.web.administration.AdministrationTabedPanel.TabContent;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;
import org.biffo.dms.web.server.ServerControlPanel;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
 


public class SMTPServersPanelViewList extends Panel{

	private static final String _className="SMTPServersPanelViewList";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	
	private StandardListParameters _thisParameters;
	
	private DPSmtpServers _qtdp = new DPSmtpServers();
	private DefaultDataTable<?, ?> _ddt;
	private Set<SmtpServerImpl> _selectionList = new HashSet<SmtpServerImpl>();
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete, _btStart, _btStop;
	private Form<?> _form;
	
	private DmsWebSession _session = DmsWebSession.getSession();

	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	
	@Override
	protected void onBeforeRender() {
		if(_qtdp!=null) {
			_qtdp.refresh();
			_qtdp.size();
		}
		super.onBeforeRender();
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public SMTPServersPanelViewList(String id) {
		super(id);
		
		initParameters();
		setOutputMarkupId(true);
		addInfobox();
		addModalDefault();
		_form = new Form("dataform");
		_form.setOutputMarkupId(true);
		add(_form);
		
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		
		columns.add(new CheckBoxColumn<SmtpServerImpl>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<SmtpServerImpl> rowModel) {
				return new AbstractCheckBoxModel() {

					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		}); 
		
		columns.add(new LinkColumn(ResourceText.getModelText("colServerName", this), "serverName", "serverName") {

			private static final long serialVersionUID = 1L;

			@Override
			public void onClick(Item item, String componentId, IModel model) {
				SmtpServerImpl fn = (SmtpServerImpl)model.getObject();
				SMTPServerPanel panel = new SMTPServerPanel(SMTPServersPanelViewList.this.getId(), fn, SMTPServersPanelViewList.this);
				SMTPServersPanelViewList.this.replaceWith(panel);
			}
			
		}); 
		
		columns.add(new PropertyColumn<SmtpServerImpl, String>(ResourceText.getModelText("colServerFolder", this), "serverFolder","serverFolder"));
		columns.add(new PropertyColumn<SmtpServerImpl, String>(ResourceText.getModelText("colFrom", this), "From","From"));
		columns.add(new PropertyColumn<SmtpServerImpl, String>(ResourceText.getModelText("colServerPort", this), "serverPort","serverPort"));
		columns.add(new AbstractColumn<IVariablesImpl, String>(new Model<String>("")){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<IVariablesImpl>> item, String componentId, IModel<IVariablesImpl> rowModel) {
				
				
				SmtpServerImpl fn = (SmtpServerImpl)rowModel.getObject();
				if(DmsWebApplication.getProcessQueue().isRunning(fn.getServerName())) {
					rowModel.getObject().setServerActive(true);
				}else {
					rowModel.getObject().setServerActive(false);
				}
				
				ServerControlPanel serverControlPanel = new ServerControlPanel(componentId, rowModel.getObject()) {
					
					private static final long serialVersionUID = 1L;
					@Override
					protected void OnServerStart(AjaxRequestTarget target, IVariablesImpl serverImpl) {
						SMTPServersPanelViewList.this.processEvent("start", target, serverImpl);
					}
					@Override
					protected void OnServerStop(AjaxRequestTarget target, IVariablesImpl serverImpl) {
						SMTPServersPanelViewList.this.processEvent("stop", target, serverImpl);
					}
					@Override
					protected void OnServerSync(AjaxRequestTarget target, IVariablesImpl serverImpl) {
						SMTPServersPanelViewList.this.processEvent("sync", target, serverImpl);
					}
				};
				serverControlPanel.setOutputMarkupId(true);
				item.add(serverControlPanel);
			}
			
		});
		
		columns.add(new AbstractColumn<SmtpServerImpl, String>(ResourceText.getModelText("colBeschreibung", this)){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<SmtpServerImpl>> item, String componentId, IModel<SmtpServerImpl> rowModel) {
				
				String temp = "NA";
				try {
					if(null!=rowModel.getObject().getDescription()) {
						temp = rowModel.getObject().getDescription();
						if(temp.length()>100)temp = temp.substring(0,100);
					};
				}catch(Exception e) {}
				Label lbl = new Label(componentId, temp);
				item.add(lbl);
				
				
			}
		});
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setOutputMarkupId(true);
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		addToolBar(); 
		_form.add(_ddt);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
	}
	
	
	
	
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			
			for(SmtpServerImpl smtpServer : _selectionList) {
				
				RelationFilterClass rfc = new RelationFilterClass();
				rfc.setIdrelation1(smtpServer.getId());
				rfc.setRelationType(RelationType.IMPORT_FOLDER_TO_IMPORT_PROCESS);
				
				if(StaticLocators.getRelations().getNumberOfEntities(rfc)>0) {
					throw new Exception(ResourceText.getText("errServerUsed", this, smtpServer.getServerName()));
				}
				if(DmsWebApplication.getProcessQueue().hasTask(smtpServer.getServerName())) {
					if(!DmsWebApplication.getProcessQueue().isStopped(smtpServer.getServerName())) {
						throw new Exception(ResourceText.getText("errServerStarted", this, smtpServer.getServerName()));
					}else {
						DmsWebApplication.getProcessQueue().remove(smtpServer.getServerName());
					}
				}
				
				Set<String> keys = smtpServer.getVariables().keySet();
				for(String key: keys) {
					StaticLocators.getVariables().deleteEntityById(smtpServer.getVariables().get(key).getId());
				}
				DmsException dmsException = new DmsException(
						2000, 
						"I", 
						_className, 
						"deleteEntities",
						ResourceText.getText("dmsInfoServerDeleted", this, smtpServer.getServerName()),
						null,
						smtpServer
					);
				StaticLocators.getMessageHandler().createMessage(dmsException, _session);
			}
			_selectionList = new HashSet<SmtpServerImpl>();
			infobox.refresh(target, ResourceText.getText("infServerDeleted",this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
			target.add(_form);
						
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errServerDeleted",this)+"\n"+e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		} 
	}
	
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target, null);
			}
		};
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		_btStart = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStart.setUtf8Code("xf0d1");
		_btStart.setEventName("start");
		_btStart.setAlternate(ResourceText.getText("tbAltStart", this));
		_toolBar.add(_btStart);

		_btStop = new ToolbarButton().setButtonType(ToolbarButtonType.USER_DEFINED).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btStop.setUtf8Code("xef71");
		_btStop.setEventName("stop");
		_btStop.setAlternate(ResourceText.getText("tbAltStop", this));
		_toolBar.add(_btStop);

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}

	
	public Set<SmtpServerImpl> getSelection(){
		return _selectionList;
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			SmtpServerImpl ql =(SmtpServerImpl)iter.next();
 
			if(select){
				_selectionList.add(ql);
			}else{
				 _selectionList.remove(ql);
			} 
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target, IVariablesImpl serverImpl) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			_qtdp.refresh();
			_qtdp.size();
			target.add(_dpn);
			target.add(_ddt);
		}else if(event.equals("start")) {
			if(_selectionList.size()>0 || serverImpl!=null)startSMTPServers(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
		}else if(event.equals("stop")) {
			if(_selectionList.size()>0 || serverImpl!=null)stopSMTPServers(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
		}else if(event.equals("sync")) {
			if(_selectionList.size()>0 || serverImpl!=null)refreshSMTPServer(target, serverImpl);
			target.add(_ddt);
			target.add(_dpn);
			target.add(_form);
		}else if(event.equals("Add")) {
			AdministrationPage ap = new AdministrationPage(getPage().getPageReference(), TabContent.SMTP, true);
			setResponsePage(ap);
		}
	}
	
	
	private void startSMTPServers( AjaxRequestTarget target, IVariablesImpl serverImpl) {
		if(serverImpl!=null) {
			startSMTPServer(target, serverImpl);
		}else {
			for(SmtpServerImpl smtpServer : _selectionList) {
				startSMTPServer(target, smtpServer);
			}
		}
		_selectionList = new HashSet<SmtpServerImpl>();
		infobox.refresh(target, ResourceText.getText("infServerStarted",this), INFOBOX_STATUS.success);
		infobox.setVisible(true);
		target.add(wmcInfo);
		target.add(_form);
	}
	private void stopSMTPServers( AjaxRequestTarget target, IVariablesImpl serverImpl) {
		if(serverImpl!=null) {
			stopSMTPServer(target, serverImpl);
		}else {
			for(SmtpServerImpl smtpServer : _selectionList) {
				stopSMTPServer(target, smtpServer);
			}
		}
		_selectionList = new HashSet<SmtpServerImpl>();
		infobox.refresh(target,ResourceText.getText("infServerStopped",this), INFOBOX_STATUS.success);
		infobox.setVisible(true);
		target.add(wmcInfo);
		target.add(_form);
	} 
	

	private void startSMTPServer(AjaxRequestTarget target, IVariablesImpl serverImpl) {
		
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		SmtpServerImpl fn = (SmtpServerImpl)serverImpl;
		
		if(!DmsWebApplication.getProcessQueue().hasTask(fn.getServerName())) {
			DmsWebApplication.getProcessQueue().add( new SMTPServerTask(fn));
			getPage().send(getPage(), Broadcast.DEPTH, new ProcessLaunchedEvent(target));
		}else {
			if(!DmsWebApplication.getProcessQueue().isRunning(fn.getServerName())){
				DmsWebApplication.getProcessQueue().start(fn.getServerName());
			}
		}
	}

	private void stopSMTPServer(AjaxRequestTarget target, IVariablesImpl serverImpl) {
		
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		SmtpServerImpl fn = (SmtpServerImpl)serverImpl;
		
		if(DmsWebApplication.getProcessQueue().hasTask(fn.getServerName())) {
			if(DmsWebApplication.getProcessQueue().isRunning(fn.getServerName())){
				DmsWebApplication.getProcessQueue().stop(fn.getServerName());
			}
		}
	}

	private void removeSMTPServer(AjaxRequestTarget target, IVariablesImpl serverImpl) {
		
		DmsWebApplication.getProcessQueue().pruneFinishedTasks();
		SmtpServerImpl fn = (SmtpServerImpl)serverImpl;
		
		
		if(DmsWebApplication.getProcessQueue().hasTask(fn.getServerName())) {
			if(DmsWebApplication.getProcessQueue().isRunning(fn.getServerName())){
				DmsWebApplication.getProcessQueue().stop(fn.getServerName());
			}
			DmsWebApplication.getProcessQueue().remove(fn.getServerName());
		}
	}
	
	private void refreshSMTPServer(AjaxRequestTarget target, IVariablesImpl serverImpl) {
		removeSMTPServer(target, serverImpl);
		startSMTPServer(target, serverImpl);
	}
	
	
	
	
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.SMTP);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_qtdp.refresh();
				_qtdp.size();
				target.add(_dpn);
				target.add(_ddt);
				target.add(_form);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	
 
	 
	
	

}
