/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.tags;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.NavigationToolbar;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
 
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.DPTags;
import org.biffo.dms.entity.Relation.Relation;
import org.biffo.dms.entity.Relation.RelationFilterClass;
import org.biffo.dms.entity.Relation.RelationStatusType;
import org.biffo.dms.entity.Relation.RelationType;
import org.biffo.dms.entity.Tags.Tag;
import org.biffo.dms.helpers.DmsDBHelpersTag;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class TagsListPanel extends Panel{

	private static final String _className = "TagsListPanel";

	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	

	private DPTags _qtdp = new DPTags();
	private Set<Tag> _selectionList = new HashSet<Tag>();
	private DefaultDataTable<?, ?> _ddt;

	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete;
 
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private DmsWebSession _session = (DmsWebSession)DmsWebSession.getSession();
	private StandardListParameters _thisParameters;
	
	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");

	
	@SuppressWarnings({ "serial", "rawtypes", "unchecked" })
	public TagsListPanel(String id) {
		super(id);
		initParameters();
		addInfobox();
		addModalDefault();
		
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<Tag>(Model.of("")) {

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<Tag> rowModel) {
				return new AbstractCheckBoxModel() {
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});
		
		columns.add(new LinkColumn(ResourceText.getModelText("colTagName", this), "TagName", "TagName") {
			@Override
			public void onClick(Item item, String componentId, IModel model) {
				Tag fn = (Tag)model.getObject();
				TagPanel panel = new TagPanel(TagsListPanel.this.getId(), fn.getId(),  TagsListPanel.this);
				TagsListPanel.this.replaceWith(panel);
			}
		});
		
		columns.add(new AbstractColumn<Tag, String>(ResourceText.getModelText("colTagBeschreibung", this), "Beschreibung"){
			 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<Tag>> item, String componentId, IModel<Tag> rowModel) {
				String temp = "NA";
				try{
					temp = rowModel.getObject().getBeschreibung();
					if(null!=temp){
						if(temp.length()>40) temp = temp.substring(0, 40) + " ...";
					}else{
						temp="NA";
					}
				}catch(Exception e){}
				item.add(new Label(componentId, temp));
			}
			
		});
		
		columns.add(new AbstractColumn<Tag, String>(ResourceText.getModelText("colTagRelations", this)){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<Tag>> item, String componentId, IModel<Tag> rowModel){
				String temp = "NA";
				try {
					
					RelationFilterClass rfc = new RelationFilterClass();
					rfc.setIdTenant(rowModel.getObject().getIdTenant());
					rfc.setIdrelation1(rowModel.getObject().getId());
					rfc.setRelationType(RelationType.TAG_TO_DOCUMENT);
					rfc.setRelationStatus(RelationStatusType.ACTIVE);
					temp = ""+StaticLocators.getRelations().getNumberOfEntities(rfc);
				}catch(Exception e) {}
				item.add(new Label(componentId, temp));
			}
			
		});
		
		columns.add(new AbstractColumn<Tag, String>(new Model<String>("")){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<Tag>> item, String componentId, IModel<Tag> rowModel){
				Tag tag = rowModel.getObject();
				Label lbl = new Label(componentId, tag.getTagValue());
				lbl.setEscapeModelStrings(false);
				lbl.add(new AttributeAppender("class", "tbpltagvalue"));
				item.add(new AttributeAppender("class", "tbpltag"));
				item.add(new AttributeAppender("style", "border: 1px solid #"+tag.getTagBgColor()+"; background-color: #"+tag.getTagBgColor()+";color: #"+tag.getTagFgColor()+";"));
				item.add(lbl);
			}
		});
		
		
		
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){
			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		_form = new Form("dataform");
		
		_ddt.addBottomToolbar(new NavigationToolbar(_ddt));
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		
		addToolBar();
		add(_form);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false) {
			 
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnNbOfItemsChanged(AjaxRequestTarget target, long itemPerPage) {
				target.add(_dpn);
			}
		};
		_dpn.setOutputMarkupId(true);
		add(_dpn);
		
		
		
		_dpn.setOutputMarkupId(true);
		add(_dpn);
	}
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			Tag ql =(Tag)iter.next();
			if(select){
				if(!_selectionList.contains(ql)) _selectionList.add(ql);
			}else{
				if(_selectionList.contains(ql)) _selectionList.remove(ql);
			}
		}
	}
	
	
	
	
	private void deleteEntities(AjaxRequestTarget target){
		try{
			
			for(Tag entityToDelete : _selectionList) {
				ArrayList<Relation> relationsForDelete = new ArrayList<Relation>();
				relationsForDelete.addAll(DmsDBHelpersTag.getRelatedRelations(entityToDelete.getId(), RelationType.TAG_TO_DOCUMENT, _session.getCurrentTenantId()));
				relationsForDelete.addAll(DmsDBHelpersTag.getRelatedRelations(entityToDelete.getId(), RelationType.TAG_TO_IMPORT_PROCESS, _session.getCurrentTenantId()));
				
				for(Relation relation : relationsForDelete) {
					StaticLocators.getRelations().deleteEntityById(relation.getId());
				}
				for(Relation relation : relationsForDelete) {
					if(relation.getRelationType()==RelationType.TAG_TO_DOCUMENT) {
						try {
							StaticLocators.getOpenSearchProcessing().updateDocument(StaticLocators.getDocuments().getEntityById(relation.getIdrelation2()));
						}catch(Exception e) {
							DmsException dmsInfo = new DmsException(1000, "E", 
								_className,
								"deleteEntities",
								null,
								e,
								entityToDelete
							);
							StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
						}
					}
				}
				StaticLocators.getTags().deleteEntityById(entityToDelete.getId());
				DmsException dmsInfo = new DmsException(2000, "I", 
					_className,
					"deleteEntities",
					ResourceText.getText("dmsInfoTagDeleted", this, entityToDelete.getTagValue()),
					null,
					entityToDelete
				);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				
			}
			infobox.refresh(target, ResourceText.getText("infTagDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
			
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errTagDeleted", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
	}
	

	
	public Set<Tag> getSelection(){
		return _selectionList;
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		

		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_form);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_form);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			target.add(_dpn);
			target.add(_form);
		}else if(event.equals("Add")) {
			TagsPage page = new TagsPage(getPage().getPageReference(), -1);
			setResponsePage(page);
		}
	}
	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.TAGS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_form);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_form);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	 
	
}

