/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.usergroups;

import java.util.Date;
 

import org.apache.wicket.Page;
import org.apache.wicket.PageReference;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.util.string.Strings;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.entity.Helpers.EntityUUID;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.models.UserGroupModel;
import org.biffo.dms.web.helpers.ErrorLabel;
import org.biffo.dms.web.helpers.PanelTitel;
import org.biffo.dms.web.helpers.ResourceText;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;
  
  
public class UserGroupPanel extends Panel{

	private static final String _className="UserGroupPanel";

	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "dataformToolbar";

	private TextField<String> _name;
	private TextArea<String> _beschreibung;
	private Label _lblTitel;
	
	private ErrorLabel _name_info;
	
	private Form<?> _form;
	private Toolbar _toolBar;
	private ToolbarButton _btEdit, _btCancel, _btSave, _btOK;
	private Object _origin;
	
	private PanelTitel _panelTitel = new PanelTitel();
	private UserGroupModel _userGroupModel;
	private int _userGroupId;
	
	private DmsWebSession _session = DmsWebSession.getSession();
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	

	public UserGroupPanel(String id, int userGroupId,  Object origin) {
		super(id);

		_origin = origin;
		_userGroupId = userGroupId;
		
		if(userGroupId==-1){
			_userGroupModel = new UserGroupModel();
		}else{
			_userGroupModel = new UserGroupModel(userGroupId);
		}
		
		_form = new Form<Void>("dataform");
		_form.setOutputMarkupId(true);
		add(_form);

		
        addTextFields();
        addCheckBoxen();
        addDropDownBoxen();
        addAdditionalControls();
        addToolBar();
        addInfobox();

        refreshFormObjects();
		setFormComponentsEditable(false);
		setOutputMarkupId(true);
		
		if(_userGroupId==-1) {
			_panelTitel.setTitel(ResourceText.getText("titNewUsergroup", this));
			_toolBar.setViewState(ToolbarState.STATE_NEW);
			setFormComponentsEditable(true);
		}else {
			_panelTitel.setTitel(ResourceText.getText("titEditUsergroup", this));
		}
        
	}

	
	
	private void addTextFields(){
		
		_name = new TextField<String>("name");
		_name.setOutputMarkupId(true);
        _form.add(_name);
        
        _name_info = new ErrorLabel("name_info");
        _form.add(_name_info);
        
		_beschreibung= new TextArea<String>("beschreibung");
		_beschreibung.setOutputMarkupId(true);
        _form.add(_beschreibung);
	}
	

 	private void addCheckBoxen(){}
	
 
	private void addDropDownBoxen(){}
	
	private void addAdditionalControls(){
		_lblTitel = new Label("paneltitel", new PropertyModel<String>(this._panelTitel, "Titel"));
		_lblTitel.setOutputMarkupId(true);
		add(_lblTitel);
	}
	
	
	private void refreshFormObjects(){
		_name.setDefaultModel(new PropertyModel<String>(_userGroupModel, "Name"));
		_beschreibung.setDefaultModel(new PropertyModel<String>(_userGroupModel, "Beschreibung"));
	}
	
	private void closeForm(AjaxRequestTarget target){
		if(_origin instanceof Panel){
			target.add(UserGroupPanel.this.replaceWith((Panel)_origin).getParent());
		}else if(_origin instanceof Page){
		}else if(_origin instanceof PageReference){
			PageReference pg = (PageReference)_origin;
			setResponsePage(pg.getPage());
		}
	}
	
	private void setFormComponentsEditable(boolean editable){
		try{
			_name.setEnabled(editable);
			_beschreibung.setEnabled(editable);
		}catch(Exception e){
			DmsException dmsException = new DmsException(1000,
					"E", 
					_className, 
					ResourceText.getText("errSetFormComponents", this), 
					null, 
					e);
			StaticLocators.getMessageHandler().createMessage(dmsException, _session);
		}
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};

		_btEdit = new ToolbarButton().setButtonType(ToolbarButtonType.EDIT).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btEdit.setAlternate(ResourceText.getText("tbAltEdit", this));
		_toolBar.add(_btEdit);
		
		_btCancel = new ToolbarButton().setButtonType(ToolbarButtonType.CANCEL).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.STATE_NEW | ToolbarState.STATE_CHANGE).setCssClass(toolbarCSS);
		_btCancel.setAlternate(ResourceText.getText("tbAltCancel", this));
		_toolBar.add(_btCancel);
		
		_btOK = new ToolbarButton(_form).setButtonType(ToolbarButtonType.OK).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btOK.setAlternate(ResourceText.getText("tbAltOk", this));
		_toolBar.add(_btOK);
		
		_btSave = new ToolbarButton(_form).setButtonType(ToolbarButtonType.SAVE).setToolbarState(ToolbarState.STATE_CHANGE | ToolbarState.STATE_NEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btSave.setAlternate(ResourceText.getText("tbAltSave", this));
		_toolBar.add(_btSave);	
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		_form.add(_toolBar);
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		
		if(event.equals("Save"))save(target, true);
		else if(event.equals("Ok")) ok(target);
		else if(event.equals("Cancel")) cancel(target);
		else if(event.equals("Edit")) edit(target);

		
		target.add(_form);
		target.add(_toolBar);
		
	}

	
	
	
	
	private void cancel(AjaxRequestTarget target){
		 try {
			 if(_toolBar.getViewState()==ToolbarState.STATE_VIEW || _toolBar.getViewState()==ToolbarState.STATE_NEW ) {
				 closeForm(target);
			 }else {
				 try {
					 refreshFormObjects();
					 _toolBar.setViewState(ToolbarState.STATE_VIEW);
					 setFormComponentsEditable(false);
				 }catch(Exception e2) {
					 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+".\n"+e2.getMessage(), INFOBOX_STATUS.error);
					 infobox.setVisible(true);
					 target.add(wmcInfo);
				 }
			 } 
		 }catch(Exception e) {
			 infobox.refresh(target, ResourceText.getText("errChangeCancel", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			 infobox.setVisible(true);
			 target.add(wmcInfo);
		 }
	}
	
 
	private void edit(AjaxRequestTarget target){
		 try {
			 if(_userGroupModel.getObject().getId()!=-1){
				 _toolBar.setViewState(ToolbarState.STATE_CHANGE);
				 setFormComponentsEditable(true);
			 }
	 	}catch(Exception e) {
	 		infobox.refresh(target, ResourceText.getText("errChangeEditMode", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
	 	}
	}
	
	private boolean ok(AjaxRequestTarget target){
		if(save(target, false)) {
			_toolBar.setViewState(ToolbarState.STATE_VIEW);
			cancel(target);
			return true;
		}
		return false;
	} 
	
	private boolean save(AjaxRequestTarget target, boolean withMessage) {
		try {
			UserGroup userGroup = _userGroupModel.getObject();
			if(checkShowError(userGroup, target))return false;

			userGroup.setUUID(EntityUUID.createUserGroupUUID(userGroup));
			userGroup.setAenderungsDatum(new Date());
			userGroup.setGeaendertVon(_session.getCurrentUser());
			 
			if(userGroup.getId()==-1){
				userGroup.setErstellungsDatum(userGroup.getAenderungsDatum());
				userGroup.setErstelltVon(userGroup.getGeaendertVon());
			}
			
			saveUserGroup(userGroup);
			if(withMessage) {
				infobox.refresh(target, ResourceText.getText("infUsergroupSaved", this), INFOBOX_STATUS.success);
				infobox.setVisible(true);
				target.add(wmcInfo);
				target.add(_lblTitel);
			}
			 return true;
		}catch(Exception e) {
			infobox.refresh(target, ResourceText.getText("errUsergroupSaved", this)+"\n"+ e.toString(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
		return false;
	}
	
	
	private void saveUserGroup(UserGroup userGroup) throws Exception{
		
		if(userGroup.getId()==-1){
			_userGroupId = StaticLocators.getUserGroups().getEntityId(userGroup);
			if(_userGroupId==-1){
				StaticLocators.getUserGroups().addEntity(userGroup);
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"saveUserGroup",
						ResourceText.getText("dmsInfoUsergroupCreated", this, userGroup.getName()),
						null,
						userGroup
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
				_panelTitel.setTitel(ResourceText.getText("titEditUsergroup", this));
				_userGroupId = userGroup.getId();
			}else{
				throw new Exception(ResourceText.getText("errUsergroupExists", this));
			}
			_userGroupModel = new UserGroupModel(_userGroupId);
		}else{
			StaticLocators.getUserGroups().updateEntity(userGroup, true);
			DmsException dmsInfo = new DmsException(2000, "I", 
					_className, 
					"saveUserGroup",
					ResourceText.getText("dmsInfoUsergroupChanged", this, userGroup.getName()),
					null,
					userGroup
					);
			StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			_userGroupModel = new UserGroupModel(_userGroupId);
		}
	}

	private boolean checkShowError(UserGroup userGroup, AjaxRequestTarget target) {
		
		boolean error=false;
		if(userGroup==null) return false;
		
		if(Strings.isEmpty(userGroup.getName())){
			_name_info.setErrorText(ResourceText.getText("errTextUsergroupEmpty", this));
			_name_info.setVisible(true);
			error = true;
		}else {
			_name_info.setVisible(false);
		}
		target.add(_name_info);
		
		return error;
		
	}
	
 
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	} 
	
}
