/**********************************************************************
*	Copyright (C) 2024  Martin Lange
*	This file is part of biffo. OpenSource DMS
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*	info@biffo.de
*	www.biffo.de
**********************************************************************/
package org.biffo.dms.web.usergroups;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.DefaultDataTable;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.model.AbstractCheckBoxModel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.biffo.dms.DmsException;
import org.biffo.dms.DmsWebSession;
import org.biffo.dms.StaticLocators;
import org.biffo.dms.components.custommodal.CustomModalWindow;
import org.biffo.dms.components.infobox.InfoboxPanel;
import org.biffo.dms.components.infobox.InfoboxPanel.INFOBOX_STATUS;
import org.biffo.dms.components.pagging.DmsPagingNavigator;
import org.biffo.dms.dataprovider.users.DPUserGroups;
import org.biffo.dms.entity.user.Usergroups.UserGroup;
import org.biffo.dms.session.parameters.SessionParameterTypes;
import org.biffo.dms.session.parameters.StandardListParameters;
import org.biffo.dms.web.helpers.ResourceText;
import org.biffo.dms.web.htmlhelpers.CheckBoxColumn;
import org.biffo.dms.web.htmlhelpers.LinkColumn;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel;
import org.biffo.dms.web.modals.AbstractModalDefaultPanel.ModalType;

import de.kleinekuh.components.toolbar.Toolbar;
import de.kleinekuh.components.toolbar.ToolbarButton;
import de.kleinekuh.components.toolbar.ToolbarButtonType;
import de.kleinekuh.components.toolbar.ToolbarState;

 
public class UserGroupsPanelViewList extends Panel{

	private static final String _className="UserGroupsPanelViewList";
	private static final long serialVersionUID = 1L;
	private static final String toolbarCSS = "viewListToolbar";
	private final static SimpleDateFormat _sdf2 = new SimpleDateFormat("dd.MM.yyyy HH:mm");

	private DPUserGroups _qtdp = new DPUserGroups();
	private Set<UserGroup> _selectionList = new HashSet<UserGroup>();
	private DefaultDataTable<?, ?> _ddt;
	private Toolbar _toolBar;
	private ToolbarButton _btAll, _btNone, _btRefresh, _btAdd, _btDelete;
	private Form<?> _form;
	
	private WebMarkupContainer wmcInfo;
	private InfoboxPanel infobox;
	private DmsPagingNavigator _dpn;
	
	private DmsWebSession _session = (DmsWebSession)DmsWebSession.getSession();
	private StandardListParameters _thisParameters;

	private CustomModalWindow modalDefault = new CustomModalWindow("modalDefault");
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public UserGroupsPanelViewList(String id) {
		super(id);
		initParameters();
		addInfobox();
		addModalDefault();
		setOutputMarkupId(true);
		
		_form = new Form<Void>("dataform");
		List<IColumn<?,?>> columns = new ArrayList<IColumn<?,?>>();
		columns.add(new CheckBoxColumn<UserGroup>(Model.of("")) {

			private static final long serialVersionUID = 1L;

			@Override
			protected IModel<Boolean> newCheckBoxModel(final IModel<UserGroup> rowModel) {
				return new AbstractCheckBoxModel() {
					private static final long serialVersionUID = 1L;
					@Override
					public void unselect() {
						_selectionList.remove(rowModel.getObject());
					}
					@Override
					public void select() {
						_selectionList.add(rowModel.getObject());
					}
					@Override
					public boolean isSelected() {
						return _selectionList.contains(rowModel.getObject());
					}
					@Override
					public void detach() {
						rowModel.detach();
					}
				};
			}
		});

		 
		
		columns.add(new LinkColumn(ResourceText.getModelText("colUserGroupsName", this), "Name", "Name") {
 
			private static final long serialVersionUID = 1L;
 
			public void onClick(Item item, String componentId, IModel model) {
				UserGroup fn = (UserGroup)model.getObject();
				UserGroupPanel panel = new  UserGroupPanel(UserGroupsPanelViewList.this.getId(), fn.getId(),  UserGroupsPanelViewList.this);
				UserGroupsPanelViewList.this.replaceWith(panel);
			}
		});
		
		
		columns.add(new AbstractColumn<UserGroup, String>(ResourceText.getModelText("colUserGroupsBeschreibung", this), "Beschreibung"){
 
			private static final long serialVersionUID = 1L;

			@Override
			public void populateItem(Item<ICellPopulator<UserGroup>> item, String componentId, IModel<UserGroup> rowModel) {
				String temp = "NA";
				try{
					temp = rowModel.getObject().getBeschreibung();
					if(null!=temp){
						if(temp.length()>40) temp = temp.substring(0, 40);
					}else{
						temp="NA";
					}
				}catch(Exception e){
				}
				item.add(new Label(componentId, temp));
			}
			
		});
		
		columns.add(new AbstractColumn<UserGroup, String>(ResourceText.getModelText("colUserGroupsAenderungsDatum", this), "AenderungsDatum"){
			private static final long serialVersionUID = 1L;
			@Override
			public void populateItem(Item<ICellPopulator<UserGroup>> item, String componentId, IModel<UserGroup> rowModel){
				String label = "NA";
				try{
					label = _sdf2.format(rowModel.getObject().getAenderungsDatum());
				}catch(Exception e){}
				Label lbl = new Label(componentId, label);
				item.add(lbl);
			}
		});
		
		
				
		_ddt = new DefaultDataTable("datatable", columns, _qtdp, 10){

			private static final long serialVersionUID = 1L;

			@Override
			protected void onPageChanged() {
				super.onPageChanged();
				_thisParameters.setCurrentPageNumber(_ddt.getCurrentPage());
				_thisParameters.setSortParam(_qtdp.getSort());
			}
		};
		_ddt.setCurrentPage(_thisParameters.getCurrentPageNumber());
		_ddt.setOutputMarkupId(true);

		
		_form.add(_ddt);
		_form.setOutputMarkupId(true);
		addToolBar();
		add(_form);
		
		_dpn = new DmsPagingNavigator("navigation", _ddt, false, false, false, false);
		_dpn.setOutputMarkupId(true);
		add(_dpn);
	}
	
	private void addToolBar() {
		_toolBar = new Toolbar("toolbar") {
			private static final long serialVersionUID = 1L;

			@Override
			protected void OnButtonClick(String event, AjaxRequestTarget target) {
				processEvent(event, target);
			}
		};
		
		_btAll= new ToolbarButton().setButtonType(ToolbarButtonType.ALL).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btAll.setAlternate(ResourceText.getText("tbAltAll", this));
		_toolBar.add(_btAll);
				
		_btNone=new ToolbarButton().setButtonType(ToolbarButtonType.NONE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btNone.setAlternate(ResourceText.getText("tbAltNone", this));
		_toolBar.add(_btNone);
		
		_btAdd=new ToolbarButton().setButtonType(ToolbarButtonType.ADD).setToolbarState(ToolbarState.STATE_VIEW | ToolbarState.WITH_EVENT).setCssClass(toolbarCSS);
		_btAdd.setAlternate(ResourceText.getText("tbAltAdd", this));
		_toolBar.add(_btAdd);
		
		_btDelete=new ToolbarButton().setButtonType(ToolbarButtonType.DELETE).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btDelete.setAlternate(ResourceText.getText("tbAltDelete", this));
		_toolBar.add(_btDelete);
		
		_btRefresh=new ToolbarButton().setButtonType(ToolbarButtonType.REFRESH).setToolbarState(ToolbarState.STATE_VIEW).setCssClass(toolbarCSS);
		_btRefresh.setAlternate(ResourceText.getText("tbAltRefresh", this));
		_toolBar.add(_btRefresh);
		
		_toolBar.setOutputMarkupId(true);
		_toolBar.setViewState(ToolbarState.STATE_VIEW);
		add(_toolBar);
		
	}
	
	
	

	private void deleteEntities(AjaxRequestTarget target){
		try{
			Iterator<?> iter = _selectionList.iterator();
			while(iter.hasNext()) {
				UserGroup ql =(UserGroup)iter.next();
				StaticLocators.getUserGroups().deleteEntityById(ql.getId());
				
				DmsException dmsInfo = new DmsException(2000, "I", 
						_className,
						"deleteEntities",
						ResourceText.getText("dmsInfoDeleted", this, ql.getName()),
						null,
						ql
						);
				StaticLocators.getMessageHandler().createMessage(dmsInfo, _session);
			}
			_selectionList = new HashSet<UserGroup>();
			infobox.refresh(target, ResourceText.getText("infUsergroupsDeleted", this), INFOBOX_STATUS.success);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}catch(Exception e){
			infobox.refresh(target, ResourceText.getText("errUsergroupsDeleted", this)+"\n"+e.getMessage(), INFOBOX_STATUS.error);
			infobox.setVisible(true);
			target.add(wmcInfo);
		}
	}
	
	
	
	private void fselectAll(boolean select){
		long itemsPerPage = _ddt.getItemsPerPage();
		if(_qtdp.size()<itemsPerPage)itemsPerPage=_qtdp.size();
		long currentPage = _ddt.getCurrentPage();
		Iterator<?> iter =_ddt.getDataProvider().iterator(itemsPerPage*currentPage, itemsPerPage);
		while(iter.hasNext()) {
			UserGroup ql =(UserGroup)iter.next();
			if(select){
				if(!_selectionList.contains(ql)) _selectionList.add(ql);
			}else{
				if(_selectionList.contains(ql)) _selectionList.remove(ql);
			}
		}
	}
	
	private void processEvent(String event, AjaxRequestTarget target) {
		if(event.equals("All")) {
			fselectAll(true);
			target.add(_ddt);
		}else if(event.equals("None")) {
			fselectAll(false);
			target.add(_ddt);
		}else if(event.equals("Delete")) {
			if(_selectionList.size()>0)modalDefault.show(target);
		}else if(event.equals("Refresh")) {
			_ddt.size();
			target.add(_dpn);
			target.add(_ddt);
		}else if(event.equals("Add")) {
			//AdministrationPage ap = new AdministrationPage(getPage().getPageReference(), TabContent.USERGROUPS, true);
			//setResponsePage(ap);	
		}
	}

	
	private void addInfobox() {
		wmcInfo = new WebMarkupContainer("info");
		wmcInfo.setOutputMarkupId(true);
		add(wmcInfo);
		
		infobox = new InfoboxPanel("infobox");
		infobox.setVisible(false);
		infobox.setOutputMarkupId(true);
		wmcInfo.add(infobox);
	}
	
	private void initParameters() {
		_thisParameters = (StandardListParameters) _session.getSessionParameters().getParametersByType(SessionParameterTypes.USERGROUPS);
		if(_thisParameters.getSortParam()!=null) _qtdp.setSort(_thisParameters.getSortParam());
	}
	
	private void addModalDefault() {

		modalDefault.setAutoSize(false);
		modalDefault.setHeightUnit("px");
		modalDefault.setInitialHeight(80);
		modalDefault.setInitialWidth(400);
		modalDefault.setResizable(false);
		modalDefault.showUnloadConfirmation(false);
    	
    	AbstractModalDefaultPanel _amdp = new AbstractModalDefaultPanel(modalDefault) {
 
			private static final long serialVersionUID = 1L;

			@Override
			public void onOk(AjaxRequestTarget target) {
				deleteEntities(target);
				modalDefault.close(target);
				_ddt.size();
				target.add(_dpn);
				target.add(_ddt);
			}
			
			@Override
			public void onCancel(AjaxRequestTarget target) {
				modalDefault.close(target);
				fselectAll(false);
				target.add(_dpn);
				target.add(_ddt);
			}
		};
		_amdp.setModalType(ModalType.DELETE);
		_amdp.setModalText(ResourceText.getText("titDialogDelete", this));
		modalDefault.setContent(_amdp);
    	add(modalDefault);
    	
    }
	
	
	
}
